// dialogs.c
// LiVES (lives-exe)
// (c) G. Finch 2003
// Released under the GPL 2.0
// see file ../COPYING for licensing details

#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "main.h"
#include "interface.h"
#include "support.h"

// for ping-pong loops
extern gboolean dirchange_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType, gpointer user_data); // defined in callbacks.c



void on_warn_mask_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  mainw->warn_mask=!mainw->warn_mask;
}



//Warning dialog
GtkWidget*
create_warn_dialog (gint warn_mask_number)
{
  GtkWidget *dialog2;
  GtkWidget *dialog_vbox2;
  GtkWidget *dialog_action_area2;
  GtkWidget *warning_cancelbutton;
  GtkWidget *warning_okbutton;
  GtkWidget *checkbutton;
  GtkWidget *label;
  GtkWidget *hbox;

  mainw->warn_mask=FALSE;

  dialog2 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog2), _("LiVES: - Warning !"));
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(dialog2),FALSE);
  }
  gtk_widget_modify_bg(dialog2, GTK_STATE_NORMAL, &palette->normal_back);


  gtk_window_set_modal (GTK_WINDOW (dialog2), TRUE);
  gtk_window_set_position (GTK_WINDOW (dialog2), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (dialog2), 350, 200);

  dialog_vbox2 = GTK_DIALOG (dialog2)->vbox;
  gtk_widget_show (dialog_vbox2);

  mainw->warning_label = gtk_label_new (_("warning"));
  gtk_widget_show (mainw->warning_label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), mainw->warning_label, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (mainw->warning_label), GTK_JUSTIFY_CENTER);
  gtk_label_set_line_wrap (GTK_LABEL (mainw->warning_label), FALSE);
  gtk_widget_modify_fg(mainw->warning_label, GTK_STATE_NORMAL, &palette->normal_fore);

  if (warn_mask_number>0) {
    checkbutton = gtk_check_button_new ();
    label=gtk_label_new_with_mnemonic (_("Do _not show this warning again"));
    gtk_label_set_mnemonic_widget (GTK_LABEL (label),checkbutton);
    if (palette->style&STYLE_1&&mainw!=NULL) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (dialog_vbox2), hbox, FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (hbox), checkbutton, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_widget_show_all (hbox);
    GTK_WIDGET_SET_FLAGS (checkbutton, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
    g_signal_connect (GTK_OBJECT (checkbutton), "toggled",
                      G_CALLBACK (on_warn_mask_toggled),
                      NULL);
  }

  dialog_action_area2 = GTK_DIALOG (dialog2)->action_area;
  gtk_widget_show (dialog_action_area2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);

  warning_cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (warning_cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog2), warning_cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (warning_cancelbutton, GTK_CAN_DEFAULT);

  warning_okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (warning_okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog2), warning_okbutton, GTK_RESPONSE_OK);


  GTK_WIDGET_SET_FLAGS (warning_okbutton, GTK_CAN_DEFAULT);

  return dialog2;
}



gint
do_warning_dialog(const gchar *text) {
  return do_warning_dialog_with_check(text, 0);
}


gint do_warning_dialog_with_check (const gchar *text, gint warn_mask_number) {
  return do_warning_dialog_with_check_transient(text,warn_mask_number,GTK_WINDOW(mainw->LiVES));
}



gint
do_warning_dialog_with_check_transient(const gchar *text, gint warn_mask_number, GtkWindow *transient) {
  // show OK/CANCEL, returns FALSE if cancelled
  GtkWidget *warning;
  gint response;

  if (prefs->warning_mask&warn_mask_number) {
    return TRUE;
  }

  warning=create_warn_dialog(warn_mask_number);
  gtk_window_set_transient_for(GTK_WINDOW(warning),transient);
  gtk_label_set_text(GTK_LABEL(mainw->warning_label),text);

  response=gtk_dialog_run (GTK_DIALOG (warning));
  gtk_widget_destroy (warning);

  if (mainw->warn_mask) {
    prefs->warning_mask|=warn_mask_number;
    set_int_pref("lives_warning_mask",prefs->warning_mask);
  }
  return (response==GTK_RESPONSE_OK);
}



void 
do_error_dialog(const gchar *text) {
  // show error/info box
  do_error_dialog_with_check_transient(text,FALSE,0,GTK_WINDOW(mainw->LiVES));
}

void 
do_error_dialog_with_check(const gchar *text, gint warn_mask_number) {
  // show error/info box
  do_error_dialog_with_check_transient(text,FALSE,warn_mask_number,GTK_WINDOW(mainw->LiVES));
}


void 
do_blocking_error_dialog(const gchar *text) {
  // show error/info box - blocks until OK is pressed
  do_error_dialog_with_check_transient(text,TRUE,0,GTK_WINDOW(mainw->LiVES));
}


void 
do_error_dialog_with_check_transient(const gchar *text, gboolean is_blocking, gint warn_mask_number, GtkWindow *transient) {
  // show error/info box

  GtkWidget *err_box=create_dialog3(text,is_blocking,warn_mask_number);
  gtk_window_set_transient_for(GTK_WINDOW(err_box),transient);
  gtk_widget_show(err_box);
  gtk_window_present (GTK_WINDOW (err_box));

  if (is_blocking) {
    gtk_dialog_run(GTK_DIALOG (err_box));
    gtk_widget_destroy(err_box);

    if (mainw!=NULL&&mainw->is_ready==TRUE) {
      gtk_widget_queue_draw(GTK_WIDGET(transient));
    }
  }
}



void 
do_memory_error_dialog (void) {
  do_error_dialog (_ ("\n\nLiVES was unable to perform this operation due to unsufficient memory.\nPlease try closing some other applications first.\n"));
}



gboolean do_progress_dialog(gboolean visible, gboolean cancellable, const gchar *text) {
  // monitor progress, return FALSE if the operation was cancelled
  gint oframeno=0;
  FILE *infofile=NULL;

  gdouble audio_start=mainw->play_start;
  gboolean accelerators_swapped=FALSE;
  gint first_frame,last_frame;
  gboolean finished=FALSE;
  gint64 last_display_ticks=0;  // ticks when last display happened (fixed)

  gint64 tc,event_start=0;

  gint64 ratio=U_SECL/1000000; // how many U_SECs (ticks) in a microsecond

  if (!visible) {
    mainw->startticks=mainw->startsecs=mainw->currticks=0;
    if (!(cfile->event_list==NULL)) {
      // this is for audio, work out the apparent frame when we have variable fps
      audio_start=calc_variable_time(mainw->play_start)*cfile->fps;
    }
  }

  if (prefs->show_player_stats) {
    mainw->fps_measure=0.;
  }

  mainw->cancelled=0;   
  mainw->error=FALSE;
  clear_mainw_msg();

  if (visible) {
    mainw->is_processing=TRUE;
    desensitize();
    procw_desensitize();
    cfile->proc_ptr=create_processing (text);
    
    gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),.01);
    gtk_widget_show(cfile->proc_ptr->processing);
    
    cfile->proc_ptr->frames_done=0;
    
    if (cancellable) {
      gtk_widget_show (cfile->proc_ptr->cancel_button);
    }
    if (cfile->opening&&capable->has_sox&&mainw->playing_file==-1) {
      gtk_widget_show (cfile->proc_ptr->preview_button);
      if (mainw->preview_box!=NULL) gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,_ ("Preview"), NULL);
      gtk_tooltips_set_tip (mainw->tooltips, mainw->m_playbutton,_ ("Preview"), NULL);
      // gtk_widget_remove_accelerator (mainw->playall, mainw->accel_group, GDK_p, 0);
      //gtk_widget_add_accelerator (cfile->proc_ptr->preview_button, "clicked", mainw->accel_group, GDK_p, 0, 0);
      accelerators_swapped=TRUE;
    }
  }

  if (cfile->next_event!=NULL) {
    event_start=get_event_timecode(cfile->next_event->event);
  }

  if (!visible) {
      cfile->last_frameno=cfile->frameno=mainw->play_start;

      mainw->deltaticks=0;
      gettimeofday(&tv, NULL);

      // we subtract this from every calculation to make the numbers smaller
      mainw->startsecs=tv.tv_sec;

      // mainw->origticks is our base for quantising (and is constant for each playback session)
      mainw->origticks=mainw->firstticks=mainw->startticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*ratio;
  }

  //try to open info file - or if internal_messaging is TRUE, we get mainw->msg
  // from the mainw->progress_fn function
  do {
    while (!mainw->internal_messaging&&((!visible&&mainw->mute)||!g_file_test(cfile->info_file,G_FILE_TEST_EXISTS))) {
      if (!visible) {
	// INTERNAL PLAYER
	if (mainw->new_clip!=-1) {
	  do_quick_switch(mainw->new_clip);
	  mainw->new_clip=-1;
	}

	gettimeofday(&tv, NULL);
	mainw->currticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*ratio;

	if (cfile->proc_ptr==NULL&&cfile->next_event!=NULL&&(tc=mainw->currticks-mainw->origticks)>=event_start) {
	  cfile->next_event=process_events (cfile->next_event,mainw->currticks-mainw->origticks,&cfile->frameno);
	  if (cfile->next_event==NULL&&mainw->preview) mainw->cancelled=CANCEL_EVENT_LIST_END;
	  if (!mainw->cancelled) continue;
	  break;
	}
	else {
	    oframeno=cfile->frameno;
	    last_frame=(mainw->playing_sel&&!mainw->is_rendering)?cfile->end:mainw->play_end;
	    first_frame=mainw->playing_sel?cfile->start:1;
	    cfile->frameno+=cfile->play_paused?0:(gint)((mainw->currticks-mainw->startticks+mainw->deltaticks)/mainw->period);
	  if (mainw->noframedrop) cfile->frameno=cfile->frameno>oframeno?oframeno+1:cfile->frameno<oframeno?oframeno-1:cfile->frameno;
	}
	// see if we reached the start (backwards...)
	if (cfile->frameno<first_frame) {
	  // loop backwards...
	  if (mainw->ping_pong==TRUE&&cfile->pb_fps<0.) {
	    dirchange_callback (NULL,NULL,0,0,NULL);
	    oframeno=(cfile->frameno=first_frame)-1;
	  }
	  else {
	    oframeno=(cfile->frameno=last_frame)+1;
	    mainw->startticks=mainw->currticks+mainw->deltaticks-(last_frame-first_frame)*mainw->period;
	  }
	}

	// check if we are done
	if ((cfile->frameno>last_frame)&&last_frame>0) {
	  if (mainw->whentostop==STOP_ON_VID_END) {
	    mainw->cancelled=CANCEL_VID_END;
	  }
	  else {
	    // loop forwards
	    if (mainw->playing_sel) mainw->play_start=cfile->start;
	    if (mainw->ping_pong==TRUE&&cfile->pb_fps>0.) {
	      dirchange_callback (NULL,NULL,0,0,NULL);
	      oframeno=(cfile->frameno=last_frame)+1;
	    }
	    else {
	      oframeno=(cfile->frameno=first_frame)-1;
	    }
	  }
	}
	
	// play next frame
	if (!mainw->cancelled) {
	  // calculate the audio 'frame'
	  mainw->aframeno=(mainw->currticks-mainw->firstticks)*cfile->fps/U_SEC+audio_start;
	  if (mainw->loop_cont==TRUE&&(mainw->aframeno>(mainw->audio_end?mainw->audio_end:cfile->laudio_time*cfile->fps))) {
	    mainw->firstticks=mainw->currticks;
	  }
	  mainw->startticks+=(cfile->frameno-oframeno)*mainw->period;
	  if ((mainw->fixed_fps==-1.&&cfile->frameno!=oframeno)||(mainw->fixed_fps>0.&&((mainw->currticks-last_display_ticks)/U_SEC*ratio*mainw->fixed_fps)>=1.)) {
	    load_frame_image(cfile->frameno,oframeno);
	    last_display_ticks=mainw->currticks;
	  }
	}
      }
      // paused
      if (cfile->play_paused) {
	mainw->startticks=mainw->currticks+mainw->deltaticks;
	if (mainw->ext_playback&&mainw->fixed_fps==-1.) (*vid_playback_plugin->send_keycodes)();
      }
      if (visible) {
	// fixes a problem with opening preview with bg generator
	if (cfile->proc_ptr==NULL) {
	  if (!mainw->cancelled) mainw->cancelled=CANCEL_NO_PROPOGATE;
	}
	//
	else gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
      }
      if (!mainw->cancelled) {
	while (g_main_context_iteration(NULL,FALSE));
      }

      else {
	if (prefs->show_player_stats&&!visible&&mainw->fps_measure>0.) {
	  // statistics
	  gettimeofday(&tv, NULL);
	  mainw->fps_measure/=(gdouble)(U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*ratio-mainw->origticks)/U_SEC;
	}
	if (visible) {
	  if (mainw->preview_box!=NULL&&!mainw->preview) gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,_ ("Play all"), NULL);
	  if (accelerators_swapped) {
	    if (!mainw->preview) gtk_tooltips_set_tip (mainw->tooltips, mainw->m_playbutton,_ ("Play all"), NULL);
	    //gtk_widget_remove_accelerator (cfile->proc_ptr->preview_button, mainw->accel_group, GDK_p, 0);
	    //gtk_widget_add_accelerator (mainw->playall, "activate", mainw->accel_group,GDK_p, 0, GTK_ACCEL_VISIBLE);
	  }
	  if (cfile->proc_ptr!=NULL) {
	    gtk_widget_destroy(GTK_WIDGET(cfile->proc_ptr->processing));
	    g_free(cfile->proc_ptr);
	    cfile->proc_ptr=NULL;
	  }
	  mainw->is_processing=FALSE;
	  if (!(cfile->menuentry==NULL)) {
	      sensitize();
	  }
	}
	else mainw->is_processing=TRUE;
	if (cfile->clip_type==CLIP_TYPE_DISK) unlink(cfile->info_file);
	return FALSE;
      }

      if (visible) {
	g_usleep(prefs->sleep_time);
      }
    }
    // end while
    if (!mainw->internal_messaging) {
      if ((infofile=fopen(cfile->info_file,"r"))) {
	// OK, now we might have some frames
	fgets(mainw->msg,512,infofile);
	fclose(infofile);
      }
    }
    else {
      (*mainw->progress_fn)(FALSE);
    }

#if defined DEBUG
    g_print("msg %s\n",mainw->msg);
#endif

  if (visible&&!accelerators_swapped&&cancellable&&cfile->nopreview==FALSE) {
    gtk_widget_show (cfile->proc_ptr->pause_button);
    gtk_widget_show (cfile->proc_ptr->preview_button);
    if (mainw->preview_box!=NULL) gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,_ ("Preview"), NULL);
      gtk_tooltips_set_tip (mainw->tooltips, mainw->m_playbutton,_ ("Preview"), NULL);
      //gtk_widget_remove_accelerator (mainw->playall, mainw->accel_group, GDK_p, 0);
      //gtk_widget_add_accelerator (cfile->proc_ptr->preview_button, "clicked", mainw->accel_group, GDK_p, 0, 0);
      accelerators_swapped=TRUE;
    }
  
  if (strncmp(mainw->msg,"completed",9)&&strncmp(mainw->msg,"error",5)&&(visible||((strncmp(mainw->msg,"video_ended",11)||mainw->whentostop!=STOP_ON_VID_END)&&(strncmp(mainw->msg,"audio_ended",11)||mainw->preview==TRUE||mainw->whentostop!=STOP_ON_AUD_END)))) {
      // processing not yet completed...
    
    if (visible) {
      // last frame processed ->> will go from cfile->start to cfile->end
      cfile->proc_ptr->frames_done=atoi(mainw->msg);
    }
    
    if (!visible) {
      // INTERNAL PLAYER
      if (mainw->new_clip!=-1) {
	do_quick_switch(mainw->new_clip);
	mainw->new_clip=-1;
      }
      
      gettimeofday(&tv, NULL);
      mainw->currticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*ratio;
      if (cfile->proc_ptr==NULL&&cfile->next_event!=NULL&&(tc=mainw->currticks-mainw->origticks)>=event_start) {
	cfile->next_event=process_events (cfile->next_event,mainw->currticks-mainw->origticks,&cfile->frameno);
	if (cfile->next_event==NULL&&mainw->preview) mainw->cancelled=CANCEL_EVENT_LIST_END;
	if (!mainw->cancelled) continue;
	break;
      }
      else {
	oframeno=cfile->frameno;
	last_frame=(mainw->playing_sel&&!mainw->is_rendering)?cfile->end:mainw->play_end;
	first_frame=mainw->playing_sel?cfile->start:1;
	cfile->frameno+=cfile->play_paused?0:(gint)((mainw->currticks-mainw->startticks+mainw->deltaticks)/mainw->period);
	if (mainw->noframedrop) cfile->frameno=cfile->frameno>oframeno?oframeno+1:cfile->frameno<oframeno?oframeno-1:cfile->frameno;
      }
      // see if we reached the start (backwards...)
      if (cfile->frameno<first_frame) {
	// loop backwards...
	if (mainw->ping_pong==TRUE&&cfile->pb_fps<0.) {
	  dirchange_callback (NULL,NULL,0,0,NULL);
	  oframeno=(cfile->frameno=first_frame)-1;
	}
	else {
	  oframeno=(cfile->frameno=mainw->playing_sel?cfile->end:mainw->play_end)+1;
	  mainw->startticks=mainw->currticks+mainw->deltaticks-(mainw->playing_sel==TRUE?cfile->end-cfile->start:mainw->play_end-1)*mainw->period;
	}
      }
      
      // check if we are done
      if (cfile->frameno>last_frame) {
	if (mainw->whentostop!=STOP_ON_VID_END) {
	  if (mainw->playing_sel) mainw->play_start=cfile->start;
	  // loop forwards
	  if (mainw->ping_pong==TRUE&&cfile->pb_fps>0.) {
	    dirchange_callback (NULL,NULL,0,0,NULL);
	    oframeno=(cfile->frameno=last_frame)+1;
	  }
	  else {
	    oframeno=(cfile->frameno=mainw->play_start)-1;
	  }
	}
	else {
	  mainw->cancelled=CANCEL_VID_END;
	}
      }
      
      // play next frame
      if (!mainw->cancelled) {
	mainw->aframeno=(mainw->currticks-mainw->firstticks)*cfile->fps/U_SEC+audio_start;
	if (mainw->loop_cont==TRUE&&(mainw->aframeno>(mainw->audio_end?mainw->audio_end:cfile->laudio_time*cfile->fps))) {
	  mainw->firstticks=mainw->currticks;
	}
	mainw->startticks+=(cfile->frameno-oframeno)*mainw->period;
	if ((mainw->fixed_fps==-1.&&cfile->frameno!=oframeno)||(mainw->fixed_fps>0.&&((mainw->currticks-last_display_ticks)/U_SEC*ratio*mainw->fixed_fps)>=1.)) {
	  load_frame_image(cfile->frameno,oframeno);
	  last_display_ticks=mainw->currticks;
	}
      }
    }
    // paused
    if (cfile->play_paused) {
      mainw->startticks=mainw->currticks+mainw->deltaticks;
      if (mainw->ext_playback&&mainw->fixed_fps==-1.) (*vid_playback_plugin->send_keycodes)();
    }
    
    // set the progress bar %
    if (visible&&cfile->proc_ptr->frames_done>=cfile->progress_start&&cfile->proc_ptr->frames_done<=cfile->progress_end) gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),(gdouble)(cfile->proc_ptr->frames_done-cfile->progress_start)/(gdouble)(cfile->progress_end-cfile->progress_start+1));
    while (g_main_context_iteration(NULL,FALSE));
    if (mainw->cancelled) {
      if (prefs->show_player_stats&&!visible&&mainw->fps_measure>0.) {
	gettimeofday(&tv, NULL);
	mainw->fps_measure/=(gdouble)(U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*ratio-mainw->origticks)/U_SEC;
      }
      if (visible) {
	if (mainw->preview_box!=NULL&&!mainw->preview) gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,_ ("Play all"), NULL);
	if (accelerators_swapped) {
	  if (!mainw->preview) gtk_tooltips_set_tip (mainw->tooltips, mainw->m_playbutton,_ ("Play all"), NULL);
	  //gtk_widget_remove_accelerator (cfile->proc_ptr->preview_button, mainw->accel_group, GDK_p, 0);
	  //gtk_widget_add_accelerator (mainw->playall, "activate", mainw->accel_group, GDK_p, 0, GTK_ACCEL_VISIBLE);
	}
	if (cfile->proc_ptr!=NULL) {
	  gtk_widget_destroy(cfile->proc_ptr->processing);
	  g_free(cfile->proc_ptr);
	  cfile->proc_ptr=NULL;
	}
	mainw->is_processing=FALSE;
	if (!(cfile->menuentry==NULL)) {
	  sensitize();
	}
      }
      else mainw->is_processing=TRUE;
      if (cfile->clip_type==CLIP_TYPE_DISK) unlink(cfile->info_file);
      return FALSE;
    }
    if (visible) {
      g_usleep(prefs->sleep_time);
    }
  }
  else finished=TRUE;
  } while (!finished);
  
  
#if defined DEBUG
  g_print ("exit pt 3 %s\n",mainw->msg);
#endif

  //play/operation ended
  if (visible) {
    if (cfile->clip_type==CLIP_TYPE_DISK) unlink(cfile->info_file);
    if (mainw->preview_box!=NULL&&!mainw->preview) gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,_ ("Play all"), NULL);
    if (accelerators_swapped) {
      if (!mainw->preview) gtk_tooltips_set_tip (mainw->tooltips, mainw->m_playbutton,_ ("Play all"), NULL);
      //gtk_widget_remove_accelerator (cfile->proc_ptr->preview_button, mainw->accel_group, GDK_p, 0);
      //gtk_widget_add_accelerator (mainw->playall, "activate", mainw->accel_group, GDK_p, 0, GTK_ACCEL_VISIBLE);
    }
    if (cfile->proc_ptr!=NULL) {
      gtk_widget_destroy(cfile->proc_ptr->processing);
      g_free(cfile->proc_ptr);
      cfile->proc_ptr=NULL;
    }
    mainw->is_processing=FALSE;
    if (!(cfile->menuentry==NULL)) {
      // note - for operations to/from clipboard (file 0) we
      // should manually call sensitize() after operation
      sensitize();
    }
  }
  else {
    if (prefs->show_player_stats) {
      if (mainw->fps_measure>0.) {
	gettimeofday(&tv, NULL);
	mainw->fps_measure/=(gdouble)(U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*ratio-mainw->origticks)/U_SEC;
      }
    }
    mainw->is_processing=TRUE;
  }
  
  // get error message (if any)
  if (!strncmp(mainw->msg,"error",5)) {
    int i;
    gchar **array;
    gint numtok=get_token_count (mainw->msg,'|');
    
    array=g_strsplit(mainw->msg,"|",numtok);
    g_snprintf(mainw->msg,512,"\n\n");
    for (i=1;i<numtok;i++) {
      g_strappend(mainw->msg,512,array[i]);
      g_strappend(mainw->msg,512,"\n");
    }
    g_strfreev(array);
    mainw->error=TRUE;
  }
#if defined DEBUG
  g_print("exiting progress dialogue\n");
#endif
  return TRUE;
}



void 
do_auto_dialog (const gchar *text) {
  FILE *infofile;

  gint count=100000./prefs->sleep_time;  // don't want to flash too fast...

  cfile->proc_ptr=create_processing (text);
  gtk_widget_hide (cfile->proc_ptr->stop_button);
  gtk_window_set_modal (GTK_WINDOW (cfile->proc_ptr->processing), TRUE);
      
  gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar),.01);
  gtk_widget_show(cfile->proc_ptr->processing);
  
  clear_mainw_msg();

  while (!(infofile=fopen(cfile->info_file,"r"))) {
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
    while (g_main_context_iteration(NULL,FALSE));
    g_usleep(prefs->sleep_time);
    count--;
  }
  
  fgets(mainw->msg,512,infofile);
  fclose(infofile);
  if (cfile->clip_type==CLIP_TYPE_DISK) unlink(cfile->info_file);

  while (count>0) {
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(cfile->proc_ptr->progressbar));
    while (g_main_context_iteration(NULL,FALSE));
    g_usleep(prefs->sleep_time);
    count--;
  }

  if (cfile->proc_ptr!=NULL) {
    gtk_widget_destroy(cfile->proc_ptr->processing);
    g_free(cfile->proc_ptr);
    cfile->proc_ptr=NULL;
  }

  // get error message (if any)
  if (!strncmp(mainw->msg,"error",5)) {
    gchar **array=g_strsplit(mainw->msg,"|",5);
    g_snprintf(mainw->msg,512,"\n\n%s\n\n%s\n\n%s\n\n%s\n",array[1],array[2],array[3],array[4]);
    g_strfreev(array);
    mainw->error=TRUE;
  }
}




void
too_many_files(void) {
  gchar *warn=g_strdup_printf(_ ("\nSorry, LiVES can only open %d files at once.\nPlease close a file and then try again."),MAX_FILES);
  do_error_dialog(warn);
  g_free(warn);
}

void
tempdir_warning (void) {
    gchar *com=g_strdup_printf(_ ("LiVES was unable to write to its temporary directory.\n\nThe current temporary directory is:\n\n%s\n\nPlease make sure you can write to this directory."),prefs->tmpdir);
    if (mainw!=NULL&&mainw->is_ready) {
      do_error_dialog(com);
    }
    g_free(com);
}


void
do_encoder_fps_error(gdouble fps_restrict) {
  gchar *msg;

  if (fps_restrict==-1) {
    msg=g_strdup_printf (_ ("%s can only save %s at integer fps values.\n\nPlease adjust the playback speed from Tools|Resample Video\nor switch to another save format in Preferences"),prefs->encoder.name,prefs->encoder.of_name);
  }
  else {
    msg=g_strdup_printf (_ ("%s can only save %s at restricted frame rates.\nYou are advised to resample to %.3f frames per second.\n\nPlease adjust the playback speed using Tools|Resample Video\nor switch to another save format in Preferences."),prefs->encoder.name,prefs->encoder.of_name,fps_restrict);
  }
  do_error_dialog(msg);
  g_free(msg);
  return;
}


void
do_encoder_size_error(gint width, gint height) {
  gchar *msg;

  msg=g_strdup_printf (_ ("%s can only save %s at certain frame sizes.\nYou should use a frame size of %dX%d.\n\nPlease adjust the frame size from Tools|Resize All\nor switch to another save format in Preferences."),prefs->encoder.name,prefs->encoder.of_name,width,height);
  do_error_dialog(msg);
  g_free(msg);
  return;
}


void
do_encoder_sox_error(void) {
  gchar *msg;
  msg=g_strdup_printf (_ ("Audio resampling is required for this format.\nPlease install 'sox'\nOr switch to another encoder format in Tools | Preferences | Encoding\n"));
  d_print (msg);
  g_free (msg);
}


gboolean
do_encoder_aspect_error(gint width, gint height, gboolean allow_override) {
  gchar *msg;
  gboolean retcode=FALSE;

  if (!allow_override) {
    msg=g_strdup_printf (_ ("Due to aspect ratio restrictions in the %s encoder,\n a frame size of \n%dX%d should be used.\n\nPlease adjust the frame size from Tools|Resize All\nor switch to another save format in Preferences."),prefs->encoder.name,width,height);
    do_error_dialog(msg);
  }
  else {
    msg=g_strdup_printf (_ ("Due to aspect ratio restrictions in the %s encoder,\n a frame size of \n%dX%d is recommended.\n\nYou may wish to adjust the frame size from Tools|Resize All\nor switch to another save format in Preferences.\nClick 'OK' to encode using the current size, or 'Cancel' to abort saving.\n"),prefs->encoder.name,width,height);
    retcode=do_warning_dialog(msg);
  }
  g_free(msg);
  return retcode;
}

void
do_encoder_arate_error(gint best_arate) {
  gchar *msg;

  if (!capable->has_sox) {
    msg=g_strdup_printf (_ ("%s can only save %s at limited audio rates.\n\nPlease adjust the audio rate from Tools|Audio|Resample audio\nor switch to another save format in Preferences.\n\nA new audio rate of %d Hz is recommended."),prefs->encoder.name,prefs->encoder.of_name,best_arate);
  }
  msg=g_strdup_printf (_ ("%s can only save %s at limited audio rates.\n\nPlease adjust the audio rate from Tools|Audio|Resample audio\nor switch to another save format in Preferences.\n\nA new audio rate of %d Hz is recommended."),prefs->encoder.name,prefs->encoder.of_name,best_arate);
  do_error_dialog(msg);
  g_free(msg);
  return;
}

void do_encoder_acodec_error (void) {
  do_error_dialog (_ ("\n\nThis encoder/format cannot use the requested audio codec.\nPlease set the audio codec in Tools|Preferences|Encoding\n"));
}


gboolean 
do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gboolean anr) {
  gchar *msg1=g_strdup_printf (_ ("\n\nDue to restrictions in the %s format\n"),prefs->encoder.of_name);
  gchar *msg2=g_strdup ("");
  gchar *msg3=g_strdup ("");
  gchar *msg4=g_strdup ("");
  gchar *msg5=g_strdup ("");
  gchar *msg6=g_strdup ("");
  gchar *msg7=g_strdup ("");
  gchar *msg_a,*msg_b;

  gboolean ret;

  GtkWidget *prep_dialog;

  if ((arate>0&&arate!=cfile->arate)||(fps>0.&&fps!=cfile->fps)||(fps_denom>0&&(fps_num*1.)/(fps_denom*1.)!=cfile->fps)||(!anr&&(cfile->hsize!=width||cfile->vsize!=height)&&height*width>0)) {
    g_free (msg2);
    msg2=g_strdup (_ ("LiVES must:\n"));
    if (arate>0&&arate!=cfile->arate) {
      g_free (msg3);
      msg3=g_strdup_printf (_ ("resample audio to %d Hz\n"),arate);
    }
    if (fps>0&&fps!=cfile->fps) {
      g_free (msg4);
      msg4=g_strdup_printf (_ ("resample video to %.3f frames per second\n"),fps);
    }
    else if (fps_denom>0&&fps_num/fps_denom!=cfile->fps) {
      g_free (msg4);
      msg4=g_strdup_printf (_ ("resample video to %d:%d frames per second\n"),fps_num,fps_denom);
    }
    if (!anr&&((cfile->hsize!=width||cfile->vsize!=height)&&height*width>0)) {
      g_free (msg5);
      msg5=g_strdup_printf (_ ("resize video to %d x %d pixels\n"),width,height);
      mainw->fx1_bool=TRUE;
    }
  }
  if (anr) {
    if ((cfile->hsize!=width||cfile->vsize!=height)&&height*width>0) {
      g_free (msg6);
      g_free (msg7);
      msg6=g_strdup_printf (_ ("\nYou may wish to:\n"));
      msg7=g_strdup_printf (_ ("resize video to %d x %d pixels\n"),width,height);
    }
    else anr=FALSE;
  }
  msg_a=g_strconcat (msg1,msg2,msg3,msg4,msg5,msg6,msg7,NULL);
  if (mainw->save_all) {
    msg_b=g_strdup (_ ("\nYou will be able to undo these changes afterwards.\n\nClick `OK` to proceed, `Cancel` to abort.\n\n"));
  }
  else {
    msg_b=g_strdup (_ ("\nChanges applied to the selection will not be permanent.\n\n"));
  }
  g_free (msg1);
  g_free (msg2);
  g_free (msg3);
  g_free (msg4);
  g_free (msg5);
  g_free (msg6);
  g_free (msg7);
  prep_dialog=create_encoder_prep_dialog(msg_a,msg_b,anr);
  g_free (msg_a);
  g_free (msg_b);
  ret=(gtk_dialog_run(GTK_DIALOG (prep_dialog))==GTK_RESPONSE_OK);
  gtk_widget_destroy (prep_dialog);
  return ret;
}


void
perf_mem_warning(void) {
  do_error_dialog(_ ("\n\nLiVES was unable to record a performance. There is currently insufficient memory available.\nTry recording for just a selection of the file."));
}

gint
do_clipboard_fps_warning(void) {
  if (prefs->warning_mask&WARN_MASK_FPS) {
    return TRUE;
  }
  return do_warning_dialog_with_check(_ ("The playback speed (fps), or the audio rate\n of the clipboard does not match\nthe playback speed or audio rate of the clip you are inserting into.\n\nThe insertion will be adjusted to fit into the clip.\n\nPlease press Cancel to abort the insert, or OK to continue."),WARN_MASK_FPS);
}



gboolean
do_comments_dialog (void) {
  gboolean response;

  commentsw=create_comments_dialog();
  if ((response=(gtk_dialog_run(GTK_DIALOG (commentsw->comments_dialog))==GTK_RESPONSE_OK))) {
    g_snprintf (cfile->title,256,"%s",gtk_entry_get_text (GTK_ENTRY (commentsw->title_entry)));
    g_snprintf (cfile->author,256,"%s",gtk_entry_get_text (GTK_ENTRY (commentsw->author_entry)));
    g_snprintf (cfile->comment,256,"%s",gtk_entry_get_text (GTK_ENTRY (commentsw->comment_entry)));
  }
  gtk_widget_destroy (commentsw->comments_dialog);
  g_free (commentsw);
  
  return response;
}


void 
do_keys_window (void) {
  do_text_window (_ ("Show Keys"),_ ("You can use the following keys during playback to control LiVES:-\n\nRecordable keys (press 'r' before playback to make a recording)\n-----------------------\nctrl-left                     skip back\nctrl-right                   skip forwards\nctrl-up                      faster/increase effect\nctrl-down                 slower/decrease effect\nctrl-space                reverse direction\nctrl-backspace         freeze frame\nn                             nervous\nctrl-page up            previous clip\nctrl-page down        next clip\n\nctrl-1                       toggle real-time effect 1\nctrl-2                       toggle real-time effect 2\n                                          ...etc...\nctrl-0                       real-time effects off\n\nk           grab keyboard for last activated effect\nm         switch effect mode (when effect has keyboard grab)\nx                     swap background/foreground\nf1                           store/switch to clip mnemonic 1\nf2                           store/switch to clip mnemonic 2\n                                          ...etc...\nf12                          clear function keys\n\n\n Other playback keys\n-----------------------------\np                             play all\ny                             play selection\nq                             stop\nf                               fullscreen\ns                              separate window\nd                             double size\ng                             ping pong loops\n"));

}

void 
do_messages_window (void) {
  GtkTextIter start_iter,end_iter;
  gtk_text_buffer_get_start_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(mainw->textview1)),&start_iter);
  gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(mainw->textview1)),&end_iter);
  do_text_window (_ ("Message History"),gtk_text_iter_get_text (&start_iter,&end_iter));
}


void 
do_text_window (const gchar *title, const gchar *text) {
  GtkWidget *text_window=create_text_window (title,text);
  gtk_widget_show (text_window);
}



void 
do_firstever_dialog (void) {
  startup_message_nonfatal (g_strdup (_ ("\n\nWelcome to LiVES. If this is your first time using LiVES, please check all the\nPreferences using Control-P. In particular make sure the temporary directory is located somewhere with plenty of diskspace.\nHave fun !\n")));
}


void 
do_upgrade_error_dialog (void) {
  startup_message_nonfatal (g_strdup (_("After upgrading/installing, you may need to adjust the <prefix_dir> setting in your ~/.lives file")));
}


