// events.h
// LiVES
// (c) G. Finch 2005 <salsaman@xs4all.nl>
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details


// functions/structs for event_lists and events

#ifndef __HAS_EVENTS_H__
#define __HAS_EVENTS_H__

#include "weed.h"

typedef struct _lives_event_list event_list_t;
typedef struct _lives_event event_t;

struct _lives_event {
  event_t *next;
  event_t *prev;
  weed_plant_t *event;
};


struct _lives_event_list {
  event_t *first;
  event_t *last;
};


#define LIVES_PORT_TYPE_EVENT 1024

#define LIVES_EVENT_HINT_FRAME 1
#define LIVES_EVENT_HINT_EFFECT_INIT 2
#define LIVES_EVENT_HINT_EFFECT_DEINIT 3
#define LIVES_EVENT_HINT_EFFECT_MAP 4
#define LIVES_EVENT_HINT_PARAM_CHANGE 5


// TODO - rename these to append_*
event_list_t *append_frame_event (event_list_t *event_list, gint64 tc, int numframes, int *clips, int *frames);
event_list_t *append_effect_init_event (event_list_t *event_list, gint64 tc, int filter_idx);
gboolean add_effect_deinit_event (event_list_t *event_list, gint64 tc, event_t *init_event);
gboolean add_effect_map_event (event_list_t *event_list, gint64 tc, event_t **init_events);

/* will either insert or replace */
event_list_t *insert_frame_event_at (event_list_t *event_list, gint64 tc, int numframes, int *clips, int *frames, event_t **shortcut);
void remove_frame_from_event (event_list_t *event_list, event_t *event, gint track);
void remove_end_blank_frames (event_list_t *event_list);

event_t *process_events (event_t *next_event, gint64 curr_tc, int *frameno); // RT playback
void event_list_add_offset (gint64 ticks, event_list_t *event_list);
void event_list_add_track (event_list_t *event_list, gint layer);
void event_list_free (event_list_t *event_list);
void event_list_replace_events (event_list_t *event_list, event_list_t *new_event_list); //replace events in event_list with events in new_event_list
event_list_t *event_copy_and_append (event_t *in_event, event_list_t *event_list); // called during quantisation 
gint count_events (event_list_t *event_list, gboolean all_events); // if all_events is FALSE we only count FRAME events

gint event_list_to_block (event_list_t *event_list); // for reorder/backup
gdouble event_list_get_end_secs (event_list_t *event_list);
gdouble event_list_get_start_secs (event_list_t *event_list);

event_t *get_last_frame_event (event_list_t *event_list);
event_t *get_first_frame_event (event_list_t *event_list);

event_t *get_next_frame_event (event_t *event);
event_t *get_prev_frame_event (event_t *event);

event_t *get_frame_event_at (event_list_t *event_list, gint64 tc, event_t *shortcut);


//////////////////////////////////////////////////////////

#include "weed-utils.h"

inline gint64 get_event_timecode (weed_plant_t *);
inline int get_event_hint (weed_plant_t *);
gboolean is_blank_frame (weed_plant_t *);
int get_frame_event_clip (weed_plant_t *, int layer);


//////////////////////////////////////////////////////////
///// render details //////////

typedef struct {
  gint width;
  gint height;
  gdouble fps;
} render_details;



////////////////////////////////////////////////////////
//// UI stuff ///////

GtkWidget *events_rec_dialog (void);
gint get_render_choice (void);
void deal_with_render_choice (gboolean add_deinit);

#define RENDER_CHOICE_DISCARD 0
#define RENDER_CHOICE_PREVIEW 1
#define RENDER_CHOICE_SAME_CLIP 2
#define RENDER_CHOICE_NEW_CLIP 3
#define RENDER_CHOICE_MULTITRACK 4
#define RENDER_CHOICE_EVENT_LIST 5

GtkWidget *create_event_list_dialog (event_list_t *event_list);
GtkWidget *create_render_details_dialog (render_details *rdet);

////////////////////////////////////////////////////////////////
/// rendering

gboolean render_to_clip (gboolean new_clip); // render to clip
gboolean start_render_effect_events (event_list_t *event_list); // render to clip







#endif // __HAS_EVENTS_H
