// framedraw.c
// LiVES
// (c) G. Finch (salsaman@xs4all.nl)
// see file COPYING for licensing details : released under the GNU GPL2

// functions for the 'framedraw' widget - lets users draw on frames :-)

#include "main.h"
#include "paramspecial.h"
#include "callbacks.h"
#include "support.h"
#include "interface.h"

// set by mouse button press
static gint xstart,ystart;
static gboolean b1_held;

void  
widget_add_framedraw (GtkVBox *box, gint start, gint end, gboolean add_reset, lives_special_framedraw_rect_t *framedraw) {
  // adds the frame draw widget to box
  // the redraw button should be connected to an appropriate redraw function
  // after calling this function

  // an example of this is in 'trim frames'

  GtkWidget *vseparator;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *hbox_rst;
  GtkWidget *label;
  GtkObject *spinbutton_adj;
  GtkWidget *spinbutton;
  GtkWidget *label2;
  GtkWidget *framedraw_scale;
  gint hsize,vsize;
  GtkWidget *frame;

  b1_held=FALSE;
  mainw->framedraw_reset=NULL;

  vseparator = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (GTK_WIDGET (box)->parent), vseparator, FALSE, FALSE, 0);
  gtk_widget_show (vseparator);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (GTK_WIDGET (box)->parent), vbox, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);

  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
  gtk_widget_modify_bg (frame, GTK_STATE_NORMAL, &palette->normal_back);

  label = gtk_label_new (_("Preview"));
  gtk_frame_set_label_widget (GTK_FRAME (frame), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  hsize=cfile->hsize;
  vsize=cfile->vsize;
  if (hsize<58) {
    hsize=58;
  }
  if (vsize<32) {
    vsize=32;
  }

  gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
  mainw->framedraw=gtk_event_box_new();
  gtk_widget_set_size_request (mainw->framedraw, hsize, vsize);
  gtk_widget_modify_bg (mainw->framedraw, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_widget_set_events (mainw->framedraw, GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE | GDK_BUTTON_PRESS);

  mainw->framedraw_frame=start;

  // create a blank bitmap mask
  if (!(mainw->framedraw_bitmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_bitmap);
  }
  mainw->framedraw_bitmap = gdk_pixmap_new (NULL, cfile->hsize, cfile->vsize, 1);

  if (!(mainw->framedraw_bitmapgc==NULL)) {
    g_object_unref (mainw->framedraw_bitmapgc);
  }

  mainw->framedraw_bitmapgc=gdk_gc_new (mainw->framedraw_bitmap);

  mainw->framedraw_image = gtk_image_new_from_pixmap (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (frame), mainw->framedraw);
  gtk_container_add (GTK_CONTAINER (mainw->framedraw), mainw->framedraw_image);

  // load the start frame and show it
  load_framedraw_image();

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);

  spinbutton_adj = gtk_adjustment_new (start, start, end, 1, 10, 0);
  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);

  label2 = gtk_label_new_with_mnemonic (_("    _Frame"));
  gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label2, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  framedraw_scale=gtk_hscale_new_with_range(0,cfile->frames,1);
  gtk_box_pack_start (GTK_BOX (hbox), framedraw_scale, TRUE, TRUE, 0);
  gtk_range_set_adjustment(GTK_RANGE(framedraw_scale),GTK_ADJUSTMENT(spinbutton_adj));
  gtk_scale_set_draw_value(GTK_SCALE(framedraw_scale),FALSE);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label2),framedraw_scale);

  mainw->framedraw_reset = gtk_button_new_from_stock ("gtk-refresh");

  if (add_reset) {
    hbox_rst = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), hbox_rst, FALSE, FALSE, 0);

    gtk_button_set_label (GTK_BUTTON (mainw->framedraw_reset),_ ("_Reset Values"));
    gtk_button_set_use_underline (GTK_BUTTON (mainw->framedraw_reset), TRUE);
    gtk_box_pack_start (GTK_BOX (hbox_rst), mainw->framedraw_reset, TRUE, FALSE, 0);
    gtk_widget_set_sensitive (mainw->framedraw_reset,FALSE);

    g_signal_connect (mainw->framedraw_reset, "clicked",G_CALLBACK (on_framedraw_reset_clicked),framedraw);
  }

  g_signal_connect_after (GTK_OBJECT (spinbutton), "value_changed",
			  G_CALLBACK (after_framedraw_frame_spinbutton_changed),
			  framedraw);


  // add mouse fn's so we can draw on frames
  g_signal_connect (GTK_OBJECT (mainw->framedraw), "motion_notify_event",
		    G_CALLBACK (on_framedraw_mouse_update),
		    framedraw);
  g_signal_connect (GTK_OBJECT (mainw->framedraw), "button_release_event",
		    G_CALLBACK (on_framedraw_mouse_reset),
		    framedraw);
  g_signal_connect (GTK_OBJECT (mainw->framedraw), "button_press_event",
		    G_CALLBACK (on_framedraw_mouse_start),
		    framedraw);
  
  if (mainw->framedraw_colourgc!=NULL) {
    g_object_unref (mainw->framedraw_colourgc);
    mainw->framedraw_colourgc=NULL;
  }

  gtk_widget_show_all (vbox);
}


void
framedraw_redraw (lives_special_framedraw_rect_t * framedraw, gboolean reload)
{
  // this will draw the mask and optionally reload the image
  // and then combine them

  gint xstart=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (framedraw->xstart_widget));
  gint ystart=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (framedraw->ystart_widget));
  gint xend=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (framedraw->xend_widget));
  gint yend=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (framedraw->yend_widget));

  if (!GDK_IS_DRAWABLE (mainw->framedraw_bitmap)) return;

  // update framedraw_bitmap
  gdk_gc_set_foreground (mainw->framedraw_bitmapgc, &palette->bm_trans);
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_bitmap),
		      mainw->framedraw_bitmapgc,
		      TRUE,
		      0, 0,
		      cfile->hsize, cfile->vsize
		      );

  gdk_gc_set_foreground (mainw->framedraw_bitmapgc, &palette->bm_opaque);
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_bitmap),
		      mainw->framedraw_bitmapgc,
		      TRUE,
		      0, 0,
		      cfile->hsize, ystart
		      );
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_bitmap),
		      mainw->framedraw_bitmapgc,
		      TRUE,
		      xend+1, ystart,
		      cfile->hsize-xend-1, yend-ystart+1
		      );
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_bitmap),
		      mainw->framedraw_bitmapgc,
		      TRUE,
		      0, yend+1,
		      cfile->hsize, cfile->vsize-yend-1
		      );
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_bitmap),
		      mainw->framedraw_bitmapgc,
		      TRUE,
		      0, ystart,
		      xstart, yend-ystart+1
		      );

  // and call (re)load_framedraw_image to composite it onto
  if (reload) {
    load_framedraw_image();
  }
  else {
    redraw_framedraw_image ();
  }
}





void
after_framedraw_frame_spinbutton_changed (GtkSpinButton *spinbutton, lives_special_framedraw_rect_t *framedraw)
{
  // update the single frame/framedraw preview
  mainw->framedraw_frame=gtk_spin_button_get_value_as_int(spinbutton);
  framedraw_redraw(framedraw, TRUE);
}




void 
load_framedraw_image(void) {
  // this is for the single frame framedraw widget
  // it should be called whenever mainw->framedraw_bitmap changes

  // it will load mainw->framedraw_frame into mainw->framedraw_pixmap
  // and combine this with mainw->framedraw_bitmap to create
  // mainw->framedraw_image

  // see for example framedraw_redraw for how to combine with a mask

  GError *error=NULL;
  gchar buf[256];
  GdkPixbuf *pixbuf=NULL;
  GdkBitmap *dummy_bitmap;

  // TODO - use load_frame func
  g_snprintf(buf,256,"%s/%s/%08d.%s",prefs->tmpdir,cfile->handle,mainw->framedraw_frame,prefs->image_ext);
  do {
    if (!(error==NULL)) {
      g_error_free(error);
      error=NULL;
    }
    pixbuf=gdk_pixbuf_new_from_file(buf,&error);
  } while (!(error==NULL));

  if (!(mainw->framedraw_orig_pixmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_orig_pixmap);
  }

  // we just want the pixmap
  gdk_pixbuf_render_pixmap_and_mask (pixbuf, &mainw->framedraw_orig_pixmap, &dummy_bitmap, 128);
  if (!(dummy_bitmap==NULL)) {
    gdk_pixmap_unref (dummy_bitmap);
  }
  if (pixbuf!=NULL) {
    gdk_pixbuf_unref (pixbuf);
  }

  if (mainw->framedraw_colourgc==NULL) {
    mainw->framedraw_colourgc=gdk_gc_new (mainw->framedraw_orig_pixmap);
  }

  redraw_framedraw_image();
}





void 
redraw_framedraw_image(void) {
  // redraw existing image and mask

  if (!(mainw->framedraw_copy_pixmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_copy_pixmap);
  }

  mainw->framedraw_copy_pixmap=gdk_pixmap_copy (mainw->framedraw_orig_pixmap);
  // do any drawing on the copy_pixmap here
  gtk_image_set_from_pixmap(GTK_IMAGE(mainw->framedraw_image), mainw->framedraw_copy_pixmap, mainw->framedraw_bitmap);
  gtk_widget_queue_draw (mainw->framedraw_image);
}










// using these 3 functions, the user can draw on frames

gboolean
on_framedraw_mouse_start (GtkWidget *widget, GdkEventButton *event, lives_special_framedraw_rect_t *framedraw)
{
  // user clicked in the framedraw widget
  //  if (event->button==1); etc

  if (event->button==1) {
    gdk_window_get_pointer(GDK_WINDOW (mainw->framedraw->window), &xstart, &ystart, NULL);
    b1_held=TRUE;
  }
  return FALSE;
}

gboolean
on_framedraw_mouse_update (GtkWidget *widget, GdkEventButton *event, lives_special_framedraw_rect_t *framedraw)
{
  // pointer moved in the framedraw widget
  gint xcurrent,ycurrent;

  gdk_window_get_pointer(GDK_WINDOW (mainw->framedraw->window), &xcurrent, &ycurrent, NULL);

  switch (framedraw->type) {
  case FD_RECT_DEMASK:
    if (b1_held) {
      // TODO - update all widgets, re-read values
      redraw_framedraw_image();

      // TODO - allow user selectable line colour
      draw_rect_demask (&palette->light_yellow,xstart,ystart,xcurrent,ycurrent,FALSE);
      gtk_image_set_from_pixmap(GTK_IMAGE(mainw->framedraw_image), mainw->framedraw_copy_pixmap, mainw->framedraw_bitmap);
    }
    break;
  }
  return FALSE;
}


gboolean
on_framedraw_mouse_reset (GtkWidget *widget, GdkEventButton *event, lives_special_framedraw_rect_t *framedraw)
{
  gint xend,yend;
  gdk_window_get_pointer(GDK_WINDOW (mainw->framedraw->window), &xend, &yend, NULL);


// user released the mouse button in framedraw widget
  if (event->button==1) {
    b1_held=FALSE;
  }

  switch (framedraw->type) {
  case FD_RECT_DEMASK:
    if (xend>=cfile->hsize) xend=cfile->hsize-1;
    if (yend>=cfile->vsize) yend=cfile->vsize-1;
    
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->xstart_widget),MIN (xstart,xend));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->ystart_widget),MIN (ystart,yend));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->xend_widget),MAX (xstart,xend));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->yend_widget),MAX (ystart,yend));
    break;
  }
  return FALSE;
}


void
after_framedraw_widget_changed (GtkWidget *widget, lives_special_framedraw_rect_t *framedraw)
{
  // redraw mask when spin values change
  framedraw_redraw (framedraw, FALSE);
  if (mainw->framedraw_reset!=NULL) {
    gtk_widget_set_sensitive (mainw->framedraw_reset,TRUE);
  }
}




// various drawing functions

void 
draw_rect_demask (GdkColor *col, gint x1, gint y1, gint x2, gint y2, gboolean filled)
{
  gdk_gc_set_rgb_fg_color (GDK_GC (mainw->framedraw_colourgc),col);
  gdk_draw_rectangle (GDK_DRAWABLE (mainw->framedraw_copy_pixmap),GDK_GC (mainw->framedraw_colourgc), filled, MIN (x1,x2), MIN (y1,y2), ABS (x2-x1), ABS (y2-y1));
}


#ifdef GOOP_TEST

  // this will make a fun toy...

  gchar *com;
  com=g_strdup_printf ("/usr/X11R6/lib/xscreensaver/goop -window-id %lu 2>/dev/null&",GDK_WINDOW_XWINDOW (mainw->framedraw->window));
  system (com);
  g_free (com);
		       
#endif





void
on_framedraw_reset_clicked (GtkButton *button, lives_special_framedraw_rect_t *framedraw)
{
  gdouble x_min,x_max,y_min,y_max;
  gtk_spin_button_get_range (GTK_SPIN_BUTTON (framedraw->xstart_widget),&x_min,NULL);
  gtk_spin_button_get_range (GTK_SPIN_BUTTON (framedraw->ystart_widget),&y_min,NULL);
  gtk_spin_button_get_range (GTK_SPIN_BUTTON (framedraw->xend_widget),NULL,&x_max);
  gtk_spin_button_get_range (GTK_SPIN_BUTTON (framedraw->yend_widget),NULL,&y_max);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->xstart_widget),x_min);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->ystart_widget),y_min);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->xend_widget),x_max);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(framedraw->yend_widget),y_max);

  if (mainw->framedraw_reset!=NULL) {
    gtk_widget_set_sensitive (mainw->framedraw_reset,FALSE);
  }
}
