// multitrack.c
// LiVES
// (c) G. Finch 2005 <salsaman@xs4all.nl>
// released under the GNU GPL 2.0
// see file ../COPYING for licensing details

// multitrack window

// layout is:

// play | clips/params | ctx menu
// ------------------------------
//            timeline
// ------------------------------
//            messages


#include "main.h"
#include "events.h"
#include "callbacks.h"
#include "effects-weed.h"
#include "resample.h"
#include "support.h"


void
scroll_tracks (GtkAdjustment *adj, gpointer user_data) {
  // TODO - remove all tracks from timeline_table, the re-pack them starting at val

  //g_print("val changed\n");
}

void multitrack_view_clips (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  polymorph (mt,POLY_CLIPS);
}


void multitrack_view_in_out (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  polymorph (mt,POLY_IN_OUT);
}


static void time_to_string (lives_mt *mt, gdouble secs, gint length) {
  // TODO - pre-pad to length
  gint hours,mins,rest;
  gchar *string;

  hours=secs/3600;
  secs-=hours*3600.;
  mins=secs/60;
  secs-=mins*60.;
  rest=(secs-((gint)secs)*1.)*100.+.5;
  secs=(gint)secs*1.;
  string=g_strdup_printf("   %02d:%02d:%02d.%02d",hours,mins,(gint)secs,rest);
  gtk_entry_set_text (GTK_ENTRY (mt->timecode),string);
  g_free(string);
}


void clip_select (lives_mt *mt) {
  GList *list=GTK_BOX (mt->clip_inner_box)->children;
  GtkBoxChild *clipbox=NULL;
  gint len;
  int i;
  mt->file_selected=-1;

  if (list==NULL) return;
  multitrack_view_clips(NULL,mt);

  if (mt->clip_selected>=(len=g_list_length(list))) mt->clip_selected=0;
  if (mt->clip_selected<0) mt->clip_selected=len-1;
  for (i=0;i<len;i++) {
    clipbox=g_list_nth_data (list,i);
    if (i==mt->clip_selected) {
      GtkAdjustment *adj;
      gint value=(adj=GTK_RANGE(GTK_SCROLLED_WINDOW(mt->clip_scroll)->hscrollbar)->adjustment)->upper*mt->clip_selected/len;
      gtk_adjustment_clamp_page(adj,value-adj->page_size/2,value+adj->page_size/2);
      gtk_widget_set_state(clipbox->widget,GTK_STATE_SELECTED);
      mt->file_selected=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(clipbox->widget),"file"));
      gtk_widget_set_sensitive (mt->insert, TRUE);
    }
    else gtk_widget_set_state(clipbox->widget,GTK_STATE_NORMAL);
  }
}


gboolean clip_focus_cb (GtkWidget *box, GtkDirectionType motion, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  clip_select(mt);
  return TRUE;
}


gboolean clip_focus_acc (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  // user can press "c" to bring clips into polymorph window
  lives_mt *mt=(lives_mt *)user_data;
  clip_select(mt);
  return TRUE;
}

gboolean mt_prevclip (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  mt->clip_selected--;
  multitrack_view_clips(NULL,mt);
  clip_select(mt);
  return TRUE;
}

gboolean mt_nextclip (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  mt->clip_selected++;
  multitrack_view_clips(NULL,mt);
  clip_select(mt);
  return TRUE;
}

void set_timeline_end_secs (lives_mt *mt, gdouble secs) {
  mt->end_secs=secs;
  gtk_ruler_set_range (GTK_RULER (mt->timeline), 0., mt->end_secs, 0., mt->end_secs);
  gtk_widget_queue_draw (mt->timeline);
  gtk_widget_queue_draw (mt->timeline_table);
}

gboolean mt_tlfor (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;

  if (mainw->playing_file>-1) return TRUE;

  gdouble pos=GTK_RULER (mt->timeline)->position;

  pos+=(gint)mt->fps/mt->fps;

  if (pos>GTK_RULER (mt->timeline)->upper) pos=GTK_RULER (mt->timeline)->upper;
  GTK_RULER (mt->timeline)->position=pos;
  gtk_widget_queue_draw (mt->timeline);
  mt->last_direction=DIRECTION_POSITIVE;

  return TRUE;
}


gboolean mt_tlback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  gdouble pos=GTK_RULER (mt->timeline)->position;

  if (mainw->playing_file>-1) return TRUE;

  pos-=(gint)mt->fps/mt->fps;

  if (pos<0.) pos=0.;

  GTK_RULER (mt->timeline)->position=pos;
  gtk_widget_queue_draw (mt->timeline);
  mt->last_direction=DIRECTION_NEGATIVE;

  return TRUE;
}


void track_select (lives_mt *mt) {
  int i;
  GtkWidget *label,*eventbox;

  for (i=0;i<mt->num_video_tracks;i++) {
    eventbox=g_list_nth_data(mt->video_draws,i);
    label=g_object_get_data (G_OBJECT(eventbox),"label");
    if (i==mt->track_selected) {
      gtk_label_select_region (GTK_LABEL(label),0,-1);
      gtk_widget_set_sensitive (mt->jumpto, g_object_get_data(G_OBJECT(eventbox),"blocks")!=NULL);
    }
    else gtk_label_select_region (GTK_LABEL(label),0,0);
  }
}

gboolean mt_trdown (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;

  mt->track_selected+=(mt->track_selected<(mt->num_video_tracks-1));
  track_select(mt);

  return TRUE;
}


gboolean mt_trup (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;

  mt->track_selected-=(mt->track_selected>0);
  track_select(mt);

  return TRUE;
}


void
paned_pos (GtkWidget *paned, gpointer user_data) {
  // TODO - why doesnt this work ??
  lives_mt *mt=(lives_mt *)user_data;
  //g_print("val changaaed\n");
  gtk_widget_queue_draw (mt->timeline_table);
}

lives_mt *multitrack (event_list_t *event_list, gint orig_file, gdouble fps) {
  GtkWidget *hseparator;
  GtkWidget *vseparator;
  GtkWidget *menubar;
  GtkWidget *menuitem;
  GtkWidget *menuitem_menu;
  GtkWidget *image;
  GtkWidget *separator;
  GtkWidget *playsel;
  GtkWidget *rewind;
  GtkWidget *full_screen;
  GtkWidget *sepwin;
  GtkWidget *fade;
  GtkWidget *loop_video;
  GtkWidget *loop_continue;
  GtkWidget *mute_audio;
  GtkWidget *sticky;
  GtkWidget *quit;
  GtkWidget *add_vid_behind;
  GtkWidget *add_vid_front;
  GtkWidget *tl_vbox;
  GtkWidget *scrollbar;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *label;
  GtkObject *vadjustment;
  GtkObject *spinbutton_adj;

  lives_mt *mt=(lives_mt *)g_malloc(sizeof(lives_mt));
  mt->tl_fixed_length=0.;
  mt->ptr_time=0.;
  mt->video_draws=NULL;
  mt->block_selected=NULL;
  mt->event_list=event_list;
  mt->accel_group = GTK_ACCEL_GROUP(gtk_accel_group_new ());
  mt->fps=fps;

  mt->region_start=mt->region_end=0.;
  mt->region_updating=FALSE;
  mt->is_rendering=FALSE;

  mt->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  if (palette->style&STYLE_1) {
    if (palette->style&STYLE_3) gtk_widget_modify_bg(mt->window, GTK_STATE_NORMAL, &palette->normal_back);
    else gtk_widget_modify_bg(mt->window, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  gtk_window_set_title (GTK_WINDOW (mt->window), g_strdup_printf(_("LiVES-%s: Multitrack Window"),LiVES_VERSION));

  mt->top_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (mt->window), mt->top_vbox);

  menubar = gtk_menu_bar_new ();
  gtk_box_pack_start (GTK_BOX (mt->top_vbox), menubar, FALSE, FALSE, 0);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menubar, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  // File

  menuitem = gtk_menu_item_new_with_mnemonic (_ ("_File"));
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
  
  menuitem_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menuitem_menu);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menuitem_menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  quit = gtk_image_menu_item_new_from_stock ("gtk-quit", mt->accel_group);
  gtk_container_add (GTK_CONTAINER (menuitem_menu), quit);

  // Edit

  menuitem = gtk_menu_item_new_with_mnemonic (_("_Edit"));
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);

  menuitem_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menuitem_menu);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menuitem_menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  mt->clipedit = gtk_image_menu_item_new_with_mnemonic (_("_CLIP EDITOR"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->clipedit);

  gtk_widget_add_accelerator (mt->clipedit, "activate", mt->accel_group,
                              GDK_e, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);


  separator = gtk_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menuitem_menu), separator);
  gtk_widget_set_sensitive (separator, FALSE);

  mt->insert = gtk_image_menu_item_new_with_mnemonic (_("_Insert selected clip"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->insert);

  gtk_widget_add_accelerator (mt->insert, "activate", mt->accel_group,
                              GDK_i, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  gtk_widget_set_sensitive (mt->insert, FALSE);


  mt->delblock = gtk_image_menu_item_new_with_mnemonic (_("_Delete selected block"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->delblock);
  gtk_widget_set_sensitive (mt->delblock, FALSE);

  gtk_widget_add_accelerator (mt->delblock, "activate", mt->accel_group,
                              GDK_d, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);


  mt->jumpto = gtk_image_menu_item_new_with_mnemonic (_("_Jump to next block boundary"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->jumpto);

  gtk_widget_add_accelerator (mt->jumpto, "activate", mt->accel_group,
                              GDK_j, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_widget_set_sensitive (mt->jumpto, FALSE);

  // Play

  menuitem = gtk_menu_item_new_with_mnemonic (_ ("_Play"));
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
  
  menuitem_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menuitem_menu);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menuitem_menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  mt->playall = gtk_image_menu_item_new_with_mnemonic (_("_Play All"));
  gtk_widget_add_accelerator (mt->playall, "activate", mt->accel_group,
                              GDK_p, 0,
                              GTK_ACCEL_VISIBLE);
  gtk_widget_set_sensitive (mt->playall, FALSE);


  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->playall);

  image = gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mt->playall), image);

  playsel = gtk_image_menu_item_new_with_mnemonic (_("Pla_y Selection"));
  gtk_widget_add_accelerator (playsel, "activate", mt->accel_group,
                              GDK_y, 0,
                              GTK_ACCEL_VISIBLE);
  gtk_container_add (GTK_CONTAINER (menuitem_menu), playsel);
  gtk_widget_set_sensitive (playsel, FALSE);

  mt->stop = gtk_image_menu_item_new_with_mnemonic (_("_Stop"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->stop);
  gtk_widget_set_sensitive (mt->stop, FALSE);
  gtk_widget_add_accelerator (mt->stop, "activate", mt->accel_group,
                              GDK_q, 0,
                              GTK_ACCEL_VISIBLE);

  image = gtk_image_new_from_stock ("gtk-stop", GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mt->stop), image);

  rewind = gtk_image_menu_item_new_with_mnemonic (_("Re_wind"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), rewind);

  image = gtk_image_new_from_stock ("gtk-back", GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (rewind), image);
  gtk_widget_set_sensitive (rewind, FALSE);

  gtk_widget_add_accelerator (rewind, "activate", mt->accel_group,
                              GDK_w, 0,
                              GTK_ACCEL_VISIBLE);

  separator = gtk_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menuitem_menu), separator);
  gtk_widget_set_sensitive (separator, FALSE);

  full_screen = gtk_check_menu_item_new_with_mnemonic (_("_Full Screen"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), full_screen);

  gtk_widget_add_accelerator (full_screen, "activate", mt->accel_group,
                              GDK_f, 0,
                              GTK_ACCEL_VISIBLE);

  sepwin = gtk_check_menu_item_new_with_mnemonic (_("Play in _Separate Window"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), sepwin);

  gtk_widget_add_accelerator (sepwin, "activate", mt->accel_group,
                              GDK_s, 0,
                              GTK_ACCEL_VISIBLE);

  fade = gtk_check_menu_item_new_with_mnemonic (_("_Blank Background"));
  gtk_widget_add_accelerator (fade, "activate", mt->accel_group,
                              GDK_b, 0,
                              GTK_ACCEL_VISIBLE);

  gtk_container_add (GTK_CONTAINER (menuitem_menu), fade);
  gtk_widget_set_sensitive (fade, FALSE);

  loop_video = gtk_check_menu_item_new_with_mnemonic (_("(Auto)_loop Video (to fit audio track)"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), loop_video);
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(loop_video),FALSE);
  gtk_widget_add_accelerator (loop_video, "activate", mt->accel_group,
                              GDK_l, 0,
                              GTK_ACCEL_VISIBLE);
  gtk_widget_set_sensitive (loop_video, FALSE);

  loop_continue = gtk_check_menu_item_new_with_mnemonic (_("L_oop Continuously"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), loop_continue);

  gtk_widget_add_accelerator (loop_continue, "activate", mt->accel_group,
                              GDK_o, 0,
                              GTK_ACCEL_VISIBLE);

  mute_audio = gtk_check_menu_item_new_with_mnemonic (_("_Mute"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mute_audio);

  separator = gtk_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menuitem_menu), separator);
  gtk_widget_set_sensitive (separator, FALSE);

  sticky = gtk_check_menu_item_new_with_mnemonic (_("Separate Window 'S_ticky' Mode"));

  gtk_widget_add_accelerator (sticky, "activate", mt->accel_group,
                              GDK_t, 0,
                              GTK_ACCEL_VISIBLE);

  gtk_container_add (GTK_CONTAINER (menuitem_menu), sticky);

  if (capable->smog_version_correct&&prefs->sepwin_type==1) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(sticky),TRUE);
  }



  // Tracks

  menuitem = gtk_menu_item_new_with_mnemonic (_ ("_Tracks"));
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
  
  menuitem_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menuitem_menu);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menuitem_menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  add_vid_behind = gtk_image_menu_item_new_with_mnemonic (_("Add Video Track at _Rear"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), add_vid_behind);

  gtk_widget_add_accelerator (add_vid_behind, "activate", mt->accel_group,
                              GDK_a, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);


  add_vid_front = gtk_image_menu_item_new_with_mnemonic (_("Add Video Track at _Front"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), add_vid_front);

  gtk_widget_add_accelerator (add_vid_front, "activate", mt->accel_group,
                              GDK_a, GDK_CONTROL_MASK|GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);


  // Render

  menuitem = gtk_menu_item_new_with_mnemonic (_ ("_Render"));
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
  
  menuitem_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menuitem_menu);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menuitem_menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  mt->render = gtk_image_menu_item_new_with_mnemonic (_("_Render all to new clip..."));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->render);

  gtk_widget_add_accelerator (mt->render, "activate", mt->accel_group,
                              GDK_r, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  // View

  menuitem = gtk_menu_item_new_with_mnemonic (_ ("_View"));
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);
  
  menuitem_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menuitem_menu);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menuitem_menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  mt->view_events = gtk_image_menu_item_new_with_mnemonic (_("_Event Window"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->view_events);
  gtk_widget_set_sensitive (mt->view_events, FALSE);

  separator = gtk_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menuitem_menu), separator);
  gtk_widget_set_sensitive (separator, FALSE);

  mt->view_clips = gtk_menu_item_new_with_mnemonic (_("_Clips"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->view_clips);

  gtk_widget_add_accelerator (mt->view_clips, "activate", mt->accel_group,
                              GDK_c, 0,
                              GTK_ACCEL_VISIBLE);

  mt->view_in_out = gtk_menu_item_new_with_mnemonic (_("_In/out points"));
  gtk_container_add (GTK_CONTAINER (menuitem_menu), mt->view_in_out);

  gtk_widget_add_accelerator (mt->view_in_out, "activate", mt->accel_group,
                              GDK_i, 0,
                              GTK_ACCEL_VISIBLE);

  g_signal_connect (GTK_OBJECT (quit), "activate",
		    G_CALLBACK (on_quit_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (mt->clipedit), "activate",
		    G_CALLBACK (multitrack_end),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->playall), "activate",
		    G_CALLBACK (multitrack_playall),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->insert), "activate",
		    G_CALLBACK (multitrack_insert),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->view_events), "activate",
		    G_CALLBACK (multitrack_view_events),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->view_clips), "activate",
		    G_CALLBACK (multitrack_view_clips),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->view_in_out), "activate",
		    G_CALLBACK (multitrack_view_in_out),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->stop), "activate",
		    G_CALLBACK (on_stop_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (sepwin), "activate",
		    G_CALLBACK (on_sepwin_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (full_screen), "activate",
		    G_CALLBACK (on_full_screen_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (fade), "activate",
		    G_CALLBACK (on_fade_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (loop_video), "activate",
		    G_CALLBACK (on_loop_video_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (loop_continue), "activate",
		    G_CALLBACK (on_loop_cont_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (mute_audio), "activate",
		    G_CALLBACK (on_mute_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (sticky), "activate",
		    G_CALLBACK (on_sticky_activate),
		    NULL);
  g_signal_connect (GTK_OBJECT (add_vid_behind), "activate",
		    G_CALLBACK (add_video_track_behind),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (add_vid_front), "activate",
		    G_CALLBACK (add_video_track_front),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->render), "activate",
		    G_CALLBACK (on_render_activate),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->jumpto), "activate",
		    G_CALLBACK (on_jumpto_activate),
		    (gpointer)mt);
  g_signal_connect (GTK_OBJECT (mt->delblock), "activate",
		    G_CALLBACK (on_delblock_activate),
		    (gpointer)mt);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_box_pack_start (GTK_BOX (mt->top_vbox), hbox, FALSE, FALSE, 0);

  label=gtk_label_new (_("Timecode"));

  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);

  mt->timecode=gtk_entry_new();
  time_to_string (mt,0.,TIMECODE_LENGTH);
  gtk_editable_set_editable (GTK_EDITABLE(mt->timecode),FALSE);
  gtk_entry_set_max_length(GTK_ENTRY (mt->timecode),TIMECODE_LENGTH);
  gtk_entry_set_width_chars (GTK_ENTRY (mt->timecode),TIMECODE_LENGTH);
  GTK_WIDGET_UNSET_FLAGS (mt->timecode, GTK_CAN_FOCUS);
  gtk_box_pack_start (GTK_BOX (hbox), mt->timecode, FALSE, FALSE, 0);

  hseparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (mt->top_vbox), hseparator, FALSE, FALSE, 0);

  mt->hbox = gtk_hbox_new (FALSE, 10);
  gtk_box_pack_start (GTK_BOX (mt->top_vbox), mt->hbox, FALSE, FALSE, 0);

  mt->play_blank = gtk_image_new_from_pixbuf (mainw->imframe);
  mt->play_box = gtk_vbox_new (FALSE, 10);
  gtk_widget_set_app_paintable(mt->play_box,TRUE);

  gtk_widget_set_size_request (mt->play_box, MT_PLAY_WIDTH, MT_PLAY_HEIGHT);
  gtk_box_pack_start (GTK_BOX (mt->hbox), mt->play_box, FALSE, FALSE, 0);
  gtk_container_add (GTK_CONTAINER (mt->play_box), mt->play_blank);

  vseparator = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (mt->hbox), vseparator, FALSE, FALSE, 0);


  // prepare polymorph box
  mt->poly_box = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mt->hbox), mt->poly_box, TRUE, TRUE, 0);

  // poly clip scroll
  mt->clip_scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (mt->clip_scroll);

  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (mt->clip_scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);

  mt->clip_inner_box = gtk_hbox_new (FALSE, 10);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (mt->clip_scroll), mt->clip_inner_box);

  init_clips (mt,orig_file);


  // poly in_out_box
  mt->in_out_box = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (mt->in_out_box);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start(GTK_BOX(mt->in_out_box),vbox,FALSE,TRUE,0);
  spinbutton_adj = gtk_adjustment_new (0.,0.,0.,1./mt->fps,1.,0.);
  mt->spinbutton_in = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1./mt->fps, 2);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (mt->spinbutton_in),TRUE);
  mt->in_image=gtk_image_new();
  gtk_box_pack_start(GTK_BOX(vbox),mt->in_image,FALSE,TRUE,0);

  mt->in_hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox),mt->in_hbox,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(mt->in_hbox),mt->spinbutton_in,FALSE,FALSE,0);

  mt->checkbutton_start_anchored = gtk_check_button_new_with_mnemonic (_ ("Anchor _start"));
  gtk_box_pack_end(GTK_BOX(mt->in_hbox),mt->checkbutton_start_anchored,FALSE,FALSE,0);

  mt->spin_in_func=g_signal_connect_after (GTK_OBJECT (mt->spinbutton_in), "value_changed",
					   G_CALLBACK (in_out_start_changed),
					   mt);

  mt->check_start_func=g_signal_connect_after (GTK_OBJECT (mt->checkbutton_start_anchored), "toggled",
					    G_CALLBACK (in_anchor_toggled),
					    mt);


  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_end(GTK_BOX(mt->in_out_box),vbox,FALSE,TRUE,0);
  spinbutton_adj = gtk_adjustment_new (0.,0.,0.,1./mt->fps,1.,0.);
  mt->spinbutton_out = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1./mt->fps, 2); // TODO
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (mt->spinbutton_out),TRUE);
  mt->out_image=gtk_image_new();
  gtk_box_pack_start(GTK_BOX(vbox),mt->out_image,FALSE,TRUE,0);
  mt->out_hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox),mt->out_hbox,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(mt->out_hbox),mt->spinbutton_out,FALSE,FALSE,0);
  mt->checkbutton_end_anchored = gtk_check_button_new_with_mnemonic (_ ("Anchor _end"));
  gtk_box_pack_end(GTK_BOX(mt->out_hbox),mt->checkbutton_end_anchored,FALSE,FALSE,0);

  mt->spin_out_func=g_signal_connect_after (GTK_OBJECT (mt->spinbutton_out), "value_changed",
					    G_CALLBACK (in_out_end_changed),
					    mt);

  mt->check_end_func=g_signal_connect_after (GTK_OBJECT (mt->checkbutton_end_anchored), "toggled",
					     G_CALLBACK (out_anchor_toggled),
					     mt);

  gtk_widget_show_all(mt->in_out_box);

  mt->poly_state=POLY_NONE;
  polymorph(mt,POLY_CLIPS);

  // context/info box
  vseparator = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (mt->hbox), vseparator, FALSE, FALSE, 0);

  mt->context_box = gtk_vbox_new (FALSE, 10);
  gtk_box_pack_start (GTK_BOX (mt->hbox), mt->context_box, FALSE, FALSE, 0);

  if (mainw->imsep==NULL) {
    mt->sep_image=NULL;
    hseparator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (mt->top_vbox), hseparator, FALSE, FALSE, 10);
  }
  else {
    hseparator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (mt->top_vbox), hseparator, FALSE, FALSE, 0);
    mt->sep_image = gtk_image_new_from_pixbuf (mainw->imsep);
    gtk_box_pack_start (GTK_BOX (mt->top_vbox), mt->sep_image, FALSE, FALSE, 0);
    hseparator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (mt->top_vbox), hseparator, FALSE, FALSE, 0);
  }

  mt->vpaned=gtk_vpaned_new();
  gtk_box_pack_start (GTK_BOX (mt->top_vbox), mt->vpaned, TRUE, TRUE, 0);

  g_signal_connect (GTK_OBJECT (mt->vpaned), "accept_position",
		    G_CALLBACK (paned_pos),
		    (gpointer)mt);

  tl_vbox = gtk_vbox_new (FALSE, 0);

  gtk_container_add (GTK_CONTAINER (mt->vpaned), tl_vbox);

  mt->timeline_table_header = gtk_table_new (1, 10, TRUE);

  hbox = gtk_hbox_new (FALSE, 10);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(hbox, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }
  gtk_box_pack_start (GTK_BOX (tl_vbox), hbox, FALSE, FALSE, 0);

  vadjustment = gtk_adjustment_new (1.0,1.0,1.0,1.0,1.0,1.0);
  scrollbar=gtk_vscrollbar_new(GTK_ADJUSTMENT(vadjustment));
  gtk_widget_set_sensitive(scrollbar,FALSE);

  gtk_box_pack_start (GTK_BOX (hbox), mt->timeline_table_header, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), scrollbar, FALSE, FALSE, 0);

  mt->timeline_table = gtk_table_new (1, 10, TRUE);

  hbox = gtk_hbox_new (FALSE, 10);

  gtk_box_pack_start (GTK_BOX (tl_vbox), hbox, FALSE, TRUE, 0);

  mt->vadjustment = gtk_adjustment_new (1.0,1.0,1.0,1.0,1.0,1.0);
  mt->scrollbar=gtk_vscrollbar_new(GTK_ADJUSTMENT(mt->vadjustment));

  g_signal_connect (GTK_OBJECT (mt->scrollbar), "value_changed",
		    G_CALLBACK (scroll_tracks),
		    (gpointer)mt);


  gtk_box_pack_start (GTK_BOX (hbox), mt->timeline_table, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), mt->scrollbar, FALSE, FALSE, 0);

  gtk_table_set_row_spacings (GTK_TABLE(mt->timeline_table),5);

  hseparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (tl_vbox), hseparator, FALSE, FALSE, 20);

  if (prefs->open_maximised) {
    gtk_window_maximize (GTK_WINDOW(mt->window));
  }

  mt->num_video_tracks=0;
  init_tracks (mt);

  gtk_widget_reparent (mainw->scrolledwindow,mt->vpaned);

  // add info bar

  gtk_window_add_accel_group (GTK_WINDOW (mt->window), mt->accel_group);

  g_signal_connect (GTK_OBJECT (mt->window), "delete_event",
		    G_CALLBACK (on_mt_delete_event),
		    (gpointer)mt);

  mainw->multitrack=mt;
  add_context_label (mt,(_("Single click on timeline to select a frame.\n")));
  add_context_label (mt,(_("Double click on timeline to select a block.\n")));
  add_context_label (mt,(_("Ctrl-page-up, ctrl-page-down to select a clip.\n")));
  add_context_label (mt,(_("Ctrl-up, ctrl-down to select a single track.\n")));
  add_context_label (mt,(_("Ctrl-left, ctrl-right to move insertion point.\n")));

  gtk_accel_group_connect (GTK_ACCEL_GROUP (mt->accel_group), GDK_Page_Up, GDK_CONTROL_MASK, 0, g_cclosure_new (G_CALLBACK (mt_nextclip),mt,NULL));
  gtk_accel_group_connect (GTK_ACCEL_GROUP (mt->accel_group), GDK_Page_Down, GDK_CONTROL_MASK, 0, g_cclosure_new (G_CALLBACK (mt_prevclip),mt,NULL));

  gtk_accel_group_connect (GTK_ACCEL_GROUP (mt->accel_group), GDK_Left, GDK_CONTROL_MASK, 0, g_cclosure_new (G_CALLBACK (mt_tlback),mt,NULL));
  gtk_accel_group_connect (GTK_ACCEL_GROUP (mt->accel_group), GDK_Right, GDK_CONTROL_MASK, 0, g_cclosure_new (G_CALLBACK (mt_tlfor),mt,NULL));

  gtk_accel_group_connect (GTK_ACCEL_GROUP (mt->accel_group), GDK_Up, GDK_CONTROL_MASK, 0, g_cclosure_new (G_CALLBACK (mt_trup),mt,NULL));
  gtk_accel_group_connect (GTK_ACCEL_GROUP (mt->accel_group), GDK_Down, GDK_CONTROL_MASK, 0, g_cclosure_new (G_CALLBACK (mt_trdown),mt,NULL));

  mt->insert_mode=INSERT_MODE_NORMAL;
  mt->last_direction=DIRECTION_POSITIVE;
  mt_sensitise(mt);

  return mt;
}



void multitrack_delete (lives_mt *mt) {
  // free lives_mt struct
  int i;
  GtkWidget *eventbox;
  track_rect *block, *block_next;
  gint new_file=-1;

  if (mt->event_list!=NULL) event_list_free(mt->event_list);

  if (cfile->frames==0) close_current_file();

  if (mt->file_selected>0&&mainw->files[mt->file_selected]!=NULL) new_file=mt->file_selected;

  gtk_widget_reparent (mainw->scrolledwindow,mainw->message_box);
  gtk_widget_show (mainw->LiVES);

  // free our track_rects
  if (mt->video_draws!=NULL) {
    for (i=0;i<mt->num_video_tracks;i++) {
      eventbox=g_list_nth_data(mt->video_draws,i);
      if (eventbox!=NULL) {
	block=(track_rect *)g_object_get_data (G_OBJECT(eventbox), "blocks");
	while (block!=NULL) {
	  block_next=block->next;
	  g_free (block);
	  block=block_next;
	}
      }
    }
    g_list_free (mt->video_draws);
  }
  gtk_widget_unref(mt->clip_scroll);
  gtk_widget_unref(mt->in_out_box);

  gtk_widget_destroy (mt->window);

  if (mainw->play_window!=NULL) {
    gtk_window_remove_accel_group (GTK_WINDOW (mainw->play_window), mt->accel_group);
    gtk_window_add_accel_group (GTK_WINDOW (mainw->play_window), mainw->accel_group);
  }

  g_free (mt);
  mainw->multitrack=NULL;

  if (new_file!=-1) switch_to_file ((mainw->current_file=0),mt->file_selected);

  reset_clip_menu();

  for (i=1;i<MAX_FILES;i++) {
    if (mainw->files[i]!=NULL) {
      if (mainw->files[i]->event_list!=NULL) event_list_free(mainw->files[i]->event_list);
      mainw->files[i]->event_list=NULL;
    }
  }
  if (mainw->current_file>0) sensitize();
  gtk_widget_hide(mainw->playframe);
  mainw->is_rendering=FALSE;
}


gboolean
on_mt_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
  multitrack_delete((lives_mt *)user_data);
  return FALSE;
}


inline track_rect *add_block_start_point (GtkWidget *eventbox, gint64 tc, gint filenum, gint64 offset, event_t *event, gboolean ordered) {
  // each mt->video_draw (eventbox) has a gulong data which points to a linked list of track_rect
  // here we create a new linked list item and set the start timecode in the timeline, offset in the source file, and start event
  // then append it to our list

  track_rect *block=(track_rect *)g_object_get_data (G_OBJECT(eventbox), "blocks");
  track_rect *new_block=(track_rect *)g_malloc (sizeof(track_rect));

  new_block->next=new_block->prev=NULL;
  new_block->state=BLOCK_UNSELECTED;
  new_block->start_anchored=new_block->end_anchored=FALSE;
  new_block->start_event=event;
  new_block->ordered=ordered;
  new_block->eventbox=eventbox;
  new_block->offset_start=offset;
  new_block->min_tc=new_block->max_tc=-1;

  while (block!=NULL) {
    if (get_event_timecode(block->start_event->event)>tc) {
      if (block->prev!=NULL) {
	block->prev->next=new_block;
	new_block->prev=block->prev;
      }
      else g_object_set_data (G_OBJECT(eventbox),"blocks",(gpointer)new_block);
      new_block->next=block;
      block->prev=new_block;
      break;
    }
    if (block->next==NULL) {
      block->next=new_block;
      new_block->prev=block;
      break;
    }
    block=block->next;
  }
  if (block==NULL) g_object_set_data (G_OBJECT(eventbox),"blocks",(gpointer)new_block);
  g_object_set_data (G_OBJECT(eventbox),"block_last",(gpointer)new_block);
  return new_block;
}


inline void add_block_end_point (GtkWidget *eventbox, event_t *event) {
  // here we add the end point to our last track_rect
  track_rect *block=(track_rect *)g_object_get_data (G_OBJECT(eventbox),"block_last");
  block->end_event=event;
}


void init_tracks (lives_mt *mt) {
  GtkWidget *label;

  if (mt->video_draws!=NULL) g_list_free (mt->video_draws);
  mt->video_draws=NULL;

  label=gtk_label_new (_("Timeline (seconds)"));

  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  gtk_table_attach (GTK_TABLE (mt->timeline_table_header), label, 0, 1, 0, 1, 0, 0, 0, 0);

  mt->track_selected=0;
  add_video_track_behind (NULL,mt);

  mt->timeline = gtk_hruler_new();
  mt->timeline_eb=gtk_event_box_new();
  gtk_widget_set_events (mt->timeline_eb, GDK_POINTER_MOTION_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE | GDK_BUTTON_PRESS);

  g_signal_connect (GTK_OBJECT (mt->timeline), "motion_notify_event",
		    G_CALLBACK (return_true),
		    NULL);

  g_signal_connect (GTK_OBJECT (mt->timeline_eb), "motion_notify_event",
		    G_CALLBACK (on_timeline_update),
		    (gpointer)mt);

  g_signal_connect (GTK_OBJECT (mt->timeline_eb), "button_release_event",
		    G_CALLBACK (on_timeline_release),
		    (gpointer)mt);

  g_signal_connect (GTK_OBJECT (mt->timeline_eb), "button_press_event",
		    G_CALLBACK (on_timeline_press),
		    (gpointer)mt);

  g_signal_connect_after (GTK_OBJECT (mt->timeline_eb), "expose_event",
			  G_CALLBACK (expose_timeline_event),
			  (gpointer)mt);

  mt->end_secs=DEF_TIME;

  gtk_container_add (GTK_CONTAINER (mt->timeline_eb), mt->timeline);

  gtk_table_attach (GTK_TABLE (mt->timeline_table_header), mt->timeline_eb, 1, 10, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (GTK_FILL), 0, 0);

  if (mt->event_list!=NULL) {
    gint current_tracks=1; // number of video tracks on timeline
    int *frame_index;
    int *clip_index;
    int hint;
    event_t *event,*last_event;
    gint num_tracks;
    int tracks[256]; // TODO - use linked list
    int j,error;
    gint64 tc;

    for (j=0;j<256;j++) {
      tracks[j]=0;
    }

    mt->end_secs=event_list_get_end_secs(mt->event_list);

    // draw coloured blocks to represent the FRAME events
    event=mt->event_list->first;

    while (event!=NULL) {
      hint=get_event_hint (event->event);
      if (hint==LIVES_EVENT_HINT_FRAME) {
	last_event=event;
	tc=get_event_timecode (event->event);
	num_tracks=weed_leaf_num_elements (event->event,"clips");

	if (num_tracks>current_tracks) {
	  for (j=current_tracks+1;j<=num_tracks;j++) {
	    add_video_track_behind(NULL,mt);
	  }
	  current_tracks=num_tracks;
	}
	clip_index=weed_get_int_array(event->event,"clips",&error);
	frame_index=weed_get_int_array(event->event,"frames",&error);
	
	for (j=0;j<num_tracks;j++) {
	  // TODO - tracks should be linked list
	  if (tracks[j]!=clip_index[j]) {
	    if (tracks[j]!=0) {
	      add_block_end_point (GTK_WIDGET(g_list_nth_data(mt->video_draws,j)),last_event); // end of previous rectangle
	    }
	    if (clip_index[j]!=0) {
	      // start a new rectangle
	      add_block_start_point (GTK_WIDGET(g_list_nth_data(mt->video_draws,j)),tc,clip_index[j],frame_index[j],event,FALSE);
	    }
	    tracks[j]=clip_index[j];
	  }
	}
	g_free(clip_index);
	g_free(frame_index);

	event=get_next_frame_event(event);
      
	if (event==NULL) {
	  // this is the last FRAME event, so close all our rectangles
	  for (j=0;j<mt->num_video_tracks;j++) {
	    if (tracks[j]!=0) {
	      add_block_end_point (GTK_WIDGET(g_list_nth_data(mt->video_draws,j)),last_event); // end of previous rectangle
	    }
	  }
	}
      }
      else event=event->next;   // keep going till we find the first FRAME event (if any)
    }
  }
  gtk_ruler_set_range (GTK_RULER (mt->timeline), 0., mt->end_secs, 0., mt->end_secs);
}



static void gtk_table_shift_down (GtkTable *table) {
  // shift all table children down by one row
  GtkTableChild *child;
  GList *children = table->children;

  while (children!=NULL) {
    child = children->data;
    children = children->next;
    
    child->top_attach++;
    child->bottom_attach++;
  }
}



static void add_video_track (lives_mt *mt, gboolean behind) {
  // add another video track to our timeline_table
  GtkWidget *label;
  GtkWidget *eventbox;  // each track has an eventbox, which we store in GList *video_draws
  int i;
  gint rows=GTK_TABLE(mt->timeline_table)->nrows;

  mt_desensitise(mt);

  if (mt->video_draws==NULL) rows--;

  gtk_table_resize (GTK_TABLE(mt->timeline_table),rows+1,10);

  mt->num_video_tracks++;

  eventbox=gtk_event_box_new();
  g_object_set_data (G_OBJECT(eventbox),"track_name",g_strdup_printf(_("\nVideo %d\n"),mt->num_video_tracks));

  g_object_set_data (G_OBJECT(eventbox), "blocks", (gpointer)NULL);

  g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
                      G_CALLBACK (on_track_click),
                      (gpointer)mt);

  g_signal_connect_after (GTK_OBJECT (eventbox), "expose_event",
			  G_CALLBACK (expose_vidtrack_event),
			  (gpointer)mt);


  GTK_ADJUSTMENT(mt->vadjustment)->upper=(gdouble)mt->num_video_tracks;

  if (!behind) {
    // shift all rows down
    gtk_table_shift_down(GTK_TABLE(mt->timeline_table));
    // increment "layer_number"s, change labels
    g_object_set_data (G_OBJECT(eventbox),"layer_number",GINT_TO_POINTER(0));
    for (i=0;i<mt->num_video_tracks-1;i++) {
      GtkWidget *xeventbox=g_list_nth_data(mt->video_draws,i);
      g_object_set_data (G_OBJECT(xeventbox),"layer_number",GINT_TO_POINTER(i+1));
      label=g_object_get_data(G_OBJECT(xeventbox),"label");
      gtk_label_set_text(GTK_LABEL(label),g_strdup_printf(_("%s (layer %d)"),g_object_get_data(G_OBJECT(xeventbox),"track_name"),i+1));
    }
    // add a -1,0 in all frame events
    // renumber "in_tacks", "out_tracks" in effect_init events
    event_list_add_track(mt->event_list,0);

    mt->video_draws=g_list_prepend (mt->video_draws, (gpointer)eventbox);
    rows=0;
    mt->track_selected++;
  }
  else {
    mt->video_draws=g_list_append (mt->video_draws, (gpointer)eventbox);
    g_object_set_data (G_OBJECT(eventbox),"layer_number",GINT_TO_POINTER(mt->num_video_tracks-1));
  }

  label=gtk_label_new (g_strdup_printf(_("%s (layer %d)"),g_object_get_data (G_OBJECT(eventbox),"track_name"),GPOINTER_TO_INT(g_object_get_data(G_OBJECT(eventbox),"layer_number"))));

  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);

  gtk_label_set_selectable (GTK_LABEL (label), TRUE);
  g_object_set_data (G_OBJECT(eventbox),"label",label);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  gtk_table_attach (GTK_TABLE (mt->timeline_table), label, 0, 1, rows, rows+1, 0, 0, 0, 0);

  gtk_table_attach (GTK_TABLE (mt->timeline_table), eventbox, 1, 10, rows, rows+1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_FILL), 0, 0);

  gtk_widget_queue_draw (mt->vpaned);
  mt_sensitise(mt);
}

void add_video_track_behind (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  add_video_track (mt,TRUE);
  gtk_widget_show_all (mt->timeline_table);
}

void add_video_track_front (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  add_video_track (mt,FALSE);
  gtk_widget_show_all (mt->timeline_table);
}

GdkPixbuf *make_thumb (gint file, gint width, gint height, gint frame) {
  // TODO - use layer
  GdkPixbuf *pixbuf,*thumbnail;
  GError *error=NULL;
  gchar *buf;
  gint retries=1;

  if (mainw->playing_file>-1) retries=10000000;

  buf=g_strdup_printf("%s/%s/%08d.%s",prefs->tmpdir,mainw->files[file]->handle,frame,prefs->image_ext);
  pixbuf=gdk_pixbuf_new_from_file(buf,&error);
  g_free(buf);
  if (error==NULL) {
    thumbnail=gdk_pixbuf_scale_simple(pixbuf,width,height,GDK_INTERP_BILINEAR);
    gdk_pixbuf_unref(pixbuf);
  }
  else return NULL;
  return thumbnail;
}


static void draw_block (track_rect *block, gdouble end_secs, gint x1, gint x2) {
  GtkWidget *eventbox=block->eventbox;
  gint offset_start=get_event_timecode(block->start_event->event)/U_SEC/end_secs*eventbox->allocation.width;
  gint offset_end;
  int i,filenum,track;
  GdkPixbuf *thumbnail;

  if (offset_start>(x1+x2)) return;

  offset_end=(get_event_timecode(block->end_event->event)/U_SEC+1./cfile->fps)/end_secs*eventbox->allocation.width;

  if (offset_end<x1) return;

  switch (block->state) {
  case BLOCK_UNSELECTED:
    track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(eventbox),"layer_number"));
    filenum=get_frame_event_clip(block->start_event->event,track);
    if (BLOCK_DRAW_TYPE==BLOCK_DRAW_SIMPLE) {
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_start, eventbox->allocation.height);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_end, eventbox->allocation.height, offset_end, 0);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_end, eventbox->allocation.height);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, eventbox->allocation.height, offset_end, 0);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_end, 0);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, eventbox->allocation.height-1, offset_end, eventbox->allocation.height-1);
    }
    for (i=offset_start;i<offset_end;i+=BLOCK_THUMB_WIDTH) {
      // create a small thumb
      filenum=get_frame_event_clip(block->start_event->event,track);
      thumbnail=make_thumb(filenum,BLOCK_THUMB_WIDTH,eventbox->allocation.height-1,1);
      gint width=BLOCK_THUMB_WIDTH;
      if (i+width>offset_end) width=offset_end-i;
      // render it in the eventbox
      if (thumbnail!=NULL) {
	gdk_pixbuf_render_to_drawable (thumbnail,GDK_DRAWABLE (eventbox->window),mainw->gc,0,0,i,0,width,-1,GDK_RGB_DITHER_NONE,0,0);
	gdk_pixbuf_unref(thumbnail);
      }
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_start, eventbox->allocation.height-1);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_end, eventbox->allocation.height-1, offset_end, 0);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_end, 0);
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, eventbox->allocation.height-1, offset_end, eventbox->allocation.height-1);
    }
    break;
  case BLOCK_SELECTED:
    gdk_draw_rectangle (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->white_gc, TRUE, offset_start, 0, offset_end-offset_start, eventbox->allocation.height-1);
    gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_start, eventbox->allocation.height-1);
    gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_end, eventbox->allocation.height-1, offset_end, 0);
    gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_end, eventbox->allocation.height-1);
    gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, eventbox->allocation.height-1, offset_end, 0);
    gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, 0, offset_end, 0);
    gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset_start, eventbox->allocation.height-1, offset_end, eventbox->allocation.height-1);
    break;
  }
}


gint expose_vidtrack_event (GtkWidget *eventbox, GdkEventExpose *event, gpointer user_data) {
  // TODO - only redraw exposed area
  track_rect *block;
  lives_mt *mt=(lives_mt *)user_data;
  GdkRegion *reg=event->region;
  GdkRectangle rect;
  gdk_region_get_clipbox(reg,&rect);

  gint startx=rect.x,width=rect.width;

  if (width>eventbox->allocation.width-startx) width=eventbox->allocation.width-startx;

  if (event!=NULL&&event->count>0) {
    return TRUE;
  }
  gdk_draw_rectangle (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->white_gc, TRUE, startx, 0, width, eventbox->allocation.height);

  block=(track_rect *)g_object_get_data (G_OBJECT(eventbox), "blocks");

  while (block!=NULL) {
    draw_block(block,mt->end_secs,startx,width);
    block=block->next;
  }
  return FALSE;
}



void init_clips (lives_mt *mt, gint orig_file) {
  GtkWidget *thumb_image=NULL;
  GtkWidget *vbox, *label;
  gchar clip_name[CLIP_LABEL_LENGTH];
  GdkPixbuf *thumbnail;
  int i,j=0;
  gint width=CLIP_THUMB_WIDTH,height=CLIP_THUMB_HEIGHT;
  gchar filename[256];

  mt->clip_selected=-1;

  for (i=1;i<MAX_FILES;i++) {
    if (mainw->files[i]!=NULL&&i!=mainw->scrap_file&&mainw->files[i]->clip_type==CLIP_TYPE_DISK) {
      if (i==orig_file||(mt->clip_selected==-1&&i==mainw->pre_src_file)) {
	mt->clip_selected=j;
      }
      // make a small thumbnail, add it to the clips box
      thumbnail=make_thumb(i,width,height,1);
      if (thumbnail!=NULL) {
	vbox = gtk_vbox_new (FALSE, 10);
	g_object_set_data (G_OBJECT(vbox),"file",GINT_TO_POINTER(i));
	//GTK_WIDGET_SET_FLAGS (vbox, GTK_CAN_FOCUS);
	thumb_image=gtk_image_new();
	gtk_image_set_from_pixbuf(GTK_IMAGE(thumb_image),thumbnail);
	gdk_pixbuf_unref(thumbnail);
	gtk_box_pack_start (GTK_BOX (mt->clip_inner_box), vbox, FALSE, FALSE, 10);
	g_snprintf (filename,256,"%s",mainw->files[i]->name);
	get_basename(filename);
	g_snprintf (clip_name,CLIP_LABEL_LENGTH,"%s",filename);
	label=gtk_label_new (clip_name);
	gtk_label_set_selectable (GTK_LABEL(label),TRUE);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), thumb_image, FALSE, FALSE, 0);
	label=gtk_label_new (g_strdup_printf (_("%d frames"),mainw->files[i]->frames));
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	label=gtk_label_new (g_strdup_printf (_("%d to %d selected"),mainw->files[i]->start,mainw->files[i]->end));
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	label=gtk_label_new (g_strdup_printf (_("%.2f sec."),(mainw->files[i]->end-mainw->files[i]->start+1.)/mainw->files[i]->fps));
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	j++;
      }
    }
  }
}


gboolean on_multitrack_activate (GtkMenuItem *menuitem, event_list_t *event_list) {
  //returns TRUE if we go into mt mode
  GtkWidget *rdet_dialog;
  render_details rdet;
  gint orig_file=mainw->current_file;
  int i;
  gboolean event_warn=FALSE;
  gboolean response;

  lives_mt *multi;

  for (i=1;i<MAX_FILES;i++) {
    if (mainw->files[i]!=NULL&&mainw->files[i]->event_list!=NULL) {
      if (!event_warn&&!do_warning_dialog(_("Some clips have frame changes recorded.\nIf you go to multitrack mode, you will lose these events\n\nClick OK to continue, Cancel to return to the clip editor.\n"))) return FALSE;
      event_warn=TRUE;
      event_list_free(mainw->files[i]->event_list);
      mainw->files[i]->event_list=NULL;
    }
  }

  rdet_dialog=create_render_details_dialog(&rdet);
  response=gtk_dialog_run(GTK_DIALOG(rdet_dialog));
  gtk_widget_destroy (rdet_dialog); 

  if (response==GTK_RESPONSE_CANCEL) return FALSE;

  // create new file for rendering to
  mainw->current_file=mainw->first_free_file;
    
  if (!get_new_handle(mainw->current_file,NULL)) {
    mainw->current_file=orig_file;
    return FALSE; // show dialog again
  }

  cfile->hsize=rdet.width;
  cfile->vsize=rdet.height;
  cfile->pb_fps=cfile->fps=rdet.fps;
  cfile->bpp=24;
  cfile->changed=TRUE;
  cfile->is_loaded=TRUE;

  cfile->old_frames=cfile->frames;

  // if we have an existing event list, we will quantise it to the selected fps
  if (event_list!=NULL) {
    event_list_t *qevent_list=quantise_events(event_list,cfile->fps,TRUE);
    event_list_replace_events(event_list,qevent_list);
  }

  multi=multitrack(event_list,orig_file,cfile->fps);
  gtk_widget_show_all (multi->window);
  gtk_widget_hide (mainw->LiVES);

  multi->render_file=mainw->current_file;

  track_select(multi);
  clip_select(multi);

  gtk_window_set_title (GTK_WINDOW (multi->window), g_strdup_printf(_("LiVES-%s: Multitrack %dx%d : %d bpp %.3f fps"),LiVES_VERSION,cfile->hsize,cfile->vsize,cfile->bpp,cfile->fps));

  if (mainw->preview_box!=NULL&&mainw->preview_box->parent!=NULL) {
    g_object_unref(mainw->preview_box);
    gtk_container_remove (GTK_CONTAINER (mainw->play_window), mainw->preview_box);
    mainw->preview_box=NULL;
  }
  
  if (mainw->play_window!=NULL) {
    g_signal_handler_unblock(mainw->play_window,mainw->pw_exp_func);
    gtk_window_remove_accel_group (GTK_WINDOW (mainw->play_window), mainw->accel_group);
    gtk_window_add_accel_group (GTK_WINDOW (mainw->play_window), multi->accel_group);
    resize_play_window();
  }

  return TRUE;
}


gboolean block_overlap (GtkWidget *eventbox, gdouble time_start, gdouble time_end) {
  gint64 tc_start=time_start*U_SECL;
  gint64 tc_end=time_end*U_SECL;
  track_rect *block=(track_rect *)g_object_get_data (G_OBJECT(eventbox),"blocks");

  while (block!=NULL) {
    if (get_event_timecode(block->start_event->event)>tc_end) return FALSE;
    if (get_event_timecode(block->end_event->event)>=tc_start) return TRUE;
    block=block->next;
  }
  return FALSE;
}  



track_rect *get_block_from_time (GtkWidget *eventbox, gdouble time) {
  gint64 tc=time*U_SECL;
  track_rect *block=(track_rect *)g_object_get_data (G_OBJECT(eventbox),"blocks");

  while (block!=NULL) {
    if (get_event_timecode(block->start_event->event)>tc) return NULL;
    if (get_event_timecode(block->end_event->event)>=tc) break;
    block=block->next;
  }
  return block;
}


track_rect *get_block_before (GtkWidget *eventbox, gdouble time) {
  gint64 tc=time*U_SECL;
  track_rect *block=(track_rect *)g_object_get_data (G_OBJECT(eventbox),"blocks"),*last_block=NULL;

  while (block!=NULL) {
    if (get_event_timecode(block->start_event->event)>tc) break;
    last_block=block;
    block=block->next;
  }
  return last_block;
}

track_rect *get_block_after (GtkWidget *eventbox, gdouble time) {
  gint64 tc=time*U_SECL;
  track_rect *block=(track_rect *)g_object_get_data (G_OBJECT(eventbox),"blocks");

  while (block!=NULL) {
    if (get_event_timecode(block->start_event->event)>tc) break;
    block=block->next;
  }
  return block;
}


void unselect_all (lives_mt *mt) {
  // unselect all blocks 
  int i;
  GtkWidget *eventbox;
  track_rect *trec;

  if (mt->block_selected!=NULL) gtk_widget_queue_draw(mt->block_selected->eventbox);

  for (i=0;i<mt->num_video_tracks;i++) {
    eventbox=g_list_nth_data(mt->video_draws,i);
    if (eventbox!=NULL) {
      trec=(track_rect *)g_object_get_data (G_OBJECT(eventbox),"blocks");
      while (trec!=NULL) {
	trec->state=BLOCK_UNSELECTED;
	trec=trec->next;
      }
    }
  }
  mt->block_selected=NULL;
  gtk_widget_set_sensitive(mt->view_in_out,FALSE);
  gtk_widget_set_sensitive (mt->delblock, FALSE);
}


void clear_context (lives_mt *mt) {
  gtk_widget_destroy(mt->context_box);
  mt->context_box = gtk_vbox_new (FALSE, 10);
  gtk_box_pack_start (GTK_BOX (mt->hbox), mt->context_box, FALSE, FALSE, 0);
  add_context_label(mt," \n");
  gtk_widget_show (mt->context_box);
}

void add_context_label (lives_mt *mt, gchar *text) {
  GtkWidget *label=gtk_label_new (text);
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (mt->context_box), label, FALSE, FALSE, 0);
}



void do_block_context (lives_mt *mt, GdkEventButton *event) {
  // pop up a context menu when a selected block is right clicked on

  // unfinished...

  GtkWidget *add_effect;
  GtkWidget *menu=gtk_menu_new();

  gtk_menu_set_title (GTK_MENU(menu),_("LiVES: Selected block"));

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(menu, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }
  
  add_effect = gtk_menu_item_new_with_mnemonic (_("_Add effect to block..."));
  gtk_container_add (GTK_CONTAINER (menu), add_effect);
  
  gtk_widget_show_all (menu);
  
  gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);

}

gboolean resize_timeline (lives_mt *mt) {
  gdouble end_secs;

  if (mt->event_list==NULL||mt->event_list->first==NULL||mt->tl_fixed_length>0.) return FALSE;

  end_secs=event_list_get_end_secs(mt->event_list);

  if (end_secs>mt->end_secs) {
    set_timeline_end_secs(mt,end_secs);
    return TRUE;
  }
  return FALSE;
}



static void set_in_out_spin_ranges(lives_mt *mt, gint64 start_tc, gint64 end_tc) {
  track_rect *block=mt->block_selected;
  gint64 min_tc=block->min_tc,max_tc=block->max_tc,offset_start=get_event_timecode(block->start_event->event);
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(block->eventbox),"layer_number"));
  gint filenum=get_frame_event_clip(block->start_event->event,track);

  gdouble in_val=start_tc/U_SEC,out_val=end_tc/U_SEC,in_start_range=0.,out_start_range=in_val+1./mt->fps,out_end_range=count_resampled_frames(mainw->files[filenum]->frames,mainw->files[filenum]->fps,mt->fps)/mt->fps,in_end_range=out_val-1./mt->fps,real_out_end_range=out_end_range,real_in_start_range=in_start_range;

   if (mt->insert_mode!=INSERT_MODE_OVERWRITE) {
     if (!block->end_anchored&&max_tc>-1&&(((max_tc-offset_start)/U_SEC+in_val)<out_end_range)) real_out_end_range=(max_tc-offset_start)/U_SEC+in_val;
     if (!block->start_anchored&&min_tc>-1&&(((min_tc-offset_start)/U_SEC+in_val)>in_start_range)) real_in_start_range=(min_tc-offset_start)/U_SEC+in_val;
     if (!block->start_anchored) out_end_range=real_out_end_range; 
     if (!block->end_anchored) in_start_range=real_in_start_range; 
   }

   if (block->end_anchored&&(out_val-in_val>out_start_range)) out_start_range=in_start_range+out_val-in_val;
   if (block->start_anchored&&(out_end_range-out_val+in_val)<in_end_range) in_end_range=out_end_range-out_val+in_val;

   gtk_spin_button_set_range(GTK_SPIN_BUTTON(mt->spinbutton_out), out_start_range, real_out_end_range);
   gtk_spin_button_set_range(GTK_SPIN_BUTTON(mt->spinbutton_in), real_in_start_range, in_end_range);

   // make sure next and prev blocks can't overwrite
   if (block->prev!=NULL) block->prev->max_tc=offset_start;
   if (block->next!=NULL) block->next->min_tc=get_event_timecode(block->end_event->event)+U_SEC/mt->fps;
}

static void update_in_image(lives_mt *mt) {
  GdkPixbuf *pixbuf,*thumb;
  track_rect *block=mt->block_selected;
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(block->eventbox),"layer_number"));
  gint filenum=get_frame_event_clip(block->start_event->event,track);
  gint frame_start=calc_variable_frame(filenum,block->offset_start/U_SEC);
  gint width=cfile->hsize;
  gint height=cfile->vsize;

  if (width>mt->poly_box->allocation.width/2) width=mt->poly_box->allocation.width/2;
  if (height>mt->poly_box->allocation.height-(block->ordered?mt->spinbutton_in->allocation.height:0)) height=mt->poly_box->allocation.height-(block->ordered?mt->spinbutton_in->allocation.height:0);
  thumb=make_thumb(filenum,width,height,frame_start);
  if ((pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(mt->in_image)))!=NULL) gdk_pixbuf_unref(pixbuf);
  gtk_image_set_from_pixbuf (GTK_IMAGE(mt->in_image),thumb);
}

static void update_out_image(lives_mt *mt, gint64 end_tc) {
  GdkPixbuf *pixbuf,*thumb;
  track_rect *block=mt->block_selected;
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(block->eventbox),"layer_number"));
  gint filenum=get_frame_event_clip(block->start_event->event,track);
  gint frame_end=calc_variable_frame(filenum,end_tc/U_SEC-1./mt->fps);
  gint width=cfile->hsize;
  gint height=cfile->vsize;
  
  if (width>mt->poly_box->allocation.width/2) width=mt->poly_box->allocation.width/2;
  if (height>mt->poly_box->allocation.height-(block->ordered?mt->spinbutton_out->allocation.height:0)) height=mt->poly_box->allocation.height-(block->ordered?mt->spinbutton_out->allocation.height:0);
  thumb=make_thumb(filenum,width,height,frame_end);
  if ((pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(mt->out_image)))!=NULL) gdk_pixbuf_unref(pixbuf);
  gtk_image_set_from_pixbuf (GTK_IMAGE(mt->out_image),thumb);
}




void
in_out_start_changed (GtkWidget *widget, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  track_rect *block=mt->block_selected;
  gdouble new_start=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
  event_t *event=block->start_event;
  gint64 new_start_tc,orig_start_tc=block->offset_start,offset_end,tl_start;
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(block->eventbox),"layer_number"));
  gint filenum;

  new_start_tc=q_dbl(new_start,mt->fps);

  if (new_start_tc==orig_start_tc||!block->ordered) {
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_in),new_start_tc/U_SEC);
    return;
  }

  tl_start=get_event_timecode(event->event);
  filenum=get_frame_event_clip(block->start_event->event,track);

  if (!block->start_anchored) {
    if (new_start_tc>block->offset_start) {
      // start increased, not anchored
      while (event!=NULL) {
	if ((get_event_timecode(event->event)-tl_start)>=new_start_tc-block->offset_start) {
	  // done
	  block->start_event=event;
	  block->offset_start=new_start_tc;
	  break;
	}
	remove_frame_from_event(mt->event_list,event,track);
	event=get_next_frame_event(event);
      }
    }
    else {
      // do an insert from offset_start down
      gint filenum=get_frame_event_clip(block->start_event->event,track);
      insert_frames (filenum,block->offset_start,new_start_tc,tl_start,DIRECTION_NEGATIVE,block->eventbox,mt,block);
    }
  }
  else {
    // start is anchored, do a re-insert from start to end
    gshort insert_mode=mt->insert_mode;
    offset_end=q_gint64((block->offset_start=new_start_tc)+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event)),mt->fps);
    mt->insert_mode=INSERT_MODE_OVERWRITE;
    insert_frames (filenum,new_start_tc,offset_end,tl_start,DIRECTION_POSITIVE,block->eventbox,mt,block);
    mt->insert_mode=insert_mode;
  }

  offset_end=q_gint64((new_start_tc=block->offset_start)+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event)),mt->fps);

  if (mt->poly_state==POLY_IN_OUT) {
    set_in_out_spin_ranges(mt,new_start_tc,offset_end);
  
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_out),offset_end/U_SEC);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_in),new_start_tc/U_SEC);

    // update images
    update_in_image(mt);
    if (block->start_anchored) update_out_image(mt,offset_end);
  }

  if (!resize_timeline(mt)) gtk_widget_queue_draw (block->eventbox); // redraw the track
}

void
in_out_end_changed (GtkWidget *widget, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  track_rect *block=mt->block_selected;
  gdouble new_end=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
  gint64 offset_end=block->offset_start+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event)),orig_end_tc;
  event_t *event=block->end_event,*prevevent;
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(block->eventbox),"layer_number"));
;
  gint filenum=get_frame_event_clip(block->end_event->event,track);
  gint64 new_end_tc,tl_end;

  if (new_end>mainw->files[filenum]->frames/mainw->files[filenum]->fps) new_end=mainw->files[filenum]->frames/mainw->files[filenum]->fps;

  new_end_tc=q_dbl(new_end,mt->fps);
  orig_end_tc=offset_end=q_gint64(offset_end,mt->fps);

  if (new_end_tc==orig_end_tc||!block->ordered) {
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_out),new_end_tc/U_SEC);
    return;
  }

  tl_end=get_event_timecode(event->event);

  if (!block->end_anchored) {
    if (new_end_tc<offset_end) {
      // end decreased, not anchored
      while (event!=NULL) {
	if ((tl_end-get_event_timecode(event->event))>=(offset_end-new_end_tc)) {
	  // done
	  block->end_event=event;
	  break;
	}
	prevevent=get_prev_frame_event(event);
	remove_frame_from_event(mt->event_list,event,track);
	event=prevevent;
      }
    }
    else {
      // end increased, not anchored
      // do an insert from end_tc up, starting with end_frame and finishing at new_end
      insert_frames (filenum,offset_end,new_end_tc,tl_end+(gint64)(U_SEC/mt->fps),DIRECTION_POSITIVE,block->eventbox,mt,block);
    }
  }
  else {
    // end is anchored, do a re-insert from end to start
    gint64 offset_start;
    gshort insert_mode=mt->insert_mode;
    offset_end=q_gint64((offset_start=block->offset_start+new_end_tc-orig_end_tc)+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event)),mt->fps);
    mt->insert_mode=INSERT_MODE_OVERWRITE;
    insert_frames (filenum,offset_end,offset_start,tl_end+(gint64)(U_SEC/mt->fps),DIRECTION_NEGATIVE,block->eventbox,mt,block);
    mt->insert_mode=insert_mode;
  }

  // get new offset_end
  new_end_tc=q_gint64(block->offset_start+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event)),mt->fps);

  if (mt->poly_state==POLY_IN_OUT) {
    set_in_out_spin_ranges(mt,block->offset_start,new_end_tc);

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_in), block->offset_start/U_SEC);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_out), new_end_tc/U_SEC);

    // update image
    update_out_image(mt,new_end_tc);
    if (block->end_anchored) update_in_image(mt);
  }

  resize_timeline(mt);

  if (!resize_timeline(mt)) gtk_widget_queue_draw (block->eventbox); // redraw the track
}

void
in_anchor_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  track_rect *block=mt->block_selected;
  gint64 offset_end=block->offset_start+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event));

  block->start_anchored=!block->start_anchored;
  set_in_out_spin_ranges(mt,block->offset_start,offset_end);
}

void
out_anchor_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  track_rect *block=mt->block_selected;
  gint64 offset_end=block->offset_start+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event));

  block->end_anchored=!block->end_anchored;
  set_in_out_spin_ranges(mt,block->offset_start,offset_end);
}




void polymorph (lives_mt *mt, gshort poly) {
  GdkPixbuf *thumb,*pixbuf;
  gint width,height,track;
  gint frame_start,frame_end;
  int filenum;
  track_rect *block=mt->block_selected;
  gint64 offset_end;

  if (poly==mt->poly_state) return;

  if (poly==POLY_IN_OUT&&block==NULL) return;

  switch (mt->poly_state) {
  case (POLY_CLIPS) :
    gtk_container_remove (GTK_CONTAINER(mt->poly_box),mt->clip_scroll);
    break;
  case (POLY_IN_OUT) :
    gtk_container_remove (GTK_CONTAINER(mt->poly_box),mt->in_out_box);
    break;
  }

  mt->poly_state=poly;

  switch (poly) {
  case (POLY_IN_OUT) :
    if (block->ordered) {
      gtk_widget_show(mt->in_hbox);
      gtk_widget_show(mt->out_hbox);
    }
    else {
      gtk_widget_hide(mt->in_hbox);
      gtk_widget_hide(mt->out_hbox);
    }
    track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(block->eventbox),"layer_number"));
    filenum=get_frame_event_clip(block->start_event->event,track);
    offset_end=block->offset_start+(gint64)(U_SEC/mt->fps)+(get_event_timecode(block->end_event->event)-get_event_timecode(block->start_event->event));
    width=cfile->hsize;
    height=cfile->vsize;
    if (width>mt->poly_box->allocation.width/2) width=mt->poly_box->allocation.width/2;
    if (height>mt->poly_box->allocation.height-(block->ordered?mainw->spinbutton_start->allocation.height:0)) height=mt->poly_box->allocation.height-(block->ordered?mainw->spinbutton_start->allocation.height:0);

    if (mainw->playing_file==filenum) {
      // we must do this, because during playback with an event_list, cfile->event_list holds a pointer to our mt->event_list
      // our resampled frames will have been moved to cfile->event_list_back
      // this is definately NOT good coding style
      mainw->files[filenum]->event_list=mainw->files[filenum]->event_list_back;
    }
    frame_start=calc_variable_frame(filenum,block->offset_start/U_SEC);
    frame_end=calc_variable_frame(filenum,offset_end/U_SEC-1./mt->fps);
    if (mainw->playing_file==filenum) {
      mainw->files[filenum]->event_list=mt->event_list;
    }

    // start image
    thumb=make_thumb(filenum,width,height,frame_start);
    if ((pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(mt->in_image)))!=NULL) gdk_pixbuf_unref(pixbuf);
    gtk_image_set_from_pixbuf (GTK_IMAGE(mt->in_image),thumb);

    g_signal_handler_block (mt->spinbutton_in,mt->spin_in_func);
    gtk_spin_button_set_range(GTK_SPIN_BUTTON(mt->spinbutton_in),0., offset_end/U_SEC-1./mt->fps);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_in),block->offset_start/U_SEC);
    g_signal_handler_unblock (mt->spinbutton_in,mt->spin_in_func);
    g_signal_handler_block (mt->checkbutton_start_anchored,mt->check_start_func);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mt->checkbutton_start_anchored),block->start_anchored);
    g_signal_handler_unblock (mt->checkbutton_start_anchored,mt->check_start_func);

    // end image
    thumb=make_thumb(filenum,width,height,frame_end);
    if ((pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(mt->out_image)))!=NULL) gdk_pixbuf_unref(pixbuf);
    gtk_image_set_from_pixbuf (GTK_IMAGE(mt->out_image),thumb);

    g_signal_handler_block (mt->spinbutton_out,mt->spin_out_func);
    gtk_spin_button_set_range(GTK_SPIN_BUTTON(mt->spinbutton_out), block->offset_start/U_SEC+1./mt->fps, count_resampled_frames(mainw->files[filenum]->frames,mainw->files[filenum]->fps,mt->fps)/mt->fps);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mt->spinbutton_out),offset_end/U_SEC);
    g_signal_handler_unblock (mt->spinbutton_out,mt->spin_out_func);

    g_signal_handler_block (mt->checkbutton_end_anchored,mt->check_end_func);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mt->checkbutton_end_anchored),block->end_anchored);
    g_signal_handler_unblock (mt->checkbutton_end_anchored,mt->check_end_func);
    gtk_box_pack_start(GTK_BOX(mt->poly_box),mt->in_out_box,TRUE,TRUE,0);

    if (mainw->playing_file>-1) mt_desensitise(mt);
    else mt_sensitise (mt);

    break;
  case (POLY_CLIPS) :
    gtk_box_pack_start(GTK_BOX(mt->poly_box),mt->clip_scroll,TRUE,TRUE,0);
    break;
  }
  gtk_widget_queue_draw(mt->poly_box);
}



gboolean on_track_click (GtkWidget *eventbox, GdkEventButton *event, gpointer user_data) {
  gint x;
  track_rect *block;
  lives_mt *mt=(lives_mt *)user_data;
  gdouble timesecs;
  gint track;
  int filenum=-1;

  gdk_window_get_pointer(GDK_WINDOW (eventbox->window), &x, NULL, NULL);
  timesecs=(gdouble)x/(gdouble)eventbox->allocation.width*mt->end_secs;
  timesecs=(gint)(timesecs*mt->fps+.5)/mt->fps; // quantise to frame timing

  unselect_all(mt);

  track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(eventbox),"layer_number"));

  if ((block=get_block_from_time(eventbox,timesecs))!=NULL) {
    if (event->button==3) {
      if (block->state==BLOCK_SELECTED) {
	// context menu for a selected block
	do_block_context(mt,event);
      }
      return FALSE;
    }
  }
  if (event->type==GDK_2BUTTON_PRESS) {
    if (block!=NULL) {
      // double click, select the rectangle
      filenum=get_frame_event_clip(block->start_event->event,track);
      block->state=BLOCK_SELECTED;
      mt->block_selected=block;
      clear_context(mt);
      add_context_label (mt,g_strdup_printf (_("Selected track: %s (layer %d)\n"),g_object_get_data(G_OBJECT(eventbox),"track_name"),track));
      add_context_label (mt,g_strdup_printf (_("%.2f sec. to %.2f sec.\n"),get_event_timecode(block->start_event->event)/U_SEC,get_event_timecode(block->end_event->event)/U_SEC+1./mt->fps));
      add_context_label (mt,g_strdup_printf (_("Source: %s"),mainw->files[filenum]->name));
      add_context_label (mt,(_("Right click for context menu.\n")));
      add_context_label (mt,(_("Single click on timeline to select a frame.\n")));
      gtk_widget_set_sensitive(mt->view_in_out,TRUE);
      gtk_widget_queue_draw(block->eventbox);
      multitrack_view_in_out(NULL,mt);
    }
  }
  else {
    // single click, TODO - locate the frame for the track in event_list
    clear_context(mt);
    add_context_label (mt,g_strdup_printf (_("Selected track: %s (layer %d)\n"),g_object_get_data(G_OBJECT(eventbox),"track_name"),track));
    add_context_label (mt,g_strdup_printf (_("%.2f sec.\n"),timesecs));
    if (block!=NULL) {
      filenum=get_frame_event_clip(block->start_event->event,track);
      add_context_label (mt,g_strdup_printf (_("Source: %s"),mainw->files[filenum]->name));
      add_context_label (mt,(_("Right click for context menu.\n")));
    }
    add_context_label (mt,(_("Double click on a block to select it.\n")));
    
    if (mainw->playing_file==-1) {
      GTK_RULER (mt->timeline)->position=timesecs;
      gtk_widget_queue_draw (mainw->multitrack->timeline);
    }
    // TODO - list effects applied to frame

    if (block!=NULL&&filenum!=mainw->scrap_file) {
      mt->clip_selected=filenum-1; // TODO !!!!!! *******
      clip_select(mt);
    }
    else polymorph (mt,POLY_CLIPS);
    mt->region_start=mt->region_end=timesecs;
  }
  mt->track_selected=track;
  track_select(mt);

  return FALSE;
}



void animate_multitrack (lives_mt *mt) {
  // update timeline pointer(s)
  /*  gint offset=-1;
  int i;
  gint len=g_list_length (mainw->multitrack->video_draws); */
  gdouble currtime=(mainw->currticks-mainw->origticks)/U_SEC;
  time_to_string(mt,currtime,TIMECODE_LENGTH);

  GTK_RULER (mt->timeline)->position=currtime;
  gtk_widget_queue_draw (mt->timeline);

  // TODO - opt
  /*  for (i=0;i<len;i++) {
    GtkWidget *eventbox=g_list_nth_data(mainw->multitrack->video_draws,i);
    if (GTK_WIDGET_VISIBLE(eventbox)) {
      if (offset==-1) offset=currtime/mainw->multitrack->end_secs*eventbox->allocation.width;
      gdk_draw_line (GDK_DRAWABLE(eventbox->window), mainw->LiVES->style->black_gc, offset, 1, offset, eventbox->allocation.height-1);
    }
    } */
} 






////////////////////////////////////////////////////
// menuitem callbacks


void multitrack_end (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  gtk_widget_show (mainw->LiVES);
  multitrack_delete (mt);
}


void on_jumpto_activate (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  GtkWidget *eventbox=g_list_nth_data(mt->video_draws,mt->track_selected);
  gdouble secs=GTK_RULER (mt->timeline)->position/GTK_RULER (mt->timeline)->upper*mt->end_secs,new_secs;
  track_rect *block=get_block_from_time(eventbox,secs);
  gint64 tc,new_tc;

  if (block==NULL) {
    if (mt->last_direction==DIRECTION_NEGATIVE) {
      block=get_block_before(eventbox,secs);
      if (block==NULL) return;
      new_secs=get_event_timecode(block->end_event->event)/U_SEC+1./mt->fps;
      new_tc=new_secs*U_SECL+.5;
      tc=secs*U_SECL+.5;
      secs=(new_tc!=tc)?new_secs:get_event_timecode(block->start_event->event)/U_SEC-1./mt->fps;
    }
    else {
      block=get_block_after(eventbox,secs);
      if (block==NULL) return;
      new_secs=get_event_timecode(block->start_event->event)/U_SEC-1./mt->fps;
      new_tc=new_secs*U_SECL+.5;
      tc=secs*U_SECL+.5;
      secs=(new_tc!=tc)?new_secs:get_event_timecode(block->end_event->event)/U_SEC+1./mt->fps;
    }
  }
  else {
    if (mt->last_direction==DIRECTION_NEGATIVE) secs=get_event_timecode(block->start_event->event)/U_SEC-1./mt->fps;
    else secs=get_event_timecode(block->end_event->event)/U_SEC+1./mt->fps;
  }

  if (secs<0.) secs=0.;
  if (secs>mt->end_secs) set_timeline_end_secs (mt, secs);

  GTK_RULER (mt->timeline)->position=secs;
  gtk_widget_queue_draw (mt->timeline);
}

void on_render_activate (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  gint current_file;

  mainw->event_list=mt->event_list;

  mt->is_rendering=TRUE;

  if (render_to_clip(FALSE)) {
    cfile->start=1;
    cfile->end=cfile->frames;
    set_undoable (NULL,FALSE);
    add_to_winmenu();
    current_file=mainw->current_file;
    d_print (g_strdup_printf (_ ("rendered %d frames to new clip.\n"),cfile->frames));
    mt->file_selected=current_file;
    mt->event_list=NULL;
    mt->is_rendering=FALSE;
    multitrack_end(menuitem,user_data);
    switch_to_file ((mainw->current_file=0),current_file);
  }
  else {
    cfile->frames=cfile->start=cfile->end=0;
    mt->is_rendering=FALSE;
    mainw->event_list=NULL;
  }
}

void on_delblock_activate (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  track_rect *block=mt->block_selected,*firstblock;
  event_t *event=block->end_event,*prevevent;
  GtkWidget *eventbox=block->eventbox;
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(eventbox),"layer_number"));
  gboolean done=FALSE;

  mt_desensitise(mt);
  while (event!=NULL&&!done) {
    prevevent=get_prev_frame_event(event);
    if (event==block->start_event) done=TRUE;
    remove_frame_from_event(mt->event_list,event,track);
    event=prevevent;
  }
  firstblock=(track_rect *)g_object_get_data (G_OBJECT(eventbox), "blocks");
  if (firstblock==block) g_object_set_data (G_OBJECT(eventbox),"blocks",(gpointer)block->next);
  if (block->prev!=NULL) block->prev->next=block->next;
  if (block->next!=NULL) block->next->prev=block->prev;
  g_free(block);
  mt->block_selected=NULL;
  remove_end_blank_frames(mt->event_list);
  unselect_all(mt);
  polymorph (mt,POLY_CLIPS);
  gtk_widget_queue_draw (eventbox);
  mt_sensitise(mt);
}


///////////////////////////////////////////////////////////


void mt_desensitise (lives_mt *mt) {
  gdouble val;
  gtk_widget_set_sensitive (mt->clipedit,FALSE);
  gtk_widget_set_sensitive (mt->insert,FALSE);
  gtk_widget_set_sensitive (mt->playall,FALSE);
  gtk_widget_set_sensitive (mt->view_events,FALSE);
  gtk_widget_set_sensitive (mt->jumpto, FALSE);
  gtk_widget_set_sensitive (mt->render, FALSE);
  gtk_widget_set_sensitive (mt->delblock, FALSE);

  if (mt->poly_state==POLY_IN_OUT) {
    val=gtk_spin_button_get_value(GTK_SPIN_BUTTON(mt->spinbutton_in));
    gtk_spin_button_set_range(GTK_SPIN_BUTTON(mt->spinbutton_in),val,val);
    
    val=gtk_spin_button_get_value(GTK_SPIN_BUTTON(mt->spinbutton_out));
    gtk_spin_button_set_range(GTK_SPIN_BUTTON(mt->spinbutton_out),val,val);
  }
}

void mt_sensitise (lives_mt *mt) {
  GtkWidget *eventbox;

  if (mt->event_list!=NULL&&mt->event_list->first!=NULL) {
    gtk_widget_set_sensitive (mt->playall,TRUE);
    gtk_widget_set_sensitive (mt->view_events,TRUE);
    gtk_widget_set_sensitive (mt->render, TRUE);
  }

  gtk_widget_set_sensitive (mt->clipedit,TRUE);
  if (mt->clip_selected>-1) gtk_widget_set_sensitive (mt->insert,TRUE);
  eventbox=g_list_nth_data (mt->video_draws,mt->track_selected);
  gtk_widget_set_sensitive (mt->jumpto, g_object_get_data(G_OBJECT(eventbox),"blocks")!=(gulong)NULL);

  if (mt->block_selected) {
    gtk_widget_set_sensitive (mt->delblock, TRUE);
    if (mt->poly_state==POLY_IN_OUT&&mt->block_selected->ordered) {
      gint64 offset_end=mt->block_selected->offset_start+(gint64)(U_SEC/mt->fps)+(get_event_timecode(mt->block_selected->end_event->event)-get_event_timecode(mt->block_selected->start_event->event));
      
      set_in_out_spin_ranges(mt,mt->block_selected->offset_start,offset_end);
    }
  }
  track_select(mt);
}


//////////////////////////////////////////////////////////////////
void multitrack_preview_clicked  (GtkWidget *button, gpointer user_data) {
  //preview during rendering
  if (mainw->playing_file==-1) multitrack_playall(button,user_data);
  else on_preview_clicked((GtkButton *)button,NULL);
}




void multitrack_playall (GtkWidget *widget, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;

  if (mainw->current_file<1) return;
  
  mt_desensitise(mt);

  gtk_widget_ref(mt->play_blank);
  gtk_container_remove (GTK_CONTAINER(mt->play_box),mt->play_blank);

  gtk_widget_ref (mainw->playarea);

  if (mainw->plug1!=NULL) {
    gtk_container_remove (GTK_CONTAINER(mainw->plug1),mainw->image274);
    gtk_widget_destroy (mainw->plug1);
    mainw->plug1=NULL;
  }

  if (mainw->playarea->parent!=NULL) gtk_container_remove (GTK_CONTAINER(mainw->playarea->parent),mainw->playarea);
  gtk_box_pack_start (GTK_BOX (mt->play_box), mainw->playarea, TRUE, TRUE, 0);
  gtk_widget_unref (mainw->playarea);

  mainw->xwin=0;

  gtk_widget_set_sensitive (mt->stop,TRUE);
  mt->ptr_time=GTK_RULER(mt->timeline)->position;

  if (mt->is_rendering) {
    // preview during rendering
    on_preview_clicked((GtkButton *)widget,NULL);
    mainw->cancelled=0;
  }
  else {
    if (mt->event_list!=NULL) {
      mainw->is_rendering=TRUE;
      on_preview_clicked (NULL,GINT_TO_POINTER(1));
      mainw->is_rendering=mainw->is_processing=FALSE;
      mainw->cancelled=0;
      cfile->event_list=NULL;
      cfile->next_event=NULL;
    }
    time_to_string (mt,0.,TIMECODE_LENGTH);
  }

  gtk_widget_set_sensitive (mt->stop,FALSE);
    
  gtk_widget_ref (mainw->playarea);

  if (mainw->plug1!=NULL) {
    gtk_container_remove (GTK_CONTAINER(mainw->plug1),mainw->image274);
    gtk_widget_destroy (mainw->plug1);
    mainw->plug1=NULL;
  }

  if (mainw->playarea->parent!=NULL) gtk_container_remove (GTK_CONTAINER(mainw->playarea->parent),mainw->playarea);
  gtk_container_add (GTK_CONTAINER(mt->play_box),mt->play_blank);
  gtk_container_add (GTK_CONTAINER (mainw->playframe), mainw->playarea);
  gtk_widget_unref (mainw->playarea);
  gtk_widget_unref(mt->play_blank);

  mainw->xwin=0;

  GTK_RULER (mt->timeline)->position=mt->ptr_time;
  gtk_widget_queue_draw (mt->timeline);
  mt_sensitise(mt);
}




void multitrack_insert (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  file *sfile=mainw->files[mt->file_selected];
  gdouble secs=GTK_RULER (mt->timeline)->position/GTK_RULER (mt->timeline)->upper*mt->end_secs;
  GtkWidget *eventbox=g_list_nth_data(mt->video_draws,mt->track_selected);

  insert_frames (mt->file_selected,(gint64)((sfile->start-1.)/sfile->fps*U_SEC),(gint64)((sfile->end/sfile->fps)*U_SEC),secs*U_SECL,DIRECTION_POSITIVE,eventbox,mt,NULL);

  if (!resize_timeline(mt)) gtk_widget_queue_draw (eventbox); // redraw the track
}

 
void insert_frames (gint filenum, gint64 offset_start, gint64 offset_end, gint64 tc, gshort direction, GtkWidget *eventbox, lives_mt *mt, track_rect *in_block) {
  // insert the selected frames from mainw->files[filenum] from source file filenum into mt->event_list starting at timeline timecode tc
  // if in_block is non-NULL, then we extend (existing) in_block with the new frames; otherwise we create a new block and insert it into eventbox

  // this is quite complex as the framerates of the sourcefile and the timeline might not match. Therefore we resample (in memory) our source file
  // After resampling, we insert resampled frames from offset_start (inclusive) to offset_end (non-inclusive) [forwards]
  // or from offset_start (non-inclusive) to offset_end (inclusive) if going backwards 

  // if we are inserting in an existing block, we can only use this to extend the end (not shrink it)


  // TODO - handle insert before, insert after

  // TODO - handle insert with/without audio

  // TODO - handle case where frames are overwritten


  int numframes,i;
  gint render_file=mainw->current_file;
  gboolean isfirst=TRUE;
  file *sfile=mainw->files[filenum];
  gint track=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(eventbox),"layer_number"));
  gint64 last_tc=0,offset_start_tc,start_tc,last_offset;
  int *clips=NULL,*frames=NULL,*rep_clips,*rep_frames,error;
  gint64 time=0;
  event_t *last_frame_event;
  event_list_t *event_list;
  int blank_clip=-1,blank_frame=0;
  event_t *event,*shortcut1=NULL,*shortcut2=NULL;
  int frame=(offset_start/U_SEC/mt->fps+1.4999);
  gint64 tl=q_dbl(1./mt->fps,mt->fps);
  track_rect *new_block=NULL;

  mt_desensitise(mt);

  last_offset=offset_start_tc=q_gint64(offset_start,mt->fps);
  offset_end=q_gint64(offset_end,mt->fps);
  start_tc=q_gint64(tc,mt->fps);
  if (direction==DIRECTION_NEGATIVE) tc-=U_SEC/mt->fps;
  last_tc=q_gint64(tc,mt->fps);

  if (direction==DIRECTION_POSITIVE) {
    // fill in blank frames in a gap
    if (mt->event_list!=NULL&&(last_frame_event=get_last_frame_event(mt->event_list))!=NULL) time=get_event_timecode(last_frame_event->event)+tl;

    for (;time<start_tc;time+=tl) {
      time=q_gint64(time,mt->fps);
      // TODO - memcheck
      event_list=insert_frame_event_at (mt->event_list,time,1,&blank_clip,&blank_frame,&shortcut1);
      if (event_list!=NULL) {
	if (mt->event_list==NULL) mt->event_list=event_list;
	else if (mt->event_list->first==NULL&&mt->event_list->last==NULL) {
	  mt->event_list->first=event_list->first;
	  mt->event_list->last=event_list->last;
	}
      }
    }
  }

  mainw->current_file=filenum;

  if (cfile->fps!=mt->fps&&cfile->event_list==NULL) {
    // resample clip to render fps
    cfile->undo1_dbl=mt->fps;
    on_resample_vid_ok (NULL,NULL);
  }

  mainw->current_file=render_file;

  while (((direction==DIRECTION_POSITIVE&&(offset_start=q_gint64(last_tc-start_tc+offset_start_tc,mt->fps))<offset_end)||(direction==DIRECTION_NEGATIVE&&(offset_start=q_gint64(last_tc+offset_start_tc-start_tc,mt->fps))>=offset_end))) {

    numframes=0;
    
    if ((event=get_frame_event_at (mt->event_list,last_tc,shortcut1))!=NULL) {
      // TODO - memcheck
      numframes=weed_leaf_num_elements(event->event,"clips");
      clips=weed_get_int_array(event->event,"clips",&error);
      frames=weed_get_int_array(event->event,"frames",&error);
      shortcut1=event;
    }

    if (numframes<=track) {
      // TODO - memcheck
      rep_clips=(int *)g_malloc(track*sizint+sizint);
      rep_frames=(int *)g_malloc(track*sizint+sizint);

      for (i=0;i<track;i++) {
	if (i<numframes) {
	  rep_clips[i]=clips[i];
	  rep_frames[i]=frames[i];
	}
	else {
	  rep_clips[i]=-1;
	  rep_frames[i]=0;
	}
      }
      numframes=track+1;
    }
    else {
      if (mt->insert_mode==INSERT_MODE_NORMAL&&frames[track]>0) {
	if (in_block==NULL&&new_block!=NULL) {
	  if (direction==DIRECTION_POSITIVE) {
	    new_block->max_tc=last_tc;
	    shortcut1=get_prev_frame_event(shortcut1);
	  }
	  else new_block->min_tc=last_tc+tl;
	}
	g_free(clips);
	g_free(frames);
	break; // do not allow overwriting in this mode
      }
      rep_clips=clips;
      rep_frames=frames;
    }

    if (sfile->event_list!=NULL) event=get_frame_event_at(sfile->event_list,offset_start,shortcut2);
    if (sfile->event_list!=NULL&&event==NULL) break; // insert finished: ran out of frames in resampled clip
    last_offset=offset_start;
    if (sfile->event_list!=NULL) {
      // frames were resampled, get new frame at the source file timecode
      frame=weed_get_int_value(event->event,"frames",&error);
      if (direction==DIRECTION_POSITIVE) shortcut2=event;
      else shortcut2=get_prev_frame_event(event); // TODO : this is not optimal for the first frame
    }
    rep_clips[track]=filenum;
    rep_frames[track]=frame;

    // TODO - memcheck
    event_list=insert_frame_event_at (mt->event_list,last_tc,numframes,rep_clips,rep_frames,&shortcut1);
    if (isfirst) {
      // TODO - memcheck
      if (in_block==NULL) new_block=add_block_start_point (eventbox,last_tc,filenum,offset_start,shortcut1,TRUE);
      isfirst=FALSE;
    }
    if (event_list!=NULL) {
      if (mt->event_list==NULL) mt->event_list=event_list;
      else if (mt->event_list->first==NULL&&mt->event_list->last==NULL) {
	mt->event_list->first=event_list->first;
	mt->event_list->last=event_list->last;
      }
    }

    g_free(clips);
    g_free(frames);
    
    clips=rep_clips=NULL;
    frames=rep_frames=NULL;
    if (direction==DIRECTION_POSITIVE) last_tc+=tl;
    else {
      if (last_tc<tl) break;
      last_tc-=tl;
    }
    last_tc=q_gint64(last_tc,mt->fps);
    if (sfile->event_list==NULL) if ((direction==DIRECTION_POSITIVE&&(++frame>sfile->frames))||(direction==DIRECTION_NEGATIVE&&(--frame<1))) break;
  }

  if (!isfirst) {
    if (direction==DIRECTION_POSITIVE) {
      if (in_block!=NULL) g_object_set_data (G_OBJECT(eventbox),"block_last",(gpointer)in_block);
      add_block_end_point(eventbox,shortcut1);
    }
    else if (in_block!=NULL) {
      in_block->offset_start=last_offset;
      in_block->start_event=shortcut1;
    }
  }

  mt->last_direction=direction;

  mt_sensitise(mt);
}


void multitrack_view_events (GtkMenuItem *menuitem, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  GtkWidget *elist_dialog=create_event_list_dialog(mt->event_list);
  gtk_dialog_run(GTK_DIALOG(elist_dialog));
  gtk_widget_destroy(elist_dialog);
}

////////////////////////////////////////////////////////
// region functions

void draw_region (lives_mt *mt) {
  gdouble start,end;
  if (mt->region_start==mt->region_end) return;

  gtk_widget_queue_draw(mt->timeline);
  while (g_main_context_iteration(NULL,FALSE));

  if (mt->region_start<mt->region_end) {
    start=mt->region_start;
    end=mt->region_end;
  }
  else {
    start=mt->region_end;
    end=mt->region_start;
  }

  gdk_draw_rectangle (GDK_DRAWABLE(mt->timeline->window), mainw->LiVES->style->black_gc, TRUE, start*mt->timeline->allocation.width/mt->end_secs, 0, (end-start)*mt->timeline->allocation.width/mt->end_secs,mt->timeline->allocation.height-1);
}

gint expose_timeline_event (GtkWidget *timeline, GdkEventExpose *event, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  draw_region (mt);
  return FALSE;
}


////////////////////////////////////////////////////

// functions for moving and clicking on the timeline


gboolean
on_timeline_update (GtkWidget *widget, GdkEventMotion *event, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  gint x;
  gdouble pos;

  if (!mt->region_updating) return TRUE;
  if (mainw->playing_file>-1) return TRUE;

  gdk_window_get_pointer(GDK_WINDOW (widget->window), &x, NULL, NULL);

  if (x<0) x=0;

  pos=x*mt->end_secs/widget->allocation.width;
  pos=(gint)(pos*mt->fps+.5)/mt->fps; // quantise to nearest frame
  if (pos>mt->end_secs) pos=mt->end_secs;

  mt->region_end=pos;
  if (mt->region_end!=mt->region_start) draw_region (mt);
  return TRUE;
}



gboolean
on_timeline_release (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  //button release
  lives_mt *mt=(lives_mt *)user_data;
  gint x;
  gdouble pos;

  if (mainw->playing_file>-1) return FALSE;

  gdk_window_get_pointer(GDK_WINDOW (widget->window), &x, NULL, NULL);

  if (x<0) x=0;

  pos=x*mt->end_secs/widget->allocation.width;
  pos=(gint)(pos*mt->fps+.5)/mt->fps; // quantise to nearest frame
  if (pos>mt->end_secs) pos=mt->end_secs;

  mt->region_end=pos;
  mt->region_updating=FALSE;

  if (mt->region_end!=mt->region_start) {
    if (mt->region_end<mt->region_start) {
      mt->region_start-=mt->region_end;
      mt->region_end+=mt->region_start;
      mt->region_start=mt->region_end-mt->region_start;
    }
    draw_region (mt);
  }
  else {
    GTK_RULER (mt->timeline)->position=pos;
    gtk_widget_queue_draw (mt->timeline);
  }
  return FALSE;
}


gboolean
on_timeline_press (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
  lives_mt *mt=(lives_mt *)user_data;
  gint x;
  gdouble pos;

  if (mainw->playing_file>-1) return FALSE;

  gdk_window_get_pointer(GDK_WINDOW (widget->window), &x, NULL, NULL);
  if (x<0) x=0;

  pos=x*mt->end_secs/widget->allocation.width;
  pos=(gint)(pos*mt->fps+.5)/mt->fps; // quantise to nearest frame
  if (pos>mt->end_secs) pos=mt->end_secs;

  mt->region_start=mt->region_end=pos;
  mt->region_updating=TRUE;

  GTK_RULER (mt->timeline)->position=pos;
  gtk_widget_queue_draw (mt->timeline);
  return FALSE;
}

