// multitrack.h
// LiVES
// (c) G. Finch 2005 <salsaman@xs4all.nl>
// released under the GNU GPL 2.0
// see file ../COPYING for licensing details

// multitrack window

#ifndef __HAS_MULTITRACK_H__
#define __HAS_MULTITRACK_H__

#include "events.h"

#define CLIP_THUMB_WIDTH 80
#define CLIP_THUMB_HEIGHT 80

#define CLIP_LABEL_LENGTH 20

#define BLOCK_THUMB_WIDTH 40

#define BLOCK_DRAW_SIMPLE 1
#define BLOCK_DRAW_THUMB 2
#define BLOCK_DRAW_TYPE BLOCK_DRAW_THUMB

#define MT_PLAY_WIDTH 320
#define MT_PLAY_HEIGHT 240

#define TIMECODE_LENGTH 14 // length of timecode text entry, must be >12

#define DEF_TIME 120 // default seconds when there is no event_list

typedef struct _mt lives_mt;

typedef struct _track_rect track_rect;

struct _mt {
  // widgets
  GtkWidget *window;
  GtkWidget *top_vbox;
  GtkWidget *hbox;
  GtkWidget *play_blank;
  GtkWidget *play_box;
  GtkWidget *poly_box;
  GtkWidget *clip_scroll;
  GtkWidget *clip_inner_box;
  GtkWidget *in_out_box;
  GtkWidget *in_hbox;
  GtkWidget *out_hbox;
  GtkWidget *in_image;
  GtkWidget *out_image;
  GtkWidget *context_box;
  GtkWidget *sep_image;
  GtkWidget *timeline_table_header;
  GtkWidget *timeline_table;
  GtkWidget *timeline;
  GtkWidget *timeline_eb;
  GtkWidget *infobar;
  GtkWidget *stop;
  GtkWidget *insert;
  GtkWidget *delblock;
  GtkWidget *clipedit;
  GtkWidget *vpaned;
  GtkWidget *scrollbar;
  GtkWidget *playall;
  GtkWidget *jumpto;
  GtkWidget *render;
  GtkWidget *view_events;
  GtkWidget *view_clips;
  GtkWidget *view_in_out;
  GtkWidget *spinbutton_in;
  GtkWidget *spinbutton_out;
  GtkWidget *checkbutton_start_anchored;
  GtkWidget *checkbutton_end_anchored;
  GtkWidget *timecode;

  GtkAccelGroup *accel_group;
  GList *video_draws; // list of video timeline eventboxes, in layer order
  GtkObject *vadjustment;

  gulong spin_in_func;
  gulong spin_out_func;
  gulong check_start_func;
  gulong check_end_func;
  
  event_list_t *event_list;

  ///////////////////////
  gint num_video_tracks;
  gdouble end_secs;  // max display time of timeline in seconds

  gint clip_selected; // clip in clip window
  gint file_selected; // actual LiVES file struct number which clip_selected matches
  gint track_selected; // TODO - allow selection of multiple tracks; starts at 0; -1 means multiple tracks selected

  gshort poly_state;  // state of polymorph window
#define POLY_NONE 0
#define POLY_CLIPS 1
#define POLY_IN_OUT 2


  gshort insert_mode;
#define INSERT_MODE_NORMAL 1  // the default
#define INSERT_MODE_OVERWRITE 2
#define INSERT_MODE_EXPAND 3
  //#define INSERT_MODE_FILL 4

  gint render_file;

  gshort last_direction; // last direction timeline cursor was moved in
#define DIRECTION_NEGATIVE 0
#define DIRECTION_POSITIVE 1

  track_rect *block_selected; // pointer to current selected block, or NULL

  gdouble ptr_time; // stored timeline cursor position before playback

  gdouble tl_fixed_length; // length of timeline can be fixed (secs) : TODO

  gdouble fps; // fps of this timeline

  gdouble region_start; // start of time region (or 0.)
  gdouble region_end; // end of time region (or 0.)
  gboolean region_updating;

  gboolean is_rendering; // TRUE if we are is the process of rendering to a clip, cf. mainw->is_rendering

};


struct _track_rect {
  // track rectangles (blocks), we translate our event_list FRAME events into these, then when exposed, the eventbox draws them
  // blocks MUST only contain frames from a single clip. They MAY NOT contain blank frames.
  // start and end events MUST be FRAME events
  track_rect *next;
  track_rect *prev;
  event_t *start_event;
  event_t *end_event;

  gint64 offset_start; // offset in sourcefile of first frame

  gshort state;
  gboolean start_anchored;
  gboolean end_anchored;
  gboolean ordered; // are frames in sequential order ?
  
  GtkWidget *eventbox; // pointer to eventbox widget which contains this block; we can use its "layer_number" to get the track/layer number

  // if block is bounded either side by another block, we set the limits here : TODO
  gint64 min_tc;
  gint64 max_tc;
};


// block state
#define BLOCK_UNSELECTED 0
#define BLOCK_SELECTED 1




//////////////////////////////////////////////////////////////////

// setup functions
gboolean on_multitrack_activate (GtkMenuItem *, event_list_t *); // widget callback to create the multitrack window
lives_mt *multitrack (event_list_t *, gint orig_file, gdouble fps); // create and return lives_mt struct
void init_tracks (lives_mt *mt);  // add basic tracks
void init_clips (lives_mt *mt, gint orig_file); // init clip window

// delete functions
gboolean on_mt_delete_event (GtkWidget *, GdkEvent *, gpointer mt);
void multitrack_delete (lives_mt *); // free the lives_mt struct

// morph the poly window
void polymorph (lives_mt *, gshort poly);

/// sens/desens
void mt_desensitise (lives_mt *);
void mt_sensitise (lives_mt *);


//// menuitem callbacks
void multitrack_end (GtkMenuItem *, gpointer mt);  // menuitem CLIP EDITOR
void on_add_video_track_activate (GtkMenuItem *, gpointer mt);
void multitrack_playall (GtkWidget *, gpointer mt); // menuitem playall
void multitrack_insert (GtkMenuItem *, gpointer mt); // menuitem insert
void multitrack_view_events (GtkMenuItem *, gpointer mt); // menuitem insert
void on_render_activate (GtkMenuItem *, gpointer mt);
void on_jumpto_activate (GtkMenuItem *, gpointer mt);
void on_delblock_activate (GtkMenuItem *, gpointer mt);

/// track functions
void add_video_track_behind (GtkMenuItem *, gpointer mt);
void add_video_track_front (GtkMenuItem *, gpointer mt);
gint expose_vidtrack_event (GtkWidget *eventbox, GdkEventExpose *, gpointer mt);
gboolean on_track_click (GtkWidget *eventbox, GdkEventButton *, gpointer mt);
void unselect_all (lives_mt *); // unselect all blocks
void insert_frames (gint filenum, gint64 offset_start, gint64 offset_end, gint64 tc, gshort direction, GtkWidget *eventbox, lives_mt *, track_rect *in_block);

// block functions
void in_out_start_changed (GtkWidget *, gpointer mt);
void in_out_end_changed (GtkWidget *, gpointer mt);
void in_anchor_toggled (GtkToggleButton *, gpointer mt);
void out_anchor_toggled (GtkToggleButton *, gpointer mt);

// timeline functions
void set_timeline_end_secs (lives_mt *, gdouble secs);
gboolean on_timeline_press (GtkWidget *, GdkEventButton *, gpointer mt);
gboolean on_timeline_release (GtkWidget *, GdkEventButton *, gpointer mt);
gboolean on_timeline_update (GtkWidget *, GdkEventMotion *, gpointer mt);
gint expose_timeline_event (GtkWidget *, GdkEventExpose *, gpointer mt);


// context box text
void add_context_label (lives_mt *, gchar *text);


// playback animation
void animate_multitrack (lives_mt *);

#endif
