// paramspecial.c
// LiVES
// (c) G. Finch 2004 <salsaman@xs4all.nl>
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details

// dynamic window generation from parameter arrays :-)
// special widgets

#include "main.h"
#include "paramspecial.h"
#include "resample.h"
#include "support.h"

static lives_special_aspect_t aspect;
static lives_special_framedraw_rect_t framedraw;

void 
init_special (void) {
  aspect.width_param=aspect.height_param=0;
  aspect.width_widget=aspect.height_widget=NULL;
  framedraw.xstart_param=framedraw.ystart_param=framedraw.xend_param=framedraw.yend_param=0;
  framedraw.xstart_widget=framedraw.ystart_widget=framedraw.xend_widget=framedraw.yend_widget=NULL;
  framedraw.added=FALSE;
  mergealign.start_param=mergealign.end_param=0;
  mergealign.start_widget=mergealign.end_widget=NULL;
}


gint 
add_to_special (gchar *sp_string, lives_rendered_effect_t *rfx) {
  gchar **array=g_strsplit (sp_string,"|",-1);

  // TODO - make sure only one of each of these


  // returns extra width in pixels
  gint extra_width=0;

  if (!strcmp (array[0],"aspect")) {
    // aspect button
    aspect.width_param=atoi (array[1]);
    aspect.height_param=atoi (array[2]);
  }
  else if (!strcmp (array[0],"mergealign")) {
    // aspect button
    mergealign.start_param=atoi (array[1]);
    mergealign.end_param=atoi (array[2]);
    mergealign.rfx=rfx;
  }
  else if (!strcmp (array[0],"framedraw")) {
    if (!strcmp (array[1],"rectdemask")) {
      // aspect button
      framedraw.type=FD_RECT_DEMASK;
      framedraw.xstart_param=atoi (array[2]);
      framedraw.ystart_param=atoi (array[3]);
      framedraw.xend_param=atoi (array[4]);
      framedraw.yend_param=atoi (array[5]);
    }
    extra_width=RFX_EXTRA_WIDTH;
  }
  g_strfreev (array);
  return extra_width;
}



void
check_for_special (lives_param_t *param, gint num, GtkBox *pbox) {
    GtkWidget *checkbutton;
    GtkWidget *hbox;
    GtkWidget *box;
    GtkWidget *label;
    // check if this parameter is part of a special window
    // as we are drawing the paramwindow
    
    num++;  // 0 based in our array, 1 based in widget structs

    if (num==framedraw.xstart_param) {
      framedraw.xstart_widget=param->widgets[0];
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),0.);
      g_signal_connect_after (GTK_OBJECT (param->widgets[0]), "value_changed", G_CALLBACK (after_framedraw_widget_changed), &framedraw);
    }
    
    if (num==framedraw.ystart_param) {
      framedraw.ystart_widget=param->widgets[0];
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),0.);
      g_signal_connect_after (GTK_OBJECT (param->widgets[0]), "value_changed", G_CALLBACK (after_framedraw_widget_changed), &framedraw);
    }
    if (num==framedraw.xend_param) {
      framedraw.xend_widget=param->widgets[0];
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)cfile->hsize);
      g_signal_connect_after (GTK_OBJECT (param->widgets[0]), "value_changed", G_CALLBACK (after_framedraw_widget_changed), &framedraw);
    }
    if (num==framedraw.yend_param) {
      framedraw.yend_widget=param->widgets[0];
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)cfile->vsize);
      g_signal_connect_after (GTK_OBJECT (param->widgets[0]), "value_changed", G_CALLBACK (after_framedraw_widget_changed), &framedraw);
    }
    if (framedraw.xstart_widget&&framedraw.ystart_widget&&framedraw.xend_widget&&framedraw.yend_widget&&!framedraw.added) {
      widget_add_framedraw (GTK_VBOX (GTK_WIDGET(pbox)->parent),cfile->start, cfile->end, TRUE, &framedraw);
      framedraw_redraw (&framedraw, TRUE);
      framedraw.added=TRUE;
    }
    
    if (num==aspect.width_param) {
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)cfile->hsize);
      aspect.width_func=g_signal_connect_after (GTK_OBJECT (param->widgets[0]), "value_changed",
						G_CALLBACK (after_aspect_width_changed),
						NULL);
      aspect.width_widget=param->widgets[0];
    }
    if (num==aspect.height_param) {
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)cfile->vsize);
      aspect.height_func=g_signal_connect_after (GTK_OBJECT (param->widgets[0]), "value_changed",
						 G_CALLBACK (after_aspect_height_changed),
						 NULL);
      
      box = gtk_hbox_new (FALSE, 10);
      gtk_box_pack_start (GTK_BOX (GTK_WIDGET (pbox)->parent), box, TRUE, TRUE, 0);
      
      checkbutton = gtk_check_button_new ();
      label=gtk_label_new_with_mnemonic (_("Maintain _Aspect Ratio"));
      gtk_label_set_mnemonic_widget (GTK_LABEL (label),checkbutton);
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
      hbox = gtk_hbox_new (FALSE, 10);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 10);
      gtk_box_pack_start (GTK_BOX (hbox), checkbutton, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS (checkbutton, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), get_bool_param(param->value));
      aspect.checkbutton=checkbutton;
      aspect.height_widget=param->widgets[0];
    }
    if (num==mergealign.start_param) mergealign.start_widget=param->widgets[0];
    if (num==mergealign.end_param) mergealign.end_widget=param->widgets[0];
}


void after_aspect_width_changed (GtkSpinButton *spinbutton, gpointer user_data) {
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (aspect.checkbutton))) {
    gint width=gtk_spin_button_get_value_as_int (spinbutton);
    g_signal_handler_block (aspect.height_widget,aspect.height_func);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (aspect.height_widget), (gdouble)(gint)(width*cfile->vsize/cfile->hsize+.5));
    g_signal_handler_unblock (aspect.height_widget,aspect.height_func);
  }
}


void after_aspect_height_changed (GtkToggleButton *spinbutton, gpointer user_data){
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (aspect.checkbutton))) {
    gint height=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
    g_signal_handler_block (aspect.width_widget,aspect.width_func);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (aspect.width_widget), (gdouble)(gint)(height*cfile->hsize/cfile->vsize+.5));
    g_signal_handler_unblock (aspect.width_widget,aspect.width_func);
  }
}


void special_cleanup (void) {
  // free some memory now
  if (!(mainw->framedraw_copy_pixmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_copy_pixmap);
    mainw->framedraw_copy_pixmap=NULL;
  }

  if (!(mainw->framedraw_bitmap==NULL)) {
    gdk_pixmap_unref (mainw->framedraw_bitmap);
    mainw->framedraw_bitmap=NULL;
  }

  if (!(mainw->framedraw_bitmapgc==NULL)) {
    g_object_unref (mainw->framedraw_bitmapgc);
    mainw->framedraw_bitmapgc=NULL;
  }

  if (mainw->framedraw_colourgc!=NULL) {
    g_object_unref (mainw->framedraw_colourgc);
    mainw->framedraw_colourgc=NULL;
  }
  mainw->framedraw_image=mainw->framedraw_reset=NULL;
}



void setmergealign (void) {
  lives_param_t *param;
  gint cb_frames=clipboard->frames;

  // TODO - tidy

  if (prefs->ins_resample==TRUE&&clipboard->fps!=cfile->fps) {
    cb_frames=count_resampled_frames(clipboard->frames,clipboard->fps,cfile->fps);
  }

  if (cfile->end-cfile->start+1>(cb_frames*gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (merge_opts->spinbutton_loops)))&&merge_opts->loop_to_fit==FALSE) {
    // set special transalign widgets to their default values
    if (mergealign.start_widget!=NULL&&GTK_IS_SPIN_BUTTON (mergealign.start_widget)&&(param=&(mergealign.rfx->params[mergealign.start_param-1]))->type==LIVES_PARAM_NUM) {
      if (param->dp) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.start_widget),get_double_param (param->def));
      }
      else {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.start_widget),(gdouble)get_int_param (param->def));
      }
    }
    if (mergealign.end_widget!=NULL&&GTK_IS_SPIN_BUTTON (mergealign.end_widget)&&(param=&(mergealign.rfx->params[mergealign.end_param-1]))->type==LIVES_PARAM_NUM) {
      if (param->dp) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.end_widget),get_double_param (param->def));
      }
      else {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.end_widget),(gdouble)get_int_param (param->def));
      }
    }
  }
  else {
    if (merge_opts->align_start) {
      // set special transalign widgets to min/max values
      if (mergealign.start_widget!=NULL&&GTK_IS_SPIN_BUTTON (mergealign.start_widget)&&(param=&(mergealign.rfx->params[mergealign.start_param-1]))->type==LIVES_PARAM_NUM) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.start_widget),(gdouble)param->min);
      }
      if (mergealign.end_widget!=NULL&&GTK_IS_SPIN_BUTTON (mergealign.end_widget)&&(param=&(mergealign.rfx->params[mergealign.end_param-1]))->type==LIVES_PARAM_NUM) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.end_widget),(gdouble)param->max);
      }
    }
    else {
      // set special transalign widgets to max/min values
      if (mergealign.start_widget!=NULL&&GTK_IS_SPIN_BUTTON (mergealign.start_widget)&&(param=&(mergealign.rfx->params[mergealign.start_param-1]))->type==LIVES_PARAM_NUM) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.start_widget),(gdouble)param->max);
      }
      if (mergealign.end_widget!=NULL&&GTK_IS_SPIN_BUTTON (mergealign.end_widget)&&(param=&(mergealign.rfx->params[mergealign.end_param-1]))->type==LIVES_PARAM_NUM) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (mergealign.end_widget),(gdouble)param->min);
      }
    }
  }
}
