// paramwindow.c
// LiVES
// (c) G. Finch 2004 <salsaman@xs4all.nl>
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details

// dynamic window generation from parameter arrays :-)



// NOTE: this is now the "house style" for all LiVES widgets, there should be a consistent look across all 
// windows.




#include "main.h"
#include "paramwindow.h"
#include "callbacks.h"
#include "support.h"
#include "paramspecial.h"
#include "resample.h"
#include "weed-utils.h"
#include "effects-weed.h"

extern gboolean do_effect (lives_rendered_effect_t *); //effects.c in LiVES

// TODO -
// use list of these in case we have multiple windows open
// right now this is single threaded because of this
static GSList *usrgrp_to_livesgrp[2]={NULL,NULL}; // ordered list of lives_widget_group_t

void
on_paramwindow_ok_clicked (GtkButton *button, lives_rendered_effect_t *rfx) {
  if (button!=NULL) {
    gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  }
  if (usrgrp_to_livesgrp[0]!=NULL) g_slist_free (usrgrp_to_livesgrp[0]);
  usrgrp_to_livesgrp[0]=NULL;
  on_render_fx_activate (NULL,rfx);
  special_cleanup();
}



void
on_paramwindow_cancel_clicked2 (GtkButton *button, lives_rendered_effect_t *rfx) {
  on_paramwindow_cancel_clicked(button,rfx);
  fx_dialog[1]=NULL;
}

void
on_paramwindow_cancel_clicked (GtkButton *button, lives_rendered_effect_t *rfx) {
  if (button!=NULL) {
    gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  }
  if (rfx==NULL) {
    if (usrgrp_to_livesgrp[1]!=NULL) g_slist_free (usrgrp_to_livesgrp[1]);
    usrgrp_to_livesgrp[1]=NULL;
  }
  else {
    if (usrgrp_to_livesgrp[0]!=NULL) g_slist_free (usrgrp_to_livesgrp[0]);
    usrgrp_to_livesgrp[0]=NULL;
    if (rfx->status==RFX_STATUS_WEED) rfx_free(rfx);
  }
  special_cleanup();
}


GtkWidget *set_groups(lives_rendered_effect_t *rfx, lives_param_t *param) {
  GtkWidget *radiobutton;
  lives_widget_group_t *group;

  if (rfx->status==RFX_STATUS_WEED) {
    if ((group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[1], param->group))==NULL) {
      radiobutton = gtk_radio_button_new (NULL);
      usrgrp_to_livesgrp[1]=add_usrgrp_to_livesgrp (usrgrp_to_livesgrp[1], gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton)), param->group);
    }
    else {
      radiobutton = gtk_radio_button_new (group->rbgroup);
      group->rbgroup=gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton));
    }
  }
  else {
    if ((group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[0], param->group))==NULL) {
      radiobutton = gtk_radio_button_new (NULL);
      usrgrp_to_livesgrp[0]=add_usrgrp_to_livesgrp (usrgrp_to_livesgrp[0], gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton)), param->group);
    }
    else {
      radiobutton = gtk_radio_button_new (group->rbgroup);
      group->rbgroup=gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton));
    }
  }
  return radiobutton;
}



void
on_render_fx_activate (GtkMenuItem *menuitem, lives_rendered_effect_t *rfx) {
  GList *onchange=NULL;
  gchar **array;
  gchar *dir;
  int i;

  // do onchange|init
  if (menuitem!=NULL) {
    switch (rfx->status) {
    case RFX_STATUS_BUILTIN:
      dir=g_strdup(PLUGIN_RENDERED_EFFECTS_BUILTIN);
      break;
    case RFX_STATUS_CUSTOM:
      dir=g_strdup(PLUGIN_RENDERED_EFFECTS_CUSTOM);
      break;
    default:
      dir=g_strdup(PLUGIN_RENDERED_EFFECTS_TEST);
      break;
    }
    if ((onchange=plugin_request_by_line (dir,rfx->name,"get_onchange"))!=NULL) {
      for (i=0;i<g_list_length (onchange);i++) {
	array=g_strsplit (g_list_nth_data (onchange,i),"|",-1);
	if (!strcmp (array[0],"init")) {
	  // onchange is init
	  // create dummy object with data
	  GtkWidget *dummy_widget=gtk_label_new(NULL);
	  g_object_set_data (G_OBJECT (dummy_widget),"param_number",GINT_TO_POINTER (-1));
	  do_onchange (G_OBJECT (dummy_widget),rfx);
	  gtk_widget_destroy (dummy_widget);
	}
	g_strfreev (array);
      }
      g_list_free (onchange);
    }
    g_free (dir);
  }
  if (rfx->min_frames>-1) {
    do_effect(rfx);
  }
}


void
on_render_fx_pre_activate (GtkMenuItem *menuitem, lives_rendered_effect_t *rfx) {
  GtkWidget *top_dialog_vbox;
  GtkWidget *dialog_action_area;
  GtkWidget *cancelbutton;
  GtkWidget *okbutton;
  GtkAccelGroup *fxw_accel_group;

  gchar *txt;
  gboolean no_process=FALSE;

  int n=0;

  if (menuitem==NULL) {
    no_process=TRUE;
    n=1;
  }

  if (rfx->min_frames<0) no_process=TRUE;
  if (!no_process) {
    // check we have a real clip open
    if (mainw->current_file<=0) return;
    if (cfile->end-cfile->start+1<rfx->min_frames) {
      txt=g_strdup_printf (_("\nYou must select at least %d frames to use this effect.\n\n"),rfx->min_frames);
      do_blocking_error_dialog (txt);
      g_free (txt);
      return;
    }
    
    // here we invalidate cfile->ohsize, cfile->ovsize
    cfile->ohsize=cfile->hsize;
    cfile->ovsize=cfile->vsize;
    
    if (cfile->undo_action==UNDO_RESIZABLE) {
      set_undoable(NULL,FALSE);
    }
  }

  fx_dialog[n] = gtk_dialog_new ();
  txt=g_strdup_printf ("LiVES: - %s",_(rfx->menu_text));
  gtk_window_set_title (GTK_WINDOW (fx_dialog[n]), txt);
  g_free (txt);

  if (n==1) gtk_window_add_accel_group (GTK_WINDOW (fx_dialog[1]), mainw->accel_group);

  gtk_container_set_border_width (GTK_CONTAINER (fx_dialog[n]), 20);
  gtk_window_set_position (GTK_WINDOW (fx_dialog[n]), GTK_WIN_POS_CENTER);

  if (menuitem!=NULL) {
    // activated from the menu for a rendered effect
    gtk_window_set_transient_for(GTK_WINDOW(fx_dialog[n]),GTK_WINDOW(mainw->LiVES));
    gtk_window_set_modal (GTK_WINDOW (fx_dialog[n]), TRUE);
  }

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(fx_dialog[n], GTK_STATE_NORMAL, &palette->normal_back);
    gtk_dialog_set_has_separator(GTK_DIALOG(fx_dialog[n]),FALSE);
  }

  top_dialog_vbox = GTK_DIALOG (fx_dialog[n])->vbox;
  make_param_box(GTK_VBOX (top_dialog_vbox), rfx);

  dialog_action_area = GTK_DIALOG (fx_dialog[n])->action_area;
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area), 80);
  gtk_box_set_spacing (GTK_BOX (dialog_action_area),10);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (dialog_action_area), DEF_BUTTON_WIDTH, -1);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);
  if (!no_process) {
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_END);
    gtk_dialog_add_action_widget (GTK_DIALOG (fx_dialog[n]), cancelbutton, GTK_RESPONSE_CANCEL);
    okbutton = gtk_button_new_from_stock ("gtk-ok");
    gtk_dialog_add_action_widget (GTK_DIALOG (fx_dialog[n]), okbutton, GTK_RESPONSE_OK);
  }
  else {
    okbutton = gtk_button_new_with_mnemonic (_("Close _window"));
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_SPREAD);
    gtk_box_pack_start (GTK_BOX (dialog_action_area), okbutton, TRUE, TRUE, 0);
  }

  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton);

  gtk_widget_show_all (fx_dialog[n]);

  fxw_accel_group = GTK_ACCEL_GROUP(gtk_accel_group_new ());
  gtk_window_add_accel_group (GTK_WINDOW (fx_dialog[n]), fxw_accel_group);

  if (no_process) {
    if (n==0) {
      g_signal_connect (GTK_OBJECT (okbutton), "clicked",
			G_CALLBACK (on_paramwindow_cancel_clicked),
			rfx);
    }
    else {
      g_signal_connect (GTK_OBJECT (okbutton), "clicked",
			G_CALLBACK (on_paramwindow_cancel_clicked2),
			rfx);
    }
    gtk_widget_add_accelerator (okbutton, "activate", fxw_accel_group,
				GDK_Escape, 0, 0);
  }
  else {
    g_signal_connect (GTK_OBJECT (okbutton), "clicked",
		      G_CALLBACK (on_paramwindow_ok_clicked),
		      (gpointer)rfx);
    g_signal_connect (GTK_OBJECT (cancelbutton), "clicked",
		      G_CALLBACK (on_paramwindow_cancel_clicked),
		      NULL);
    gtk_widget_add_accelerator (cancelbutton, "activate", fxw_accel_group,
				GDK_Escape, 0, 0);
  }
  g_signal_connect (GTK_OBJECT (fx_dialog[n]), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);
  
}
  




void make_param_box(GtkVBox *top_vbox, lives_rendered_effect_t *rfx) {
  // make a dynamic parameter window
  GtkWidget *param_vbox;
  GtkWidget *top_hbox;
  GtkWidget *hbox=NULL;

  gchar **array;
  gboolean used[rfx->num_params];
  gboolean has_box=FALSE;
  gchar *line;
  gchar *dir;
  GList *hints=NULL;
  GList *onchange=NULL;
  GSList *layout=NULL;
  int i,j,k,pnum;
  int length;

  gint num_tok;
  gchar label_text[256]; // max length of a label in layout hints

  // put whole thing in scrolled window
  GtkWidget *scrolledwindow;
  gint winsize_h=0,max_winsize_h=0;

  gboolean internal=FALSE;
  GtkTooltips *tooltips = gtk_tooltips_new ();

  if (mainw->current_file>0) {
    // initialise special widgets
    init_special();
  }

  if (rfx->status==RFX_STATUS_WEED) usrgrp_to_livesgrp[1]=NULL;
  else usrgrp_to_livesgrp[0]=NULL;

  // paramwindow start, everything goes in top_hbox

  top_hbox = gtk_hbox_new (FALSE, 10);

  // param_vbox holds the dynamic parameters
  param_vbox = gtk_vbox_new (FALSE, 10);
  gtk_box_pack_start (GTK_BOX (top_hbox), param_vbox, TRUE, TRUE, 10);
  gtk_box_set_spacing (GTK_BOX (param_vbox), 5);
  switch (rfx->status) {
  case RFX_STATUS_BUILTIN:
    dir=g_strdup(PLUGIN_RENDERED_EFFECTS_BUILTIN);
    break;
  case RFX_STATUS_CUSTOM:
    dir=g_strdup(PLUGIN_RENDERED_EFFECTS_CUSTOM);
    break;
  case RFX_STATUS_WEED:
    internal=TRUE;
    break;
  default:
    dir=g_strdup(PLUGIN_RENDERED_EFFECTS_TEST);
    break;
  }


  if (!internal) {
    // do onchange|init
    if ((onchange=plugin_request_by_line (dir,rfx->name,"get_onchange"))!=NULL) {
      for (i=0;i<g_list_length (onchange);i++) {
	array=g_strsplit (g_list_nth_data (onchange,i),"|",-1);
	if (!strcmp (array[0],"init")) {
	  // onchange is init
	  // create dummy object with data
	  GtkWidget *dummy_widget=gtk_label_new(NULL);
	  g_object_set_data (G_OBJECT (dummy_widget),"param_number",GINT_TO_POINTER (-1));
	  do_onchange (G_OBJECT (dummy_widget),rfx);
	  gtk_widget_destroy (dummy_widget);
	}
	else {
	  // note other onchanges so we don't have to keep parsing the list
	  gint which=atoi (array[0]);
	  if (which>0&&which<=rfx->num_params) {
	    rfx->params[which-1].onchange=TRUE;
	  }
	}
	g_strfreev (array);
      }
      g_list_free (onchange);
    }
    hints=plugin_request_by_line (dir,rfx->name,"get_param_window");
    g_free(dir);
  }
  else hints=get_external_window_hints(rfx);

  // do param window hints
  if (hints!=NULL) {
    for (i=0;i<g_list_length (hints);i++) {
      if (!strncmp (g_list_nth_data (hints,i),"layout|",7)) {
	layout=g_slist_append (layout,(gchar *)g_list_nth_data (hints,i)+7);
      }
      else if (!strncmp (g_list_nth_data (hints,i),"special|",8)&&mainw->current_file>0) {
	if ((winsize_h=add_to_special((gchar *)g_list_nth_data (hints,i)+8,rfx))>max_winsize_h) max_winsize_h=winsize_h;
      }
    }
    g_list_free (hints);  // no longer needed
  }

  for (i=0;i<rfx->num_params;i++) {
    used[i]=FALSE;
    for (j=0;j<MAX_PARAM_WIDGETS;rfx->params[i].widgets[j++]=NULL);
  }

  // use layout hints to build as much as we can
  for (i=0;i<g_slist_length (layout);i++) {
    has_box=FALSE;
    line=g_slist_nth_data (layout,i);
    num_tok=get_token_count (line,'|');
    // ignore | inside strings
    array=g_strsplit (line,"|",num_tok);
    for (j=0;j<num_tok;j++) {
      if (!strncmp (array[j],"p",1)&&(pnum=atoi ((gchar *)array[j]+1))>0&&pnum<=rfx->num_params&&!used[pnum-1]) {
	// parameter, eg. p1
	if (!has_box) {
	  hbox = gtk_hbox_new (FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (param_vbox), hbox, FALSE, FALSE, 10);
	  has_box=TRUE;
	}
	add_param_to_box (GTK_BOX (hbox),rfx,pnum-1,tooltips);
	used[pnum-1]=TRUE;
      }
      else if (!j&&!strcmp (array[j],"hseparator")) {
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (param_vbox), hbox, FALSE, FALSE, 10);
	add_hsep_to_box (GTK_BOX (hbox));
	j=num_tok;  // ignore anything after hseparator
      }
      else if (!strncmp (array[j],"fill",4)) {
	// can be filln
	if (strlen (array[j])==4||!(length=atoi (array[j]+4))) length=1;
	if (!has_box) {
	  hbox = gtk_hbox_new (FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (param_vbox), hbox, FALSE, FALSE, 10);
	  has_box=TRUE;
	}
	for (k=0;k<length;k++) {
	  add_fill_to_box (GTK_BOX (hbox));
	}
      }
      else if (!strncmp (array[j],"\"",1)) {
	// label
	if (!has_box) {
	  hbox = gtk_hbox_new (FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (param_vbox), hbox, FALSE, FALSE, 10);
	  has_box=TRUE;
	}
	g_snprintf (label_text,256,"%s",array[j]+1);
	while (strcmp (array[j]+strlen (array[j])-1,"\"")&&j<num_tok-1) {
	  g_strappend (label_text,256,array[++j]);
	}
	if (strlen (label_text)>1) {
	  if (!strcmp (label_text+strlen (label_text)-1,"\"")) {
	    memset (label_text+strlen (label_text)-1,0,1);
	  }
	  add_label_to_box (GTK_BOX (hbox),label_text);
	}}}
    g_strfreev (array);
  }
  if (layout!=NULL) g_slist_free (layout);


  // add any unused parameters
  for (i=0;i<rfx->num_params;i++) {
    if (!used[i]) {
      // TODO - handle weed parameters which need re-init
      add_param_to_box (GTK_BOX (param_vbox),rfx,i,tooltips);
    }
  }

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindow), top_hbox);
  gtk_box_pack_start (GTK_BOX (top_vbox), scrolledwindow, TRUE, TRUE, 0);
  gtk_widget_set_size_request (scrolledwindow, RFX_WINSIZE_H+max_winsize_h, RFX_WINSIZE_V);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(gtk_bin_get_child (GTK_BIN (scrolledwindow)), GTK_STATE_NORMAL, &palette->normal_back);
  }
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (scrolledwindow))),GTK_SHADOW_IN);

}









void add_param_to_box (GtkBox *box, lives_rendered_effect_t *rfx, gint pnum, GtkTooltips *tooltips) {
  // box here is vbox inside top_hbox inside top_dialog

  GtkWidget *label;
  GtkWidget *eventbox;
  GtkWidget *labelcname;
  GtkWidget *checkbutton;
  GtkWidget *radiobutton;
  GtkWidget *spinbutton;
  GtkWidget *scale;
  GtkWidget *spinbutton_red;
  GtkWidget *spinbutton_green;
  GtkWidget *spinbutton_blue;
  GtkWidget *button;
  GtkWidget *entry=NULL;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *combo;
  GtkWidget *dlabel=NULL;
  GtkWidget *textview=NULL;
  GtkWidget *scrolledwindow;
  GtkObject *spinbutton_adj;
  GtkObject *spinbutton_red_adj;
  GtkObject *spinbutton_green_adj;
  GtkObject *spinbutton_blue_adj;

  GtkTextBuffer *textbuffer=NULL;

  lives_param_t *param;
  lives_widget_group_t *group;
  gint maxlen;
  gulong satfunc,tatfunc,catfunc,spinfunc,blockfunc;
  gchar *name;
  gchar *txt;
  gboolean use_mnemonic;

  lives_colRGB24_t rgb;
  lives_colRGB24_t rgb_def;
  gboolean not_default=FALSE;
  char *disp_string;

  if (pnum>=rfx->num_params) {
    add_label_to_box (box,(_("Invalid parameter")));
    return;
  }

  param=&rfx->params[pnum];
  if (param->hidden||param->type==LIVES_PARAM_UNDISPLAYABLE) return;

  name=g_strdup_printf ("%s",param->label);
  use_mnemonic=param->use_mnemonic;
  switch (param->type) {
  case LIVES_PARAM_BOOL :
    if (!param->group) {
      checkbutton = gtk_check_button_new ();
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, checkbutton, param->desc, NULL);
      eventbox=gtk_event_box_new();
      if (use_mnemonic) {
	label=gtk_label_new_with_mnemonic (_ (name));
	gtk_label_set_mnemonic_widget (GTK_LABEL (label),checkbutton);
      }
      else label=gtk_label_new (_ (name));
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);
      gtk_container_add(GTK_CONTAINER(eventbox),label);
      g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
			G_CALLBACK (label_act_toggle),
			checkbutton);
      if (palette->style&STYLE_1) {
	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
      }
      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 10);
      gtk_box_pack_start (GTK_BOX (hbox), checkbutton, FALSE, FALSE, 10);
      gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
      GTK_WIDGET_SET_FLAGS (checkbutton, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), get_bool_param(param->value));
      g_signal_connect_after (GTK_OBJECT (checkbutton), "toggled",
			      G_CALLBACK (after_boolean_param_toggled),
			      (gpointer)rfx);
      
      // store parameter so we know whose trigger to use
      g_object_set_data (G_OBJECT (checkbutton),"param_number",GINT_TO_POINTER (pnum+1));
      param->widgets[0]=checkbutton;
    }
    else {
      radiobutton=set_groups(rfx,param);
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, radiobutton, param->desc, NULL);
      if (rfx->status==RFX_STATUS_WEED) group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[1], param->group);
      else group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[0], param->group);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), get_bool_param(param->value));
      if (get_bool_param(param->value)) {
	group->active_param=pnum+1;
      }

      GTK_WIDGET_SET_FLAGS (radiobutton, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 10);
      if (rfx->status==RFX_STATUS_WEED&&(disp_string=get_weed_display_string(rfx->source,pnum))!=NULL) {
	dlabel=gtk_label_new (g_strdup_printf("(%s)",_ (disp_string)));
	weed_free(disp_string);
	gtk_box_pack_start (GTK_BOX (hbox), dlabel, FALSE, FALSE, 10);
	param->widgets[1]=dlabel;
      }

      gtk_box_pack_start (GTK_BOX (hbox), radiobutton, FALSE, FALSE, 10);
      
      if (use_mnemonic) {
	label=gtk_label_new_with_mnemonic (_ (name));
	gtk_label_set_mnemonic_widget (GTK_LABEL (label),radiobutton);
      }
      else label=gtk_label_new (_ (name));
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);
      eventbox=gtk_event_box_new();
      gtk_container_add(GTK_CONTAINER(eventbox),label);
      g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
			G_CALLBACK (label_act_toggle),
			radiobutton);
      if (palette->style&STYLE_1) {
	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
      }
      gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
      
      g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
			      G_CALLBACK (after_boolean_param_toggled),
			      (gpointer)rfx);
      // store parameter so we know whose trigger to use
      g_object_set_data (G_OBJECT (radiobutton),"param_number",GINT_TO_POINTER (pnum+1));
      param->widgets[0]=radiobutton;
      }
    break;
    
  case LIVES_PARAM_NUM :
    if (param->dp) {
      spinbutton_adj = gtk_adjustment_new (get_double_param(param->value), param->min, param->max, param->step_size, param->step_size*10., 0.);
    }
    else {
      spinbutton_adj = gtk_adjustment_new ((gdouble)(get_int_param(param->value)), param->min, param->max, param->step_size, param->step_size*10., 0.);
    }

    spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, param->dp);
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, spinbutton, param->desc, NULL);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton),TRUE);
    txt=g_strdup_printf ("%d",(int)param->max);
    maxlen=strlen (txt);
    g_free (txt);
    txt=g_strdup_printf ("%d",(int)param->min);
    if (strlen (txt)>maxlen) maxlen=strlen (txt);
    g_free (txt);

    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton),param->wrap);
    gtk_entry_set_width_chars (GTK_ENTRY (spinbutton),maxlen+param->dp<4?4:maxlen+param->dp+1);
    GTK_WIDGET_SET_FLAGS (spinbutton, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton)->entry)), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinbutton),GTK_UPDATE_ALWAYS);
    
    if (use_mnemonic) {
      label = gtk_label_new_with_mnemonic (_ (name));
      gtk_label_set_mnemonic_widget (GTK_LABEL (label), spinbutton);
    }
    else label=gtk_label_new (_ (name));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);
    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }

    if (param->group) {
      radiobutton=set_groups(rfx,param);
      if (rfx->status==RFX_STATUS_WEED) group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[1], param->group);
      else group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[0], param->group);

      if (param->dp) {
	if (get_double_param (param->value)!=get_double_param (param->def)) not_default=TRUE;
      }
      else {
	if (get_int_param (param->value)!=get_int_param (param->def)) not_default=TRUE;
      }
      if (!group->active_param||not_default) {
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
	group->active_param=pnum+1;
      }
      else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), FALSE);

      // store parameter so we know whose default to use
      g_object_set_data (G_OBJECT (radiobutton),"param",(gpointer)param);
      
      GTK_WIDGET_SET_FLAGS (radiobutton, 0);
      gtk_box_pack_start (GTK_BOX (box), radiobutton, FALSE, FALSE, 10);
      
      // activate this radiobutton when value is changed
      // and when deactivated, set back to default
      satfunc=g_signal_connect_after (GTK_OBJECT (spinbutton), "value_changed",
			      G_CALLBACK (widget_act_toggle),
			      radiobutton);
      
      g_object_set_data(G_OBJECT(spinbutton),"satfunc",(gpointer)satfunc);

      g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
			      G_CALLBACK (toggle_set_default_value),
			      spinbutton);
      
    }
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    if (rfx->status==RFX_STATUS_WEED&&(disp_string=get_weed_display_string(rfx->source,pnum))!=NULL) {
      dlabel=gtk_label_new (g_strdup_printf("%s",_ (disp_string)));
      weed_free(disp_string);
      gtk_box_pack_start (GTK_BOX (hbox), dlabel, FALSE, FALSE, 10);
      param->widgets[1]=dlabel;
    }
    else gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 10);
    
    spinfunc=g_signal_connect_after (GTK_OBJECT (spinbutton), "value_changed",
			    G_CALLBACK (after_param_value_changed),
			    (gpointer)rfx);
    g_object_set_data(G_OBJECT(spinbutton),"spinfunc",(gpointer)spinfunc);
    
    // store parameter so we know whose trigger to use
    g_object_set_data (G_OBJECT (spinbutton),"param_number",GINT_TO_POINTER (pnum+1));
    param->widgets[0]=spinbutton;

    if (GTK_IS_VBOX(box)||dlabel!=NULL) {
      scale=gtk_hscale_new_with_range(param->min,param->max>param->min?param->max:param->min,1);
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, scale, param->desc, NULL);
      gtk_box_pack_start (GTK_BOX (hbox), scale, TRUE, TRUE, 10);
      gtk_range_set_adjustment(GTK_RANGE(scale),GTK_ADJUSTMENT(spinbutton_adj));
      gtk_scale_set_draw_value(GTK_SCALE(scale),FALSE);
    }
    break;
    
  case LIVES_PARAM_COLRGB24 :
    get_colRGB24_param(param->value,&rgb);

    // TODO - scale step_size
    spinbutton_red_adj = gtk_adjustment_new (rgb.red, 0, 255, 1, 10, 0.);
    spinbutton_red = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_red_adj), 1, 0);
    spinbutton_green_adj = gtk_adjustment_new (rgb.green, 0, 255, 1, 10, 0.);
    spinbutton_green = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_green_adj), 1, 0);
    spinbutton_blue_adj = gtk_adjustment_new (rgb.blue, 0, 255, 1, 10, 0.);
    spinbutton_blue = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_blue_adj), 1, 0);
    if (param->desc!=NULL) {
      gtk_tooltips_set_tip (tooltips, spinbutton_red, param->desc, NULL);
      gtk_tooltips_set_tip (tooltips, spinbutton_green, param->desc, NULL);
      gtk_tooltips_set_tip (tooltips, spinbutton_blue, param->desc, NULL);
    }

    gtk_entry_set_width_chars (GTK_ENTRY (spinbutton_red),4);
    gtk_entry_set_width_chars (GTK_ENTRY (spinbutton_green),4);
    gtk_entry_set_width_chars (GTK_ENTRY (spinbutton_blue),4);
    GTK_WIDGET_SET_FLAGS (spinbutton_red, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
    GTK_WIDGET_SET_FLAGS (spinbutton_green, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
    GTK_WIDGET_SET_FLAGS (spinbutton_blue, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinbutton_red),GTK_UPDATE_ALWAYS);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinbutton_green),GTK_UPDATE_ALWAYS);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinbutton_blue),GTK_UPDATE_ALWAYS);
    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton_red)->entry)), TRUE);
    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton_green)->entry)), TRUE);
    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton_blue)->entry)), TRUE);
    
    if (param->group) {
      radiobutton=set_groups(rfx,param);
      if (rfx->status==RFX_STATUS_WEED) group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[1], param->group);
      else group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[0], param->group);

      get_colRGB24_param (param->value,&rgb);
      get_colRGB24_param (param->def,&rgb_def);
      if (rgb.red!=rgb_def.red||rgb.green!=rgb_def.green||rgb.blue!=rgb_def.blue) not_default=TRUE;

      if (!group->active_param||not_default) {
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
	group->active_param=pnum+1;
      }
      else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), FALSE);
      GTK_WIDGET_SET_FLAGS (radiobutton, 0);
      gtk_box_pack_start (GTK_BOX (box), radiobutton, FALSE, FALSE, 10);
      
      // store parameter so we know whose default to use
      g_object_set_data (G_OBJECT (radiobutton),"param",(gpointer)param);
      
      // activate this radiobutton when value is changed
      // and when deactivated, set back to default
      satfunc=g_signal_connect_after (GTK_OBJECT (spinbutton_red), "value_changed",
				      G_CALLBACK (widget_act_toggle),
				      radiobutton);
      g_object_set_data(G_OBJECT(spinbutton_red),"satfunc",(gpointer)satfunc);

      satfunc=g_signal_connect_after (GTK_OBJECT (spinbutton_green), "value_changed",
				      G_CALLBACK (widget_act_toggle),
				      radiobutton);
      g_object_set_data(G_OBJECT(spinbutton_green),"satfunc",(gpointer)satfunc);
      satfunc=g_signal_connect_after (GTK_OBJECT (spinbutton_blue), "value_changed",
				      G_CALLBACK (widget_act_toggle),
				      radiobutton);
      g_object_set_data(G_OBJECT(spinbutton_red),"satfunc",(gpointer)satfunc);
      
      g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
			      G_CALLBACK (toggle_set_default_red),
			      spinbutton_red);
      g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
			      G_CALLBACK (toggle_set_default_green),
			      spinbutton_green);
      g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
			      G_CALLBACK (toggle_set_default_blue),
			      spinbutton_blue);
    }
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 10);
    
    // colsel button : TODO - use gtk_color_button
    button = gtk_button_new_from_stock ("gtk-select-color");
    g_object_set_data (G_OBJECT (button),"param_number",GINT_TO_POINTER (pnum+1));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, button, param->desc, NULL);

    if (use_mnemonic) {
      labelcname=gtk_label_new_with_mnemonic (_(name));
      gtk_label_set_mnemonic_widget (GTK_LABEL (labelcname),button);
    }
    else labelcname=gtk_label_new (_(name));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, labelcname, param->desc, NULL);

    gtk_label_set_justify (GTK_LABEL (labelcname), GTK_JUSTIFY_LEFT);
    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(labelcname, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    gtk_box_pack_start (GTK_BOX (hbox), labelcname, FALSE, FALSE, 10);
    
    label=gtk_label_new (_("Red"));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (hbox), spinbutton_red, FALSE, FALSE, 0);
    
    label=gtk_label_new (_("Green"));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    
    gtk_box_pack_start (GTK_BOX (hbox), spinbutton_green, FALSE, FALSE, 0);
    label=gtk_label_new (_("Blue"));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    
    gtk_box_pack_start (GTK_BOX (hbox), spinbutton_blue, FALSE, FALSE, 0);
    
    

    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 20);
    g_signal_connect (GTK_OBJECT (button), "clicked",
		      G_CALLBACK (on_pwcolourbutton_clicked),
		      (gpointer)rfx);
    
    g_signal_connect_after (GTK_OBJECT (spinbutton_red), "value_changed",
			    G_CALLBACK (after_param_red_changed),
			    (gpointer)rfx);
    g_signal_connect_after (GTK_OBJECT (spinbutton_green), "value_changed",
			    G_CALLBACK (after_param_green_changed),
			    (gpointer)rfx);
    g_signal_connect_after (GTK_OBJECT (spinbutton_blue), "value_changed",
			    G_CALLBACK (after_param_blue_changed),
			    (gpointer)rfx);
    
    // store parameter so we know whose trigger to use
    g_object_set_data (G_OBJECT (spinbutton_red),"param_number",GINT_TO_POINTER (pnum+1));
    g_object_set_data (G_OBJECT (spinbutton_green),"param_number",GINT_TO_POINTER (pnum+1));
    g_object_set_data (G_OBJECT (spinbutton_blue),"param_number",GINT_TO_POINTER (pnum+1));
    
    param->widgets[0]=spinbutton_red;
    param->widgets[1]=spinbutton_green;
    param->widgets[2]=spinbutton_blue;
    break;

  case LIVES_PARAM_STRING:
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), vbox, TRUE, TRUE, 0);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    if (rfx->status==RFX_STATUS_WEED&&(disp_string=get_weed_display_string(rfx->source,pnum))!=NULL) {
      if (param->max==0.) txt=g_strdup (disp_string);
      else txt=g_strndup (disp_string,(gint)param->max);
      weed_free(disp_string);
    }
    else {
      if (param->max==0.) txt=g_strdup (param->value);
      else txt=g_strndup (param->value,(gint)param->max);
    }

    if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
      textview = gtk_text_view_new ();
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, textview, param->desc, NULL);
      textbuffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
      
      gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), TRUE);
      gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD);
      gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), TRUE);

      gtk_text_buffer_set_text (textbuffer, txt, -1);
    }
    else {
      entry=gtk_entry_new();
      if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, entry, param->desc, NULL);
      gtk_entry_set_text (GTK_ENTRY (entry),txt);
      gtk_entry_set_max_length(GTK_ENTRY (entry),(gint)param->max);
      gtk_entry_set_width_chars (GTK_ENTRY (entry),(gint)param->max);
    }
    g_free (txt);

    if (param->group) {
      radiobutton=set_groups(rfx,param);
      if (rfx->status==RFX_STATUS_WEED) group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[1], param->group);
      else group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[0], param->group);
      
      if (strcmp (param->value,param->def)) not_default=TRUE;
      
      if (!group->active_param||not_default) {
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
	group->active_param=pnum+1;
      }
      else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), FALSE);
      GTK_WIDGET_SET_FLAGS (radiobutton, 0);
      gtk_box_pack_start (GTK_BOX (hbox), radiobutton, FALSE, FALSE, 10);
      
      // store parameter so we know whose default to use
      g_object_set_data (G_OBJECT (radiobutton),"param",(gpointer)param);
      
      // activate this radiobutton when text is changed
      // and when deactivated, set back to default
      if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
	param->widgets[0]=textview;
	tatfunc=g_signal_connect_after (G_OBJECT (textbuffer), "changed", G_CALLBACK (widget_act_toggle), radiobutton);
	g_object_set_data(G_OBJECT(textview),"tatfunc",(gpointer)tatfunc);
	
	g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
				G_CALLBACK (toggle_set_default_text),
				textview);
      }
      else {
	param->widgets[0]=entry;
	tatfunc=g_signal_connect_after (G_OBJECT (entry), "changed", G_CALLBACK (widget_act_toggle), radiobutton);
	g_object_set_data(G_OBJECT(entry),"tatfunc",(gpointer)tatfunc);
	
	g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
				G_CALLBACK (toggle_set_default_text),
				entry);
      }
    }

    if (use_mnemonic) label = gtk_label_new_with_mnemonic (_(name));
    else label = gtk_label_new (_(name));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);

    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);

    
    if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 10);
      
      scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
      
      gtk_container_add (GTK_CONTAINER (scrolledwindow), textview);
      if (use_mnemonic) gtk_label_set_mnemonic_widget (GTK_LABEL (label),textview);
      gtk_box_pack_start (GTK_BOX (hbox), scrolledwindow, TRUE, TRUE, 10);
      
      blockfunc=g_signal_connect_after (G_OBJECT (textbuffer), "changed", G_CALLBACK (after_param_text_changed), (gpointer) rfx);
      g_object_set_data(G_OBJECT(textbuffer),"blockfunc",(gpointer)blockfunc);
      
      // store parameter so we know whose trigger to use
      g_object_set_data (G_OBJECT (textbuffer),"param_number",GINT_TO_POINTER (pnum+1));
    }
    else {
      if (use_mnemonic) gtk_label_set_mnemonic_widget (GTK_LABEL (label),entry);
      gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 10);
      blockfunc=g_signal_connect_after (G_OBJECT (entry), "changed", G_CALLBACK (after_param_text_changed), (gpointer) rfx);
      g_object_set_data(G_OBJECT(entry),"blockfunc",(gpointer)blockfunc);
      // store parameter so we know whose trigger to use
      g_object_set_data (G_OBJECT (entry),"param_number",GINT_TO_POINTER (pnum+1));

    }
    break;

  case LIVES_PARAM_STRING_LIST:
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 10);
    combo = gtk_combo_new ();
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, combo, param->desc, NULL);

    if (param->group) {
      radiobutton=set_groups(rfx,param);
      if (rfx->status==RFX_STATUS_WEED) group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[1], param->group);
      else group=livesgrp_from_usrgrp (usrgrp_to_livesgrp[0], param->group);
      
      if (get_int_param (param->value)==get_int_param (param->def)) not_default=TRUE;
      
      if (!group->active_param||not_default) {
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
	group->active_param=pnum+1;
      }
      else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), FALSE);
      GTK_WIDGET_SET_FLAGS (radiobutton, 0);
      gtk_box_pack_start (GTK_BOX (hbox), radiobutton, FALSE, FALSE, 10);
      
      // store parameter so we know whose default to use
      g_object_set_data (G_OBJECT (radiobutton),"param",(gpointer)param);
      
      // activate this radiobutton when entry text is changed
      // and when deactivated, set back to default
      catfunc=g_signal_connect_after (G_OBJECT (GTK_COMBO (combo)->entry), "changed", G_CALLBACK (widget_act_toggle), radiobutton);
      g_object_set_data(G_OBJECT(combo),"catfunc",(gpointer)catfunc);
      
      g_signal_connect_after (GTK_OBJECT (radiobutton), "toggled",
			      G_CALLBACK (toggle_set_default_combo),
			      combo);
    }

    if (use_mnemonic) {
      label = gtk_label_new_with_mnemonic (_(name));
      gtk_label_set_mnemonic_widget (GTK_LABEL (label),combo);
    }
    else label = gtk_label_new (_(name));
    if (param->desc!=NULL) gtk_tooltips_set_tip (tooltips, label, param->desc, NULL);

    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }

    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 10);
    gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO (combo))->entry),FALSE);

    combo_set_popdown_strings (GTK_COMBO (combo), param->list);


    if (rfx->status==RFX_STATUS_WEED&&(disp_string=get_weed_display_string(rfx->source,pnum))!=NULL) {
      gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(combo))->entry),disp_string);
      weed_free(disp_string);
    }
    else if (param->list!=NULL) {
      gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(combo))->entry),g_list_nth_data (param->list,get_int_param (param->value)));
    }

    blockfunc=g_signal_connect_after (G_OBJECT (GTK_COMBO(combo)->entry), "changed", G_CALLBACK (after_string_list_changed), (gpointer) rfx);
    g_object_set_data(G_OBJECT(GTK_COMBO(combo)->entry),"blockfunc",(gpointer)blockfunc);

    // store parameter so we know whose trigger to use
    g_object_set_data (G_OBJECT (GTK_COMBO(combo)->entry),"param_number",GINT_TO_POINTER (pnum+1));
    param->widgets[0]=combo;

    break;
  }
  
  if (mainw->current_file>0) {
    // see if there were any 'special' hints
    check_for_special (param,pnum,box);
  }

  g_free (name);
}

void add_hsep_to_box (GtkBox *box) {
  GtkWidget *hseparator = gtk_hseparator_new ();
  gtk_box_pack_start (box, hseparator, TRUE, TRUE, 0);
}

void add_fill_to_box (GtkBox *box) {
  GtkWidget *blank_label = gtk_label_new ("");
  gtk_box_pack_start (box, blank_label, TRUE, TRUE, 0);
}

void add_label_to_box (GtkBox *box, const gchar *text) {
  GtkWidget *label = gtk_label_new_with_mnemonic (_ (text));
  gtk_box_pack_start (box, label, FALSE, FALSE, 10);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
}

GSList *add_usrgrp_to_livesgrp (GSList *u2l, GSList *rbgroup, gint usr_number) {
  lives_widget_group_t *wgroup=g_malloc (sizeof(lives_widget_group_t));
  wgroup->usr_number=usr_number;
  wgroup->rbgroup=rbgroup;
  wgroup->active_param=0;
  u2l=g_slist_append (u2l, (gpointer)wgroup);
  return u2l;
}




lives_widget_group_t *livesgrp_from_usrgrp (GSList *u2l, gint usrgrp) {
  int i;
  lives_widget_group_t *group;

  for (i=0;i<g_slist_length (u2l);i++) {
    group=g_slist_nth_data (u2l,i);
    if (group->usr_number==usrgrp) return group;
  }
  return NULL;
}


gboolean
label_act_toggle (GtkWidget *widget, GdkEventButton *event, GtkToggleButton *togglebutton) {
  gtk_toggle_button_set_active (togglebutton, TRUE);
  return FALSE;
}



void
after_boolean_param_toggled        (GtkToggleButton *togglebutton,
				    lives_rendered_effect_t *         rfx)
{
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (togglebutton),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  gboolean old_bool=get_bool_param(param->value),new_bool;

  set_bool_param(param->value,(new_bool=gtk_toggle_button_get_active (togglebutton)));

  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) {
      char *disp_string=get_weed_display_string(inst,param_number-1);
      weed_set_boolean_value(weed_inst_in_param(inst,param_number-1),"value",(int)new_bool);
      if (param->copy_to!=-1) weed_set_boolean_value(weed_inst_in_param(inst,param->copy_to),"value",(int)new_bool);
      if (disp_string!=NULL) {
	gtk_label_set_text(GTK_LABEL(param->widgets[1]),disp_string);
	weed_free(disp_string);
      }
    }
  }
  if (get_bool_param(param->value)!=old_bool&&param->onchange) {
    do_onchange (G_OBJECT (togglebutton), rfx);
  }
}



void
after_param_value_changed           (GtkSpinButton   *spinbutton,
				     lives_rendered_effect_t *rfx)
{
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (spinbutton),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  gdouble new_double,old_double=0.;
  gint new_int,old_int=0;
  

  if (param->dp>0) {
    old_double=get_double_param(param->value);
  }
  else {
    old_int=get_int_param(param->value);
  }

  if (param->dp>0) {
    set_double_param(param->value,(new_double=gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton))));
    if (rfx->status==RFX_STATUS_WEED) {
      weed_plant_t *inst=(weed_plant_t *)rfx->source;
      if (inst!=NULL) weed_set_double_value(weed_inst_in_param(inst,param_number-1),"value",(double)new_double);
      if (param->copy_to!=-1) weed_set_double_value(weed_inst_in_param(inst,param->copy_to),"value",(double)new_double);
    }
  }
  else {
    set_int_param(param->value,(new_int=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton))));
    if (rfx->status==RFX_STATUS_WEED) {
      weed_plant_t *inst=(weed_plant_t *)rfx->source;
      if (inst!=NULL) weed_set_int_value(weed_inst_in_param(inst,param_number-1),"value",(int)new_int);
      if (param->copy_to!=-1) weed_set_int_value(weed_inst_in_param(inst,param->copy_to),"value",(int)new_int);
    }
  }

  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) {
      char *disp_string=get_weed_display_string(inst,param_number-1);
      if (disp_string!=NULL) {
	gtk_label_set_text(GTK_LABEL(param->widgets[1]),disp_string);
	weed_free(disp_string);
      }
    }
  }
    
  if (((param->dp>0&&(get_double_param(param->value)!=old_double))||(param->dp==0&&(get_int_param(param->value)!=old_int)))&&param->onchange) {
    do_onchange (G_OBJECT (spinbutton), rfx);
  }
}


static void update_weed_color_value(weed_plant_t *param, weed_plant_t *inst, int c1, int c2, int c3, int c4) {
  int error;
  int cols[4]={c1,c2,c3,c4};
  double colds[4];
  weed_plant_t *ptmpl=weed_get_plantptr_value(param,"template",&error);
  int cspace=weed_get_int_value(ptmpl,"colorspace",&error);
  int rmax,rmin,gmax,gmin,bmax,bmin;
  int *maxs=NULL,*mins=NULL;
  double rmaxd,rmind,gmaxd,gmind,bmaxd,bmind;
  double *maxds=NULL,*minds=NULL;

  gboolean is_int=(weed_leaf_seed_type(ptmpl,"default")==WEED_SEED_INT);

  switch (cspace) {
    // TODO - other cspaces
  case WEED_COLORSPACE_RGB:
    if (is_int) {
      if (weed_leaf_num_elements(ptmpl,"max")==3) {
	maxs=weed_get_int_array(ptmpl,"max",&error);
	rmax=maxs[0];
	gmax=maxs[1];
	bmax=maxs[2];
	weed_free(maxs);
      }
      else rmax=gmax=bmax=weed_get_int_value(ptmpl,"max",&error);
      if (weed_leaf_num_elements(ptmpl,"min")==3) {
	mins=weed_get_int_array(ptmpl,"min",&error);
	rmin=mins[0];
	gmin=mins[1];
	bmin=mins[2];
	weed_free(mins);
      }
      else rmin=gmin=bmin=weed_get_int_value(ptmpl,"min",&error);

      cols[0]=rmin+(int)((double)cols[0]/255.*(double)(rmax-rmin)+.5);
      cols[1]=gmin+(int)((double)cols[1]/255.*(double)(gmax-gmin)+.5);
      cols[2]=bmin+(int)((double)cols[2]/255.*(double)(bmax-bmin)+.5);
      weed_set_int_array(param,"value",3,cols);
      break;
    }
    else {
      // double
      if (weed_leaf_num_elements(ptmpl,"max")==3) {
	maxds=weed_get_double_array(ptmpl,"max",&error);
	rmaxd=maxds[0];
	gmaxd=maxds[1];
	bmaxd=maxds[2];
	weed_free(maxds);
      }
      else rmaxd=gmaxd=bmaxd=weed_get_double_value(ptmpl,"max",&error);
      if (weed_leaf_num_elements(ptmpl,"min")==3) {
	minds=weed_get_double_array(ptmpl,"min",&error);
	rmind=minds[0];
	gmind=minds[1];
	bmind=minds[2];
	weed_free(minds);
      }
      else rmind=gmind=bmind=weed_get_double_value(ptmpl,"min",&error);
      colds[0]=rmind+(double)cols[0]/255.*(rmaxd-rmind);
      colds[1]=gmind+(double)cols[1]/255.*(gmaxd-gmind);
      colds[2]=bmind+(double)cols[2]/255.*(bmaxd-bmind);
      weed_set_double_array(param,"value",3,colds);
    }
    break;
  }


}


void
after_param_red_changed           (GtkSpinButton   *spinbutton,
				   lives_rendered_effect_t *rfx) {
  //  after_param_value_changed (spinbutton, rfx);
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (spinbutton),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  lives_colRGB24_t old_value;
  gint new_red;

  get_colRGB24_param(param->value,&old_value);
  new_red=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
  set_colRGB24_param(param->value,new_red,old_value.green,old_value.blue);
  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) update_weed_color_value(weed_inst_in_param(inst,param_number-1),inst,new_red,old_value.green,old_value.blue,0);
  }
  if (new_red!=old_value.red&&param->onchange) {
    do_onchange (G_OBJECT (spinbutton), rfx);
  }
}

void
after_param_green_changed           (GtkSpinButton   *spinbutton,
				     lives_rendered_effect_t *rfx) {
  //  after_param_value_changed (spinbutton, rfx);
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (spinbutton),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  lives_colRGB24_t old_value;
  gint new_green;

  get_colRGB24_param(param->value,&old_value);
  new_green=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
  set_colRGB24_param(param->value,old_value.red,new_green,old_value.blue);
  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) update_weed_color_value(weed_inst_in_param(inst,param_number-1),inst,old_value.red,new_green,old_value.blue,0);
  }
  if (new_green!=old_value.green&&param->onchange) {
    do_onchange (G_OBJECT (spinbutton), rfx);
  }
}


void
after_param_blue_changed           (GtkSpinButton   *spinbutton,
				    lives_rendered_effect_t *rfx) {
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (spinbutton),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  lives_colRGB24_t old_value;
  gint new_blue;

  get_colRGB24_param(param->value,&old_value);
  new_blue=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
  set_colRGB24_param(param->value,old_value.red,old_value.green,new_blue);
  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) update_weed_color_value(weed_inst_in_param(inst,param_number-1),inst,old_value.red,old_value.green,new_blue,0);
  }
  if (new_blue!=old_value.blue&&param->onchange) {
    do_onchange (G_OBJECT (spinbutton), rfx);
  }
}
 


void
after_param_alpha_changed           (GtkSpinButton   *spinbutton,
				     lives_rendered_effect_t *rfx) {
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (spinbutton),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  lives_colRGBA32_t old_value;

  gint new_alpha=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));

  get_colRGBA32_param(param->value,&old_value);
  
  set_colRGBA32_param(param->value,old_value.red,old_value.green,old_value.blue,new_alpha);
  if (new_alpha!=old_value.alpha&&param->onchange) {
    do_onchange (G_OBJECT (spinbutton), rfx);
  }
}


void 
after_param_text_changed (GtkWidget *textwidget, lives_rendered_effect_t *rfx) {
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (textwidget),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  gchar *old_text=param->value;

  if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
    GtkTextIter start_iter,end_iter;
    GtkTextBuffer *textbuffer=GTK_TEXT_BUFFER (textwidget);

    gtk_text_buffer_get_start_iter(textbuffer,&start_iter);
    gtk_text_buffer_get_end_iter(textbuffer,&end_iter);
    
    param->value=gtk_text_buffer_get_text (textbuffer,&start_iter,&end_iter,FALSE);
  }
  else {
    param->value=g_strdup (gtk_entry_get_text (GTK_ENTRY (textwidget)));
  }

  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) {
    char *disp_string=get_weed_display_string(inst,param_number-1);
      weed_set_string_value(weed_inst_in_param(inst,param_number-1),"value",(char *)param->value);
      if (param->copy_to!=-1) weed_set_string_value(weed_inst_in_param(inst,param->copy_to),"value",(char *)param->value);

      if (disp_string!=NULL) {
	gulong blockfunc=(gulong)g_object_get_data(G_OBJECT(textwidget),"blockfunc");
	gulong tatfunc=(gulong)g_object_get_data(G_OBJECT(textwidget),"tatfunc");
	g_signal_handler_block(textwidget,blockfunc);
	if ((gpointer)tatfunc!=NULL) g_signal_handler_block(textwidget,tatfunc);
	if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
	  gtk_text_buffer_set_text (GTK_TEXT_BUFFER (textwidget), param->value, -1);
	}
	else {
	  gtk_entry_set_text(GTK_ENTRY(textwidget),disp_string);
	}
	g_signal_handler_unblock(textwidget,blockfunc);
	if ((gpointer)tatfunc!=NULL) g_signal_handler_unblock(textwidget,tatfunc);
	weed_free(disp_string);
      }
    }
  }

  if (strcmp (old_text,param->value)&&param->onchange) {
    do_onchange (G_OBJECT (textwidget), rfx);
  }
  g_free (old_text);
}


void 
after_string_list_changed (GtkEntry *entry, lives_rendered_effect_t *rfx) {
  gint param_number=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (entry),"param_number"));
  lives_param_t *param=&rfx->params[param_number-1];
  gint old_index=get_int_param(param->value);
  gint new_index=lives_list_index(param->list,gtk_entry_get_text(entry));

  if (new_index==-1) return;

  set_int_param(param->value,new_index);

  if (rfx->status==RFX_STATUS_WEED) {
    weed_plant_t *inst=(weed_plant_t *)rfx->source;
    if (inst!=NULL) {
      char *disp_string=get_weed_display_string(inst,param_number-1);
      weed_set_int_value(weed_inst_in_param(inst,param_number-1),"value",(int)new_index);
      if (param->copy_to!=-1) weed_set_int_value(weed_inst_in_param(inst,param->copy_to),"value",(int)new_index);
      if (disp_string!=NULL) {
	gulong blockfunc=(gulong)g_object_get_data(G_OBJECT(entry),"blockfunc");
	gulong tatfunc=(gulong)g_object_get_data(G_OBJECT(entry),"tatfunc");
	g_signal_handler_block(entry,blockfunc);
	if ((gpointer)tatfunc!=NULL) g_signal_handler_block(entry,tatfunc);
	gtk_entry_set_text(entry,disp_string);
	g_signal_handler_unblock(entry,blockfunc);
	if ((gpointer)tatfunc!=NULL) g_signal_handler_unblock(entry,tatfunc);
	weed_free(disp_string);
      } 
    }
  }

  if (old_index!=new_index&&param->onchange) {
    do_onchange(G_OBJECT(entry), rfx);
  }
}


gchar *param_marshall (lives_rendered_effect_t *rfx, gboolean with_min_max) {
  // this function will marshall all parameters into a space separated string
  // in case of string parameters, these will be surrounded by " and all 
  // quotes will be escaped \"

  // the returned string should be g_free()'ed after use
  gchar *new_return=g_strdup ("");
  gchar *old_return=new_return;
  gchar *return_pattern;
  lives_colRGB24_t rgb;
  int i;
 
  for (i=0;i<rfx->num_params;i++) {
    switch (rfx->params[i].type) {
    case LIVES_PARAM_COLRGB24:
      get_colRGB24_param(rfx->params[i].value,&rgb);
      if (!with_min_max) {
	new_return=g_strdup_printf ("%s %d",old_return,(((rgb.red<<8)+rgb.green)<<8)+rgb.blue);
      }
      else {
	new_return=g_strdup_printf ("%s %d %d %d",old_return,rgb.red,rgb.green,rgb.blue);
      }
      g_free (old_return);
      old_return=new_return;
      break;

    case LIVES_PARAM_STRING:
      // escape strings
      new_return=g_strdup_printf ("%s \"%s\"",old_return,U82L (subst (rfx->params[i].value,"\"","\\\"")));
      g_free (old_return);
      old_return=new_return;
      break;

    case LIVES_PARAM_STRING_LIST:
      // escape strings
      new_return=g_strdup_printf ("%s \"%s\"",old_return,U82L (subst (g_list_nth_data(rfx->params[i].list,get_int_param(rfx->params[i].value)),"\"","\\\"")));
      g_free (old_return);
      old_return=new_return;
      break;
      
    default:
      if (rfx->params[i].dp) {
	return_pattern=g_strdup_printf ("%%s %%.%df",rfx->params[i].dp);
	new_return=g_strdup_printf (return_pattern,old_return,get_double_param(rfx->params[i].value));
	if (with_min_max) {
	  g_free (old_return);
	  old_return=new_return;
	  new_return=g_strdup_printf (return_pattern,old_return,rfx->params[i].min);
	  g_free (old_return);
	  old_return=new_return;
	  new_return=g_strdup_printf (return_pattern,old_return,rfx->params[i].max);
	}
	g_free (return_pattern);
      }
      else {
	new_return=g_strdup_printf ("%s %d",old_return,get_int_param(rfx->params[i].value));
	if (with_min_max&&rfx->params[i].type!=LIVES_PARAM_BOOL) {
	  g_free (old_return);
	  old_return=new_return;
	  new_return=g_strdup_printf ("%s %d",old_return,(int)rfx->params[i].min);
	  g_free (old_return);
	  old_return=new_return;
	  new_return=g_strdup_printf ("%s %d",old_return,(int)rfx->params[i].max);
	}
      }
      g_free (old_return);
      old_return=new_return;
    }
  }
  if (mainw->current_file>0&&with_min_max) {
    if (rfx->num_in_channels<2) {
      new_return=g_strdup_printf ("%s %d %d %d %d %d",old_return,cfile->hsize,cfile->vsize,cfile->start,cfile->end,cfile->frames);
    }
    else {
      // for transitions, change the end to indicate the merge section
      // this is better for length calculations
      gint cb_frames=clipboard->frames;
      gint start=cfile->start,end=cfile->end,ttl;

      if (prefs->ins_resample==TRUE&&clipboard->fps!=cfile->fps) {
	cb_frames=count_resampled_frames(clipboard->frames,clipboard->fps,cfile->fps);
      }

      if (cfile->end-cfile->start+1>(cb_frames*(ttl=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (merge_opts->spinbutton_loops))))&&!merge_opts->loop_to_fit) {
	end=cb_frames*ttl;
	if (!merge_opts->align_start) {
	  start=cfile->end-end+1;
	  end=cfile->end;
	}
	else {
	  start=cfile->start;
	  end+=start-1;
	}
      }
      new_return=g_strdup_printf ("%s %d %d %d %d %d %d %d",old_return,cfile->hsize,cfile->vsize,start,end,cfile->frames,clipboard->hsize,clipboard->vsize);
    }
  }
  else {
    new_return=g_strdup (old_return);
  }
  g_free (old_return);
  return new_return;
}


gchar *reconstruct_string (GList *plist, gint start, gint *offs) {
  // convert each piece from locale to utf8
  // concat list entries to get reconstruct
  // replace \" with "

  gchar *word=NULL;
  int i;
  gboolean lastword=FALSE;
  gchar *ret=g_strdup (""),*ret2;

  word=L2U8 (g_list_nth_data (plist,start));
  if (word==NULL||!strlen (word)||word[0]!='\"') {
    if (word!=NULL) g_free (word);
    return 0;
  }

  word++;

  for (i=start;i<g_list_length (plist);i++) {
    if (strlen (word)) {
      if ((word[strlen (word)-1]=='\"')&&(strlen (word)==1||word[strlen (word)-2]!='\\')) {
	lastword=TRUE;
	memset (word+strlen (word)-1,0,1);
      }
    }

    ret2=g_strconcat (ret,subst (word,"\\\"","\"")," ",NULL);
    if (ret2!=ret) g_free (ret);
    ret=ret2;

    if (i==start) word--;
    g_free (word);

    if (lastword) break;

    if (i<g_list_length (plist)-1) word=L2U8 (g_list_nth_data (plist,i+1));
  }

  set_int_param (offs,i-start+1);

  // remove trailing space
  memset (ret+strlen (ret)-1,0,1);
  return ret;

}


void param_demarshall (lives_rendered_effect_t *rfx, GList *plist, gboolean upd) {
  int i;
  gint pnum=0;
  lives_param_t *param;

  // param->widgets[n] are only valid if upd==TRUE

  if (plist==NULL) return;

  for (i=0;i<rfx->num_params;i++) {
    param=&rfx->params[i];
    pnum=set_param_from_list(plist,param,pnum,upd);
  }
}



gint set_param_from_list(GList *plist, lives_param_t *param, gint pnum, gboolean upd) {
  // update values for param using values in plist
  // if upd is TRUE, the widgets for that param also are updated
  gint red,green,blue;
  gint offs=0;

  switch (param->type) {
  case LIVES_PARAM_BOOL:
    set_bool_param(param->value,(atoi (g_list_nth_data (plist,pnum++))));
    if (upd) {
      if (param->widgets[0]&&GTK_IS_TOGGLE_BUTTON (param->widgets[0])) {
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (param->widgets[0]),get_bool_param(param->value));
      }
    }
    break;
  case LIVES_PARAM_NUM:
    if (param->dp) {
      gdouble double_val=g_strtod (g_list_nth_data (plist,pnum++),NULL);
      param->min=g_strtod (g_list_nth_data (plist,pnum++),NULL);
      param->max=g_strtod (g_list_nth_data (plist,pnum++),NULL);
      if (double_val<param->min) double_val=param->min;
      if (double_val>param->max) double_val=param->max;
      set_double_param(param->value,double_val);
      if (upd) {
	if (param->widgets[0]&&GTK_IS_SPIN_BUTTON (param->widgets[0])) {
	  gulong spinfunc=(gulong)g_object_get_data(G_OBJECT(param->widgets[0]),"spinfunc");
	  g_signal_handler_block(param->widgets[0],spinfunc);
	  gtk_spin_button_set_range (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)param->min,(gdouble)param->max);
	  gtk_spin_button_update(GTK_SPIN_BUTTON(param->widgets[0]));
	  g_signal_handler_unblock(param->widgets[0],spinfunc);
	  gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),get_double_param(param->value));
	  gtk_spin_button_update(GTK_SPIN_BUTTON(param->widgets[0]));
	}
      }
    }
    else {
      gint int_value=atoi (g_list_nth_data (plist,pnum++));
      gint int_min=atoi (g_list_nth_data (plist,pnum++));
      gint int_max=atoi (g_list_nth_data (plist,pnum++));
      if (int_value<int_min) int_value=int_min;
      if (int_value>int_max) int_value=int_max;
      param->min=(gdouble)int_min;
      param->max=(gdouble)int_max;
      set_int_param(param->value,int_value);
      
      if (upd) {
	if (param->widgets[0]&&GTK_IS_SPIN_BUTTON (param->widgets[0])) {
	  gulong spinfunc=(gulong)g_object_get_data(G_OBJECT(param->widgets[0]),"spinfunc");
	  g_signal_handler_block(param->widgets[0],spinfunc);
	  gtk_spin_button_set_range (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)param->min,(gdouble)param->max);
	  gtk_spin_button_update(GTK_SPIN_BUTTON(param->widgets[0]));
	  g_signal_handler_unblock(param->widgets[0],spinfunc);
	  gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)get_int_param(param->value));
	  gtk_spin_button_update(GTK_SPIN_BUTTON(param->widgets[0]));
	}
      }
    }
    break;
  case LIVES_PARAM_COLRGB24:
    red=atoi (g_list_nth_data (plist,pnum++)); // red
    green=atoi (g_list_nth_data (plist,pnum++)); // green
    blue=atoi (g_list_nth_data (plist,pnum++)); // blue
    set_colRGB24_param(param->value,red,green,blue);
    
    if (upd) {
      if (param->widgets[0]&&GTK_IS_SPIN_BUTTON (param->widgets[0])) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[0]),(gdouble)red);
      }
      if (param->widgets[1]&&GTK_IS_SPIN_BUTTON (param->widgets[1])) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[1]),(gdouble)green);
      }
      if (param->widgets[2]&&GTK_IS_SPIN_BUTTON (param->widgets[2])) {
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (param->widgets[2]),(gdouble)blue);
      }
    }
    break;
  case LIVES_PARAM_STRING:
    param->value=reconstruct_string (plist,pnum,&offs);
    if (upd) {
      if (param->widgets[0]!=NULL) {
	if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
	  gtk_text_buffer_set_text (GTK_TEXT_BUFFER (param->widgets[0]), param->value, -1);
	}
	else {
	  gtk_entry_set_text (GTK_ENTRY (param->widgets[0]),param->value);
	}
      }
    }
    pnum+=offs;
    break;
  case LIVES_PARAM_STRING_LIST:
    {
      gchar *buf=reconstruct_string (plist,pnum,&offs);
      gint listindex;
      
      pnum+=offs;
      
      if ((listindex=lives_list_index(param->list,buf))!=-1) {
	set_int_param(param->value,listindex);
      }
      else {
	// value not in list, add it...
	param->list=g_list_prepend(param->list,g_strdup(buf));
	set_int_param(param->value,0);
      }
      if (upd&&param->widgets[0]!=NULL&&GTK_IS_COMBO(param->widgets[0])) gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(param->widgets[0])->entry),buf);
      g_free (buf);
      break;
    }
  }
  return pnum;
}















void toggle_set_default_value (GtkToggleButton *button, GtkSpinButton *spinbutton) {
  lives_param_t *param;
  gulong satfunc;
  
  if (gtk_toggle_button_get_active (button)) return;
  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  satfunc=(gulong)g_object_get_data(G_OBJECT(spinbutton),"satfunc");
  g_signal_handler_block(spinbutton,satfunc);
  if (param->dp) {
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton),get_double_param(param->def));
  }
  else {
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton),(gdouble)(get_int_param(param->def)));
  }
  g_signal_handler_unblock(spinbutton,satfunc);
}

void toggle_set_default_red (GtkToggleButton *button, GtkSpinButton *spinbutton) {
  lives_param_t *param;
  lives_colRGB24_t rgb;
  gulong satfunc;

  if (gtk_toggle_button_get_active (button)) return;

  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  get_colRGB24_param(param->def,&rgb);
  satfunc=(gulong)g_object_get_data(G_OBJECT(spinbutton),"satfunc");
  g_signal_handler_block(spinbutton,satfunc);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton),(gdouble)(rgb.red));
  g_signal_handler_unblock(spinbutton,satfunc);
}

void toggle_set_default_green (GtkToggleButton *button, GtkSpinButton *spinbutton) {
  lives_param_t *param;
  lives_colRGB24_t rgb;
  gulong satfunc;

  if (gtk_toggle_button_get_active (button)) return;

  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  get_colRGB24_param(param->def,&rgb);
  satfunc=(gulong)g_object_get_data(G_OBJECT(spinbutton),"satfunc");
  g_signal_handler_block(spinbutton,satfunc);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton),(gdouble)(rgb.green));
  g_signal_handler_unblock(spinbutton,satfunc);
}

void toggle_set_default_blue (GtkToggleButton *button, GtkSpinButton *spinbutton) {
  lives_param_t *param;
  lives_colRGB24_t rgb;
  gulong satfunc;

  if (gtk_toggle_button_get_active (button)) return;

  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  get_colRGB24_param(param->def,&rgb);
  satfunc=(gulong)g_object_get_data(G_OBJECT(spinbutton),"satfunc");
  g_signal_handler_block(spinbutton,satfunc);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton),(gdouble)(rgb.blue));
  g_signal_handler_unblock(spinbutton,satfunc);
}

void toggle_set_default_alpha (GtkToggleButton *button, GtkSpinButton *spinbutton) {
  lives_param_t *param;
  lives_colRGBA32_t rgba;
  gulong satfunc;

  if (gtk_toggle_button_get_active (button)) return;

  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  get_colRGBA32_param(param->def,&rgba);
  satfunc=(gulong)g_object_get_data(G_OBJECT(spinbutton),"satfunc");
  g_signal_handler_block(spinbutton,satfunc);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton),(gdouble)(rgba.alpha));
  g_signal_handler_unblock(spinbutton,satfunc);
}

void toggle_set_default_text (GtkToggleButton *button, GtkWidget *textwidget) {
  lives_param_t *param;
  gulong tatfunc;
  gchar *txt;

  if (gtk_toggle_button_get_active (button)) return;
  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  tatfunc=(gulong)g_object_get_data(G_OBJECT(textwidget),"tatfunc");
  txt=g_strndup (param->def,(gint)param->max);

  if ((gint)param->max>RFX_TEXT_MAGIC||param->max==0.) {
    GtkTextBuffer *textbuffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (textwidget));
    g_signal_handler_block(textbuffer,tatfunc);
    gtk_text_buffer_set_text (textbuffer, txt, -1);
    g_signal_handler_unblock(textbuffer,tatfunc);
  }
  else {
    g_signal_handler_block(textwidget,tatfunc);
    gtk_entry_set_text (GTK_ENTRY (textwidget), txt);
    g_signal_handler_unblock(textwidget,tatfunc);
  }
  g_free (txt);
}

void toggle_set_default_combo (GtkToggleButton *button, GtkCombo *combo) {
  lives_param_t *param;
  gulong catfunc;
  gchar *txt;

  if (gtk_toggle_button_get_active (button)) return;
  param=(lives_param_t *)g_object_get_data (G_OBJECT (button),"param");
  catfunc=(gulong)g_object_get_data(G_OBJECT (combo),"catfunc");
  txt=g_strdup (g_list_nth_data (param->list,get_int_param (param->def)));

  g_signal_handler_block(GTK_COMBO (combo)->entry,catfunc);
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), txt);
  g_signal_handler_unblock(GTK_COMBO (combo)->entry,catfunc);

  g_free (txt);
}


void do_onchange (GObject *object, lives_rendered_effect_t *rfx) {
  gint which=GPOINTER_TO_INT (g_object_get_data (object,"param_number"));
  gchar *com;
  GList *retvals;
  void **param_store=NULL;

  if (which<=0) {
    // backup param values, if any change here we will also set defaults
    param_store=store_rfx_params (rfx);
    com=g_strdup_printf ("smogrify fxinit_%s %d%s",rfx->name,rfx->status,param_marshall (rfx,TRUE));
    retvals=plugin_request_by_space (NULL,NULL,com);
  }
  else {
    com=g_strdup_printf ("onchange_%d%s",which,param_marshall (rfx,TRUE));
    switch (rfx->status) {
    case RFX_STATUS_BUILTIN:
      retvals=plugin_request_by_space (PLUGIN_RENDERED_EFFECTS_BUILTIN,rfx->name,com);
      break;
    case RFX_STATUS_CUSTOM:
      retvals=plugin_request_by_space (PLUGIN_RENDERED_EFFECTS_CUSTOM,rfx->name,com);
      break;
    default:
      retvals=plugin_request_by_space (PLUGIN_RENDERED_EFFECTS_TEST,rfx->name,com);
    }
  }
  if (retvals!=NULL) {
    param_demarshall (rfx,retvals,which>0);
    g_list_free (retvals);
  }
  else {
    if (which<=0&&mainw->error) {
      mainw->error=FALSE;
      do_blocking_error_dialog (g_strdup_printf ("\n\n%s\n\n",mainw->msg));
    }
  }
  g_free (com);
  if (param_store!=NULL) {
    set_rfx_params_from_store (rfx,param_store);
    rfx_params_store_free (rfx,param_store);
    g_free (param_store);
  }
}



void
on_pwcolourbutton_clicked              (GtkButton       *button,
					lives_rendered_effect_t *rfx)
{
  GtkWidget *selector;
  GtkWidget *csdialog;
  GtkWidget *okbutton;
  GtkWidget *cancelbutton;
  gint pnum=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button),"param_number"));

  csdialog = gtk_color_selection_dialog_new (_("LiVES: - Select Colour"));
  gtk_window_set_position (GTK_WINDOW (csdialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(csdialog),GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(button))));

  gtk_window_set_modal (GTK_WINDOW (csdialog), TRUE);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_fg (csdialog, GTK_STATE_NORMAL, &palette->normal_fore);
    gtk_widget_modify_bg (csdialog, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (csdialog, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }
  okbutton = GTK_COLOR_SELECTION_DIALOG (csdialog)->ok_button;
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
  g_object_set_data (G_OBJECT (okbutton),"param_number",GINT_TO_POINTER (pnum));

  cancelbutton = GTK_COLOR_SELECTION_DIALOG (csdialog)->cancel_button;
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  selector = GTK_COLOR_SELECTION_DIALOG (csdialog)->colorsel;
  gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION (selector), FALSE);

  g_signal_connect (GTK_OBJECT (cancelbutton), "clicked",
		    G_CALLBACK (on_pwcolsel_cancel_clicked),
		    NULL);
  g_signal_connect (GTK_OBJECT (okbutton),
		    "clicked",
		    G_CALLBACK (on_pwcolsel_ok_clicked),
		    (gpointer)rfx);
  gtk_widget_show_all(csdialog);
}


void
on_pwcolsel_ok_clicked (GtkButton *button, lives_rendered_effect_t *rfx)
{
  GdkColor selected;
  gint pnum=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button),"param_number"));
  lives_param_t *param=&rfx->params[pnum-1];

  gtk_color_selection_get_current_color(GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (gtk_widget_get_toplevel(GTK_WIDGET(button)))->colorsel),&selected);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(param->widgets[0]),(gdouble)(gint)((selected.red+128)/257));
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(param->widgets[1]),(gdouble)(gint)((selected.green+128)/257));
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(param->widgets[2]),(gdouble)(gint)((selected.blue+128)/257));
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void
on_pwcolsel_cancel_clicked (GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}



void
widget_act_toggle (GtkWidget *widget, GtkToggleButton *togglebutton) {
  // this function allows a widget to activate a togglebutton
  // TODO - only do this if value was set by user
  gtk_toggle_button_set_active (togglebutton, TRUE);
}



