// preferences.c
// LiVES (lives-exe)
// (c) G. Finch 2004
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details

// functions dealing with getting/setting user preferences
// TODO - use atom type system for prefs

#include "main.h"
#include "paramwindow.h"
#include "callbacks.h"
#include "support.h"

#define PREFS_TIMEOUT 10000000 // 10 seconds

#ifdef ENABLE_OSC
void on_osc_enable_toggled (GtkToggleButton *t1, gpointer t2) {
  if (prefs->osc_udp_started) return;
  gtk_widget_set_sensitive (prefsw->spinbutton_osc_udp,gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (t1))||gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (t2)));
}
#endif


void
get_pref(const gchar *key, gchar *val, gint maxlen) {
  FILE *valfile;
  gchar *vfile;
  gchar *com=g_strdup_printf("smogrify get_pref %s %d %d",key,getuid(),getpid());
  gint timeout=PREFS_TIMEOUT/prefs->sleep_time;

  memset(val,0,1);

  if (system(com)) {
    tempdir_warning();
    g_free(com);
    return;
  }

  vfile=g_strdup_printf("%s/.smogval.%d.%d",prefs->tmpdir,getuid(),getpid());
  do {
    if (!(valfile=fopen(vfile,"r"))) {
      if (!(mainw==NULL)) {
	while (g_main_context_iteration(NULL,FALSE));
      }
      g_usleep(prefs->sleep_time);
      timeout--;
    }
  } while (!valfile&&timeout>0);
  if (timeout<=0) {
    tempdir_warning();
  }
  else {
    fgets(val,maxlen,valfile);
    fclose(valfile);
    unlink(vfile);
  }
  g_free(vfile);
  g_free(com);
}



void
get_pref_default(const gchar *key, gchar *val, gint maxlen) {
  FILE *valfile;
  gchar *vfile;
  gint timeout=PREFS_TIMEOUT/prefs->sleep_time;
  gchar *com=g_strdup_printf("smogrify get_pref_default %s",key);

  memset(val,0,1);

  if (system(com)) {
      tempdir_warning();
      g_free(com);
      return;
  }
  vfile=g_strdup_printf("%s/.smogval.%d.%d",prefs->tmpdir,getuid(),getgid());
  do {
    if (!(valfile=fopen(vfile,"r"))) {
      if (!(mainw==NULL)) {
	while (g_main_context_iteration(NULL,FALSE));
      }
      g_usleep(prefs->sleep_time);
      timeout--;
    }
  } while (!valfile&&timeout>0);
  if (timeout<=0) {
    tempdir_warning();
  }
  else {
    fgets(val,maxlen,valfile);
    fclose(valfile);
    unlink(vfile);
  }
  g_free(vfile);
  g_free(com);
}


gboolean
get_boolean_pref(const gchar *key) {
  gchar buffer[512];
  get_pref(key,buffer,512);
  if (!strcmp(buffer,"true")) return TRUE;
  return FALSE;
}

gint
get_int_pref(const gchar *key) {
  gchar buffer[512];
  get_pref(key,buffer,512);
  return atoi(buffer);
}

gdouble
get_double_pref(const gchar *key) {
  gchar buffer[512];
  get_pref(key,buffer,512);
  return strtod(buffer,NULL);
}


void
set_pref(const gchar *key, const gchar *value) {
  gchar *com=g_strdup_printf("smogrify set_pref %s \"%s\"",key,value);
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}


void
set_int_pref(const gchar *key, gint value) {
  gchar *com=g_strdup_printf("smogrify set_pref %s %d",key,value);
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}

void
set_double_pref(const gchar *key, gdouble value) {
  gchar *com=g_strdup_printf("smogrify set_pref %s %.3f",key,value);
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}


void
set_boolean_pref(const gchar *key, gboolean value) {
  gchar *com;

  if (value==TRUE) {
    com=g_strdup_printf("smogrify set_pref %s %s",key,"true");
  }
  else {
    com=g_strdup_printf("smogrify set_pref %s %s",key,"false");
  }
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}


gboolean 
apply_prefs(gboolean skip_warn) {
  // set current prefs from prefs dialog
  int idx;

  gboolean needs_restart=FALSE;
  const gchar *bgcolour=gtk_entry_get_text(GTK_ENTRY(prefsw->bgcolour_entry));
  const gchar *video_play_command=gtk_entry_get_text(GTK_ENTRY(prefsw->video_command_entry));
  const gchar *video_open_command=gtk_entry_get_text(GTK_ENTRY(prefsw->video_open_entry));
  const gchar *audio_play_command=gtk_entry_get_text(GTK_ENTRY(prefsw->audio_command_entry));
  const gchar *def_vid_load_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->vid_load_dir_entry));
  const gchar *def_vid_save_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->vid_save_dir_entry));
  const gchar *def_audio_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->audio_dir_entry));
  const gchar *def_image_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->image_dir_entry));
  const gchar *def_proj_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->proj_dir_entry));
  const gchar *cdplay_device=gtk_entry_get_text(GTK_ENTRY(prefsw->cdplay_entry));
  gchar tmpdir[256];
  const gchar *theme=gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(prefsw->theme_combo))->entry));
  const gchar *audp=gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry));
  const gchar *audio_codec=gtk_entry_get_text(GTK_ENTRY(prefsw->acodec_entry));

  gdouble default_fps=gtk_spin_button_get_value(GTK_SPIN_BUTTON(prefsw->spinbutton_def_fps));
  gboolean pause_xmms=FALSE;
  gboolean antialias=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->stop_screensaver_check));
  gboolean stop_screensaver=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->stop_screensaver_check));
  gboolean open_maximised=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->open_maximised_check));
  gboolean fs_maximised=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->fs_max_check));
  gboolean show_recent=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->recent_check));
  gboolean warn_fps=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_fps));
  gboolean warn_set=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_set));
  gboolean warn_fsize=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_fsize));
  gboolean warn_mplayer=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_mplayer));
  gboolean warn_save_quality=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_save_quality));
  gboolean midisynch=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->check_midi));
  gboolean ins_speed=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->ins_speed));
  gboolean show_player_stats=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_show_stats));
  gboolean debug_encoders=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->debug_encoders));
  gboolean ext_jpeg=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->jpeg));
  gboolean show_tool=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->show_tool));
  gint fsize_to_warn=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_warn_fsize));
  gint dl_bwidth=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_bwidth));
  gint ocp=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_ocp));
  gboolean rec_frames=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->rframes));
  gboolean rec_fps=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->rfps));
  gboolean rec_effects=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->reffects));
  gboolean rec_clips=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->rclips));

#ifdef ENABLE_OSC
  gint osc_udp_port=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_osc_udp));
#endif
  gboolean save_warn_mask=FALSE;

  gchar audio_player[256];
  gint listlen=g_list_length (prefsw->acodec_list);

  for (idx=0;idx<listlen&&strcmp(g_list_nth_data (prefsw->acodec_list,idx),audio_codec);idx++);

  if (idx==listlen) future_prefs->encoder.audio_codec=0;
  else future_prefs->encoder.audio_codec=prefsw->acodec_list_to_format[idx];

  g_snprintf (tmpdir,256,"%s",gtk_entry_get_text(GTK_ENTRY(prefsw->tmpdir_entry)));

  g_snprintf(audio_player,256,"mplayer");
  if (strcmp (audp,"mplayer")) {
    g_snprintf(audio_player,256,"sox");
  }


  if (!(prefs->rec_opts&REC_FRAMES)&&rec_frames) prefs->rec_opts|=REC_FRAMES;
  if ((prefs->rec_opts&REC_FRAMES)&&!rec_frames) prefs->rec_opts^=REC_FRAMES;
  if (!(prefs->rec_opts&REC_FPS)&&rec_fps) prefs->rec_opts|=REC_FPS;
  if ((prefs->rec_opts&REC_FPS)&&!rec_fps) prefs->rec_opts^=REC_FPS;
  if (!(prefs->rec_opts&REC_EFFECTS)&&rec_effects) prefs->rec_opts|=REC_EFFECTS;
  if ((prefs->rec_opts&REC_EFFECTS)&&!rec_effects) prefs->rec_opts^=REC_EFFECTS;
  if (!(prefs->rec_opts&REC_CLIPS)&&rec_clips) prefs->rec_opts|=REC_CLIPS;
  if ((prefs->rec_opts&REC_CLIPS)&&!rec_clips) prefs->rec_opts^=REC_CLIPS;

  // fps warning
  if (warn_fps*WARN_MASK_FPS==(prefs->warning_mask&WARN_MASK_FPS)) {
      prefs->warning_mask^=WARN_MASK_FPS;
      save_warn_mask=TRUE;
  }

  // set warning
  if (warn_set*WARN_MASK_SAVE_SET==(prefs->warning_mask&WARN_MASK_SAVE_SET)) {
      prefs->warning_mask^=WARN_MASK_SAVE_SET;
      save_warn_mask=TRUE;
  }

  // save quality warning
  if (warn_save_quality*WARN_MASK_SAVE_QUALITY==(prefs->warning_mask&WARN_MASK_SAVE_QUALITY)) {
      prefs->warning_mask^=WARN_MASK_SAVE_QUALITY;
      save_warn_mask=TRUE;
  }

  // file size warning
  if (warn_fsize*WARN_MASK_FSIZE==(prefs->warning_mask&WARN_MASK_FSIZE)) {
      prefs->warning_mask^=WARN_MASK_FSIZE;
      save_warn_mask=TRUE;
  }

  // mplayer warning
  if (warn_mplayer*WARN_MASK_NO_MPLAYER==(prefs->warning_mask&WARN_MASK_NO_MPLAYER)) {
      prefs->warning_mask^=WARN_MASK_NO_MPLAYER;
      save_warn_mask=TRUE;
  }

  if (save_warn_mask) {
    set_int_pref("lives_warning_mask",prefs->warning_mask);
  }

  if (fsize_to_warn!=(prefs->warn_file_size)) {
    prefs->warn_file_size=fsize_to_warn;
    set_int_pref("warn_file_size",fsize_to_warn);
  }

  if (dl_bwidth!=(prefs->dl_bandwidth)) {
    prefs->dl_bandwidth=dl_bwidth;
    set_int_pref("dl_bandwidth_K",dl_bwidth);
  }

  if (ocp!=(prefs->ocp)) {
    prefs->ocp=ocp;
    set_int_pref("open_compression_percent",ocp);
  }

  if (show_tool!=(prefs->show_tool)) {
    future_prefs->show_tool=prefs->show_tool=show_tool;
    set_boolean_pref("show_toolbar",show_tool);
  }

  // temp dir
  g_strappend (tmpdir,256,"/");
  get_dirname(tmpdir);

  if (strcmp(prefs->tmpdir,tmpdir)||strcmp (future_prefs->tmpdir,tmpdir)) {
    if (g_file_test (tmpdir, G_FILE_TEST_EXISTS)&&(strlen (tmpdir)<10||strncmp (tmpdir+strlen (tmpdir)-10,"/livestmp/",10))) g_strappend (tmpdir,256,"livestmp/");
    if (strcmp(prefs->tmpdir,tmpdir)||strcmp (future_prefs->tmpdir,tmpdir)) {
      gchar *msg;
      if (!check_dir_access (tmpdir)) {
	msg=g_strdup_printf (_ ("Unable to create or write to the new temporary directory.\nYou may need to create it as the root user first, e.g:\n\nmkdir %s; chmod 777 %s\n\nThe directory will not be changed now.\n"),tmpdir,tmpdir);
	do_blocking_error_dialog (msg);
      }
      else {
	g_snprintf(future_prefs->tmpdir,256,"%s",tmpdir);
	msg=g_strdup (_ ("You have chosen to change the temporary directory.\nPlease make sure you have no other copies of LiVES open.\n\nIf you do have other copies of LiVES open, please close them now, *before* pressing OK.\n\nAlternatively, press Cancel to restore the temporary directory to its original setting."));	
	if (!skip_warn) {
	  if (do_warning_dialog(msg)) {
	    mainw->prefs_changed=needs_restart=TRUE;
	  }
	  else {
	    g_snprintf(future_prefs->tmpdir,256,"%s",prefs->tmpdir);
	  }}}
      g_free (msg);
    }
  }

  // background colour
  if (strcmp(prefs->bgcolour,bgcolour)) {
    GdkColor newcolour;
    
    if (gdk_color_parse (bgcolour, &newcolour)==TRUE) {
      gdk_color_parse (bgcolour, &palette->fade_colour);
      sprintf(prefs->bgcolour,bgcolour);
      set_pref("bgcolour",prefs->bgcolour);
      
      // toolbar buttons
      gtk_widget_modify_bg (mainw->toolbar, GTK_STATE_NORMAL, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_stopbutton, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_bckground, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_sepwin, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_double, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_single, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_fullscreen, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_infobutton, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_forward, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_back, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_faster, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_slower, GTK_STATE_PRELIGHT, &palette->fade_colour);
    }
  }
  
  // stop xscreensaver
  if (prefs->stop_screensaver!=stop_screensaver) {
    prefs->stop_screensaver=stop_screensaver;
    set_boolean_pref("stop_screensaver",prefs->stop_screensaver);
  }

  // antialias
  if (prefs->antialias!=antialias) {
    prefs->antialias=antialias;
    set_boolean_pref("antialias",antialias);
  }

  // open maximised
  if (prefs->open_maximised!=open_maximised) {
    prefs->open_maximised=open_maximised;
    set_boolean_pref("open_maximised",open_maximised);
  }

  // filesel maximised
  if (prefs->fileselmax!=fs_maximised) {
    prefs->fileselmax=fs_maximised;
    set_boolean_pref("filesel_maximised",fs_maximised);
  }

  // fps stats
  if (prefs->show_player_stats!=show_player_stats) {
    prefs->show_player_stats=show_player_stats;
    set_boolean_pref("show_player_stats",show_player_stats);
  }

  // show recent
  if (prefs->show_recent!=show_recent) {
    prefs->show_recent=show_recent;
    set_boolean_pref("show_recent_files",show_recent);
    if (prefs->show_recent==TRUE) {
      gtk_widget_show (mainw->recent_menu);
    }
    else {
      gtk_widget_hide (mainw->recent_menu);
    }
  }

  if (capable->has_xmms==TRUE) {
    pause_xmms=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->check_xmms_pause));
    // pause xmms during playback
    if (prefs->pause_xmms!=pause_xmms) {
      prefs->pause_xmms=pause_xmms;
      set_boolean_pref("pause_xmms_on_playback",pause_xmms);
    }
  }
  
  // midi synch
  if (prefs->midisynch!=midisynch) {
    prefs->midisynch=midisynch;
    set_boolean_pref("midisynch",midisynch);
  }

  // jpeg/png
  if (strcmp (prefs->image_ext,"jpg")&&ext_jpeg) {
    set_pref("default_image_format","jpeg");
    g_snprintf (prefs->image_ext,16,"jpg");
  }
  else if (!strcmp(prefs->image_ext,"jpg")&&!ext_jpeg) {
    set_pref("default_image_format","png");
    g_snprintf (prefs->image_ext,16,"png");
  }

  // encoder
  if (strcmp(prefs->encoder.name,future_prefs->encoder.name)) {
    sprintf(prefs->encoder.name,future_prefs->encoder.name);
    set_pref("encoder",prefs->encoder.name);
    sprintf(prefs->encoder.of_restrict,future_prefs->encoder.of_restrict);
    prefs->encoder.of_allowed_acodecs=future_prefs->encoder.of_allowed_acodecs;
  }

  // debug encoder
  if (prefs->debug_encoders!=debug_encoders) {
    set_boolean_pref("debug_encoders",(prefs->debug_encoders=debug_encoders));
  }

  // output format
  if (strcmp(prefs->encoder.of_name,future_prefs->encoder.of_name)) {
    sprintf(prefs->encoder.of_name,future_prefs->encoder.of_name);
    sprintf(prefs->encoder.of_restrict,future_prefs->encoder.of_restrict);
    prefs->encoder.of_allowed_acodecs=future_prefs->encoder.of_allowed_acodecs;
    set_pref("output_type",prefs->encoder.of_name);
  }

  if (prefs->encoder.audio_codec!=future_prefs->encoder.audio_codec) {
    prefs->encoder.audio_codec=future_prefs->encoder.audio_codec;
    if (prefs->encoder.audio_codec<AUDIO_CODEC_UNKNOWN) {
      set_int_pref ("encoder_acodec",prefs->encoder.audio_codec);
    }
  }


  // video play command
  if (strcmp(prefs->video_play_command,video_play_command)) {
    sprintf(prefs->video_play_command,video_play_command);
    set_pref("video_play_command",prefs->video_play_command);
  }

  // video open command
  if (strcmp(prefs->video_open_command,video_open_command)) {
    sprintf(prefs->video_open_command,video_open_command);
    set_pref("video_open_command",prefs->video_open_command);
  }

  //playback plugin
  if (strlen (future_prefs->vpp_name)) {
    if (!g_strcasecmp (future_prefs->vpp_name,"none")) {
      if (vid_playback_plugin!=NULL) {
	g_module_close (vid_playback_plugin->gmodule);
	g_free (vid_playback_plugin);
	vid_playback_plugin=NULL;
	set_pref ("vid_playback_plugin","None");
      }
    }
    else {
      if (open_vid_playback_plugin (future_prefs->vpp_name)) {
	set_pref ("vid_playback_plugin",vid_playback_plugin->name);
	do_error_dialog (_ ("\n\nLocal video playback plugins are only activated in\nfull screen, separate window (fs) mode\n"));
      }
    }
    memset (future_prefs->vpp_name,0,64);
  }


  // audio play command
  if (strcmp(prefs->audio_play_command,audio_play_command)) {
    sprintf(prefs->audio_play_command,audio_play_command);
    set_pref("audio_play_command",prefs->audio_play_command);
  }

  // cd play device
  if (strcmp(prefs->cdplay_device,cdplay_device)) {
    sprintf(prefs->cdplay_device,cdplay_device);
    set_pref("cdplay_device",prefs->cdplay_device);
  }

  // default video load directory
  if (strcmp(prefs->def_vid_load_dir,def_vid_load_dir)) {
      g_snprintf(prefs->def_vid_load_dir,256,"%s/",def_vid_load_dir);
      get_dirname(prefs->def_vid_load_dir);
      set_pref("vid_load_dir",prefs->def_vid_load_dir);
      g_snprintf(mainw->vid_load_dir,256,"%s",prefs->def_vid_load_dir);
  }

  // default video save directory
  if (strcmp(prefs->def_vid_save_dir,def_vid_save_dir)) {
      g_snprintf(prefs->def_vid_save_dir,256,"%s/",def_vid_save_dir);
      get_dirname(prefs->def_vid_save_dir);
      set_pref("vid_save_dir",prefs->def_vid_save_dir);
      g_snprintf(mainw->vid_save_dir,256,"%s",prefs->def_vid_save_dir);
  }

  // default audio directory
  if (strcmp(prefs->def_audio_dir,def_audio_dir)) {
      g_snprintf(prefs->def_audio_dir,256,"%s/",def_audio_dir);
      get_dirname(prefs->def_audio_dir);
      set_pref("audio_dir",prefs->def_audio_dir);
      g_snprintf(mainw->audio_dir,256,"%s",prefs->def_audio_dir);
  }

  // default image directory
  if (strcmp(prefs->def_image_dir,def_image_dir)) {
      g_snprintf(prefs->def_image_dir,256,"%s/",def_image_dir);
      get_dirname(prefs->def_image_dir);
      set_pref("image_dir",prefs->def_image_dir);
      g_snprintf(mainw->image_dir,256,"%s",prefs->def_image_dir);
  }

  // default project directory - for backup and restore
  if (strcmp(prefs->def_proj_dir,def_proj_dir)) {
      g_snprintf(prefs->def_proj_dir,256,"%s/",def_proj_dir);
      get_dirname(prefs->def_proj_dir);
      set_pref("proj_dir",prefs->def_proj_dir);
      g_snprintf(mainw->proj_load_dir,256,"%s",prefs->def_proj_dir);
      g_snprintf(mainw->proj_save_dir,256,"%s",prefs->def_proj_dir);
  }

  // the theme
  if (strcmp(future_prefs->theme,theme)) {
      g_snprintf(future_prefs->theme,64,"%s",theme);
      set_pref("gui_theme",future_prefs->theme);
      mainw->prefs_changed=TRUE;
  }

  // default fps
  if (prefs->default_fps!=default_fps) {
    prefs->default_fps=default_fps;
    set_double_pref("default_fps",prefs->default_fps);
  }

  if (ins_speed==prefs->ins_resample) {
    prefs->ins_resample=!ins_speed;
    set_boolean_pref ("insert_resample",prefs->ins_resample);
  }

    // switch to sox
  if (!(strcmp(audio_player,"sox"))&&prefs->audio_player==AUD_PLAYER_MPLAYER) {
    switch_aud_to_sox();
  }

  // switch to mplayer audio
  else if (!(strcmp (audio_player,"mplayer"))&&prefs->audio_player==AUD_PLAYER_SOX) {
    switch_aud_to_mplayer();
 }

  

#ifdef ENABLE_OSC
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->enable_OSC))) {
    if (prefs->osc_udp_started&&osc_udp_port!=prefs->osc_udp_port) {
      // port number changed
      lives_osc_end();
      prefs->osc_udp_started=FALSE;
    }
    prefs->osc_udp_port=osc_udp_port;
    // try to start on new port number
    if (!prefs->osc_udp_started) prefs->osc_udp_started=lives_osc_init(prefs->osc_udp_port);
  }
  else {
    if (prefs->osc_udp_started) {
      lives_osc_end();
      prefs->osc_udp_started=FALSE;
    }
  }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->enable_OSC_start))) {
    if (future_prefs->osc_udp_port!=osc_udp_port) set_int_pref ("osc_opts",osc_udp_port);
    future_prefs->osc_udp_port=osc_udp_port;
  }
  else {
    if (future_prefs->osc_udp_port>0) set_int_pref ("osc_opts",0);
    future_prefs->osc_udp_port=0;
  }
#endif
  return needs_restart;
}




void
save_future_prefs(void) {
  // save future prefs on exit, if they have changed

  // show_recent is a special case, future prefs has our original value
  if (prefs->show_recent==FALSE&&future_prefs->show_recent==TRUE) {
    set_pref("recent1","");
    set_pref("recent2","");
    set_pref("recent3","");
    set_pref("recent4","");
  }

  if (strncmp(future_prefs->tmpdir,"NULL",4)) {
    set_pref("tempdir",future_prefs->tmpdir);
  }
  if (prefs->show_tool!=future_prefs->show_tool) {
    set_boolean_pref ("show_toolbar",future_prefs->show_tool);
  }
}









void set_acodec_list_from_allowed (_prefsw *prefsw) {
  // could be done better, but no time...
  // get settings for current format


  int count=0,idx;
  gboolean is_allowed=FALSE;
  
  if (prefsw->acodec_list!=NULL) {
    g_list_free (prefsw->acodec_list);
    prefsw->acodec_list=NULL;
  }

  if (future_prefs->encoder.of_allowed_acodecs==0) {
    prefsw->acodec_list=g_list_append (prefsw->acodec_list,_ ("None"));
    future_prefs->encoder.audio_codec=prefsw->acodec_list_to_format[0]=AUDIO_CODEC_NONE;
    combo_set_popdown_strings (GTK_COMBO (prefsw->acodec_combo), prefsw->acodec_list);
    gtk_entry_set_text (GTK_ENTRY (prefsw->acodec_entry),_ ("None"));
    return;
  }
  if (future_prefs->encoder.of_allowed_acodecs&(1<<AUDIO_CODEC_PCM)) {
    prefsw->acodec_list=g_list_append (prefsw->acodec_list,_ ("PCM (highest quality; largest files)"));
    prefsw->acodec_list_to_format[count++]=AUDIO_CODEC_PCM;
    if (future_prefs->encoder.audio_codec==AUDIO_CODEC_PCM) is_allowed=TRUE;
  }
  if (future_prefs->encoder.of_allowed_acodecs&(1<<AUDIO_CODEC_MP3)) {
    prefsw->acodec_list=g_list_append (prefsw->acodec_list,"mp3");
    prefsw->acodec_list_to_format[count++]=AUDIO_CODEC_MP3;
    if (future_prefs->encoder.audio_codec==AUDIO_CODEC_MP3) is_allowed=TRUE;
  }
  if (future_prefs->encoder.of_allowed_acodecs&(1<<AUDIO_CODEC_MP2)) {
    prefsw->acodec_list=g_list_append (prefsw->acodec_list,"mp2");
    prefsw->acodec_list_to_format[count++]=AUDIO_CODEC_MP2;
    if (future_prefs->encoder.audio_codec==AUDIO_CODEC_MP2) is_allowed=TRUE;
  }
  if (future_prefs->encoder.of_allowed_acodecs&(1<<AUDIO_CODEC_VORBIS)) {
    prefsw->acodec_list=g_list_append (prefsw->acodec_list,"vorbis");
    prefsw->acodec_list_to_format[count++]=AUDIO_CODEC_VORBIS;
    if (future_prefs->encoder.audio_codec==AUDIO_CODEC_VORBIS) is_allowed=TRUE;
  }
  if (future_prefs->encoder.of_allowed_acodecs&(1<<AUDIO_CODEC_AC3)) {
    prefsw->acodec_list=g_list_append (prefsw->acodec_list,"ac3");
    prefsw->acodec_list_to_format[count++]=AUDIO_CODEC_AC3;
    if (future_prefs->encoder.audio_codec==AUDIO_CODEC_AC3) is_allowed=TRUE;
  }
  combo_set_popdown_strings (GTK_COMBO (prefsw->acodec_combo), prefsw->acodec_list);
  if (!is_allowed) future_prefs->encoder.audio_codec=prefsw->acodec_list_to_format[0];
  for (idx=0;idx<g_list_length (prefsw->acodec_list);idx++) {
    if (prefsw->acodec_list_to_format[idx]==future_prefs->encoder.audio_codec) {
      gtk_entry_set_text (GTK_ENTRY (prefsw->acodec_entry),g_list_nth_data (prefsw->acodec_list,idx));
      break;
    }
  }
}






_prefsw *
create_prefs_dialog (void)
{
  GtkWidget *dialog_vbox8;
  GtkWidget *vbox7;
  GtkWidget *ins_resample;
  GtkWidget *hbox6;
  GtkWidget *hbox100;
  GtkWidget *hbox;
  GtkObject *spinbutton_adj;
  GtkObject *spinbutton_warn_fsize_adj;
  GtkObject *spinbutton_bwidth_adj;
  GtkWidget *label28;
  GtkWidget *label157;
  GtkWidget *hseparator2;
  GtkWidget *hbox8;
  GtkWidget *hseparator3;
  GtkWidget *hbox7;
  GtkWidget *pref_gui;
  GtkWidget *vbox9;
  GtkWidget *vbox109;
  GtkWidget *vbox99;
  GtkWidget *vbox69;
  GtkWidget *vbox98;
  GtkWidget *frame4;
  GtkWidget *hbox9;
  GtkWidget *hbox109;
  GtkWidget *hbox99;
  GtkWidget *hbox19;
  GtkWidget *label33;
  GtkWidget *label133;
  GtkWidget *label34;
  GtkWidget *label134;
  GtkWidget *label31;
  GtkWidget *frame5;
  GtkWidget *hbox10;
  GtkWidget *label35;
  GtkWidget *label36;
  GtkWidget *label32;
  GtkWidget *label25;
  GtkWidget *label84;
  GtkWidget *label86;
  GtkWidget *label87;
  GtkWidget *label97;
  GtkWidget *label88;
  GtkWidget *vbox18;
  GtkWidget *vbox12;
  GtkWidget *hbox11;
  GtkWidget *hbox94;
  GtkWidget *hbox101;
  GtkWidget *label37;
  GtkWidget *hseparator8;
  GtkWidget *hbox15;
  GtkWidget *hbox115;
  GtkWidget *label56;
  GtkWidget *label94;
  GtkWidget *hbox93;
  GtkWidget *table4;
  GtkWidget *label39;
  GtkWidget *label99;
  GtkWidget *label98;
  GtkWidget *label100;
  GtkWidget *label40;
  GtkWidget *label41;
  GtkWidget *label42;
  GtkWidget *label52;
  GtkWidget *label43;
  GtkWidget *label27;
  GtkWidget *vbox14;
  GtkWidget *hbox13;
  GtkWidget *label44;
  GtkObject *spinbutton_def_fps_adj;
  GtkWidget *label29;
  GtkWidget *dialog_action_area8;
  GtkWidget *okbutton5;
  GtkWidget *applybutton;
  GtkWidget *dirbutton1;
  GtkWidget *dirbutton2;
  GtkWidget *dirbutton3;
  GtkWidget *dirbutton4;
  GtkWidget *dirbutton5;
  GtkWidget *dirbutton6;
  GtkWidget *dirimage1;
  GtkWidget *dirimage2;
  GtkWidget *dirimage3;
  GtkWidget *dirimage4;
  GtkWidget *dirimage5;
  GtkWidget *dirimage6;
  GtkWidget *hseparator13;
  GtkWidget *hbox31;
  GtkWidget *label126;
  GtkWidget *pp_combo;
  GtkWidget *combo_entry4;
  GtkWidget *label128;
  GtkWidget *entry20;
  GtkWidget *label129;
  GtkWidget *entry21;
  GtkWidget *png;
  GtkWidget *label158;
  GtkWidget *label159;
  GtkWidget *hbox116;
  GtkObject *spinbutton_ocp_adj;

  GtkWidget *label;

  // tooltips
  GtkTooltips *tooltips;

  // radio button groups
  GSList *rb_group = NULL;
  GSList *jpeg_png = NULL;
  GSList *rb_group2 = NULL;

  // drop down lists
  GList *themes = NULL;
  GList *ofmt = NULL;
  GList *ofmt_all = NULL;
  GList *audp = NULL;
  GList *encoders = NULL;
  GList *vid_playback_plugins = NULL;
  
  gchar **array;

  int i;

  gint osc_udp_port;

  _prefsw *prefsw=(_prefsw*)(g_malloc(sizeof(_prefsw)));

  tooltips = gtk_tooltips_new ();

  prefsw->prefs_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->prefs_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (prefsw->prefs_dialog), _("LiVES: - Preferences"));
  gtk_window_set_position (GTK_WINDOW (prefsw->prefs_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (prefsw->prefs_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (prefsw->prefs_dialog), 660, 440);
  gtk_window_set_transient_for(GTK_WINDOW(prefsw->prefs_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(prefsw->prefs_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(prefsw->prefs_dialog),FALSE);
  }

  dialog_vbox8 = GTK_DIALOG (prefsw->prefs_dialog)->vbox;
  gtk_widget_show (dialog_vbox8);

  prefsw->prefs_notebook = gtk_notebook_new ();
  gtk_widget_show (prefsw->prefs_notebook);
  gtk_box_pack_start (GTK_BOX (dialog_vbox8), prefsw->prefs_notebook, TRUE, TRUE, 0);
  gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (prefsw->prefs_notebook), 10);
  gtk_notebook_popup_enable(GTK_NOTEBOOK(prefsw->prefs_notebook));

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox7);
  gtk_container_set_border_width (GTK_CONTAINER (vbox7), 20);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox6, TRUE, TRUE, 0);

  label28 = gtk_label_new (_("Background colour (for Blank Background and effects)"));
  gtk_widget_show (label28);
  gtk_box_pack_start (GTK_BOX (hbox6), label28, FALSE, FALSE, 16);
  gtk_label_set_justify (GTK_LABEL (label28), GTK_JUSTIFY_LEFT);

  prefsw->bgcolour_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->bgcolour_entry),255);

  gtk_entry_set_text(GTK_ENTRY(prefsw->bgcolour_entry),prefs->bgcolour);

  gtk_widget_show (prefsw->bgcolour_entry);
  gtk_box_pack_start (GTK_BOX (hbox6), prefsw->bgcolour_entry, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, prefsw->bgcolour_entry, _("What is your favourite colour ?"), NULL);

  prefsw->show_tool = gtk_check_button_new_with_mnemonic (_("Show toolbar when background is blanked"));
  gtk_widget_show (prefsw->show_tool);
  gtk_box_pack_start (GTK_BOX (vbox7), prefsw->show_tool, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->show_tool), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->show_tool), future_prefs->show_tool);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox7), hseparator2, FALSE, TRUE, 0);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox8, TRUE, TRUE, 0);

  prefsw->fs_max_check = gtk_check_button_new_with_mnemonic (_("Open file selection maximised"));
  gtk_widget_show (prefsw->fs_max_check);
  gtk_box_pack_start (GTK_BOX (hbox8), prefsw->fs_max_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->fs_max_check), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->fs_max_check), prefs->fileselmax);

  prefsw->recent_check = gtk_check_button_new_with_mnemonic (_("Show recent files in the File menu"));
  gtk_widget_show (prefsw->recent_check);
  gtk_box_pack_start (GTK_BOX (hbox8), prefsw->recent_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->recent_check), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->recent_check), prefs->show_recent);

  hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (hseparator3);
  gtk_box_pack_start (GTK_BOX (vbox7), hseparator3, FALSE, TRUE, 0);
  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox7, FALSE, FALSE, 0);

  prefsw->stop_screensaver_check = gtk_check_button_new_with_mnemonic (_("Stop xscreensaver on playback    "));
  gtk_widget_show (prefsw->stop_screensaver_check);
  gtk_box_pack_start (GTK_BOX (hbox7), prefsw->stop_screensaver_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->stop_screensaver_check), 22);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->stop_screensaver_check), prefs->stop_screensaver);

  prefsw->open_maximised_check = gtk_check_button_new_with_mnemonic (_("Open main window maximised"));
  gtk_widget_show (prefsw->open_maximised_check);
  gtk_box_pack_start (GTK_BOX (hbox7), prefsw->open_maximised_check, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->open_maximised_check), prefs->open_maximised);

  pref_gui = gtk_label_new (_("GUI"));
  gtk_widget_show (pref_gui);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 0), pref_gui);
  gtk_label_set_justify (GTK_LABEL (pref_gui), GTK_JUSTIFY_LEFT);


  // decoding

  vbox109 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox109);
  gtk_container_set_border_width (GTK_CONTAINER (vbox109), 10);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox109);

  hbox109 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox109);
  gtk_box_pack_start (GTK_BOX (vbox109), hbox109, TRUE, FALSE, 0);

  label133 = gtk_label_new (_("Video open command             "));
  gtk_widget_show (label133);
  gtk_box_pack_start (GTK_BOX (hbox109), label133, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label133), GTK_JUSTIFY_LEFT);

  prefsw->video_open_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->video_open_entry),255);
  gtk_widget_show (prefsw->video_open_entry);
  gtk_box_pack_start (GTK_BOX (hbox109), prefsw->video_open_entry, FALSE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(prefsw->video_open_entry),prefs->video_open_command);

  // TODO - more lazy loading...
  if (prefs->ocp==-1) prefs->ocp=get_int_pref ("open_compression_percent");

  hbox116 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox116);
  gtk_box_pack_start (GTK_BOX (vbox109), hbox116, TRUE, FALSE, 0);

  label158 = gtk_label_new (_("Open compression                  "));
  gtk_widget_show (label158);
  gtk_label_set_justify (GTK_LABEL (label158), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (hbox116), label158, FALSE, TRUE, 0);

  spinbutton_ocp_adj = gtk_adjustment_new (prefs->ocp, 0, 100, 1, 5, 10);
  prefsw->spinbutton_ocp = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_ocp_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_ocp);
  gtk_box_pack_start (GTK_BOX (hbox116), prefsw->spinbutton_ocp, FALSE, TRUE, 0);

  label159 = gtk_label_new (_ (" %     ( lower = slower, larger files; for jpeg, higher quality )"));
  gtk_widget_show (label159);
  gtk_box_pack_start (GTK_BOX (hbox116), label159, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label159), GTK_JUSTIFY_LEFT);

  hbox115 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox115);
  gtk_box_pack_start (GTK_BOX (vbox109), hbox115, TRUE, FALSE, 0);

  label157 = gtk_label_new (_("Default image format          "));
  gtk_widget_show (label157);
  gtk_label_set_justify (GTK_LABEL (label157), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (hbox115), label157, FALSE, TRUE, 0);

  prefsw->jpeg = gtk_radio_button_new_with_mnemonic (NULL, _("_jpeg"));
  gtk_widget_show (prefsw->jpeg);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->jpeg), 18);
  gtk_box_pack_start (GTK_BOX (hbox115), prefsw->jpeg, FALSE, FALSE, 0);

  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->jpeg), jpeg_png);
  jpeg_png = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->jpeg));

  png = gtk_radio_button_new_with_mnemonic (jpeg_png, _("_png (experimental, requires editing of smogrify)"));
  gtk_widget_show (png);
  gtk_container_set_border_width (GTK_CONTAINER (png), 18);
  gtk_box_pack_start (GTK_BOX (hbox115), png, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (png),!strcmp (prefs->image_ext,"png"));


  label126 = gtk_label_new (_("Decoding"));
  gtk_widget_show (label126);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 1), label126);
  gtk_label_set_justify (GTK_LABEL (label126), GTK_JUSTIFY_LEFT);


  // playback

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox9);

  frame4 = gtk_frame_new (NULL);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox9), frame4, TRUE, TRUE, 0);

  vbox69=gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox69);
  gtk_container_add (GTK_CONTAINER (frame4), vbox69);
  gtk_container_set_border_width (GTK_CONTAINER (vbox69), 10);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (vbox69), hbox9);

  label33 = gtk_label_new (_("Player      "));
  gtk_widget_show (label33);
  gtk_box_pack_start (GTK_BOX (hbox9), label33, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label33), GTK_JUSTIFY_LEFT);

  label34 = gtk_label_new (_("     Video play command      "));
  gtk_widget_show (label34);
  gtk_box_pack_start (GTK_BOX (hbox9), label34, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label34), GTK_JUSTIFY_LEFT);

  prefsw->video_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->video_command_entry),255);
  gtk_widget_show (prefsw->video_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox9), prefsw->video_command_entry, TRUE, TRUE, 0);

  gtk_entry_set_text(GTK_ENTRY(prefsw->video_command_entry),prefs->video_play_command);

  gtk_widget_hide(prefsw->video_command_entry);
  gtk_widget_hide(label34);
  
  hbox101 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox101);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox101, FALSE, FALSE, 0);

  prefsw->checkbutton_show_stats = gtk_check_button_new_with_mnemonic (_ ("_Show FPS statistics"));
  gtk_widget_show (prefsw->checkbutton_show_stats);
  gtk_box_pack_start (GTK_BOX (hbox101), prefsw->checkbutton_show_stats, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_show_stats),prefs->show_player_stats);

  hseparator13 = gtk_hseparator_new ();
  gtk_widget_show (hseparator13);
  gtk_box_pack_start (GTK_BOX (vbox69), hseparator13, TRUE, TRUE, 0);

  hbox31 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox31);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox31, FALSE, FALSE, 0);

  label126 = gtk_label_new (_("Plugin      "));
  gtk_widget_show (label126);
  gtk_box_pack_start (GTK_BOX (hbox31), label126, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label126), GTK_JUSTIFY_LEFT);

  pp_combo = gtk_combo_new ();
  gtk_widget_show (pp_combo);
  gtk_box_pack_start (GTK_BOX (hbox31), pp_combo, FALSE, FALSE, 0);

  combo_entry4 = GTK_COMBO (pp_combo)->entry;
  gtk_editable_set_editable (GTK_EDITABLE(combo_entry4),FALSE);
  gtk_widget_show (combo_entry4);

  vid_playback_plugins=get_plugin_list (PLUGIN_VID_PLAYBACK,FALSE,NULL);
  vid_playback_plugins=g_list_prepend (vid_playback_plugins, _ ("None"));
  combo_set_popdown_strings (GTK_COMBO (pp_combo), vid_playback_plugins);
  g_list_free (vid_playback_plugins);

  if (vid_playback_plugin!=NULL) {
    gtk_entry_set_text (GTK_ENTRY(combo_entry4),vid_playback_plugin->name);
  }
  else {
    gtk_entry_set_text (GTK_ENTRY(combo_entry4),_ ("None"));
  }

  label128 = gtk_label_new (_("    Host "));
  gtk_widget_show (label128);
  gtk_box_pack_start (GTK_BOX (hbox31), label128, TRUE, TRUE, 6);
  gtk_label_set_justify (GTK_LABEL (label128), GTK_JUSTIFY_LEFT);

  entry20 = gtk_entry_new ();
  gtk_widget_show (entry20);
  gtk_entry_set_max_length(GTK_ENTRY(entry20),15);

  // fixed for now
  gtk_entry_set_text(GTK_ENTRY(entry20),"127.0.0.1");
  gtk_box_pack_start (GTK_BOX (hbox31), entry20, FALSE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry20), FALSE);

  label129 = gtk_label_new (_("    Port"));
  gtk_widget_show (label129);
  gtk_box_pack_start (GTK_BOX (hbox31), label129, TRUE, TRUE, 6);
  gtk_label_set_justify (GTK_LABEL (label129), GTK_JUSTIFY_LEFT);

  entry21 = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(entry21),5);

  // fixed for now
  gtk_entry_set_text(GTK_ENTRY(entry21),"N/A");
  gtk_widget_show (entry21);
  gtk_box_pack_start (GTK_BOX (hbox31), entry21, FALSE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry21), FALSE);

  label31 = gtk_label_new (_("Video"));
  gtk_widget_show (label31);
  gtk_frame_set_label_widget (GTK_FRAME (frame4), label31);
  gtk_label_set_justify (GTK_LABEL (label31), GTK_JUSTIFY_LEFT);

  frame5 = gtk_frame_new (NULL);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox9), frame5, TRUE, TRUE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox10);
  gtk_container_add (GTK_CONTAINER (frame5), hbox10);
  gtk_container_set_border_width (GTK_CONTAINER (hbox10), 10);

  label35 = gtk_label_new (_("Player     "));
  gtk_widget_show (label35);
  gtk_box_pack_start (GTK_BOX (hbox10), label35, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label35), GTK_JUSTIFY_LEFT);

  audp = g_list_append (audp, _ ("sox (recommended)"));
  audp = g_list_append (audp, "mplayer");

  prefsw->audp_combo = gtk_combo_new ();
  combo_set_popdown_strings (GTK_COMBO (prefsw->audp_combo), audp);
  gtk_box_pack_start (GTK_BOX (hbox10), prefsw->audp_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->audp_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->audp_combo))->entry),FALSE);
  if (prefs->audio_player==AUD_PLAYER_SOX) {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry),_ ("sox (recommended)"));
  }
  else {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry),"mplayer");
  }

  label36 = gtk_label_new (_("     Audio play command      "));
  gtk_widget_show (label36);
  gtk_box_pack_start (GTK_BOX (hbox10), label36, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label36), GTK_JUSTIFY_LEFT);

  prefsw->audio_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->audio_command_entry),255);
  gtk_widget_show (prefsw->audio_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox10), prefsw->audio_command_entry, TRUE, TRUE, 0);

  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->audio_command_entry),prefs->audio_play_command);

  label32 = gtk_label_new (_("Audio"));
  gtk_widget_show (label32);
  gtk_frame_set_label_widget (GTK_FRAME (frame5), label32);
  gtk_label_set_justify (GTK_LABEL (label32), GTK_JUSTIFY_LEFT);

  label25 = gtk_label_new (_("Playback"));
  gtk_widget_show (label25);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 2), label25);
  gtk_label_set_justify (GTK_LABEL (label25), GTK_JUSTIFY_LEFT);


  // recording


  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 20);


  label37 = gtk_label_new (_("      What to record when 'r' is pressed   "));
  gtk_widget_show (label37);
  gtk_box_pack_start (GTK_BOX (vbox12), label37, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label37), GTK_JUSTIFY_LEFT);

  prefsw->rframes = gtk_check_button_new_with_mnemonic (_("_Frame changes"));
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->rframes), 18);
  gtk_widget_show (prefsw->rframes);

  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->rframes, TRUE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->rframes),prefs->rec_opts&REC_FRAMES);

  if (prefs->rec_opts&REC_FPS||prefs->rec_opts&REC_CLIPS) gtk_widget_set_sensitive (prefsw->rframes,FALSE); // we must record these if recording fps changes or clip switches
  if (mainw->playing_file>0&&mainw->record) gtk_widget_set_sensitive (prefsw->rframes,FALSE);

  prefsw->rfps = gtk_check_button_new_with_mnemonic (_("_FPS changes"));
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->rfps), 18);
  gtk_widget_show (prefsw->rfps);

  if (prefs->rec_opts&REC_CLIPS) gtk_widget_set_sensitive (prefsw->rfps,FALSE); // we must record these if recording clip switches
  if (mainw->playing_file>0&&mainw->record) gtk_widget_set_sensitive (prefsw->rfps,FALSE);

  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->rfps, TRUE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->rfps),prefs->rec_opts&REC_FPS);

  prefsw->reffects = gtk_check_button_new_with_mnemonic (_("_Real time effects (requires rendering)"));
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->reffects), 18);
  gtk_widget_show (prefsw->reffects);
  if (mainw->playing_file>0&&mainw->record) gtk_widget_set_sensitive (prefsw->reffects,FALSE);

  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->reffects, TRUE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->reffects),prefs->rec_opts&REC_EFFECTS);

  prefsw->rclips = gtk_check_button_new_with_mnemonic (_("_Clip switches (requires rendering)"));
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->rclips), 18);
  gtk_widget_show (prefsw->rclips);

  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->rclips, TRUE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->rclips),prefs->rec_opts&REC_CLIPS);
  if (mainw->playing_file>0&&mainw->record) gtk_widget_set_sensitive (prefsw->rclips,FALSE);

  label = gtk_label_new (_("Recording"));
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 3), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  

  // encoding



  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 20);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox11);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox11, TRUE, TRUE, 0);

  label37 = gtk_label_new (_("      Encoder                  "));
  gtk_widget_show (label37);
  gtk_box_pack_start (GTK_BOX (hbox11), label37, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label37), GTK_JUSTIFY_LEFT);

  prefsw->encoder_combo = gtk_combo_new ();
  gtk_box_pack_start (GTK_BOX (hbox11), prefsw->encoder_combo, FALSE, FALSE, 0);

  prefsw->debug_encoders = gtk_check_button_new_with_mnemonic (_("_Debug Mode"));
  gtk_widget_show (prefsw->debug_encoders);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox15);

  gtk_box_pack_start (GTK_BOX (vbox12), hbox15, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox15), prefsw->debug_encoders, TRUE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->debug_encoders),prefs->debug_encoders);

  if (capable->has_encoder_plugins==TRUE) {
    // scan for encoder plugins
    if ((encoders=get_plugin_list (PLUGIN_ENCODERS,FALSE,NULL))==NULL) {
      do_plugin_error(PLUGIN_ENCODERS,NULL);
    }
    else {
      // TODO - check if prefs->img_ext == ".png", if so remove any encoders
      // which cannot handle png
      combo_set_popdown_strings (GTK_COMBO (prefsw->encoder_combo), encoders);
      g_list_free (encoders);
    }
  }
  else {
    gtk_widget_set_sensitive (prefsw->debug_encoders,FALSE);
  }

  gtk_widget_show(prefsw->encoder_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->encoder_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->encoder_combo))->entry),prefs->encoder.name);

  hseparator8 = gtk_hseparator_new ();
  gtk_widget_show (hseparator8);
  gtk_box_pack_start (GTK_BOX (vbox12), hseparator8, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox, TRUE, TRUE, 0);

  label56 = gtk_label_new (_("Output format"));
  gtk_box_pack_start (GTK_BOX (hbox), label56, TRUE, FALSE, 0);
  
  prefsw->ofmt_combo = gtk_combo_new ();

  if (capable->has_encoder_plugins) {
    // reqest formats from the encoder plugin
    if ((ofmt_all=plugin_request_by_line(PLUGIN_ENCODERS,prefs->encoder.name,"get_formats"))!=NULL) {
      for (i=0;i<g_list_length(ofmt_all);i++) {
	if (get_token_count (g_list_nth_data (ofmt_all,i),'|')>2) {
	  array=g_strsplit (g_list_nth_data (ofmt_all,i),"|",-1);
	  if (!strcmp(array[0],prefs->encoder.of_name)) {
	    prefs->encoder.of_allowed_acodecs=atoi(array[2]);
	  }
	  ofmt=g_list_append(ofmt,g_strdup (array[1]));
	  g_strfreev (array);
	}
      }
      memcpy (&future_prefs->encoder,&prefs->encoder,sizeof(_encoder));
      combo_set_popdown_strings (GTK_COMBO (prefsw->ofmt_combo), ofmt);
      g_list_free(ofmt);
      g_list_free(ofmt_all);
    }
    else {
      do_plugin_error(PLUGIN_ENCODERS,prefs->encoder.name);
      future_prefs->encoder.of_allowed_acodecs=0;
    }
    
    gtk_box_pack_start (GTK_BOX (hbox), prefsw->ofmt_combo, TRUE, TRUE, 10);
    add_fill_to_box (GTK_BOX (hbox));
    gtk_widget_show_all(hbox);

    gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->ofmt_combo))->entry),FALSE);
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->ofmt_combo))->entry),prefs->encoder.of_name);
  
    prefsw->acodec_combo = gtk_combo_new ();
    prefsw->acodec_entry=(GtkWidget*)(GTK_ENTRY((GTK_COMBO(prefsw->acodec_combo))->entry));
    prefsw->acodec_list=NULL;

    set_acodec_list_from_allowed(prefsw);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox12), hbox, TRUE, TRUE, 0);

    label = gtk_label_new (_("Audio codec"));
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 10);
    
    gtk_box_pack_start (GTK_BOX (hbox), prefsw->acodec_combo, TRUE, TRUE, 10);
    add_fill_to_box (GTK_BOX (hbox));
    gtk_widget_show_all (hbox);

    gtk_editable_set_editable (GTK_EDITABLE(prefsw->acodec_entry),FALSE);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label),prefsw->acodec_entry);
  }

  label = gtk_label_new (_("Encoding"));
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 4), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  

  // effects

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 20);
  
  prefsw->checkbutton_antialias = gtk_check_button_new_with_mnemonic (_("Use _antialiasing when resizing"));
  gtk_widget_show (prefsw->checkbutton_antialias);
  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->checkbutton_antialias, FALSE, FALSE, 10);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_antialias),prefs->antialias);

  label = gtk_label_new (_("Effects"));
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 5), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  table4 = gtk_table_new (7, 3, FALSE);
  gtk_widget_show (table4);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), table4);
  gtk_container_set_border_width (GTK_CONTAINER (table4), 20);
  
  label39 = gtk_label_new (_("      Video load directory (default)     "));
  gtk_widget_show (label39);
  gtk_table_attach (GTK_TABLE (table4), label39, 0, 1, 0, 1,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label39), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label39), 0, 0.5);
  
  label40 = gtk_label_new (_("      Video save directory (default)"));
  gtk_widget_show (label40);
  gtk_table_attach (GTK_TABLE (table4), label40, 0, 1, 1, 2,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label40), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label40), 0, 0.5);
  
  label41 = gtk_label_new (_("      Audio load directory (default)"));
  gtk_widget_show (label41);
  gtk_table_attach (GTK_TABLE (table4), label41, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label41), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label41), 0, 0.5);
  
  label42 = gtk_label_new (_("      Image directory (default)"));
  gtk_widget_show (label42);
  gtk_table_attach (GTK_TABLE (table4), label42, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label42), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label42), 0, 0.5);
  
  label52 = gtk_label_new (_("      Backup/Restore directory (default)"));
  gtk_widget_show (label52);
  gtk_table_attach (GTK_TABLE (table4), label52, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label52), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label52), 0, 0.5);
  
  label43 = gtk_label_new (_("      Temp directory"));
  gtk_widget_show (label43);
  gtk_table_attach (GTK_TABLE (table4), label43, 0, 1, 5, 6,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label43), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label43), 0, 0.5);








  
  prefsw->vid_load_dir_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->vid_load_dir_entry),255);
  gtk_widget_show (prefsw->vid_load_dir_entry);
  gtk_table_attach (GTK_TABLE (table4), prefsw->vid_load_dir_entry, 1, 2, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->vid_load_dir_entry),prefs->def_vid_load_dir);
  
  prefsw->vid_save_dir_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->vid_save_dir_entry),255);
  gtk_widget_show (prefsw->vid_save_dir_entry);
  gtk_table_attach (GTK_TABLE (table4), prefsw->vid_save_dir_entry, 1, 2, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->vid_save_dir_entry),prefs->def_vid_save_dir);
  
  prefsw->audio_dir_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->audio_dir_entry),255);
  gtk_widget_show (prefsw->audio_dir_entry);
  gtk_table_attach (GTK_TABLE (table4), prefsw->audio_dir_entry, 1, 2, 2, 3,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
  // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->audio_dir_entry),prefs->def_audio_dir);
   
   prefsw->image_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->image_dir_entry),255);
   gtk_widget_show (prefsw->image_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->image_dir_entry, 1, 2, 3, 4,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->image_dir_entry),prefs->def_image_dir);
   
   prefsw->proj_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->proj_dir_entry),255);
   gtk_widget_show (prefsw->proj_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->proj_dir_entry, 1, 2, 4, 5,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->proj_dir_entry),prefs->def_proj_dir);
   
   prefsw->tmpdir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->tmpdir_entry),255);
   gtk_widget_show (prefsw->tmpdir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->tmpdir_entry, 1, 2, 5, 6,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->tmpdir_entry),future_prefs->tmpdir);
   
   dirbutton1 = gtk_button_new ();
   gtk_widget_show (dirbutton1);
   
   
   dirimage1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage1);
   gtk_container_add (GTK_CONTAINER (dirbutton1), dirimage1);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton1, 2, 3, 0, 1,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton2 = gtk_button_new ();
   gtk_widget_show (dirbutton2);
   
   
   dirimage2 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage2);
   gtk_container_add (GTK_CONTAINER (dirbutton2), dirimage2);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton2, 2, 3, 1, 2,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton3 = gtk_button_new ();
   gtk_widget_show (dirbutton3);

   
   dirimage3 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage3);
   gtk_container_add (GTK_CONTAINER (dirbutton3), dirimage3);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton3, 2, 3, 2, 3,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton4 = gtk_button_new ();
   gtk_widget_show (dirbutton4);
   
   
   dirimage4 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage4);
   gtk_container_add (GTK_CONTAINER (dirbutton4), dirimage4);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton4, 2, 3, 3, 4,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   

   dirbutton5 = gtk_button_new ();
   gtk_widget_show (dirbutton5);

   
   dirimage5 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage5);
  gtk_container_add (GTK_CONTAINER (dirbutton5), dirimage5);
  
  gtk_table_attach (GTK_TABLE (table4), dirbutton5, 2, 3, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  
  
  dirbutton6 = gtk_button_new ();
  gtk_widget_show (dirbutton6);


  dirimage6 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (dirimage6);
  gtk_container_add (GTK_CONTAINER (dirbutton6), dirimage6);

  gtk_table_attach (GTK_TABLE (table4), dirbutton6, 2, 3, 5, 6,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);


  label27 = gtk_label_new (_("Directories"));
  gtk_widget_show (label27);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 6), label27);
  gtk_label_set_justify (GTK_LABEL (label27), GTK_JUSTIFY_LEFT);

  vbox18 = gtk_vbox_new (FALSE, 10);
  gtk_widget_show (vbox18);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox18);
  gtk_container_set_border_width (GTK_CONTAINER (vbox18), 20);

  prefsw->checkbutton_warn_fps = gtk_check_button_new_with_mnemonic (_("Warn on Insert / Merge if _frame rate of clipboard does not match frame rate of selection"));
  gtk_widget_show (prefsw->checkbutton_warn_fps);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_fps, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_fps), !(prefs->warning_mask&WARN_MASK_FPS));

  hbox100 = gtk_hbox_new(FALSE,10);
  gtk_box_pack_start (GTK_BOX (vbox18), hbox100, FALSE, FALSE, 0);
  gtk_widget_show(hbox100);
  prefsw->checkbutton_warn_fsize = gtk_check_button_new_with_mnemonic (_("Warn on Open if file _size exceeds "));
  gtk_widget_show (prefsw->checkbutton_warn_fsize);
  gtk_box_pack_start (GTK_BOX (hbox100), prefsw->checkbutton_warn_fsize, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_fsize), !(prefs->warning_mask&WARN_MASK_FSIZE));
  spinbutton_warn_fsize_adj = gtk_adjustment_new (prefs->warn_file_size, 1, 2048, 1, 10, 100);

  prefsw->spinbutton_warn_fsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_warn_fsize_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_warn_fsize);
  gtk_box_pack_start (GTK_BOX (hbox100), prefsw->spinbutton_warn_fsize, FALSE, TRUE, 0);

  label100 = gtk_label_new (_ (" MB"));
  gtk_widget_show (label100);
  gtk_box_pack_start (GTK_BOX (hbox100), label100, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);

  prefsw->checkbutton_warn_save_quality = gtk_check_button_new_with_mnemonic (_("Warn about loss of _quality when saving to an existing file"));
  gtk_widget_show (prefsw->checkbutton_warn_save_quality);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_save_quality, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_save_quality), !(prefs->warning_mask&WARN_MASK_SAVE_QUALITY));

  prefsw->checkbutton_warn_set = gtk_check_button_new_with_mnemonic (_("Show a warning before saving a se_t"));
  gtk_widget_show (prefsw->checkbutton_warn_set);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_set, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_set), !(prefs->warning_mask&WARN_MASK_SAVE_SET));

  prefsw->checkbutton_warn_mplayer = gtk_check_button_new_with_mnemonic (_("Show a warning if _mplayer is not found when LiVES is started."));
  gtk_widget_show (prefsw->checkbutton_warn_mplayer);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_mplayer, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_mplayer), !(prefs->warning_mask&WARN_MASK_NO_MPLAYER));

  label86 = gtk_label_new ("");
  gtk_widget_show (label86);
  gtk_box_pack_start (GTK_BOX (vbox18), label86, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label86), GTK_JUSTIFY_LEFT);

  label87 = gtk_label_new ("");
  gtk_widget_show (label87);
  gtk_box_pack_start (GTK_BOX (vbox18), label87, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label87), GTK_JUSTIFY_LEFT);

  label84 = gtk_label_new (_("Warnings"));
  gtk_widget_show (label84);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 7), label84);
  gtk_label_set_justify (GTK_LABEL (label84), GTK_JUSTIFY_LEFT);

  vbox14 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox14);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox14);
  gtk_container_set_border_width (GTK_CONTAINER (vbox14), 20);

  prefsw->check_midi = gtk_check_button_new_with_mnemonic (_("Midi synch (requires the files midistart and midistop)"));
  gtk_widget_show (prefsw->check_midi);
  gtk_box_pack_start (GTK_BOX (vbox14), prefsw->check_midi, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->check_midi), 10);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->check_midi), prefs->midisynch);
  gtk_widget_set_sensitive(prefsw->check_midi,capable->has_midistartstop);

  hbox99 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox99);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox99, TRUE, TRUE, 0);

  label97 = gtk_label_new (_("When inserting/merging frames:  "));
  gtk_widget_show (label97);
  gtk_box_pack_start (GTK_BOX (hbox99), label97, FALSE, FALSE, 16);
  gtk_label_set_justify (GTK_LABEL (label97), GTK_JUSTIFY_LEFT);

  prefsw->ins_speed = gtk_radio_button_new_with_mnemonic (NULL, _("_Speed Up/Slow Down Insertion"));
  gtk_widget_show (prefsw->ins_speed);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->ins_speed), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->ins_speed), rb_group2);
  rb_group2 = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->ins_speed));
  gtk_box_pack_start (GTK_BOX (hbox99), prefsw->ins_speed, FALSE, FALSE, 0);

  ins_resample = gtk_radio_button_new_with_mnemonic (NULL, _("_Resample Insertion"));
  gtk_widget_show (ins_resample);
  gtk_container_set_border_width (GTK_CONTAINER (ins_resample), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (ins_resample), rb_group2);
  rb_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (ins_resample));
  gtk_box_pack_start (GTK_BOX (hbox99), ins_resample, FALSE, FALSE, 0);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ins_resample),prefs->ins_resample);

  prefsw->check_xmms_pause = gtk_check_button_new_with_mnemonic (_("Pause xmms during audio playback"));
  gtk_box_pack_start (GTK_BOX (vbox14), prefsw->check_xmms_pause, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->check_xmms_pause), 10);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->check_xmms_pause), prefs->pause_xmms);

  if (capable->has_xmms==TRUE) {
    gtk_widget_show (prefsw->check_xmms_pause);
  }

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox19, TRUE, TRUE, 0);
  
  label134 = gtk_label_new (_("CD device           "));
  gtk_widget_show (label134);
  gtk_box_pack_start (GTK_BOX (hbox19), label134, FALSE, FALSE, 18);
  gtk_label_set_justify (GTK_LABEL (label134), GTK_JUSTIFY_LEFT);
  
  prefsw->cdplay_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->cdplay_entry),255);
  gtk_box_pack_start (GTK_BOX (hbox19), prefsw->cdplay_entry, FALSE, TRUE, 0);

  if (capable->has_cdda2wav==TRUE) {
    gtk_widget_show (prefsw->cdplay_entry);
    gtk_widget_show (hbox19);
  }
  else {
    gtk_widget_hide (prefsw->cdplay_entry);
    gtk_widget_hide (hbox19);
  }

  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->cdplay_entry),prefs->cdplay_device);

  gtk_tooltips_set_tip (tooltips, prefsw->cdplay_entry, _("LiVES can load audio tracks from this CD"), NULL);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox13, TRUE, TRUE, 0);

  label44 = gtk_label_new (_("Default FPS        "));
  gtk_widget_show (label44);
  gtk_box_pack_start (GTK_BOX (hbox13), label44, FALSE, FALSE, 18);
  gtk_label_set_justify (GTK_LABEL (label44), GTK_JUSTIFY_LEFT);

  spinbutton_def_fps_adj = gtk_adjustment_new (prefs->default_fps, 1, 2048, 1, 10, 10);

  prefsw->spinbutton_def_fps = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_def_fps_adj), 1, 2);
  gtk_widget_show (prefsw->spinbutton_def_fps);
  gtk_box_pack_start (GTK_BOX (hbox13), prefsw->spinbutton_def_fps, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, prefsw->spinbutton_def_fps, _("Frames per second to use when none is specified"), NULL);

  label29 = gtk_label_new (_("Misc"));
  gtk_widget_show (label29);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 8), label29);
  gtk_label_set_justify (GTK_LABEL (label29), GTK_JUSTIFY_LEFT);

  vbox99 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox99);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox99);
  gtk_container_set_border_width (GTK_CONTAINER (vbox99), 20);

  hbox93 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox93);
  gtk_box_pack_start (GTK_BOX (vbox99), hbox93, TRUE, TRUE, 0);

  label94 = gtk_label_new (_("New theme:           "));
  gtk_widget_show (label94);
  gtk_box_pack_start (GTK_BOX (hbox93), label94, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_LEFT);

  prefsw->theme_combo = gtk_combo_new ();


  // scan for themes
  themes=get_plugin_list (PLUGIN_THEMES,FALSE,NULL);
  themes=g_list_prepend (themes, "none");

  combo_set_popdown_strings (GTK_COMBO (prefsw->theme_combo), themes);
  g_list_free (themes);

  gtk_box_pack_start (GTK_BOX (hbox93), prefsw->theme_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->theme_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->theme_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->theme_combo))->entry),future_prefs->theme);

  label99 = gtk_label_new (_("Themes"));
  gtk_widget_show (label99);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 9), label99);
  gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);

  vbox98 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox98);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox98);
  gtk_container_set_border_width (GTK_CONTAINER (vbox98), 20);

  hbox94 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox94);
  gtk_box_pack_start (GTK_BOX (vbox98), hbox94, TRUE, TRUE, 0);

  label88 = gtk_label_new (_("Download bandwidth (Kb/s)       "));
  gtk_widget_show (label88);
  gtk_box_pack_start (GTK_BOX (hbox94), label88, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label88), GTK_JUSTIFY_LEFT);

  spinbutton_bwidth_adj = gtk_adjustment_new (prefs->dl_bandwidth, 0, 100000, 1, 10, 100);

  prefsw->spinbutton_bwidth = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_bwidth_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_bwidth);
  gtk_box_pack_start (GTK_BOX (hbox94), prefsw->spinbutton_bwidth, FALSE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox98), hbox, TRUE, TRUE, 0);

  prefsw->enable_OSC = gtk_check_button_new_with_mnemonic (_("OMC remote control enabled"));
  gtk_widget_show (prefsw->enable_OSC);
  gtk_box_pack_start (GTK_BOX (hbox), prefsw->enable_OSC, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->enable_OSC), 22);

  label = gtk_label_new (_("UDP port       "));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  if (!prefs->osc_udp_started&&future_prefs->osc_udp_port<1) {
    osc_udp_port=OSC_DEFAULT_UDP_PORT;
  }
  else {
    if ((osc_udp_port=prefs->osc_udp_port)<1) osc_udp_port=future_prefs->osc_udp_port;
  }
    
  spinbutton_adj = gtk_adjustment_new (osc_udp_port, 1, 65535, 1, 10, 100);

  prefsw->spinbutton_osc_udp = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_osc_udp);

  if (!prefs->osc_udp_started&&future_prefs->osc_udp_port<1) {
    gtk_widget_set_sensitive (prefsw->spinbutton_osc_udp,FALSE);
  }
  else osc_udp_port=prefs->osc_udp_port;

  gtk_box_pack_start (GTK_BOX (hbox), prefsw->spinbutton_osc_udp, FALSE, TRUE, 0);

  prefsw->enable_OSC_start = gtk_check_button_new_with_mnemonic (_("Start OMC on startup"));
  gtk_widget_show (prefsw->enable_OSC_start);
  gtk_box_pack_start (GTK_BOX (vbox98), prefsw->enable_OSC_start, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->enable_OSC_start), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->enable_OSC_start), future_prefs->osc_udp_port>0);

#ifndef ENABLE_OSC
  gtk_widget_set_sensitive (prefsw->spinbutton_osc_udp,FALSE);
  gtk_widget_set_sensitive (prefsw->enable_OSC,FALSE);
  gtk_widget_set_sensitive (prefsw->enable_OSC_start,FALSE);
  gtk_widget_set_sensitive (label,FALSE);
#else
  if (prefs->osc_udp_started) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->enable_OSC), TRUE);
    gtk_widget_set_sensitive (prefsw->spinbutton_osc_udp,FALSE);
    gtk_widget_set_sensitive (prefsw->enable_OSC,FALSE);
  }
#endif

  label98 = gtk_label_new (_("Streaming/Networking"));
  gtk_widget_show (label98);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 10), label98);
  gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);

  dialog_action_area8 = GTK_DIALOG (prefsw->prefs_dialog)->action_area;
  gtk_widget_show (dialog_action_area8);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area8), GTK_BUTTONBOX_END);

  prefsw->cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (prefsw->cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), prefsw->cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (prefsw->cancelbutton, GTK_CAN_DEFAULT);

  applybutton = gtk_button_new_from_stock ("gtk-apply");
  gtk_widget_show (applybutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), applybutton, 0);
  GTK_WIDGET_SET_FLAGS (applybutton, GTK_CAN_DEFAULT);

  okbutton5 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton5);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), okbutton5, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton5, GTK_CAN_DEFAULT);


  g_signal_connect(dirbutton1, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->vid_load_dir_entry);
  g_signal_connect(dirbutton2, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->vid_save_dir_entry);
  g_signal_connect(dirbutton3, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->audio_dir_entry);
  g_signal_connect(dirbutton4, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->image_dir_entry);
  g_signal_connect(dirbutton5, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->proj_dir_entry);
  g_signal_connect(dirbutton6, "clicked", G_CALLBACK (on_filesel_complex_clicked),prefsw->tmpdir_entry);


  if (capable->has_encoder_plugins==TRUE) {
    prefsw->encoder_name_fn=g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->encoder_combo))->entry),"changed",G_CALLBACK (on_encoder_entry_changed),NULL);
    prefsw->encoder_ofmt_fn=g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->ofmt_combo))->entry),"changed",G_CALLBACK (on_encoder_ofmt_changed),NULL);
  }
  g_signal_connect (GTK_OBJECT(combo_entry4),"changed",G_CALLBACK (on_vid_playback_plugin_changed),NULL);

#ifdef ENABLE_OSC
  g_signal_connect (GTK_OBJECT (prefsw->enable_OSC), "toggled",
		    G_CALLBACK (on_osc_enable_toggled),
		    (gpointer)prefsw->enable_OSC_start);
  g_signal_connect (GTK_OBJECT (prefsw->enable_OSC_start), "toggled",
		    G_CALLBACK (on_osc_enable_toggled),
		    (gpointer)(prefsw->enable_OSC));
#endif
  g_signal_connect (GTK_OBJECT (prefsw->cancelbutton), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    prefsw);


  g_signal_connect (GTK_OBJECT (okbutton5), "clicked",
		    G_CALLBACK (on_prefs_ok_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (applybutton), "clicked",
		    G_CALLBACK (on_prefs_apply_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (prefsw->prefs_dialog), "delete_event",
		    G_CALLBACK (on_prefs_delete_event),
		    prefsw);

  g_list_free (audp);

  return prefsw;
}




void
on_preferences_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  prefsw=create_prefs_dialog();
  gtk_widget_show(prefsw->prefs_dialog);

}


void
on_prefs_ok_clicked                   (GtkButton       *button,
				       gpointer         user_data)
{
  gboolean needs_restart;
  gboolean dont_show_warnings=FALSE;
  if (!(user_data==NULL)) {
    dont_show_warnings=TRUE;
  }
  needs_restart=apply_prefs(dont_show_warnings);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  while (g_main_context_iteration(NULL,FALSE));
  g_list_free (prefsw->acodec_list);
  g_free(prefsw);
  prefsw=NULL;
  if (needs_restart==TRUE&&dont_show_warnings==FALSE) {
    on_quit_activate (NULL,GINT_TO_POINTER (1));
  }
  else {
    if (mainw->prefs_changed==TRUE&&dont_show_warnings==FALSE) {
      do_error_dialog(_ ("Theme changes will not take effect until the next time you start LiVES."));
      mainw->prefs_changed=FALSE;
    }
  }
}

void
on_prefs_apply_clicked                   (GtkButton       *button,
					  gpointer         user_data)
{
  gint current_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(prefsw->prefs_notebook));

  on_prefs_ok_clicked (button, GINT_TO_POINTER (1));
  on_preferences_activate (NULL, NULL);

  gtk_notebook_set_current_page (GTK_NOTEBOOK(prefsw->prefs_notebook),current_page);

}


gboolean
on_prefs_delete_event                  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)


{
  g_list_free (prefsw->acodec_list);
  on_cancel_button1_clicked(GTK_BUTTON (((_prefsw *)user_data)->cancelbutton),user_data);
  return FALSE;
}


