// resample.c
// LiVES
// (c) G. Finch 2004 <salsaman@xs4all.nl>
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details


// functions for reordering, resampling video and audio

#include "main.h"
#include "resample.h"
#include "support.h"
#include "callbacks.h"
#include "effects.h"


/////////////////////////////////////////////////////
inline weed_timecode_t q_gint64 (weed_timecode_t in, gdouble fps) {
  if (in>0) return (((weed_timecode_t)(in/U_SEC*fps+.5)/fps)*U_SECL+.5); // quantise to frame timing
  if (in<0) return (((weed_timecode_t)(in/U_SEC*fps-.5)/fps)*U_SECL-.5); // quantise to frame timing
  return (weed_timecode_t)0;
}

inline weed_timecode_t q_dbl (gdouble in, gdouble fps) {
  if (in>0) return (((weed_timecode_t)(in*fps+.5)/fps)*U_SECL+.5); // quantise to frame timing
  if (in<0) return (((weed_timecode_t)(in*fps-.5)/fps)*U_SECL-.5); // quantise to frame timing
  return (weed_timecode_t)0;
}

///////////////////////////////////////////////////////

inline gint count_resampled_frames (gint in_frames, gdouble orig_fps, gdouble resampled_fps) {
  gint res_frames;
  return ((res_frames=in_frames/orig_fps*resampled_fps+.49999)<1)?1:res_frames;
}

/////////////////////////////////////////////////////

gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,gint fps_denom, gint arate) {
  // do a block atomic: resample audio, then resample video/resize or joint resample/resize

  gchar *com;
  gint current_file=mainw->current_file;
  gboolean audio_resampled=FALSE;
  gboolean video_resampled=FALSE;
  gboolean video_resized=FALSE;
  gint frames=cfile->frames;

  if (arate>0&&arate!=cfile->arate) {
    cfile->undo1_int=arate;
    cfile->undo2_int=cfile->achans;
    cfile->undo3_int=cfile->asampsize;
    cfile->undo1_uint=cfile->signed_endian;
    on_resaudio_ok_clicked (NULL,NULL);
    if (mainw->error) return FALSE;
    audio_resampled=TRUE;
  }

  if (fps_denom>0) {
    fps=(fps_num*1.)/(fps_denom*1.);
  }
  if (fps>0.&&fps!=cfile->fps) {
    // FPS CHANGE...
    if ((width!=cfile->hsize||height!=cfile->vsize)&&width*height>0) {
      // CHANGING SIZE..
      if (fps>cfile->fps) {
	// we will have more frames...
	// ...do resize first
	cfile->ohsize=cfile->hsize;
	cfile->ovsize=cfile->vsize;
	com=g_strdup_printf ("smogrify resize_all %s %d %d %d",cfile->handle,cfile->frames,width,height);

	cfile->progress_start=1;
	cfile->progress_end=cfile->frames;
	
	unlink(cfile->info_file);
	system(com);
	g_free (com);
	if (!(do_progress_dialog(TRUE,TRUE,_ ("Resizing")))) {
	  cfile->undo_action=UNDO_ATOMIC_RESAMPLE_RESIZE;
	  if (!audio_resampled) {
	    cfile->undo1_int=arate;
	    cfile->undo2_int=cfile->achans;
	    cfile->undo3_int=cfile->asampsize;
	    cfile->undo4_int=cfile->arps;
	    cfile->undo1_uint=cfile->signed_endian;
	  }
	  cfile->hsize=width;
	  cfile->vsize=height;
	  
	  cfile->undo1_dbl=fps;
	  cfile->undo_start=1;
	  cfile->undo_end=frames;
	  on_undo_activate(NULL,NULL);
	  return FALSE;
	}
	    
	    
	cfile->hsize=width;
	cfile->vsize=height;
	
	cfile->undo1_dbl=fps;
	cfile->undo_start=1;
	cfile->undo_end=frames;

	// now resample

	// special "cheat" mode for LiVES
	prefs->reorder_leave_back=TRUE;
	    
	on_resample_vid_ok (NULL,NULL);

	prefs->reorder_leave_back=FALSE;
	cfile->undo_action=UNDO_ATOMIC_RESAMPLE_RESIZE;
	if (mainw->error) {
	  on_undo_activate(NULL,NULL);
	  return FALSE;
	}

	video_resized=TRUE;
	video_resampled=TRUE;
      }
      else {
	// fewer frames
	// do resample *with* resize
	cfile->ohsize=cfile->hsize;
	cfile->ovsize=cfile->vsize;
	cfile->undo1_dbl=fps;

	// another special "cheat" mode for LiVES
	prefs->reorder_width=width;
	prefs->reorder_height=height;

	on_resample_vid_ok (NULL,NULL);

	prefs->reorder_width=prefs->reorder_height=0;
	cfile->undo_action=UNDO_ATOMIC_RESAMPLE_RESIZE;
	cfile->hsize=width;
	cfile->vsize=height;
	if (mainw->error) {
	  on_undo_activate(NULL,NULL);
	  return FALSE;
	}
       	video_resampled=TRUE;
	video_resized=TRUE;
      }
    }
    else {
      //////////////////////////////////////////////////////////////////////////////////
      cfile->undo1_dbl=fps;
      cfile->undo_start=1;
      cfile->undo_end=cfile->frames;
      
      on_resample_vid_ok (NULL,NULL);
      
      if (audio_resampled) cfile->undo_action=UNDO_ATOMIC_RESAMPLE_RESIZE;
      if (mainw->error) {
	on_undo_activate(NULL,NULL);
	return FALSE;
      }
      //////////////////////////////////////////////////////////////////////////////////////
      video_resampled=TRUE;
    }
  }
  else {
    // NO FPS CHANGE
    if ((width!=cfile->hsize||height!=cfile->vsize)&&width*height>0) {
      // no fps change - just a normal resize
      cfile->undo_start=1;
      cfile->undo_end=cfile->frames;

      com=g_strdup_printf ("smogrify resize_all %s %d %d %d",cfile->handle,cfile->frames,width,height);
    
      cfile->progress_start=1;
      cfile->progress_end=cfile->frames;

      cfile->ohsize=cfile->hsize;
      cfile->ovsize=cfile->vsize;
      
      unlink(cfile->info_file);
      system(com);
      g_free (com);
      if (!do_progress_dialog(TRUE,TRUE,_ ("Resizing"))) {
	cfile->hsize=width;
	cfile->vsize=height;
	if (audio_resampled) cfile->undo_action=UNDO_ATOMIC_RESAMPLE_RESIZE;
	else {
	  cfile->undo_action=UNDO_RESIZABLE;
	  set_undoable (_ ("Resize"),TRUE);
	}
	on_undo_activate (NULL,NULL);
	return FALSE;
      }
      cfile->hsize=width;
      cfile->vsize=height;
      if (audio_resampled) cfile->undo_action=UNDO_ATOMIC_RESAMPLE_RESIZE;
      else {
	cfile->undo_action=UNDO_RESIZABLE;
	set_undoable (_ ("Resize"),TRUE);
      }
      video_resized=TRUE;
      switch_to_file ((mainw->current_file=0),current_file);
    }
  }

  if (cfile->undo_action==UNDO_ATOMIC_RESAMPLE_RESIZE) {
    // just in case we missed anything...
    
    set_undoable (_ ("Resample/Resize"),TRUE);
    if (!audio_resampled) {
      cfile->undo1_int=arate;
      cfile->undo2_int=cfile->achans;
      cfile->undo3_int=cfile->asampsize;
      cfile->undo4_int=cfile->arps;
      cfile->undo1_uint=cfile->signed_endian;
    }
    if (!video_resampled) {
      cfile->undo1_dbl=fps;
    }
    if (!video_resized) {
      cfile->ohsize=cfile->hsize;
      cfile->ovsize=cfile->vsize;
    }
    cfile->undo_start=1;
    cfile->undo_end=frames;
  }
  return TRUE;
}


//////////////////////////////////////////////////////////////////

event_list_t *
quantise_events (event_list_t *in_list, gdouble qfps, gboolean allow_gap) { 
  // new style event system, now we quantise from event_list_t *in_list to *out_list with period tl/U_SEC

  // the timecode of the midpoint of our last frame events will match as near as possible the old length
  // but out_list will have regular period of tl microseconds

  // for optimal resampling we compare the midpoints of each frame

  // only FRAME events are moved, other event types retain the same timecodes

  // TODO ** - throw memory errors here

  event_list_t *out_list;
  event_t *event,*last_frame_event,*penultimate_frame_event,*next_frame_event,*shortcut=NULL;
  weed_timecode_t out_tc=0,in_tc=-1,nearest_tc=LONG_MAX;
  gboolean is_first=TRUE;
  weed_timecode_t tc_end,tp;
  int *out_clips=NULL,*out_frames=NULL;
  int numframes=0;
  weed_timecode_t tl=q_dbl(1./qfps,qfps);
  int error;

  if (in_list==NULL) return NULL;

  if ((out_list=g_malloc (sizeof(event_list_t)))==NULL) return NULL;

  out_list->first=out_list->last=NULL;

  last_frame_event=get_last_frame_event(in_list);

  tc_end=get_event_timecode (last_frame_event->event);

  penultimate_frame_event=get_prev_frame_event(last_frame_event);

  // tp is the duration of the last frame
  if (penultimate_frame_event!=NULL) {
    tp=get_event_timecode (penultimate_frame_event->event);
    tp=tc_end-tp;
  }
  else {
    // only one event, use cfile->fps
    tp=(weed_timecode_t)(U_SEC/cfile->fps);
  }

  tc_end+=tp;

  // our "slot" is the middle of the out frame
  // length of old clip is assumed as last timecode + (last timecode - previous timecode)

  // we fill out frames until the new slot>=old length

  event=in_list->first;

  while ((out_tc+tl/2)<=tc_end) {
    // walk list of in events

    while (event!=NULL&&get_event_hint(event->event)!=LIVES_EVENT_HINT_FRAME) {
      // copy non-FRAME events
      event_copy_and_append (event,out_list);
      is_first=FALSE;
      event=event->next;
    }

    // now we are dealing with a FRAME event
    if (event!=NULL) {
      in_tc=get_event_timecode (event->event);
      if ((next_frame_event=get_next_frame_event(event))!=NULL) {
	tp=get_event_timecode (next_frame_event->event);
	tp-=in_tc;
      }
      else {
	// only one event, use cfile->fps
	tp=(weed_timecode_t)(U_SEC/cfile->fps);
      }
      // calc mid-point of in frame
      in_tc+=tp/2;
    }

    if (in_tc<=(out_tc+tl/2)&&event!=NULL) {
      // event is before slot, note it and get next event
      if (out_clips!=NULL) g_free(out_clips);
      if (out_frames!=NULL) g_free(out_frames);

      numframes=weed_leaf_num_elements(event->event,"clips");
      out_clips=weed_get_int_array(event->event,"clips",&error);
      out_frames=weed_get_int_array(event->event,"frames",&error);

      nearest_tc=(out_tc+tl/2)-in_tc;
      if (event!=NULL) event=event->next;
      allow_gap=FALSE;
    }   
    else {
      // event is after slot, or we reached the end of in_list
      
      //  in some cases we allow a gap before writing our first FRAME out event
      if (!(is_first&&allow_gap)) {
	if (in_tc-(out_tc+tl/2)<nearest_tc) {
	  if (event!=NULL) {
	    if (out_clips!=NULL) g_free(out_clips);
	    if (out_frames!=NULL) g_free(out_frames);

	    numframes=weed_leaf_num_elements(event->event,"clips");
	    out_clips=weed_get_int_array(event->event,"clips",&error);
	    out_frames=weed_get_int_array(event->event,"frames",&error);
	  }
	}
	if (out_clips!=NULL) {
	  // TODO - memcheck
	  insert_frame_event_at (out_list,out_tc,numframes,out_clips,out_frames,&shortcut);
	  nearest_tc=LONG_MAX;
	  if (is_first) {
	    out_list->first=out_list->last;
	    is_first=FALSE;
	  }
	}
      }
      out_tc+=tl;
      out_tc=q_gint64(out_tc,qfps);
    }
  }

  if (event!=NULL&&get_event_hint(event->event)==LIVES_EVENT_HINT_FRAME) event=event->next;

  while (event!=NULL&&get_event_hint(event->event)!=LIVES_EVENT_HINT_FRAME) {
    // copy remaining non-FRAME events
    event_copy_and_append (event,out_list);
    event=event->next;
  }
  
  if (get_first_frame_event(out_list)==NULL) {
    // make sure we have at least one frame
    if ((event=get_last_frame_event(in_list))!=NULL) {

      if (out_clips!=NULL) g_free(out_clips);
      if (out_frames!=NULL) g_free(out_frames);
    
      numframes=weed_leaf_num_elements(event->event,"clips");
      out_clips=weed_get_int_array(event->event,"clips",&error);
      out_frames=weed_get_int_array(event->event,"frames",&error);

      insert_frame_event_at (out_list,0.,numframes,out_clips,out_frames,NULL);
      if (out_list->first==NULL) out_list->first=out_list->last;
    }
  }

  if (out_clips!=NULL) g_free(out_clips);
  if (out_frames!=NULL) g_free(out_frames);

  return out_list;
}


//////////////////////////////////////////////////////////////////


void
on_reorder_activate                (GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
  gchar *msg;

  if (menuitem!=NULL) {
    cfile->undo_action=UNDO_REORDER;
    if (mainw->current_file>0) {
      d_print(_ ("Reordering frames..."));
    }
    else {
      d_print(_ ("Resampling clipboard video..."));
    }
  }
  cfile->old_frames=cfile->frames;

  // if we have variable fps, we will instead resample to fixed framerate; this will also reorder
  if (cfile->variable_fps) {
    cfile->undo1_dbl=cfile->fps;
    cfile->undo_start=1;
    cfile->undo_end=cfile->frames;
    on_resample_vid_ok (NULL,NULL);
    return;
  }


  //  we  do the reorder in reorder_frames()
  // this will clear event_list and set it in event_list_back
  if ((cfile->frames=reorder_frames())<0) {
    // reordering error
    if (!(cfile->undo_action==UNDO_RESAMPLE)) {
     cfile->frames=-cfile->frames;
    }
    return;
  }

  if (cfile->start>cfile->frames) {
    cfile->start=cfile->frames;
  }

  if (cfile->end>cfile->frames) {
    cfile->end=cfile->frames;
  }

  gtk_widget_set_sensitive(mainw->reorder,FALSE);

  cfile->event_list=NULL;
  cfile->next_event=NULL;
  
  cfile->undo_action=UNDO_REORDER;
  set_undoable ("Reorder",TRUE);
  cfile->changed=TRUE;

  switch_to_file(mainw->current_file,mainw->current_file);
  if (mainw->current_file>0) {
    d_print(_ ("done.\n"));
    msg=g_strdup_printf(_ ("Length of video is now %d frames.\n"),cfile->frames);
  }
  else {
    msg=g_strdup_printf(_ ("Clipboard was resampled to %d frames.\n"),cfile->frames);
  }

  d_print(msg);
  g_free(msg);

}




void
on_resample_audio_activate (GtkMenuItem     *menuitem,
			    gpointer         user_data)
{
   // show the playback rate - real audio rate is cfile->arps
  mainw->fx1_val=cfile->arate;
  mainw->fx2_val=cfile->achans;
  mainw->fx3_val=cfile->asampsize;
  resaudw=create_resaudw(1);
  gtk_widget_show (resaudw->dialog);

}

void
on_resaudio_ok_clicked                      (GtkButton *button,
					     GtkEntry *entry)
{
  gchar *com,*msg;  
  gint arate,achans,asampsize,arps;
  int asigned=1,aendian=1;
  gint cur_signed,cur_endian;
  gboolean noswitch=mainw->noswitch;

  if (!(button==NULL)) {
    arps=arate=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_arate)));
    achans=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_achans)));
    asampsize=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_asamps)));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->rb_unsigned))==TRUE) {
      asigned=0;
    }
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->rb_bigend))==TRUE) {
      aendian=0;
    }

    gtk_widget_destroy (resaudw->dialog);
    mainw->noswitch=TRUE;
    while (g_main_context_iteration(NULL,FALSE));
    mainw->noswitch=noswitch;
    g_free (resaudw);
    
    if (arate<=0) {
      do_error_dialog (_ ("\n\nNew rate must be greater than 0\n"));
      return;
    }
  }
  else {
    // called from on_redo or other places
    arate=arps=cfile->undo1_int;
    achans=cfile->undo2_int;
    asampsize=cfile->undo3_int;
    asigned=!(cfile->undo1_uint&AFORM_UNSIGNED);
    aendian=!(cfile->undo1_uint&AFORM_BIG_ENDIAN);
  }

  // store old values for undo/redo
  cfile->undo1_int=cfile->arate;
  cfile->undo2_int=cfile->achans;
  cfile->undo3_int=cfile->asampsize;
  cfile->undo4_int=cfile->arps;
  cfile->undo1_uint=cfile->signed_endian;

  cur_signed=!(cfile->signed_endian&AFORM_UNSIGNED);
  cur_endian=!(cfile->signed_endian&AFORM_BIG_ENDIAN);

  if (!(arate==cfile->arate&&arps==cfile->arps&&achans==cfile->achans&&asampsize==cfile->asampsize&&asigned==cur_signed&&aendian==cur_endian)) {
    if (cfile->arps!=cfile->arate) {
      gdouble audio_stretch=(gdouble)cfile->arps/(gdouble)cfile->arate;
     // pb rate != real rate - stretch to pb rate and resample 
      unlink (cfile->info_file);
      com=g_strdup_printf ("smogrify resample_audio %s %d %d %d %d %d %d %d %d %d %d %.4f",cfile->handle,cfile->arps,cfile->achans,cfile->asampsize,cur_signed,cur_endian,arps,cfile->achans,cfile->asampsize,cur_signed,cur_endian,audio_stretch);
      system (com);
      do_progress_dialog (TRUE,FALSE,_ ("Resampling audio")); // TODO - allow cancel ??
      g_free (com);
      cfile->arate=cfile->arps=arps;
    }

    if (cfile->arate!=arate) {
      unlink (cfile->info_file);
      com=g_strdup_printf ("smogrify resample_audio %s %d %d %d %d %d %d %d %d %d %d",cfile->handle,cfile->arps,cfile->achans,cfile->asampsize,cur_signed,cur_endian,arps,achans,asampsize,asigned,aendian);
      system (com);
      do_progress_dialog (TRUE,FALSE,_ ("Resampling audio"));
      g_free (com);
    }
  }

  cfile->arate=arate;
  cfile->achans=achans;
  cfile->asampsize=asampsize;
  cfile->arps=arps;
  cfile->signed_endian=get_signed_endian (asigned, aendian);
  cfile->changed=TRUE;

  cfile->undo_action=UNDO_AUDIO_RESAMPLE;
  mainw->error=FALSE;
  reget_afilesize(cfile);

  if (cfile->afilesize==0) {
    do_error_dialog (_ ("LiVES was unable to resample the audio as requested.\n"));
    on_undo_activate (NULL,NULL);
    set_undoable ("Resample Audio",FALSE);
    mainw->error=TRUE;
    return;
  }
  set_undoable ("Resample Audio",!prefs->conserve_space);

  switch_to_file(mainw->current_file,mainw->current_file);

  msg=g_strdup_printf (_ ("Audio was resampled to %d Hz, %d channels, %d bit"),arate,achans,asampsize);
  d_print (msg);
  g_free (msg);
  if (cur_signed!=asigned) {
    if (asigned==1) {
      d_print (_ (", signed"));
    }
    else {
      d_print (_ (", unsigned"));
    }
  }
  if (cur_endian!=aendian) {
    if (aendian==1) {
      d_print (_ (", little-endian"));
    }
    else {
      d_print (_ (", big-endian"));
    }
  }
  d_print ("\n");
}


void 
on_resaudw_asamps_changed (GtkWidget irrelevant, gpointer rubbish) {
  if (atoi (gtk_entry_get_text (GTK_ENTRY (resaudw->entry_asamps)))==8) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_unsigned), TRUE);
    gtk_widget_set_sensitive (resaudw->rb_signed,FALSE);
    gtk_widget_set_sensitive (resaudw->rb_unsigned,FALSE);
    gtk_widget_set_sensitive (resaudw->rb_bigend,FALSE);
    gtk_widget_set_sensitive (resaudw->rb_littleend,FALSE);
  }
  else {
    if (cfile->signed_endian&AFORM_UNSIGNED) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_unsigned), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);
    }
    gtk_widget_set_sensitive (resaudw->rb_signed,TRUE);
    gtk_widget_set_sensitive (resaudw->rb_unsigned,TRUE);
    gtk_widget_set_sensitive (resaudw->rb_bigend,TRUE);
    gtk_widget_set_sensitive (resaudw->rb_littleend,TRUE);
  }
}



void
on_resample_video_activate (GtkMenuItem     *menuitem,
			    gpointer         user_data)
{
  // change speed from the menu
  GtkWidget *resample_vid=create_new_pb_speed(2);
  mainw->fx1_val=cfile->fps;
  gtk_widget_show(resample_vid);
}



void
on_resample_vid_ok (GtkButton *button, GtkEntry *entry)
{
  int i;
  gint old_frames;
  gint ostart=cfile->start;
  gint oend=cfile->end;
  gdouble oundo1_dbl=cfile->undo1_dbl;
  gchar *msg;
  weed_timecode_t in_time=0;
  gchar *test_fps_string1;
  gchar *test_fps_string2;
  gdouble old_fps=cfile->fps;
  gboolean ratio_fps;
  gboolean variable_fps=cfile->variable_fps;
  event_list_t *real_back_list=NULL;

  mainw->error=FALSE;

  if (button!=NULL) {
    gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
    if (mainw->fx1_val==0.) mainw->fx1_val=1.;
  }
  else {
    mainw->fx1_val=cfile->undo1_dbl;
  }

  if (mainw->current_file<0||cfile->frames==0) return;

  if (mainw->fx1_val==cfile->fps&&cfile->event_list==NULL) return;

  real_back_list=cfile->event_list;

  if (cfile->event_list==NULL) {
    for (i=1;i<=cfile->frames;i++) {
      // TODO ** - memcheck !!
      cfile->event_list=append_frame_event (cfile->event_list,in_time,1,&(mainw->current_file),&i);
      in_time+=(weed_timecode_t)(1./cfile->fps*U_SEC+.5);
    }
  }
  cfile->undo1_boolean=cfile->variable_fps;
  cfile->undo1_dbl=cfile->fps;

  if (cfile->event_list_back!=NULL) event_list_free (cfile->event_list_back);
  cfile->event_list_back=cfile->event_list;

  //QUANTISE
  cfile->event_list=quantise_events(cfile->event_list_back,mainw->fx1_val,real_back_list!=NULL);

  if (real_back_list==NULL) event_list_free (cfile->event_list_back);
  cfile->event_list_back=NULL;

  // TODO - end_threaded_dialog

  if (cfile->event_list==NULL) {
    do_memory_error_dialog();
    cfile->event_list=real_back_list;
    cfile->undo1_dbl=oundo1_dbl;
    cfile->variable_fps=variable_fps;
    mainw->error=TRUE;
    return;
  }

  if (mainw->multitrack!=NULL) return;

  test_fps_string1=g_strdup_printf ("%.3f00000",mainw->fx1_val);
  test_fps_string2=g_strdup_printf ("%.8f",mainw->fx1_val);
  
  if (strcmp (test_fps_string1,test_fps_string2)) {
    // got a ratio
    ratio_fps=TRUE;
  }
  else {
    ratio_fps=FALSE;
  }
  g_free (test_fps_string1);
  g_free (test_fps_string2);
  

  // we have now quantised to fixed fps; we have come here from reorder
  if (variable_fps) {
    cfile->variable_fps=FALSE;
  }
  else {
    if (ratio_fps) {
      // got a ratio
      msg=g_strdup_printf(_ ("Resampling video at %.8f frames per second..."),mainw->fx1_val);
    }
    else {
      msg=g_strdup_printf(_ ("Resampling video at %.3f frames per second..."),mainw->fx1_val);
    }
    if (mainw->current_file>0) {
      d_print(msg);
    }
    g_free(msg);
  }
  old_frames=cfile->frames;
  cfile->frames=0;

  // must set these before calling reorder
  cfile->start=(gint)((cfile->start-1.)/old_fps*mainw->fx1_val+1.);
  if ((cfile->end=(gint)((cfile->end*mainw->fx1_val)/old_fps+.49999))<cfile->start) cfile->end=cfile->start;
  cfile->variable_fps=FALSE;

  if (variable_fps) cfile->undo_action=UNDO_REORDER;
  else cfile->undo_action=UNDO_RESAMPLE;

  // REORDER
  // this calls reorder_frames, which sets event_list_back==event_list, and clears event_list
  on_reorder_activate(NULL,NULL);
  cfile->undo_action=UNDO_RESAMPLE;
  
  if (cfile->frames<=0) {
    // reordering error...
    cfile->event_list=real_back_list;
    if (cfile->event_list_back!=NULL) event_list_free (cfile->event_list_back);
    cfile->event_list_back=NULL;
    cfile->frames=old_frames;
    cfile->start=ostart;
    cfile->end=oend;
    cfile->undo1_dbl=oundo1_dbl;
    cfile->variable_fps=variable_fps;
    sensitize();
    mainw->error=TRUE;
    do_error_dialog ("Reordering error !\n");
    return;
  }

  if (cfile->event_list_back!=NULL) event_list_free (cfile->event_list_back);
  cfile->event_list_back=real_back_list;

  cfile->ratio_fps=ratio_fps;
  cfile->fps=mainw->fx1_val;
  cfile->old_frames=old_frames;

  if (variable_fps) set_undoable ("Reorder",TRUE);
  else set_undoable ("Resample",TRUE);

  switch_to_file(mainw->current_file,mainw->current_file);
}



///////// GUI stuff /////////////////////////////////////////////////////



_resaudw *
create_resaudw (gshort type)
{
  // type 1 == resample
  // type 2 == insert silence

  GtkWidget *dialog_vbox15;
  GtkWidget *vbox21;
  GtkWidget *frame6;
  GtkWidget *hbox24;
  GtkWidget *label94;
  GtkWidget *combo_entry2;
  GtkWidget *label95;
  GtkWidget *combo_entry3;
  GtkWidget *label96;
  GtkWidget *combo_entry1;
  GtkWidget *vseparator3;
  GtkWidget *vbox22;
  GtkWidget *radiobutton26;
  GSList *radiobutton26_group = NULL;
  GtkWidget *radiobutton27;
  GtkWidget *vseparator4;
  GtkWidget *vbox23;
  GtkWidget *radiobutton28;
  GSList *radiobutton28_group = NULL;
  GtkWidget *radiobutton29;
  GtkWidget *label92;
  GtkWidget *frame7;
  GtkWidget *hbox25;
  GtkWidget *label98;
  GtkWidget *combo4;
  GtkWidget *label99;
  GtkWidget *combo5;
  GtkWidget *label100;
  GtkWidget *combo6;
  GtkWidget *vseparator5;
  GtkWidget *vbox24;
  GSList *radiobutton30_group = NULL;
  GtkWidget *vseparator6;
  GtkWidget *vbox25;
  GSList *radiobutton32_group = NULL;
  GtkWidget *label93;
  GtkWidget *dialog_action_area15;
  GtkWidget *cancelbutton13;
  GtkWidget *okbutton12;
  GList *channels = NULL;
  GList *sampsize = NULL;
  GList *rate = NULL;
  gchar *tmp;

  _resaudw *resaudw=(_resaudw*)(g_malloc(sizeof(_resaudw)));

  channels = g_list_append (channels, "1");
  channels = g_list_append (channels, "2");

  sampsize = g_list_append (sampsize, "8");
  sampsize = g_list_append (sampsize, "16");

  rate = g_list_append (rate, "5512");
  rate = g_list_append (rate, "8000");
  rate = g_list_append (rate, "11025");
  rate = g_list_append (rate, "22050");
  rate = g_list_append (rate, "32000");
  rate = g_list_append (rate, "44100");
  rate = g_list_append (rate, "48000");
  rate = g_list_append (rate, "88200");
  rate = g_list_append (rate, "96000");
  rate = g_list_append (rate, "128000");


  resaudw->dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (resaudw->dialog), 10);
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (resaudw->dialog), _("LiVES: - Resample Audio"));
  }
  if (type==2) {
    gtk_window_set_title (GTK_WINDOW (resaudw->dialog), _("LiVES: - Insert Silence"));
  }
  gtk_window_set_position (GTK_WINDOW (resaudw->dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (resaudw->dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(resaudw->dialog),GTK_WINDOW(mainw->LiVES));

  dialog_vbox15 = GTK_DIALOG (resaudw->dialog)->vbox;
  gtk_widget_show (dialog_vbox15);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (resaudw->dialog, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (resaudw->dialog, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }


  vbox21 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox21);
  gtk_box_pack_start (GTK_BOX (dialog_vbox15), vbox21, TRUE, TRUE, 0);

  frame6 = gtk_frame_new (NULL);
 
  if (type==1) {

    gtk_widget_show (frame6);
    gtk_box_pack_start (GTK_BOX (vbox21), frame6, TRUE, TRUE, 0);
    
    if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
      gtk_widget_modify_bg (frame6, GTK_STATE_NORMAL, &palette->normal_back);
    }
    else {
      gtk_widget_modify_bg (frame6, GTK_STATE_NORMAL, &palette->menu_and_bars);
    }
    
    hbox24 = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox24);
    gtk_container_add (GTK_CONTAINER (frame6), hbox24);
    gtk_container_set_border_width (GTK_CONTAINER (hbox24), 10);
    
    label94 = gtk_label_new (_("Rate (Hz) "));
    gtk_widget_show (label94);
    gtk_box_pack_start (GTK_BOX (hbox24), label94, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_LEFT);
    
    combo_entry2 = gtk_entry_new();
    gtk_widget_show (combo_entry2);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry2, TRUE, TRUE, 1);

    gtk_editable_set_editable (GTK_EDITABLE (combo_entry2), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry2), 8);
    
    tmp=g_strdup_printf ("%d",(gint)mainw->fx1_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry2),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry2, GTK_CAN_FOCUS);
    
    label95 = gtk_label_new (_("         Channels "));
    gtk_widget_show (label95);
    gtk_box_pack_start (GTK_BOX (hbox24), label95, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label95), GTK_JUSTIFY_LEFT);
    
    combo_entry3 = gtk_entry_new();
    gtk_widget_show (combo_entry3);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry3, TRUE, TRUE, 1);
    
    gtk_editable_set_editable (GTK_EDITABLE (combo_entry3), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry3), 3);
    
    tmp=g_strdup_printf ("%d",(gint)mainw->fx2_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry3),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry3, GTK_CAN_FOCUS);
    
    label96 = gtk_label_new (_("        Sample Size "));
    gtk_widget_show (label96);
    gtk_box_pack_start (GTK_BOX (hbox24), label96, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label96), GTK_JUSTIFY_LEFT);

    combo_entry1 = gtk_entry_new();
    gtk_widget_show (combo_entry1);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry1, TRUE, TRUE, 1);

    gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry1), 3);

    tmp=g_strdup_printf ("%d",(gint)mainw->fx3_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry1),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry1, GTK_CAN_FOCUS);

    vseparator3 = gtk_vseparator_new ();
    gtk_widget_show (vseparator3);
    gtk_box_pack_start (GTK_BOX (hbox24), vseparator3, FALSE, FALSE, 12);

    vbox22 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox22);
    gtk_box_pack_start (GTK_BOX (hbox24), vbox22, TRUE, TRUE, 0);

    radiobutton26 = gtk_radio_button_new_with_mnemonic (NULL, _("Signed"));
    gtk_widget_show (radiobutton26);
    gtk_box_pack_start (GTK_BOX (vbox22), radiobutton26, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton26), radiobutton26_group);
    radiobutton26_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton26));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton26), TRUE);

    radiobutton27 = gtk_radio_button_new_with_mnemonic (NULL, _("Unsigned"));
    gtk_widget_show (radiobutton27);
    gtk_box_pack_start (GTK_BOX (vbox22), radiobutton27, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton27), radiobutton26_group);
    radiobutton26_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton27));

    if ((gint)mainw->fx4_val&AFORM_UNSIGNED) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton27), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton26), TRUE);
    }

    gtk_widget_set_sensitive (radiobutton26, FALSE);
    gtk_widget_set_sensitive (radiobutton27, FALSE);

    vseparator4 = gtk_vseparator_new ();
    gtk_widget_show (vseparator4);
    gtk_box_pack_start (GTK_BOX (hbox24), vseparator4, FALSE, FALSE, 12);
    
    vbox23 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox23);
    gtk_box_pack_start (GTK_BOX (hbox24), vbox23, TRUE, TRUE, 0);

    radiobutton28 = gtk_radio_button_new_with_mnemonic (NULL, _("Little Endian"));
    gtk_widget_show (radiobutton28);
    gtk_box_pack_start (GTK_BOX (vbox23), radiobutton28, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton28), radiobutton28_group);
    radiobutton28_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton28));
    
    radiobutton29 = gtk_radio_button_new_with_mnemonic (NULL, _("Big Endian"));
    gtk_widget_show (radiobutton29);
    gtk_box_pack_start (GTK_BOX (vbox23), radiobutton29, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton29), radiobutton28_group);
    radiobutton28_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton29));
    
    if ((gint)mainw->fx4_val&AFORM_BIG_ENDIAN) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton29), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton28), TRUE);
    }

    gtk_widget_set_sensitive (radiobutton29, FALSE);
    gtk_widget_set_sensitive (radiobutton28, FALSE);

    label92 = gtk_label_new (_("Current"));
    
    gtk_widget_show (label92);
    gtk_frame_set_label_widget (GTK_FRAME (frame6), label92);
    gtk_label_set_justify (GTK_LABEL (label92), GTK_JUSTIFY_LEFT);
    
  }

  frame7 = gtk_frame_new (NULL);
  gtk_widget_show (frame7);
  gtk_box_pack_start (GTK_BOX (vbox21), frame7, TRUE, TRUE, 0);

  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  hbox25 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox25);
  gtk_container_add (GTK_CONTAINER (frame7), hbox25);
  gtk_container_set_border_width (GTK_CONTAINER (hbox25), 10);

  label98 = gtk_label_new (_("Rate (Hz) "));
  gtk_widget_show (label98);
  gtk_box_pack_start (GTK_BOX (hbox25), label98, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);

  combo4 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo4), rate);
  gtk_widget_show (combo4);
  gtk_box_pack_start (GTK_BOX (hbox25), combo4, TRUE, TRUE, 1);

  resaudw->entry_arate = GTK_COMBO (combo4)->entry;
  gtk_widget_show (resaudw->entry_arate);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_arate), 8);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx1_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_arate),tmp);
  g_free (tmp);

  label99 = gtk_label_new (_("    Channels "));
  gtk_widget_show (label99);
  gtk_box_pack_start (GTK_BOX (hbox25), label99, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);

  combo5 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo5), channels);
  gtk_widget_show (combo5);
  gtk_box_pack_start (GTK_BOX (hbox25), combo5, FALSE, FALSE, 0);

  resaudw->entry_achans = GTK_COMBO (combo5)->entry;
  gtk_widget_show (resaudw->entry_achans);
  gtk_editable_set_editable (GTK_EDITABLE (resaudw->entry_achans), FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_achans), 3);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx2_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_achans),tmp);
  g_free (tmp);

  label100 = gtk_label_new (_("    Sample Size "));
  gtk_widget_show (label100);
  gtk_box_pack_start (GTK_BOX (hbox25), label100, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);

  combo6 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo6), sampsize);
  gtk_widget_show (combo6);
  gtk_box_pack_start (GTK_BOX (hbox25), combo6, TRUE, TRUE, 0);

  resaudw->entry_asamps = GTK_COMBO (combo6)->entry;
  gtk_widget_show (resaudw->entry_asamps);
  gtk_entry_set_max_length (GTK_ENTRY (resaudw->entry_asamps), 2);
  gtk_editable_set_editable (GTK_EDITABLE (resaudw->entry_asamps), FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_asamps), 3);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx3_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_asamps),tmp);
  g_free (tmp);

  vseparator5 = gtk_vseparator_new ();
  gtk_widget_show (vseparator5);
  gtk_box_pack_start (GTK_BOX (hbox25), vseparator5, FALSE, FALSE, 12);

  vbox24 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox24);
  gtk_box_pack_start (GTK_BOX (hbox25), vbox24, TRUE, TRUE, 0);

  resaudw->rb_signed = gtk_radio_button_new_with_mnemonic (NULL, _("Signed"));
  gtk_widget_show (resaudw->rb_signed);
  gtk_box_pack_start (GTK_BOX (vbox24), resaudw->rb_signed, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_signed), radiobutton30_group);
  radiobutton30_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_signed));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);

  resaudw->rb_unsigned = gtk_radio_button_new_with_mnemonic (NULL, _("Unsigned"));
  gtk_widget_show (resaudw->rb_unsigned);
  gtk_box_pack_start (GTK_BOX (vbox24), resaudw->rb_unsigned, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_unsigned), radiobutton30_group);
  radiobutton30_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_unsigned));

  if ((gint)mainw->fx4_val&AFORM_UNSIGNED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_unsigned), TRUE);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);
  }

  if (cfile->asampsize==8) {
    gtk_widget_set_sensitive (resaudw->rb_signed, FALSE);
    gtk_widget_set_sensitive (resaudw->rb_unsigned, FALSE);
  }

  vseparator6 = gtk_vseparator_new ();
  gtk_widget_show (vseparator6);
  gtk_box_pack_start (GTK_BOX (hbox25), vseparator6, FALSE, FALSE, 12);

  vbox25 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox25);
  gtk_box_pack_start (GTK_BOX (hbox25), vbox25, TRUE, TRUE, 0);

  resaudw->rb_littleend = gtk_radio_button_new_with_mnemonic (NULL, _("Little Endian"));
  gtk_widget_show (resaudw->rb_littleend);
  gtk_box_pack_start (GTK_BOX (vbox25), resaudw->rb_littleend, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_littleend), radiobutton32_group);
  radiobutton32_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_littleend));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_littleend), TRUE);

  resaudw->rb_bigend = gtk_radio_button_new_with_mnemonic (NULL, _("Big Endian"));
  gtk_widget_show (resaudw->rb_bigend);
  gtk_box_pack_start (GTK_BOX (vbox25), resaudw->rb_bigend, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_bigend), radiobutton32_group);
  radiobutton32_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_bigend));

  if ((gint)mainw->fx4_val&AFORM_BIG_ENDIAN) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_bigend), TRUE);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_littleend), TRUE);
  }

  if (cfile->asampsize==8) {
    gtk_widget_set_sensitive (resaudw->rb_littleend, FALSE);
    gtk_widget_set_sensitive (resaudw->rb_bigend, FALSE);
    }

  label93 = gtk_label_new (_("New"));
  gtk_widget_show (label93);
  gtk_frame_set_label_widget (GTK_FRAME (frame7), label93);
  gtk_label_set_justify (GTK_LABEL (label93), GTK_JUSTIFY_LEFT);

  dialog_action_area15 = GTK_DIALOG (resaudw->dialog)->action_area;
  gtk_widget_show (dialog_action_area15);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area15), GTK_BUTTONBOX_END);

  cancelbutton13 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton13);
  gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), cancelbutton13, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton13, GTK_CAN_DEFAULT);

  okbutton12 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton12);
  gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), okbutton12, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton12, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton12);

  g_signal_connect (GTK_OBJECT(resaudw->entry_asamps), "changed",
		    G_CALLBACK (on_resaudw_asamps_changed),
		    NULL);

  g_signal_connect (GTK_OBJECT (cancelbutton13), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    resaudw);

  if (type==1) {
    g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
		      G_CALLBACK (on_resaudio_ok_clicked),
		      NULL);
  }
  if (type==2) {
    g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
		      G_CALLBACK (on_ins_silence_details_clicked),
		      NULL);
  }

  g_signal_connect (GTK_OBJECT (resaudw->dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  g_list_free (channels);
  g_list_free (sampsize);
  g_list_free (rate);

  return resaudw;
}



GtkWidget*
create_new_pb_speed (gshort type)
{
  GtkWidget *new_pb_speed;
  GtkWidget *dialog_vbox6;
  GtkWidget *vbox6;
  GtkWidget *pb_speed_text;
  GtkObject *spinbutton_pb_speed_adj;
  GtkWidget *spinbutton_pb_speed;
  GtkWidget *dialog_action_area6;
  GtkWidget *cancelbutton4;
  GtkWidget *change_pb_ok;
  GtkWidget *change_audio_speed;
  gchar label_text[256];

  new_pb_speed = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (new_pb_speed), 10);
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (new_pb_speed), _("LiVES: - Change playback speed"));
  }
  else if (type==2) {
    gtk_window_set_title (GTK_WINDOW (new_pb_speed), _("LiVES: - Resample Video"));
  }
  gtk_window_set_position (GTK_WINDOW (new_pb_speed), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (new_pb_speed), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (new_pb_speed), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(new_pb_speed),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(new_pb_speed, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(new_pb_speed),FALSE);
  }

  dialog_vbox6 = GTK_DIALOG (new_pb_speed)->vbox;
  gtk_widget_show (dialog_vbox6);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox6), vbox6, TRUE, TRUE, 0);

  if (type==1) {
    g_snprintf(label_text,256,_("\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the desired playback speed\nin frames per second"),cfile->fps);
  }
  else if (type==2) {
    g_snprintf(label_text,256,_("\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the resampled rate\nin frames per second"),cfile->fps);
  }
  pb_speed_text = gtk_label_new (label_text);
  gtk_widget_show (pb_speed_text);
  gtk_box_pack_start (GTK_BOX (vbox6), pb_speed_text, TRUE, TRUE, 0);
  gtk_widget_modify_fg(pb_speed_text, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton_pb_speed_adj = gtk_adjustment_new (cfile->fps, 1, FPS_MAX, 0.01, .1, .5);
  spinbutton_pb_speed = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_pb_speed_adj), 1, 3);
  gtk_widget_show (spinbutton_pb_speed);
  gtk_box_pack_start (GTK_BOX (vbox6), spinbutton_pb_speed, FALSE, FALSE, 8);
  gtk_widget_set_size_request (spinbutton_pb_speed, 10, -1);
  gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton_pb_speed)->entry)), TRUE);

  change_audio_speed = gtk_check_button_new_with_mnemonic (_("Change the audio speed as well"));
  if (type==1&&cfile->achans>0) {
    gtk_widget_show (change_audio_speed);
  }
  gtk_box_pack_start (GTK_BOX (vbox6), change_audio_speed, FALSE, FALSE, 0);

  dialog_action_area6 = GTK_DIALOG (new_pb_speed)->action_area;
  gtk_widget_show (dialog_action_area6);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area6), GTK_BUTTONBOX_END);

  cancelbutton4 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton4);
  gtk_dialog_add_action_widget (GTK_DIALOG (new_pb_speed), cancelbutton4, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton4, GTK_CAN_DEFAULT);

  change_pb_ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (change_pb_ok);
  gtk_dialog_add_action_widget (GTK_DIALOG (new_pb_speed), change_pb_ok, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (change_pb_ok, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (change_pb_ok);

  g_signal_connect (GTK_OBJECT (change_audio_speed), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (cancelbutton4), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    NULL);
  if (type==1) {
    g_signal_connect (GTK_OBJECT (change_pb_ok), "clicked",
		      G_CALLBACK (on_change_speed_ok_clicked),
		      NULL);
  }
  else if (type==2) {
    g_signal_connect (GTK_OBJECT (change_pb_ok), "clicked",
		      G_CALLBACK (on_resample_vid_ok),
		      NULL);
  }
  g_signal_connect_after (GTK_OBJECT (spinbutton_pb_speed), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (1));


  return new_pb_speed;
}



void
on_change_speed_activate                (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  // change speed from the menu
  GtkWidget *new_pb_speed=create_new_pb_speed(1);
  mainw->fx1_bool=FALSE;
  mainw->fx1_val=cfile->fps;
  gtk_widget_show(new_pb_speed);
}



void
on_change_speed_ok_clicked                (GtkButton *button,
					   gpointer         user_data)
{
  gdouble arate=cfile->arate/cfile->fps;
  gchar *msg;

  // change playback rate
  if (!(button==NULL)) {
    gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  }
  else {
    mainw->fx1_bool=!(cfile->undo1_int==cfile->arate);
    mainw->fx1_val=cfile->undo1_dbl;
  }

  set_undoable ("Speed Change",TRUE);
  cfile->undo1_dbl=cfile->fps;
  cfile->undo1_int=cfile->arate;
  cfile->undo_action=UNDO_CHANGE_SPEED;

  if (mainw->fx1_val==0.) mainw->fx1_val=1.;
  cfile->pb_fps=cfile->fps=mainw->fx1_val;
  if (mainw->fx1_bool==TRUE) {
    cfile->arate=(gint)(arate*cfile->fps+.5);
    msg=g_strdup_printf (_ ("Changed playback speed to %.3f frames per second and audio to %d Hz.\n"),cfile->fps,cfile->arate);
  }
  else {
    msg=g_strdup_printf (_ ("Changed playback speed to %.3f frames per second.\n"),cfile->fps);
  }
  d_print (msg);
  g_free (msg);

  // resize the timer bars, etc
  set_sel_label();
  get_play_times();
}






gint
reorder_frames(void) {
  int new_frames=cfile->old_frames;
  gchar **array;
  gchar *com;

  if (prefs->reorder_width*prefs->reorder_height==0) com=g_strdup_printf("smogrify reorder %s %d 0 0 %d",cfile->handle,!mainw->endian,prefs->reorder_leave_back);
  else com=g_strdup_printf("smogrify reorder %s %d %d %d 0",cfile->handle,!mainw->endian,prefs->reorder_width,prefs->reorder_height);

  cfile->progress_start=1;
  cfile->progress_end=save_event_frames();  // we convert cfile->event_list to a block and save it

  cfile->next_event=NULL;
  if (cfile->event_list!=NULL) {
    if (cfile->event_list_back!=NULL) event_list_free (cfile->event_list_back);
    cfile->event_list_back=cfile->event_list;
    cfile->event_list=NULL;
  }

  unlink(cfile->info_file);
  // TODO - maybe use auto_dialog
  system(com);
  if (cfile->undo_action==UNDO_RESAMPLE) {
    if (mainw->current_file>0) {
      do_progress_dialog(TRUE,FALSE,_ ("Resampling video"));
    }
    else {
      do_progress_dialog(TRUE,FALSE,_ ("Resampling clipboard video"));
    }
  }
  else {
    do_progress_dialog(TRUE,FALSE,_ ("Reordering frames"));
  }
  g_free(com);
  
  if (mainw->error==TRUE) {
    do_error_dialog (_ ("\n\nLiVES was unable to reorder the frames."));
    deorder_frames(new_frames);
    new_frames=-new_frames;
  }
  else {
    array=g_strsplit(mainw->msg,"|",2);
  
    new_frames=atoi(array[1]);
    g_strfreev(array);
    
    if (cfile->frames>new_frames) {
      new_frames=cfile->frames;
    }
  }
  return new_frames;
}



gint
deorder_frames(gint old_frames) {
  gchar *com;
  weed_timecode_t time_start;
  gint perf_start,perf_end;

  if (cfile->event_list!=NULL) return cfile->frames;

  cfile->event_list=cfile->event_list_back;
  cfile->event_list_back=NULL;

  if (cfile->event_list==NULL) {
    perf_start=1;
    perf_end=old_frames;
  }
  else {
    time_start=get_event_timecode (cfile->event_list->first->event);

    perf_start=(gint)(cfile->fps*(gdouble)time_start/U_SEC)+1;
    perf_end=perf_start+count_events (cfile->event_list,FALSE)-1;
  }
  com=g_strdup_printf("smogrify deorder %s %d %d %d",cfile->handle,perf_start,perf_end,old_frames);

  unlink(cfile->info_file);
  system(com);
  do_progress_dialog(TRUE,FALSE,_ ("Deordering frames"));
  g_free(com);

  return old_frames;
}



