
// rte_window.c
// LiVES (lives-exe)
// (c) G. Finch 2005
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details


#include <errno.h>

#include "main.h"

#include "support.h"
#include "rte_window.h"
#include "effects.h"
#include "effects-weed.h"
#include "paramwindow.h"

#define RTE_INFO_WIDTH 350
#define RTE_INFO_HEIGHT 200

static GtkWidget **key_checks;
static GtkWidget **key_grabs;
static GtkWidget **mode_radios;
static GtkWidget **combo_entries;
static GtkWidget *dummy_radio;
static GtkWidget **nlabels;
static GtkWidget **type_labels;
static GtkWidget **param_buttons;
static GtkWidget **clear_buttons;
static GtkWidget **info_buttons;
static GtkWidget *clear_all_button;
static GtkWidget *save_keymap_button;
static GtkWidget *load_keymap_button;

static gulong *ch_fns;
static gulong *gr_fns;
static gulong *mode_ra_fns;

static gint keyw=-1,modew=-1;

static GList *hash_list;
static GList *name_list;
static GList *name_type_list;


//////////////////////////////////////////////////////////////////////////////

void type_label_set_text (gint key, gint mode) {
  int modes=rte_getmodespk();
  int idx=key*modes+mode;
  gchar *type=rte_keymode_get_type(key+1,mode);

  if (strlen(type)) {
    gtk_label_set_text (GTK_LABEL(type_labels[idx]),g_strdup_printf(_("Type: %s"),type));
    gtk_widget_set_sensitive(info_buttons[idx],TRUE);
    gtk_widget_set_sensitive(clear_buttons[idx],TRUE);
    gtk_widget_set_sensitive(mode_radios[idx],TRUE);
    gtk_widget_set_sensitive(nlabels[idx],TRUE);
    gtk_widget_set_sensitive(type_labels[idx],TRUE);
  }
  else {
    gtk_widget_set_sensitive(info_buttons[idx],FALSE);
    gtk_widget_set_sensitive(clear_buttons[idx],FALSE);
    gtk_widget_set_sensitive(mode_radios[idx],FALSE);
    gtk_widget_set_sensitive(nlabels[idx],FALSE);
    gtk_widget_set_sensitive(type_labels[idx],FALSE);
  }
  g_free(type);
  gtk_widget_set_sensitive(param_buttons[idx],(rte_keymode_get_instance(key+1,mode)!=NULL));
}


void 
on_rtei_ok_clicked (GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

gboolean 
on_clear_all_clicked (GtkButton *button, gpointer user_data) {
  int modes=rte_getmodespk();
  int i,j;

  // prompt for "are you sure ?"
  if (user_data!=NULL) if (!do_warning_dialog_with_check_transient((_("\n\nUnbind all effects from all keys/modes.\n\nAre you sure ?\n\n")),0,GTK_WINDOW(rte_window))) return FALSE;

  for (i=0;i<FX_KEYS_PHYSICAL;i++) {
    if (rte_window!=NULL) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(key_checks[i]),FALSE);
    for (j=modes-1;j>=0;j--) {
      weed_delete_effectkey (i+1,j);
      if (rte_window!=NULL) {
	gtk_entry_set_text (GTK_ENTRY(combo_entries[i*modes+j]),g_strdup(""));
	type_label_set_text(i,j);
      }
    }
  }

  if (button!=NULL) gtk_widget_set_sensitive (GTK_WIDGET(button), FALSE);

  return FALSE;
}


gboolean 
on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
  // quick and dirty implementation
  int modes=rte_getmodespk();
  int i,j;
  FILE *kfile;
  gchar *msg;
  GList *list=NULL;
  gboolean update=FALSE;

  const gchar *keymap_file=g_strdup_printf("%s/.lives-dir/default.keymap",capable->home_dir);

  if (button!=NULL) {
    if (!do_warning_dialog_with_check_transient((_("\n\nClick 'OK' to save this keymap as your default\n\n")),0,GTK_WINDOW(rte_window))) return FALSE;

    d_print (g_strdup_printf(_("Saving keymap to %s\n"),keymap_file));
  }
  else {
    update=TRUE;
    list=(GList *)user_data;
    if (list==NULL) return FALSE;
    d_print (g_strdup_printf(_("\nUpdating keymap file %s..."),keymap_file));
  }

  if (!(kfile=fopen(keymap_file,"w"))) {
    if (!update) {
      msg=g_strdup_printf (_("\n\nUnable to write keymap file\n%s\nError code %d\n"),keymap_file,errno);
      do_error_dialog_with_check_transient (msg,FALSE,0,GTK_WINDOW(rte_window));
      g_free (msg);
      d_print (_("failed.\n"));
    }
    return FALSE;
  }

  fputs("LiVES keymap file version 4\n",kfile);

  if (!update) {
    for (i=1;i<=FX_KEYS_PHYSICAL;i++) {
      for (j=0;j<modes;j++) {
	if (rte_keymode_valid(i,j)) {
	  fputs(g_strdup_printf("%d|%s\n",i,make_weed_hashname(rte_keymode_get_filter_idx(i,j))),kfile);
	}
      }
    }
  }
  else {
    for (i=0;i<g_list_length(list);i++) {
      fputs(g_list_nth_data(list,i),kfile);
    }
  }

  fclose (kfile);
  d_print("done.\n");

  return FALSE;
}



gboolean 
on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
  // quick and dirty implementation
  int modes=rte_getmodespk();
  int i;
  FILE *kfile;
  gchar *msg;
  gint key,mode;
  gchar buff[65536];
  size_t linelen;
  gchar *whole=g_strdup (""),*whole2;
  GList *list=NULL,*new_list=NULL;
  gchar *hashname,*hashname_new=NULL;
  gboolean notfound=FALSE;
  gint update=0;
  gchar *line=NULL;

  const gchar *keymap_file=g_strdup_printf("%s/.lives-dir/default.keymap",capable->home_dir);

  d_print (g_strdup_printf(_("Loading default keymap from %s..."),keymap_file));

  if (!(kfile=fopen (keymap_file,"r"))) {
    msg=g_strdup_printf (_("\n\nUnable to read from keymap file\n%s\nError code %d\n"),keymap_file,errno);
    do_error_dialog_with_check_transient (msg,FALSE,0,GTK_WINDOW(rte_window));
    g_free (msg);
    d_print (_("failed.\n"));
    return FALSE;
  }

  while (fgets(buff,65536,kfile)) {
    if (buff!=NULL) {
      // TODO - should we g_free() line ?
      line=(g_strchomp (g_strchug(buff)));
      if ((linelen=strlen (line))) {
	whole2=g_strconcat (whole,line,NULL);
	if (whole2!=whole) g_free (whole);
	whole=whole2;
	if (linelen<(size_t)65535) {
	  list=g_list_append (list, g_strdup (whole));
	  g_free (whole);
	  whole=g_strdup ("");
	}
      }
    }
  }
  g_free (whole);
  fclose (kfile);

  on_clear_all_clicked(NULL,user_data);

  if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 2")||!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 1")) update=1;
  if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 3")) update=2;
  
  line=NULL;

  for (i=1;i<g_list_length(list);i++) {
    gchar **array;
    if (line!=NULL) g_free(line);
    line=(gchar *)g_list_nth_data(list,i);

    if (get_token_count(line,'|')<2) {
      d_print(g_strdup_printf(_("Invalid line %d in %s\n"),i,keymap_file));
      continue;
    }

    array=g_strsplit (line,"|",-1);

    key=atoi(array[0]);
    
    hashname=line+1+strlen(array[0]);
    g_strfreev(array);

    if (update>0) {
      if (update==1) hashname_new=g_strdup_printf("%d|Weed%s1\n",key,hashname);
      if (update==2) hashname_new=g_strdup_printf("%d|Weed%s\n",key,hashname);
      new_list=g_list_append(new_list,hashname_new);
      continue;
    }

    if (key<1||key>FX_KEYS_PHYSICAL) {
      d_print(g_strdup_printf(_("Invalid key %d in %s\n"),key,keymap_file));
      notfound=TRUE;
      continue;
    }

    if ((mode=weed_add_effectkey(key,hashname))==-1) {
      // could not locate effect
      notfound=TRUE;
      continue;
    }
    if (mode==-2){
      d_print(g_strdup_printf(_("This version of LiVES cannot mix generators/non-generators on the same key (%d) !\n"),key));
      continue;
    }
    if (mode==-3){
      d_print(g_strdup_printf(_("Too many effects bound to key %d.\n"),key));
      continue;
    }
    else if (rte_window!=NULL) {
      gtk_entry_set_text (GTK_ENTRY(combo_entries[(key-1)*modes+mode]),rte_keymode_get_filter_name(key,mode));
      type_label_set_text(key-1,mode);
    }
  }
  if (line!=NULL) g_free(line);
  g_list_free(list);

  if (update>0) {
    d_print(_("update required.\n"));
    on_save_keymap_clicked(NULL,new_list);
    g_list_free(new_list);
    on_load_keymap_clicked(NULL,NULL);
  }
  else d_print("done.\n");
    
  if (notfound&&mainw->is_ready) do_warning_dialog_with_check_transient(_("\n\nSome effects could not be located.\n\n"),0,GTK_WINDOW(rte_window));
  return FALSE;
}




void 
on_rte_info_clicked (GtkButton *button, gpointer user_data) {
  gint key_mode=GPOINTER_TO_INT(user_data);
  int modes=rte_getmodespk();
  gint key=(gint)(key_mode/modes);
  gint mode=key_mode-key*modes;

  gchar *type=rte_keymode_get_type(key+1,mode);
  weed_plant_t *filter;
  gchar *plugin_name;

  GtkWidget *rte_info_window;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *textview;

  GtkTextBuffer *textbuffer;

  GtkWidget *hbuttonbox;
  GtkWidget *ok_button;

  gchar *filter_name;
  gchar *filter_author;
  gchar *filter_description;
  int filter_version;
  int weed_error;

  ////////////////////////

  if (!rte_keymode_valid(key+1,mode)) return;

  plugin_name=rte_keymode_get_plugin_name(key+1,mode);
  filter=rte_keymode_get_filter(key+1,mode);
  filter_name=weed_get_string_value(filter,"name",&weed_error);
  filter_author=weed_get_string_value(filter,"author",&weed_error);
  if (weed_plant_has_leaf(filter,"description")) filter_description=weed_get_string_value(filter,"description",&weed_error);
  else filter_description=g_strdup(_("No Description"));

  filter_version=weed_get_int_value(filter,"version",&weed_error);

  rte_info_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (rte_info_window), g_strdup_printf(_("LiVES: Information for %s"),filter_name));
  gtk_widget_modify_bg(rte_info_window, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_container_set_border_width (GTK_CONTAINER (rte_info_window), 40);
  gtk_window_set_transient_for(GTK_WINDOW(rte_info_window),GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(button))));
  gtk_window_set_position (GTK_WINDOW (rte_info_window), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (rte_info_window), RTE_INFO_WIDTH, RTE_INFO_HEIGHT);

  gtk_widget_show(rte_info_window);

  vbox = gtk_vbox_new (FALSE, 20);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (rte_info_window), vbox);

  label = gtk_label_new (g_strdup_printf(_("Effect name: %s"),filter_name));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }
    
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);

  label = gtk_label_new (g_strdup_printf(_("Type: %s"),type));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);

  label = gtk_label_new (g_strdup_printf(_("Plugin name: %s"),plugin_name));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);

  label = gtk_label_new (g_strdup_printf(_("Author: %s"),filter_author));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);

  label = gtk_label_new (g_strdup_printf(_("Version: %d"),filter_version));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);

  label = gtk_label_new (_("Description: "));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  textview = gtk_text_view_new ();
  gtk_widget_show (textview);
  textbuffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
  
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), FALSE);
  
  gtk_text_buffer_set_text (textbuffer, filter_description, -1);
  gtk_box_pack_start (GTK_BOX (hbox), textview, TRUE, TRUE, 0);
  
  hbuttonbox = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbuttonbox, TRUE, TRUE, 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox), DEF_BUTTON_WIDTH, -1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);

  ok_button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (ok_button);

  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (ok_button);

  g_signal_connect (GTK_OBJECT (ok_button), "clicked",
		    G_CALLBACK (on_rtei_ok_clicked),
		    NULL);

  g_free(filter_name);
  g_free(filter_author);
  if (weed_plant_has_leaf(filter,"description")) g_free(filter_description);
  else weed_free(filter_description);
  g_free(plugin_name);
  g_free(type);
}



void check_clear_all_button (void) {
  int modes=rte_getmodespk();
  int i,j;
  gboolean hasone=FALSE;

  for (i=0;i<FX_KEYS_PHYSICAL;i++) {
    for (j=modes-1;j>=0;j--) {
      if (rte_keymode_valid(i+1,j)) hasone=TRUE;
    }
  }

  gtk_widget_set_sensitive (GTK_WIDGET(clear_all_button), hasone);
}




void 
on_clear_clicked (GtkButton *button, gpointer user_data) {
  // this is for the "delete" buttons, c.f. clear_all

  gint idx=GPOINTER_TO_INT(user_data);
  int modes=rte_getmodespk();
  gint key=(gint)(idx/modes);
  gint mode=idx-key*modes;

  int i,newmode;

  weed_delete_effectkey (key+1,mode);

  newmode=rte_key_getmode(key+1);
  g_signal_handler_block(mode_radios[key*modes+newmode],mode_ra_fns[key*modes+newmode]);
  rtew_set_mode_radio(key,newmode);
  g_signal_handler_unblock(mode_radios[key*modes+newmode],mode_ra_fns[key*modes+newmode]);
    
  for (i=mode;i<rte_getmodespk()-1;i++) {
    idx=key*modes+i;
    gtk_entry_set_text (GTK_ENTRY(combo_entries[idx]),gtk_entry_get_text(GTK_ENTRY(combo_entries[idx+1])));
    type_label_set_text(key,i);
  }
  idx++;
  gtk_entry_set_text (GTK_ENTRY(combo_entries[idx]),g_strdup(""));
  type_label_set_text(key,i);

  if (!rte_keymode_valid(key+1,0)) rtew_set_keych(key,FALSE);

  check_clear_all_button();

}


void 
on_params_clicked (GtkButton *button, gpointer user_data) {
  gint idx=GPOINTER_TO_INT(user_data);
  int modes=rte_getmodespk();
  gint key=(gint)(idx/modes);
  gint mode=idx-key*modes;

  weed_plant_t *inst;
  lives_rendered_effect_t *rfx;

  if ((inst=rte_keymode_get_instance(key+1,mode))==NULL) return;
  if (fx_dialog[1]!=NULL) {
    rfx=g_object_get_data (G_OBJECT (fx_dialog[1]),"rfx");
    gtk_widget_destroy(fx_dialog[1]);
    on_paramwindow_cancel_clicked2(NULL,rfx);
  }

  rfx=weed_to_rfx(inst);
  keyw=key;
  modew=mode;
  on_render_fx_pre_activate(NULL,rfx);
  g_object_set_data (G_OBJECT (fx_dialog[1]),"key",GINT_TO_POINTER (key));
  g_object_set_data (G_OBJECT (fx_dialog[1]),"mode",GINT_TO_POINTER (mode));
  g_object_set_data (G_OBJECT (fx_dialog[1]),"rfx",rfx);
}


gboolean
on_rtew_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
  GList *list;

  if (hash_list!=NULL) {
    list=hash_list;
    while (list!=NULL) {
      g_free(list->data);
      list=list->next;
    }
    g_list_free(hash_list);
  }

  if (name_list!=NULL) {
    list=name_list;
    while (list!=NULL) {
      g_free(list->data);
      list=list->next;
    }
    g_list_free(name_list);
  }

  if (name_type_list!=NULL) {
    list=name_type_list;
    while (list!=NULL) {
      g_free(list->data);
      list=list->next;
    }
    g_list_free(name_type_list);
  }

  g_free(key_checks);
  g_free(key_grabs);
  g_free(mode_radios);
  g_free(combo_entries);
  g_free(ch_fns);
  g_free(mode_ra_fns);
  g_free(gr_fns);
  g_free(nlabels);
  g_free(type_labels);
  g_free(info_buttons);
  g_free(param_buttons);
  g_free(clear_buttons);
  rte_window=NULL;
  return FALSE;
}


void 
on_rtew_ok_clicked (GtkButton *button, gpointer user_data) {
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  on_rtew_delete_event (NULL,NULL,NULL);
}




void do_mix_error(void) {
  do_error_dialog_with_check_transient(_("\n\nThis version of LiVES does not allowing mixing of generators and non-generators on the same key.\n\n"),FALSE,0,GTK_WINDOW(rte_window));
  return;
}





void 
fx_changed (GtkItem *item, gpointer user_data) {
  gint key_mode=GPOINTER_TO_INT(user_data);
  int modes=rte_getmodespk();
  gint key=(gint)(key_mode/modes);
  gint mode=key_mode-key*modes;
  gchar *hashname1=g_strdup(g_object_get_data(G_OBJECT(item),"hashname"));
  gchar *hashname2=g_strdup(g_object_get_data(G_OBJECT(combo_entries[key_mode]),"hashname"));
  gint error;

  int i;

  if (!strcmp(hashname1,hashname2)) {
    g_free(hashname1);
    g_free(hashname2);
    return;
  }

  if (!rte_keymode_valid(key+1,mode)) {
    for (i=mode-1;i>=0;i--) {
      if (rte_keymode_valid(key+1,i)) {
	mode=i+1;
	i=-1;
      }
      if (i==0) mode=0;
    }
  }

  gtk_widget_grab_focus (combo_entries[key_mode]);

  if ((error=rte_switch_keymode (key+1, mode, hashname1))<0) {
    gtk_entry_set_text (GTK_ENTRY (combo_entries[key_mode]),rte_keymode_get_filter_name(key+1,mode));

    // this gets called twice, unfortunately...may be a bug in gtk+
    if (error==-2) do_mix_error();
    if (error==-1) d_print(g_strdup_printf(_("LiVES could not locate the effect %s.\n"),rte_keymode_get_filter_name(key+1,mode)));
    g_free(hashname1);
    g_free(hashname2);
    return;
  }

  g_object_set_data(G_OBJECT(combo_entries[key_mode]),"hashname",g_strdup(hashname1));

  g_free(hashname2);
  g_free(hashname1);
    
  type_label_set_text(key,mode);

  check_clear_all_button();

}




GtkWidget*
create_rte_window (void)
{
  int i,j,idx,fx_idx;

  gint winsize_h=mainw->scr_width-100;
  gint winsize_v=mainw->scr_height-100;

  GtkWidget *rte_window;
  GtkWidget *table;
  GtkWidget *hbox;
  GSList *mode_group = NULL;
  GSList *grab_group = NULL;
  GtkWidget *vbox;
  GtkWidget *label;
  GtkWidget *combo;
  GtkWidget *ok_button;
  GtkWidget *top_vbox;
  GtkWidget *hbuttonbox;

  GtkWidget *scrolledwindow;

  GtkAccelGroup *rtew_accel_group;

  int modes=rte_getmodespk();

  GList *list;
  GtkWidget *item;

  ///////////////////////////////////////////////////////////////////////////

  key_checks=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*sizeof(GtkWidget *));
  key_grabs=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*sizeof(GtkWidget *));
  mode_radios=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));
  combo_entries=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));
  info_buttons=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));
  param_buttons=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));
  clear_buttons=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));
  nlabels=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));
  type_labels=(GtkWidget **)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(GtkWidget *));

  ch_fns=(gulong *)g_malloc((FX_KEYS_PHYSICAL)*sizeof(gulong));
  gr_fns=(gulong *)g_malloc((FX_KEYS_PHYSICAL)*sizeof(gulong));
  mode_ra_fns=(gulong *)g_malloc((FX_KEYS_PHYSICAL)*modes*sizeof(gulong));

  rte_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_modify_bg(rte_window, GTK_STATE_NORMAL, &palette->menu_and_bars);
  gtk_window_set_title (GTK_WINDOW (rte_window), _("LiVES: Real time effect mapping"));
  gtk_window_add_accel_group (GTK_WINDOW (rte_window), mainw->accel_group);

  table = gtk_table_new (FX_KEYS_PHYSICAL, modes+1, FALSE);

  gtk_table_set_row_spacings (GTK_TABLE (table), 16);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);

  // dummy button for "no grab", we dont show this...there is a button instead
  dummy_radio = gtk_radio_button_new_with_label (grab_group, _("Key grab"));
  grab_group = gtk_radio_button_group (GTK_RADIO_BUTTON (dummy_radio));

  name_list=weed_get_all_names(1);
  name_type_list=weed_get_all_names(2);
  hash_list=weed_get_all_names(3);

  for (i=0;i<FX_KEYS_PHYSICAL;i++) {

    hbox = gtk_hbox_new (FALSE, 10);
    gtk_table_attach (GTK_TABLE (table), hbox, i, i+1, 0, 1,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
    
    label = gtk_label_new (g_strdup_printf(_("Ctrl-%d"),i+1));
    
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);

    key_checks[i] = gtk_check_button_new_with_label (_("Key active"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(key_checks[i]),mainw->rte&(GU641<<i));

    ch_fns[i]=g_signal_connect_after (GTK_OBJECT (key_checks[i]), "toggled",
                      G_CALLBACK (rte_on_off_callback_hook),GINT_TO_POINTER (i+1));

    gtk_box_pack_start (GTK_BOX (hbox), key_checks[i], FALSE, FALSE, 0);
    
    key_grabs[i] = gtk_radio_button_new_with_label (grab_group, _("Key grab"));
    grab_group = gtk_radio_button_group (GTK_RADIO_BUTTON (key_grabs[i]));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(key_grabs[i]),mainw->rte_keys==i);

    gr_fns[i]=g_signal_connect_after (GTK_OBJECT (key_grabs[i]), "toggled",
				      G_CALLBACK (grabkeys_callback_hook),GINT_TO_POINTER (i));

    gtk_box_pack_start (GTK_BOX (hbox), key_grabs[i], FALSE, FALSE, 0);

    mode_group=NULL;

    for (j=0;j<modes;j++) {
      idx=i*modes+j;
      hbox = gtk_hbox_new (FALSE, 0);
      gtk_table_attach (GTK_TABLE (table), hbox, i, i+1, j+1, j+2,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
      gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);

      mode_radios[idx] = gtk_radio_button_new_with_label (mode_group, _("Mode active"));
      mode_group = gtk_radio_button_group (GTK_RADIO_BUTTON (mode_radios[idx]));

      gtk_box_pack_start (GTK_BOX (hbox), mode_radios[idx], FALSE, FALSE, 0);

      if (rte_key_getmode(i+1)==j) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mode_radios[idx]),TRUE);

      mode_ra_fns[idx]=g_signal_connect_after (GTK_OBJECT (mode_radios[idx]), "toggled",
					       G_CALLBACK (rtemode_callback_hook),GINT_TO_POINTER (idx));

      type_labels[idx] = gtk_label_new ("");

      info_buttons[idx] = gtk_button_new_with_label (_("Info"));
      param_buttons[idx] = gtk_button_new_with_label (_("Set Parameters"));
      clear_buttons[idx] = gtk_button_new_with_label (_("Clear"));

      vbox = gtk_vbox_new (FALSE, 15);
      gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
      gtk_container_set_border_width (GTK_CONTAINER (vbox), 13);

      hbox = gtk_hbox_new (FALSE, 12);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      nlabels[idx] = gtk_label_new (_("Effect name:"));

      gtk_box_pack_start (GTK_BOX (hbox), nlabels[idx], FALSE, FALSE, 0);

      combo = gtk_combo_new ();
      gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);

      combo_entries[idx] = GTK_COMBO (combo)->entry;
      g_object_set_data (G_OBJECT(combo_entries[idx]),"hashname","");

      // fill names of our effects
      fx_idx=0;
      list=name_type_list;

      while (list!=NULL) {
	item = gtk_list_item_new_with_label ((gchar *) list->data);
	gtk_widget_show(item);
	gtk_container_add (GTK_CONTAINER (GTK_COMBO(combo)->list), item);
	g_object_set_data (G_OBJECT(item),"hashname",g_list_nth_data(hash_list,fx_idx));
	gtk_combo_set_item_string(GTK_COMBO(combo),GTK_ITEM(item),g_list_nth_data(name_list,fx_idx));
	if (fx_idx==rte_keymode_get_filter_idx(i+1,j)) {
	  g_object_set_data (G_OBJECT(combo_entries[idx]),"hashname",g_list_nth_data(hash_list,fx_idx));
	}
	g_signal_connect (G_OBJECT(item), "select", G_CALLBACK (fx_changed), GINT_TO_POINTER(idx));
	list = list->next;
	fx_idx++;
      }

      gtk_entry_set_text (GTK_ENTRY (combo_entries[idx]),rte_keymode_get_filter_name(i+1,j));
      gtk_entry_set_editable (GTK_ENTRY (combo_entries[idx]), FALSE);

      hbox = gtk_hbox_new (FALSE, 12);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      g_signal_connect (GTK_OBJECT (info_buttons[idx]), "clicked",
			G_CALLBACK (on_rte_info_clicked),GINT_TO_POINTER (idx));

      g_signal_connect (GTK_OBJECT (clear_buttons[idx]), "clicked",
			G_CALLBACK (on_clear_clicked),GINT_TO_POINTER (idx));

      g_signal_connect (GTK_OBJECT (param_buttons[idx]), "clicked",
			G_CALLBACK (on_params_clicked),GINT_TO_POINTER (idx));

      type_label_set_text(i,j);

      gtk_box_pack_start (GTK_BOX (hbox), type_labels[idx], FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), info_buttons[idx], FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), param_buttons[idx], FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), clear_buttons[idx], FALSE, FALSE, 0);

      }
  }


  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindow), table);

  gtk_widget_set_size_request (scrolledwindow, winsize_h, winsize_v);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(gtk_bin_get_child (GTK_BIN (scrolledwindow)), GTK_STATE_NORMAL, &palette->normal_back);
  }
  
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (scrolledwindow))),GTK_SHADOW_IN);

  top_vbox = gtk_vbox_new (FALSE, 15);

  gtk_box_pack_start (GTK_BOX (top_vbox), dummy_radio, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (top_vbox), scrolledwindow, TRUE, TRUE, 0);

  gtk_container_add (GTK_CONTAINER (rte_window), top_vbox);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_box_pack_start (GTK_BOX (top_vbox), hbuttonbox, TRUE, TRUE, 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox), DEF_BUTTON_WIDTH, -1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);

  clear_all_button = gtk_button_new_with_mnemonic (_("_Clear all effects"));
  gtk_container_add (GTK_CONTAINER (hbuttonbox), clear_all_button);
  GTK_WIDGET_SET_FLAGS (clear_all_button, GTK_CAN_DEFAULT);

  save_keymap_button = gtk_button_new_with_mnemonic (_("_Save as default keymap"));

  gtk_container_add (GTK_CONTAINER (hbuttonbox), save_keymap_button);
  GTK_WIDGET_SET_FLAGS (save_keymap_button, GTK_CAN_DEFAULT);

  load_keymap_button = gtk_button_new_with_mnemonic (_("_Load default keymap"));

  gtk_container_add (GTK_CONTAINER (hbuttonbox), load_keymap_button);
  GTK_WIDGET_SET_FLAGS (load_keymap_button, GTK_CAN_DEFAULT);

  ok_button = gtk_button_new_with_mnemonic (_("Close _window"));

  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  

  rtew_accel_group = GTK_ACCEL_GROUP(gtk_accel_group_new ());
  gtk_window_add_accel_group (GTK_WINDOW (rte_window), rtew_accel_group);

  gtk_widget_add_accelerator (ok_button, "activate", rtew_accel_group,
                              GDK_Escape, 0, 0);
  g_signal_connect (GTK_OBJECT (rte_window), "delete_event",
		    G_CALLBACK (on_rtew_ok_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (ok_button), "clicked",
		    G_CALLBACK (on_rtew_ok_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (save_keymap_button), "clicked",
		    G_CALLBACK (on_save_keymap_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (load_keymap_button), "clicked",
		    G_CALLBACK (on_load_keymap_clicked),
		    GINT_TO_POINTER(1));
  g_signal_connect (GTK_OBJECT (clear_all_button), "clicked",
		    G_CALLBACK (on_clear_all_clicked),
		    GINT_TO_POINTER(1));

  gtk_widget_show_all(rte_window);
  gtk_widget_hide(dummy_radio);
  gtk_window_maximize (GTK_WINDOW(rte_window));
  return rte_window;
}




void 
on_assign_rte_keys_activate (GtkMenuItem *menuitem, gpointer user_data) {
  if (rte_window!=NULL) {
    on_rtew_ok_clicked(GTK_BUTTON(dummy_radio), user_data);
    return;
  }
  
  rte_window=create_rte_window();
  gtk_widget_show (rte_window);
}


void rtew_set_keych (gint key, gboolean on) {
  g_signal_handler_block(key_checks[key],ch_fns[key]);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(key_checks[key]),on);
  g_signal_handler_unblock(key_checks[key],ch_fns[key]);
}


void rtew_set_keygr (gint key) {
  if (key>=0) {
    g_signal_handler_block(key_grabs[key],gr_fns[key]);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(key_grabs[key]),TRUE);
    g_signal_handler_unblock(key_grabs[key],gr_fns[key]);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(dummy_radio),TRUE);
  }
}

void rtew_set_mode_radio (gint key, gint mode) {
  int modes=rte_getmodespk();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mode_radios[key*modes+mode]),TRUE);
}


void rtew_set_param_button (gint key, gint mode, gboolean on) {
  int modes=rte_getmodespk();
  gtk_widget_set_sensitive(param_buttons[key*modes+mode],on);

}


void close_pwindow (gint key, gint mode, gboolean is_reinit) {
  gint keyw,modew;
  GList *child_list;
  int i;
  lives_rendered_effect_t *rfx;

  if (fx_dialog[1]!=NULL) {
    keyw=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"key"));
    modew=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"mode"));
    rfx=g_object_get_data(G_OBJECT(fx_dialog[1]),"rfx");
    if (key==keyw&&mode==modew) {
      if (!is_reinit) {
	gtk_widget_destroy(fx_dialog[1]);  // bye bye param window !
	on_paramwindow_cancel_clicked2(NULL,rfx);
      }
      else {
	// or just rip out the contents
	child_list=gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(fx_dialog[1])->vbox));
	for (i=0;i<g_list_length(child_list);i++) {
	 if (g_list_nth_data(child_list,i)!=GTK_DIALOG (fx_dialog[1])->action_area) gtk_widget_destroy(GTK_WIDGET(g_list_nth_data(child_list,i)));
	}
	if (child_list!=NULL) g_list_free(child_list);
	on_paramwindow_cancel_clicked(NULL,rfx);
	restore_pwindow(key);
      }
    }
  }
}



void restore_pwindow (gint key) {
  weed_plant_t *inst;
  lives_rendered_effect_t *rfx;
  gint keyw,modew;

  if (fx_dialog[1]!=NULL) {
    keyw=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"key"));
    modew=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"mode"));
    if (key==keyw) {
      if ((inst=rte_keymode_get_instance(key+1,modew))==NULL) return;
      rfx=weed_to_rfx(inst);
      make_param_box(GTK_VBOX (GTK_DIALOG(fx_dialog[1])->vbox), rfx);
      gtk_widget_show_all (fx_dialog[1]);
      g_object_set_data (G_OBJECT (fx_dialog[1]),"rfx",rfx);
    }
  }
}


void update_pwindow (gint key, gint i, GList *list) {
  const weed_plant_t *inst;
  lives_rendered_effect_t *rfx;
  gint keyw,modew;

  if (fx_dialog[1]!=NULL) {
    keyw=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"key"));
    modew=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"mode"));
    if (key==keyw) {
      if ((inst=rte_keymode_get_instance(key+1,modew))==NULL) return;
      rfx=g_object_get_data(G_OBJECT(fx_dialog[1]),"rfx");
      set_param_from_list(list,&rfx->params[i],0,TRUE);
    }
  }
}



void load_default_keymap(void) {
  // called on startup
  const gchar *keymap_file=g_strdup_printf("%s/.lives-dir/default.keymap",capable->home_dir);
  const gchar *keymap_template=g_strdup_printf("%s%sdefault.keymap",prefs->prefix_dir,DATA_DIR);
  gchar *com;

  if (!g_file_test (keymap_file, G_FILE_TEST_EXISTS)) {
    com=g_strdup_printf("/bin/cp %s %s",keymap_template,keymap_file);
    system(com);
    g_free(com);
  }
  if (!g_file_test (keymap_file, G_FILE_TEST_EXISTS)) {
    // give up
    d_print(g_strdup_printf(_("Unable to create default keymap file: %s\nPlease make sure your home directory is writable.\n"),keymap_file));
    return;
  }
  on_load_keymap_clicked(NULL,NULL);
}
