// utils.c
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// released under the GNU GPL 2
// see file ../COPYING or www.gnu.org for licensing details


#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "main.h"
#include "support.h"

static gboolean  omute,  osepwin,  ofs,  ofaded,  odouble;

inline gchar *
g_strappend (gchar *string, gint len, const gchar *new) {
  g_snprintf(string,len,"%s",g_strconcat (string,new,NULL));
  return string;
}

void d_print(const gchar *text) {
  gchar *switchtext;
  GtkTextIter end_iter;
  GtkTextMark *mark;

  if (!capable->smog_version_correct) return;

  if (GTK_IS_TEXT_VIEW (mainw->textview1)) {
    gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(mainw->textview1)),&end_iter);
    if (mainw->current_file!=mainw->last_dprint_file&&mainw->current_file!=0) {
      if (mainw->current_file>0) {
	switchtext=g_strdup_printf (_ ("\n==============================\nSwitched to clip %s\n"),cfile->name);
      }
      else {
	switchtext=g_strdup_printf (_ ("\n==============================\nSwitched to empty clip\n"));
      }
    gtk_text_buffer_insert(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),&end_iter,g_locale_to_utf8 (text,-1,NULL,NULL,NULL),-1);
      gtk_text_buffer_insert(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),&end_iter,switchtext,-1);
      g_free (switchtext);
      mainw->last_dprint_file=mainw->current_file;
    }
    gtk_text_buffer_insert(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),&end_iter,text,-1);
    mark=gtk_text_buffer_create_mark(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),NULL,&end_iter,FALSE);
    gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW (mainw->textview1),mark);
    gtk_text_buffer_delete_mark (gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),mark);
  }
}


void
get_next_free_file(void) {
  // get next free file slot, or -1 if we are full
  while ((mainw->first_free_file!=-1)&&mainw->files[mainw->first_free_file]!=NULL) {
    mainw->first_free_file++;
    if (mainw->first_free_file>=MAX_FILES) mainw->first_free_file=-1;
  }
}


void
get_dirname(gchar *filename) {
  // get directory name from a file
  //filename should point to gchar[256]
  g_snprintf (filename,256,"%s%s",g_path_get_dirname (filename),G_DIR_SEPARATOR_S);
}

void
get_basename(gchar *filename) {
  // get basename from a file
  //filename should point to gchar[256]
  g_snprintf (filename,256,"%s",g_path_get_basename (filename));
}

void
get_filename(gchar *filename) {
  // get filename (part without extension) of a file
  //filename should point to gchar[256]
  gchar **array;
  get_basename(filename);
  array=g_strsplit(filename,".",-1);
  g_snprintf(filename,256,"%s",array[0]);
  g_strfreev(array);
}


void
get_location(const gchar *exe, gchar *val, gint maxlen) {
  gchar *loc;
  if ((loc=g_find_program_in_path (exe))!=NULL) {
    g_snprintf (val,maxlen,"%s",loc);
    g_free (loc);
  }
  else {
    memset (val,0,1);
  }
}








void
get_play_times(void) {
  // update the on-screen timer bars,
  // and if we are not playing,
  // get play times for video, audio channels, and total (gint64est) time
  gchar *tmpstr;
  gdouble offset=0;
  gdouble offset_left=0;
  gdouble offset_right=0;
  gdouble allocwidth;
  gdouble allocheight;

  if (mainw->current_file==-1||mainw->foreign==TRUE||cfile==NULL||mainw->multitrack!=NULL) return;

  if (mainw->playing_file==-1) {
    get_total_time (cfile);
    set_main_title (cfile->name,0);
  }
  // draw timer bars
  allocwidth=mainw->video_draw->allocation.width;
  allocheight=mainw->video_draw->allocation.height;

  gdk_draw_rectangle (mainw->laudio_drawable,
		      mainw->laudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->laudio_draw)],
		      TRUE,
		      0, 0,
		      allocwidth,
		      allocheight
		      );
  
  gdk_draw_rectangle (mainw->raudio_drawable,
		      mainw->raudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->raudio_draw)],
		      TRUE,
		      0, 0,
		      allocwidth,
		      allocheight
		      );
      
  gdk_draw_rectangle (mainw->video_drawable,
		      mainw->video_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->video_draw)],
		      TRUE,
		      0, 0,
		      allocwidth,
		      allocheight
		      );


  if (cfile->frames>0) {
    if (cfile->event_list==NULL) {
      offset_left=(cfile->start-1)/cfile->fps/cfile->total_time*allocwidth;
      offset_right=(cfile->end)/cfile->fps/cfile->total_time*allocwidth;
    }
    else {
      offset_left=calc_variable_time(cfile->start)/cfile->total_time*allocwidth;
      offset_right=calc_variable_time(cfile->end+1)/cfile->total_time*allocwidth;
    }
  
    gdk_draw_rectangle (mainw->video_drawable,
			mainw->video_draw->style->black_gc,
			TRUE,
			0, 0,
			cfile->video_time/cfile->total_time*allocwidth-1,
			prefs->bar_height);
    
    gdk_draw_rectangle (mainw->video_drawable,
			mainw->video_draw->style->white_gc,
			TRUE,
			offset_left, 0,
			offset_right-offset_left,
			prefs->bar_height);
  }

  if (cfile->achans>0) {
    if (mainw->playing_file>-1) {
      offset_left=(mainw->audio_start-1.)/cfile->fps/cfile->total_time*allocwidth;
      if (mainw->audio_end) {
	offset_right=(mainw->audio_end)/cfile->fps/cfile->total_time*allocwidth;
      }
      else {
	offset_right=allocwidth*cfile->laudio_time/cfile->total_time;
      }
    }
    
    
    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->black_gc,
			TRUE,
			0, 0,
			cfile->laudio_time/cfile->total_time*allocwidth-1,
			prefs->bar_height);
    
    
    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->white_gc,
			TRUE,
			offset_left, 0,
			offset_right-offset_left,
			prefs->bar_height);
  
    
    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->laudio_draw)],
			TRUE,
			cfile->laudio_time/cfile->total_time*allocwidth, 0,
			allocwidth-(cfile->laudio_time/cfile->total_time*allocwidth),
			prefs->bar_height);
    
    if (cfile->achans>1) {
      gdk_draw_rectangle (mainw->raudio_drawable,
			  mainw->raudio_draw->style->black_gc,
			  TRUE,
			  0, 0,
			  cfile->raudio_time/cfile->total_time*allocwidth-1,
			  prefs->bar_height);
      
      gdk_draw_rectangle (mainw->raudio_drawable,
			  mainw->raudio_draw->style->white_gc,
			  TRUE,
			  offset_left, 0,
			  offset_right-offset_left,
			  prefs->bar_height);
      
      
      gdk_draw_rectangle (mainw->raudio_drawable,
			  mainw->raudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->raudio_draw)],
			  TRUE,
			  cfile->raudio_time/cfile->total_time*allocwidth, 0,
			  allocwidth-(cfile->raudio_time/cfile->total_time*allocwidth),
			  prefs->bar_height);
    }
  }

  // playback cursors
  if (mainw->playing_file>-1) {
    if (cfile->frames>0) {
      if (cfile->event_list==NULL) {
	offset=(mainw->actual_frame-.5)/cfile->fps;
      }
      else {
	offset=calc_variable_time(mainw->actual_frame);
      }
      offset/=cfile->total_time/allocwidth;
      if (offset>=offset_left&&offset<=offset_right) {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->black_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->white_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      
      if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->black_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight-prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->white_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight-prefs->bar_height);
      }
    }
      
    if (cfile->achans>0&&cfile->is_loaded) {
      offset=allocwidth*(mainw->aframeno-.5)/cfile->fps/cfile->total_time;
      
      if ((offset>=offset_left&&offset<=offset_right)||mainw->loop==TRUE) {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->black_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->white_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      
      if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->black_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight-prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->white_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight-prefs->bar_height);
      }
      
      if (cfile->achans>1) {
	if ((offset>=offset_left&&offset<=offset_right)||mainw->loop==TRUE) {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->black_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->white_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	
	
	if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->black_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight-prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->white_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight-prefs->bar_height);
	}
      }
    }
    GTK_RULER (mainw->hruler)->position=offset*cfile->total_time/allocwidth;
    gtk_widget_queue_draw (mainw->hruler);
  }
  
  if (mainw->playing_file==-1||(mainw->switch_during_pb&&!mainw->faded)) {
    if (cfile->total_time>0.) {
      // set the range of the timeline
      if (cfile->opening_loc==FALSE) {
	gtk_widget_show (mainw->hruler);
      }
      gtk_widget_show (mainw->eventbox5);
      gtk_widget_show (mainw->video_draw);
      gtk_widget_show (mainw->laudio_draw);
      gtk_widget_show (mainw->raudio_draw);
      GTK_RULER (mainw->hruler)->upper=cfile->total_time;
      draw_little_bars();
      if (mainw->playing_file==-1&&prefs->sepwin_type==1&&mainw->sep_win&&cfile->is_loaded) {
	if (mainw->preview_box==NULL) {
	  // create the preview box that shows frames
	  make_preview_box();
	}
	// and add it the play window
	if (mainw->preview_box->parent==NULL&&cfile->clip_type==CLIP_TYPE_DISK&&!mainw->is_rendering) {
	  gtk_widget_queue_draw(mainw->play_window);
	  gtk_container_add (GTK_CONTAINER (mainw->play_window), mainw->preview_box);
	}
	load_preview_image(FALSE);
      }
    }
    else {
      gtk_widget_hide (mainw->hruler);
      gtk_widget_hide (mainw->eventbox5);
    }

    if (cfile->opening_loc==TRUE||(cfile->frames==123456789&&cfile->opening==TRUE)) {
      tmpstr=g_strdup_printf(_ ("Video [opening...]"));
    }
    else {
      if (cfile->video_time>0.) {
	tmpstr=g_strdup_printf(_ ("Video [%.2f sec]"),cfile->video_time);
      }
      else {
	if (cfile->video_time<=0.&&cfile->frames>0) {
	  tmpstr=g_strdup (_ ("(Undefined)"));
	}
	else {
	  tmpstr=g_strdup (_ ("(No video)"));
	}
      }
    }
    gtk_label_set_text(GTK_LABEL(mainw->vidbar),tmpstr);
    g_free(tmpstr);
    if (cfile->achans==0) {
      tmpstr=g_strdup (_ ("(No audio)"));
    }
    else {
      if (cfile->opening_audio==TRUE) {
	if (cfile->achans==1) {
	  tmpstr=g_strdup_printf (_ ("Mono  [opening...]"));
	}
	else {
	  tmpstr=g_strdup_printf(_ ("Left Audio [opening...]"));
	}
      }
      else {
	if (cfile->achans==1) {
	  tmpstr=g_strdup_printf(_ ("Mono [%.2f sec]"),cfile->laudio_time);
	}
	else {
	  tmpstr=g_strdup_printf(_ ("Left Audio [%.2f sec]"),cfile->laudio_time);
	}
      }
    }
    gtk_label_set_text(GTK_LABEL(mainw->laudbar),tmpstr);
    g_free(tmpstr);
    if (cfile->achans>1) {
      if (cfile->opening_audio==TRUE) {
	tmpstr=g_strdup_printf(_ ("Right Audio [opening...]"));
      }
      else {
	tmpstr=g_strdup_printf(_ ("Right Audio [%.2f sec]"),cfile->raudio_time);
      }
      gtk_label_set_text(GTK_LABEL(mainw->raudbar),tmpstr);
      gtk_widget_show (mainw->raudbar);
      g_free(tmpstr);
    }
    else {
      gtk_widget_hide (mainw->raudbar);
    }
  }
  gtk_widget_queue_draw(mainw->video_draw);
  gtk_widget_queue_draw(mainw->laudio_draw);
  gtk_widget_queue_draw(mainw->raudio_draw);
  gtk_widget_queue_draw(mainw->vidbar);
  gtk_widget_queue_draw(mainw->hruler);
}
    

void 
draw_little_bars (void) {
//draw the vertical bars when we are not playing
  gdouble allocheight=mainw->video_draw->allocation.height-prefs->bar_height;
  gdouble offset=cfile->pointer_time/cfile->total_time*mainw->vidbar->allocation.width;
  gint frame;

  if (!(frame=calc_variable_frame(mainw->current_file,cfile->pointer_time))) frame=cfile->frames;

    if (cfile->frames>0) {
      if (frame>=cfile->start&&frame<=cfile->end) {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->black_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->white_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      
      if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->black_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }
      else {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->white_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }
    }

    if (cfile->achans>0) {
      if ((frame>=cfile->start&&frame<=cfile->end)||mainw->loop==TRUE) {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->black_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->white_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      
      if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->black_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }
      else {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->white_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }

      if (cfile->achans>1) {
	
	if ((frame>=cfile->start&&frame<=cfile->end)||mainw->loop==TRUE) {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->black_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->white_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	

	if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->black_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight);
	}
	else {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->white_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight);
	}
      }
    }
}



void 
get_total_time (file *file) {
  // get times (video, left and right audio)

  file->laudio_time=file->raudio_time=file->video_time=file->total_time=0.;

  if (file->opening==TRUE&&file->frames!=123456789) {
    if (file->frames*file->fps>0) {
      file->video_time=file->total_time=file->frames/file->fps;
    }
    return;
  }

  if (cfile->event_list!=NULL&&file->frames>0) {
    // calculate variable time if we have fps changes
    file->total_time=file->video_time=calc_variable_time(file->frames+1);
  }
  else {
    if (file->fps>0.) {
      file->total_time=file->video_time=file->frames/file->fps;
    }
  }

  if (file->asampsize*file->arate*file->achans) {
    file->laudio_time=file->afilesize/file->asampsize*8./file->arate/file->achans;
    if (file->achans>1) {
      file->raudio_time=file->laudio_time;
    }
  }

  if (file->laudio_time>file->total_time) file->total_time=file->laudio_time;
  if (file->raudio_time>file->total_time) file->total_time=file->raudio_time;

  if (file->laudio_time+file->raudio_time==0&&file->opening==FALSE) {
    file->achans=file->afilesize=file->asampsize=file->arate=file->arps=0;
  }
}


gdouble 
calc_variable_time (gint frame) {
  // return the absolute time of the first occurance of a frame
  // even with an event_list

  // if the frame is not referenced anywhere we return the timecode it would have had if there was no event_list

  gdouble time=(frame-1.)/cfile->fps;
  int list_frame;
  gint64 tc;
  event_t *event;
  int hint,error;

  if (cfile->event_list==NULL||time<event_list_get_start_secs (cfile->event_list)||time>event_list_get_end_secs (cfile->event_list)) {
    return time;
  }

  event=cfile->event_list->first;
  while (event!=NULL) {
    hint=get_event_hint (event->event);
    if (hint==LIVES_EVENT_HINT_FRAME) {
      list_frame=weed_get_int_value (event->event,"frames",&error);
      if (frame==list_frame) {
	tc=get_event_timecode (event->event);
	return (gdouble)(tc/U_SEC);
      }
    }
    event=event->next;
  }
  // undefined
  return time;
}



gint
calc_variable_frame (gint filenum, gdouble time) {
  // return the nearest frame for a given time even with an event_list 
  int frame=0;
  gint64 tc;
  gdouble delta=1000000.;
  event_t *event;
  int hint,error;

  if (time<0.) return mainw->files[filenum]->frames?1:0;
  if (mainw->files[filenum]->event_list==NULL||mainw->files[filenum]->event_list->first==NULL||time<event_list_get_start_secs (mainw->files[filenum]->event_list)||time>event_list_get_end_secs (mainw->files[filenum]->event_list)) {
    frame=(gint)(time*mainw->files[filenum]->fps+1.49999);
    return (frame<mainw->files[filenum]->frames)?frame:mainw->files[filenum]->frames;
  }
  event=mainw->files[filenum]->event_list->first;
  while (event!=NULL) {
    hint=get_event_hint (event->event);
    if (hint==LIVES_EVENT_HINT_FRAME) {
      tc=get_event_timecode (event->event);
      if ((gdouble)(tc/U_SEC)<time) {
	delta=time-(gdouble)(tc/U_SEC);
	frame=weed_get_int_value (event->event,"frames",&error);
      }
      else {
	if ((gdouble)(tc/U_SEC)-time<delta) {
	  frame=weed_get_int_value (event->event,"frames",&error);
	  return frame;
	}
	return frame;
      }
    }
    event=event->next;
  }
  return frame;
}



void 
find_when_to_stop (void) {
  // work out when to stop playing
  //
  // ---------------
  //        no loop              loop to fit                 loop cont
  //        -------              -----------                 ---------
  // a>v    stop on video end    stop on audio end           no stop
  // v>a    stop on video end    stop on video end           no stop
  // generator start - not playing : stop on vid_end, unless pure audio;
  if (cfile->clip_type==CLIP_TYPE_GENERATOR) mainw->whentostop=STOP_ON_VID_END;
  else if (cfile->clip_type!=CLIP_TYPE_DISK) mainw->whentostop=NEVER_STOP;
  else if (cfile->opening_only_audio) mainw->whentostop=STOP_ON_AUD_END;
  else if (cfile->opening_audio) mainw->whentostop=STOP_ON_VID_END;
  else if (mainw->loop_cont&&!mainw->preview) mainw->whentostop=NEVER_STOP;
  else if (mainw->loop&&cfile->achans>0&&(mainw->audio_end/cfile->fps)<MAX (cfile->laudio_time,cfile->raudio_time)) mainw->whentostop=STOP_ON_AUD_END;
  else mainw->whentostop=STOP_ON_VID_END; // tada...
}

void 
minimise_aspect_delta (gdouble aspect,gint hblock,gint vblock,gint hsize,gint vsize,gint *width,gint *height) {
  // we will use trigonometry to calculate the smallest difference between a given
  // aspect ratio and the actual frame size. If the delta is smaller than current 
  // we set the height and width
  gint cw=width[0];
  gint ch=height[0];

  gint real_width,real_height;
  gint delta,current_delta=(hsize-cw)*(hsize-cw)+(vsize-ch)*(vsize-ch);

  // minimise d[(x-x1)^2 + (y-y1)^2]/d[x1], to get approximate values
  gint calc_width=(gint)((vsize+aspect*hsize)*aspect/(aspect*aspect+1.));

  int i;

#ifdef DEBUG_ASPECT
  g_printerr ("aspect %.8f : width %d height %d is best fit\n",aspect,calc_width,(gint)(calc_width/aspect));
#endif
  // use the block size to find the nearest allowed size
  for (i=-1;i<2;i++) {
    real_width=(gint)(calc_width/hblock+i)*hblock;
    real_height=(gint)(real_width/aspect/vblock+.5)*vblock;
    delta=(hsize-real_width)*(hsize-real_width)+(vsize-real_height)*(vsize-real_height);
#ifdef DEBUG_ASPECT
    g_printerr ("block quantise to %d x %d\n",real_width,real_height);
#endif
    if (delta<current_delta) {
#ifdef DEBUG_ASPECT
      g_printerr ("is better fit\n");
#endif
      current_delta=delta;
      width[0]=real_width;
      height[0]=real_height;
    }
  }
}

void 
zero_spinbuttons (void) {
  g_signal_handler_block(mainw->spinbutton_start,mainw->spin_start_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_start),0,0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),0);
  g_signal_handler_unblock(mainw->spinbutton_start,mainw->spin_start_func);
  g_signal_handler_block(mainw->spinbutton_end,mainw->spin_end_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_end),0,0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),0);
  g_signal_handler_unblock(mainw->spinbutton_end,mainw->spin_end_func);
}


void 
hide_cursor(GdkWindow *window) {
  unsigned char cursor_bits[] = {0x00};
  unsigned char cursormask_bits[] = {0x00};
  GdkCursor *cursor;
  GdkPixmap *source, *mask;
  GdkColor fg = { 0, 0, 0, 0 };
  GdkColor bg = { 0, 0, 0, 0 };

  //make the cursor invisible in playback windows
  source = gdk_bitmap_create_from_data (NULL, cursor_bits,
					1, 1);
  mask = gdk_bitmap_create_from_data (NULL, cursormask_bits,
				      1, 1);
  cursor = gdk_cursor_new_from_pixmap (source, mask, &fg, &bg, 0, 0);
  gdk_pixmap_unref (source);
  gdk_pixmap_unref (mask);

  gdk_window_set_cursor (window, cursor);
}


void 
unhide_cursor(GdkWindow *window) {
  gdk_window_set_cursor(window,NULL);
}




void
switch_aud_to_sox(void) {
  prefs->audio_player=AUD_PLAYER_SOX;
  get_pref_default("sox_command",prefs->audio_play_command,256);
  set_pref("audio_player","sox");
  set_pref("audio_play_command",prefs->audio_play_command);
  if (mainw->current_file>0) {
    gtk_widget_set_sensitive (mainw->resample_audio, cfile->achans>0);
  }
}



void
switch_aud_to_mplayer(void) {
  int i;
  for (i=1;i<MAX_FILES;i++) {
      if (!(mainw->files[i]==NULL)) {
	if (i!=mainw->current_file&&mainw->files[i]->opening==TRUE) {
	  do_error_dialog(_ ("LiVES cannot switch to mplayer whilst clips are loading."));
	  return;
	}
      }
    }
  
  prefs->audio_player=AUD_PLAYER_MPLAYER;
  get_pref_default("mplayer_audio_command",prefs->audio_play_command,256);
  set_pref("audio_player","mplayer");
  set_pref("audio_play_command",prefs->audio_play_command);
  gtk_widget_set_sensitive (mainw->resample_audio, FALSE);
}



void
prepare_to_play_foreign(void) {
  // here we are going to 'play' a captured external window
  gint new_file=mainw->first_free_file;

  // create a new 'file' to play into
  if (get_new_handle(new_file,NULL)==FALSE) {
    return;
  }

  mainw->current_file=new_file;

  cfile->hsize=mainw->foreign_width/2+1;
  cfile->vsize=mainw->foreign_height/2+3;

  cfile->fps=prefs->default_fps;

  resize(-2);

  gtk_widget_show (mainw->playframe);
  gtk_widget_show (mainw->playarea);
  while (g_main_context_iteration(NULL,FALSE));

  // size must be exact, must not be larger than play window or we end up with nothing
  mainw->pwidth=mainw->playframe->allocation.width-H_RESIZE_ADJUST+2;
  mainw->pheight=mainw->playframe->allocation.height-V_RESIZE_ADJUST+2;

  cfile->hsize=mainw->pwidth;
  cfile->vsize=mainw->pheight;

  // for some strange reason we now have to do this twice...and now it doesn't
  // always work
  gtk_socket_add_id(GTK_SOCKET(mainw->playarea),mainw->foreign_id);
  gtk_socket_add_id(GTK_SOCKET(mainw->playarea),mainw->foreign_id);
  while (g_main_context_iteration(NULL,FALSE));

  mainw->foreign_map=gdk_pixmap_foreign_new(mainw->foreign_id);
  mainw->foreign_cmap=gdk_colormap_get_system();

  mainw->play_start=1;
  mainw->play_end=prefs->max_record_frames;

  omute=mainw->mute;
  osepwin=mainw->sep_win;
  ofs=mainw->fs;
  ofaded=mainw->faded;
  odouble=mainw->double_size;

  mainw->mute=TRUE;
  mainw->sep_win=FALSE;
  mainw->fs=FALSE;
  mainw->faded=TRUE;
  mainw->double_size=FALSE;

  gtk_widget_hide(mainw->t_double);
  gtk_widget_hide(mainw->t_single);
  gtk_widget_hide(mainw->t_fullscreen);
  gtk_widget_hide(mainw->t_bckground);
  gtk_widget_hide(mainw->t_sepwin);
  gtk_widget_hide(mainw->t_infobutton);
}


gboolean
after_foreign_play(void) {
  // read details from capture file
  int capture_fd;
  gchar *capfile=g_strdup_printf("%s/.capture.%d",prefs->tmpdir,getpid());
  gchar capbuf[256];
  ssize_t length;
  gint new_file;
  gint new_frames=0;
  gint old_file=mainw->current_file;

  gchar *com;
  gchar **array;
  gchar file_name[256];

  // assume for now we only get one clip passed back
  if ((capture_fd=open(capfile,O_RDONLY))>0) {
    memset(capbuf,0,256);
    if ((length=read(capture_fd,capbuf,256))) {
      if (get_token_count (capbuf,'|')>2) {
	array=g_strsplit(capbuf,"|",3);
	new_frames=atoi(array[1]);
	new_file=mainw->first_free_file;
	mainw->current_file=new_file;
	cfile=(file *)(g_malloc(sizeof(file)));
	g_snprintf(cfile->handle,255,"%s",array[0]);
	g_strfreev(array);
	create_cfile();
	g_snprintf(cfile->file_name,256,"Capture %d",mainw->cap_number);
	g_snprintf(cfile->name,256,"Capture %d",mainw->cap_number++);
	g_snprintf(cfile->type,40,"Frames");
	cfile->progress_start=cfile->start=1;
	cfile->progress_end=cfile->frames=cfile->end=new_frames;

	g_snprintf(file_name,256,"%s/%s/",prefs->tmpdir,cfile->handle);
	
	com=g_strdup_printf("smogrify fill_and_redo_frames %s %d %d %d %d",cfile->handle,1,cfile->frames,mainw->foreign_width,mainw->foreign_height);
	unlink(cfile->info_file);
	system(com);
	cfile->nopreview=TRUE;
	if (do_progress_dialog(TRUE,TRUE,_ ("Cleaning up clip"))) {
	  array=g_strsplit(mainw->msg,"|",5);
	  new_frames=atoi(array[1]);
	  cfile->hsize=atoi(array[2]);
	  cfile->vsize=atoi(array[3]);
	  cfile->bpp=atoi(array[4]);
	  g_strfreev(array);
	  
	  if (new_frames>0) {
	    cfile->start=1;
	    cfile->end=cfile->frames=new_frames;
	    get_next_free_file();
	  }
	}
	else {
	  // cancelled cleanup
	  cfile->frames=cfile->start=cfile->end=new_frames=0;
	}
	g_free(com);
      }
    }
    close(capture_fd);
    unlink(capfile);
  }
  else {
    // nothing captured
    g_free(capfile);
    if (mainw->current_file>0) {
      switch_to_file (old_file,old_file);
      cfile->nopreview=FALSE;
    }
    else {
      close_current_file();
    }
    return FALSE;
  }
  cfile->nopreview=FALSE;
  g_free(capfile);

  if (new_frames==0) {
    // failed to get any frames from this clip
    close_current_file();
    return FALSE;
  }

  add_to_winmenu();
  switch_to_file(old_file,mainw->current_file);
  cfile->is_loaded=TRUE;
  cfile->changed=TRUE;
  return TRUE;
}


void
set_menu_text(GtkWidget *menuitem, const gchar *text, gboolean use_mnemonic) {
  GtkWidget *label;
  if (GTK_IS_MENU_ITEM (menuitem)) {
    label=gtk_bin_get_child(GTK_BIN(menuitem));
    if (use_mnemonic) {
      gtk_label_set_text_with_mnemonic(GTK_LABEL(label),text);
    }
    else {
      gtk_label_set_text(GTK_LABEL(label),text);
    }
  }
}


void
get_menu_text(GtkWidget *menuitem, gchar *text) {
  GtkWidget *label=gtk_bin_get_child(GTK_BIN(menuitem));
  g_snprintf(text,255,"%s",gtk_label_get_text(GTK_LABEL(label)));
}


void 
reset_clip_menu (void) {
  // sometimes the clip menu gets messed up, e.g. after reloading a set.
  // This function will clean up the 'x's and so on.

  int i;
  GtkWidget *active_image=NULL;

  for (i=1;i<=MAX_FILES;i++) {
    if (!(mainw->files[i]==NULL)) {
      if (!(active_image==NULL)) {
	active_image=NULL;
      }
      if (mainw->files[i]->opening==TRUE) {
	active_image = gtk_image_new_from_stock ("gtk-no", GTK_ICON_SIZE_MENU);
      }
      else {
	if (i==mainw->current_file) {
	  active_image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
	}
      }
      if (!(active_image==NULL)) {
	gtk_widget_show (active_image);
      }
      if (mainw->files[i]->menuentry!=NULL) {
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mainw->files[i]->menuentry), active_image);
      }
    }
  }
}



gboolean
check_file(const gchar *file_name) {
  int check;
  gboolean exists=FALSE;
  gchar *msg;

  // check if file exists
  if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
    msg=g_strdup_printf (_ ("\n%s\nalready exists.\n\nOverwrite ?\n"),file_name);
    if (!do_warning_dialog(msg)) {
      g_free (msg);
      return FALSE;
    }
    g_free (msg);
    check=open(file_name,O_RDWR);
    exists=TRUE;
  }
  // if not, check if we can write to it
  else {
    check=open(file_name,O_CREAT|O_EXCL|O_RDWR);
  }

  if (check<0) {
    if (errno==EACCES&&mainw!=NULL&&mainw->is_ready) {
      g_snprintf(mainw->msg,512,_ ("\nLiVES was unable to write to the file:\n%s\nPlease check the file permissions and try again."),file_name);
      do_error_dialog(mainw->msg);
    }
    return FALSE;
  }
  if (check) {
    close(check);
    if (!exists) {
      unlink (file_name);
    }
    return TRUE;
  }
  return FALSE;
}


gboolean 
check_dir_access (gchar *dir) {
  // if a directory exists, make sure it is readable and writable
  // otherwise create it and then check
  gboolean exists=g_file_test (dir, G_FILE_TEST_EXISTS);
  gchar *com;
  gchar *testfile;
  gboolean is_OK=FALSE;

  if (!exists) {
    com=g_strdup_printf ("mkdir %s",dir);
    system (com);
    g_free (com);
  }
  if (!g_file_test(dir, G_FILE_TEST_IS_DIR)) return FALSE;

  testfile=g_strdup_printf ("%s/livestst.txt",dir);
  com=g_strdup_printf ("touch %s",testfile);
  system (com);
  g_free (com);
  if ((is_OK=g_file_test(testfile, G_FILE_TEST_EXISTS))) {
    unlink (testfile);
  }
  g_free (testfile);
  if (!exists) {
    com=g_strdup_printf ("rm -r %s",dir);
    system (com);
    g_free (com);
  }
  return is_OK;
}






guint
get_file_size(int fd) {
  // get the size of file fd
  struct stat filestat;
  fstat(fd,&filestat);
  return (guint)(filestat.st_size);
}

guint
sget_file_size(gchar *name) {
  // get the size of file fd
  struct stat filestat;
  int fd;

  if ((fd=open(name,O_RDONLY))>0) {
    close (fd);
  }
  else {
    return (guint)0;
  }
  stat(name,&filestat);
  return (guint)(filestat.st_size);
}


void 
reget_afilesize (file *file) {
  // re-get the audio file size
  // NB. file here is a struct _file
  gchar *afile=g_strdup_printf ("%s/%s/audio",prefs->tmpdir,file->handle);
  if (!(file->afilesize=sget_file_size (afile))) {
    if (file->opening==FALSE) {
      file->arate=file->achans=file->asampsize=file->arps=0;
    }
  }
  g_free (afile);
}





void
colour_equal(GdkColor *c1, const GdkColor *c2) {
  c1->pixel=c2->pixel;
  c1->red=c2->red;
  c1->green=c2->green;
  c1->blue=c2->blue;
}


gboolean
create_event_space(gint length) {
  // try to create desired events
  // if we run out of memory, all events requested are freed, and we return FALSE
  // otherwise we return TRUE

  // NOTE: this is the OLD event system, it's only used for reordering in the clip editor

  if (cfile->events[0]!=NULL) {
    g_free(cfile->events[0]);
  }
  if ((cfile->events[0]=(event *)(g_try_malloc(sizeof(event)*length)))==NULL) {
    // memory overflow
    return FALSE;
  }
  memset(cfile->events[0],0,length*sizeof(event));
  return TRUE;
}




gint lives_list_index (GList *list, const gchar *data) {
  // find data in list, GTK's version is broken

  int i;
  gint len;
  if (list==NULL) return -1;

  len=g_list_length (list);

  for (i=0;i<len;i++) {
    if (!strcmp (g_list_nth_data (list,i),data)) return i;
  }
  return -1;
}






void 
add_to_recent(const gchar *filename, gdouble start, gint frames) {
  gchar buff[256];
  gchar *file;

  if (frames>0) {
    file=g_strdup_printf ("%s|%.2f|%d",filename,start,frames);
  }
  else {
    file=g_strdup (filename);
  }


  get_menu_text(mainw->recent1,buff);
  if (strcmp(file,buff)) {
    get_menu_text(mainw->recent2,buff);
    if (strcmp(file,buff)) {
      get_menu_text(mainw->recent3,buff);
      if (strcmp(file,buff)) {
	// not in list, or at pos 4
	get_menu_text(mainw->recent3,buff);
	set_menu_text(mainw->recent4,buff,FALSE);
	set_pref("recent4",buff);
	
	get_menu_text(mainw->recent2,buff);
	set_menu_text(mainw->recent3,buff,FALSE);
	set_pref("recent3",buff);
	
	get_menu_text(mainw->recent1,buff);
	set_menu_text(mainw->recent2,buff,FALSE);
	set_pref("recent2",buff);
	
	set_menu_text(mainw->recent1,file,FALSE);
	set_pref("recent1",file);
      }
      else {
	// #3 in list
	get_menu_text(mainw->recent2,buff);
	set_menu_text(mainw->recent3,buff,FALSE);
	set_pref("recent3",buff);
	
	get_menu_text(mainw->recent1,buff);
	set_menu_text(mainw->recent2,buff,FALSE);
	set_pref("recent2",buff);
	
	set_menu_text(mainw->recent1,file,FALSE);
	set_pref("recent1",filename);
      }
    }
    else {
      // #2 in list
	get_menu_text(mainw->recent1,buff);
	set_menu_text(mainw->recent2,buff,FALSE);
	set_pref("recent2",buff);
	
	set_menu_text(mainw->recent1,file,FALSE);
	set_pref("recent1",filename);
    }
  }
  else {
    // I'm number 1, so why change ;-)
  }

    get_menu_text(mainw->recent1,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent1);
    }
    get_menu_text(mainw->recent2,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent2);
    }
    get_menu_text(mainw->recent3,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent3);
    }
    get_menu_text(mainw->recent4,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent4);
    }
}


gint 
verhash (gchar *version) {
  gchar *s;
  gint major=0;
  gint minor=0;
  gint micro=0;

  if (!(strlen(version))) return 0;

  s=strtok (version,".");
  if (!(s==NULL)) {
    major=atoi (s);
    s=strtok (NULL,".");
    if (!(s==NULL)) {
      minor=atoi (s);
      s=strtok (NULL,".");
      if (!(s==NULL)) {
	micro=atoi (s);
      }
    }
  }
  return major*1000000+minor*1000+micro;
}



// TODO - move into undo.c
void 
set_undoable (const gchar *what, gboolean sensitive) {
  if (mainw->current_file>-1) {
    cfile->redoable=FALSE;
    cfile->undoable=sensitive;
    if (!(what==NULL)) {
      gchar *what_safe=g_strdelimit (g_strdup (what),"_",' ');
      g_snprintf(cfile->undo_text,32,_ ("_Undo %s"),what_safe);
      g_snprintf(cfile->redo_text,32,_ ("_Redo %s"),what_safe);
      g_free (what_safe);
    }
    else {
      cfile->undoable=FALSE;
      cfile->undo_action=UNDO_NONE;
      g_snprintf(cfile->undo_text,32,_ ("_Undo"));
      g_snprintf(cfile->redo_text,32,_ ("_Redo"));
    }
    set_menu_text(mainw->undo,cfile->undo_text,TRUE);
    set_menu_text(mainw->redo,cfile->redo_text,TRUE);
  }

  gtk_widget_hide(mainw->redo);
  gtk_widget_show(mainw->undo);
  gtk_widget_set_sensitive (mainw->undo,sensitive);
}

void 
set_redoable (const gchar *what, gboolean sensitive) {
  if (mainw->current_file>-1) {
    cfile->undoable=FALSE;
    cfile->redoable=sensitive;
    if (!(what==NULL)) {
      gchar *what_safe=g_strdelimit (g_strdup (what),"_",' ');
      g_snprintf(cfile->undo_text,32,_ ("_Undo %s"),what_safe);
      g_snprintf(cfile->redo_text,32,_ ("_Redo %s"),what_safe);
      g_free (what_safe);
    }
    else {
      cfile->redoable=FALSE;
      cfile->undo_action=UNDO_NONE;
      g_snprintf(cfile->undo_text,32,_ ("_Undo"));
      g_snprintf(cfile->redo_text,32,_ ("_Redo"));
    }
    set_menu_text(mainw->undo,cfile->undo_text,TRUE);
    set_menu_text(mainw->redo,cfile->redo_text,TRUE);
  }

  gtk_widget_hide(mainw->undo);
  gtk_widget_show(mainw->redo);
  gtk_widget_set_sensitive (mainw->redo,sensitive);
}


void 
set_sel_label (void) {
  gchar *tstr,*frstr;

  if (mainw->current_file==-1||!cfile->frames) {
    gtk_label_set_text(GTK_LABEL(mainw->sel_label),_ ("-------------Selection------------"));
  }
  else {
    tstr=g_strdup_printf ("%.2f",calc_variable_time (cfile->end+1)-calc_variable_time (cfile->start));
    frstr=g_strdup_printf ("%d",cfile->end-cfile->start+1);
    gtk_label_set_text(GTK_LABEL(mainw->sel_label),g_strconcat (_ ("---------- [ "),tstr,_(" sec ] ----------Selection---------- [ "),frstr,_ (" frames ] ----------"),NULL));
    g_free (frstr);
    g_free (tstr);
  }
  gtk_widget_queue_draw (mainw->sel_label);
}




guint 
get_signed_endian (int asigned, int endian) {
  if (asigned==1) {
    if (endian==1) {
      return 0;
    }
    if (endian==0) {
      return AFORM_BIG_ENDIAN;
    }
  }
  else {
    if (asigned==0) { 
      if (endian==1) {
	return AFORM_UNSIGNED;
      }
      if (endian==0) {
	return AFORM_UNSIGNED|AFORM_BIG_ENDIAN;
      }
    }
  }
  return AFORM_UNKNOWN;
}



inline void 
clear_mainw_msg (void) {
  memset (mainw->msg,0,512);
}


gint 
get_token_count (gchar *string, int delim) {
  gint pieces=1;
  if (string==NULL) return 0;
  if (delim<=0||delim>255) return 1;

  while ((string=index (string,delim))!=NULL) {
    pieces++;
    string++;
  }
  return pieces;
}



gchar *subst (gchar *string, gchar *from, gchar *to) {
  // return a string with all occurrences of from replaced with to
  gchar *search,*lastsearch;
  gchar *ret="";

  search=lastsearch=string;

  while (search!=NULL) {
    if ((search=strstr (lastsearch,from))==NULL) {
      ret=g_strconcat (ret,lastsearch,NULL);
    }
    else {
      memset (search,0,1);
      ret=g_strconcat (ret,lastsearch,to,NULL);
      lastsearch=search+strlen (from);
    }
  }
  return ret;
}





void combo_set_popdown_strings (GtkCombo *combo, GList *list) {
  // this avoids an assert in some versions of GTK+ when list==NULL
  GList *empty_list=NULL;
  empty_list=g_list_append (empty_list,"");
  if (list==NULL) gtk_combo_set_popdown_strings (combo,empty_list);
  else gtk_combo_set_popdown_strings (combo,list);
  g_list_free (empty_list);
}


void 
get_border_size (GtkWidget *win, gint *bx, gint *by) {
  GdkRectangle rect;
  gint wx,wy;
  gdk_window_get_frame_extents (GDK_WINDOW (win->window),&rect);
  gdk_window_get_origin (GDK_WINDOW (win->window), &wx, &wy);
  bx[0]=wx-rect.x;
  by[0]=wy-rect.y;
}



gint hextodec (const gchar *string) {
  int i;
  gint tot=0;
  gchar test[2];

  memset (test+1,0,1);

  for (i=0;i<strlen (string);i++) {
    tot*=16;
    memcpy (test,&string[i],1);
    tot+=get_hex_digit (test);
  }
  return tot;
}

gint get_hex_digit (gchar *c) {
  if (!strcmp (c,"a")||!strcmp (c,"A")) return 10;
  if (!strcmp (c,"b")||!strcmp (c,"B")) return 11;
  if (!strcmp (c,"c")||!strcmp (c,"C")) return 12;
  if (!strcmp (c,"d")||!strcmp (c,"D")) return 13;
  if (!strcmp (c,"e")||!strcmp (c,"E")) return 14;
  if (!strcmp (c,"f")||!strcmp (c,"F")) return 15;
  return(atoi (c));
}



static guint32 fastrand_val;

inline guint32 fastrand(void)
{
#define rand_a 1073741789L
#define rand_c 32749L

  return (fastrand_val= rand_a * fastrand_val + rand_c);
}

void fastsrand(guint32 seed)
{
	fastrand_val = seed;
}




GdkPixmap*
gdk_pixmap_copy (GdkPixmap *pixmap)
{
  GdkPixmap *pixmap_out;
  GdkGC *gc;
  gint width, height, depth;
  
  if (pixmap==NULL) return NULL;
  
  gdk_drawable_get_size (pixmap, &width, &height);
  depth = gdk_drawable_get_depth (pixmap);
  pixmap_out = gdk_pixmap_new (NULL, width, height, depth);
  gc = gdk_gc_new (pixmap);
  gdk_draw_drawable (pixmap_out, gc, pixmap, 0, 0, 0, 0, width, height);
  g_object_unref (gc);
  return pixmap_out;
}

