// callbacks.c
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// released under the GNU GPL 2.0
// see file ../COPYING for licensing details


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <inttypes.h>

#include "support.h"
#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "effects.h"
#include "resample.h"
#include "rte_window.h"
#include "events.h"

#ifdef ENABLE_JACK
#include "jack.h"
#endif


static gchar file_name[256];


gboolean
on_LiVES_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
  catch_sigint (0);
  return FALSE;
}


void 
lives_exit (void) {
  if (mainw->is_ready==TRUE) {
    int i,j;
    gchar *com;

    if (mainw->playing_file>-1) {
      gtk_timeout_remove (mainw->kb_timer);
      if (mainw->ext_playback==TRUE) {
	(*vid_playback_plugin->exit_screen)(mainw->ptr_x,mainw->ptr_y);
      }
      if (vid_playback_plugin!=NULL) {
	g_module_close (vid_playback_plugin->gmodule);
      }

      com=g_strdup_printf ("touch %s/%s/.stoploop 2>/dev/null",prefs->tmpdir,mainw->files[mainw->playing_file]->handle);
      system(com);
      g_free (com);
      com=g_strdup_printf("smogrify stop %s",mainw->files[mainw->playing_file]->handle);
      system(com);
      g_free(com);
    }

    if (mainw->current_file>-1) {
      if (cfile->handle!=NULL) {
	com=g_strdup_printf("smogrify cancel %s -1",cfile->handle);
	system(com);
	g_free(com);
      }
    }
    
#ifdef ENABLE_JACK
    lives_jack_end();
#endif

    // hide the main window
    while (g_main_context_iteration (NULL,FALSE));

    if (strcmp(future_prefs->tmpdir,prefs->tmpdir)) {
      // if we changed the tempdir, remove everything but sets from the old dir
      // create the new directory, and then move any sets over
      com=g_strdup_printf("smogrify weed %s &",future_prefs->tmpdir);
      system (com);
      g_free (com);
    }
    else {
      for (i=0;i<=MAX_FILES;i++) {
	if (!(mainw->files[i]==NULL)) {
	  if (mainw->leave_files==FALSE||i==0) {
	    // close all open clips
	    com=g_strdup_printf("smogrify close %s",mainw->files[i]->handle);
	    system(com);
	    g_free(com);
	  }
	  else {
	    // or just clean them up
	    com=g_strdup_printf("smogrify clear_tmp_files %s",mainw->files[i]->handle);
	    system(com);
	    g_free(com);
	  }
	  for (j=0;j<1;j++) {
	    if (mainw->files[i]->events[j]!=NULL) {
	      g_free(mainw->files[i]->events[j]);
	    }
	  }
	  g_free(mainw->files[i]);
	}
      }
    }
    save_future_prefs();
    if (mainw->sep_win==TRUE&&(mainw->playing_file>-1||prefs->sepwin_type==1)) {
      kill_play_window();
    }
  }
  
  if (capable->smog_version_correct) {
    plugin_request("encoders",prefs->encoder.name,"finalise");

    weed_unload_all();

    rfx_free_all();

#ifdef ENABLE_OSC
    if (prefs->osc_udp_started) lives_osc_end();
#endif
  }
  g_free (mainw);
  exit(0);
}



gboolean
return_true                  (GtkWidget       *widget,
			      GdkEvent        *event,
		      	      gpointer         user_data)
{
  return TRUE;
}


void
on_filesel_complex_clicked                      (GtkButton *button,
						 GtkEntry *entry)
{
  on_filesel_simple_clicked (button,entry);
  g_signal_handler_disconnect (GTK_FILE_SELECTION (mainw->fs1)->ok_button,mainw->hnd_id);
  g_signal_connect (GTK_FILE_SELECTION(mainw->fs1)->ok_button, "clicked",G_CALLBACK (on_filesel_complex_ok_clicked),entry);
}


void
on_filesel_simple_clicked (GtkButton *button, GtkEntry *entry) {
  // callback for directory browsing buttons
  while (g_main_context_iteration(NULL,FALSE));
  mainw->fs1 = create_fileselection (_ ("Choose Directory"),0,NULL);

  g_snprintf(file_name,256,"%s",gtk_entry_get_text(entry));
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(mainw->fs1), g_filename_from_utf8 (file_name,-1,NULL,NULL,NULL));
  
  mainw->hnd_id=g_signal_connect (GTK_FILE_SELECTION(mainw->fs1)->ok_button, "clicked",G_CALLBACK (on_filesel_simple_ok_clicked),entry);
  gtk_widget_show (mainw->fs1);
}


void
on_filesel_simple_ok_clicked (GtkButton *button, GtkEntry *entry) {
  g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(mainw->fs1)),-1,NULL,NULL,NULL));
  get_dirname(file_name);
  gtk_widget_destroy(mainw->fs1);
  gtk_entry_set_text(entry,file_name);
  gtk_widget_queue_draw(mainw->LiVES);
  gtk_widget_queue_draw(gtk_widget_get_toplevel(GTK_WIDGET(entry)));
  while (g_main_context_iteration(NULL,FALSE));
}


void
on_filesel_complex_ok_clicked                  (GtkButton       *button,
						GtkEntry         *entry)
{
  // append '/livestmp' at the end
  g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(mainw->fs1)),-1,NULL,NULL,NULL));
  get_dirname(file_name);
  if (strlen (file_name)<10||strncmp (file_name+strlen (file_name)-10,"/livestmp/",10)) g_strappend (file_name,256,"livestmp/");
  gtk_widget_destroy(mainw->fs1);
  gtk_entry_set_text(entry,file_name);
  gtk_widget_queue_draw(mainw->LiVES);
  gtk_widget_queue_draw(gtk_widget_get_toplevel(GTK_WIDGET(entry)));
  while (g_main_context_iteration(NULL,FALSE));
}



void
on_open_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *fileselection;

  while (g_main_context_iteration(NULL,FALSE));

  fileselection = create_fileselection (_ ("Select File"),1,NULL);

  if (strlen(mainw->vid_load_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->vid_load_dir,-1,NULL,NULL,NULL));
    }
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_ok_button1_clicked),NULL);
  gtk_widget_show (fileselection);
}

void
on_open_sel_activate                      (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  GtkWidget *fileselection;

  while (g_main_context_iteration(NULL,FALSE));

  fileselection = create_fileselection (_ ("Select File"),1,NULL);

  if (strlen(mainw->vid_load_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->vid_load_dir,-1,NULL,NULL,NULL));
  } 
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_open_sel_ok_button_clicked),NULL);
  gtk_widget_show (fileselection);
}

void
on_open_vcd_activate                      (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  GtkWidget *vcdtrack_dialog;
  
  mainw->fx1_val=1;
  mainw->fx2_val=1;
  vcdtrack_dialog = create_cdtrack_dialog (GPOINTER_TO_INT (user_data));
  gtk_widget_show (vcdtrack_dialog);
}


void
on_open_loc_activate                      (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  locw=create_location_dialog();
  gtk_widget_show(locw->loc_dialog);

}

void
on_locbw_toggled                (GtkToggleButton *togglebutton,
				 gpointer         user_data)
{
  prefs->no_bandwidth=!prefs->no_bandwidth;
}


void
on_open_fw_activate                      (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  open_fw_device();
}


void
on_recent_activate                      (GtkMenuItem     *menuitem,
					 gpointer         user_data) {

  gchar file[256];
  gdouble start=0.;
  gint end=0;

  get_menu_text(GTK_WIDGET(menuitem),file);
  while (g_main_context_iteration (NULL,FALSE));
  if (get_token_count (file,'|')>2) {
    gchar **array=g_strsplit(file,"|",3);
    g_snprintf (file,256,"%s",array[0]);
    start=g_strtod (array[1],NULL);
    end=atoi (array[2]);
    g_strfreev (array);
  }
  deduce_file(file,start,end);
}



void
on_location_select                   (GtkButton       *button,
				      gpointer         user_data)
{
  g_snprintf(file_name,256,"%s",gtk_entry_get_text(GTK_ENTRY(locw->entry)));
  gtk_widget_destroy(locw->loc_dialog);
  while (g_main_context_iteration (NULL,FALSE));
  g_free(locw);

  // the next line tells us to add the 'stop' button
  mainw->opening_loc=TRUE;
  if (prefs->no_bandwidth) {
    mainw->file_open_params=g_strdup ("nobandwidth");
  }
  else mainw->file_open_params=g_strdup ("sendbandwidth");
  open_file(file_name);
}


void
on_stop_clicked (GtkMenuItem     *menuitem,
		 gpointer         user_data)
{
// 'enough' button for open location
  gchar *com=g_strdup_printf("smogrify stop %s",cfile->handle);
  system(com);
  g_free(com);
  gtk_widget_set_sensitive(cfile->proc_ptr->stop_button, FALSE);
  gtk_widget_set_sensitive(cfile->proc_ptr->preview_button, FALSE);
  gtk_widget_set_sensitive(cfile->proc_ptr->cancel_button, FALSE);
}



void
on_save_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // save
  while (g_main_context_iteration(NULL,FALSE));

  if (cfile->event_list!=NULL) {
    do_error_dialog (_ ("Sorry, cannot save with reordered frames.\nPlease reorder frames from the Tools menu, \nor clear the recording by pressing 'r' twice.\n"));
    return;
  }

  mainw->save_all=TRUE;
  save_file(TRUE,cfile->file_name);
}


void
on_save_as_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  if (cfile->frames==0) {
    on_export_audio_activate (NULL,NULL);
    return;
  }

  while (g_main_context_iteration(NULL,FALSE));

  if (cfile->event_list!=NULL) {
    do_error_dialog (_ ("Sorry, cannot save with reordered frames.\nPlease reorder frames from the Tools menu, \nor clear the recording by pressing 'r' twice.\n"));
    return;
  }

  mainw->save_all=TRUE;
  fileselection = create_fileselection (_ ("Select File"),0,NULL);

  if (strlen(mainw->vid_save_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->vid_save_dir,-1,NULL,NULL,NULL));
  }
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_ok_button3_clicked),NULL);
  gtk_widget_show (fileselection);
}


void
on_save_selection_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  if (!(cfile->event_list==NULL)) {
    do_error_dialog(_ ("\n\nThis file has frame changes recorded. You must click on Tools|Reorder Frames before you can save a selection.\n\nYou can choose to Undo the reorder afterwards.\n"));
    return;
  }

  while (g_main_context_iteration(NULL,FALSE));

  mainw->save_all=FALSE;

  fileselection = create_fileselection ("Select File",0,NULL);
  if (strlen(mainw->vid_save_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->vid_save_dir,-1,NULL,NULL,NULL));
  }
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_ok_button3_clicked),NULL);
  gtk_widget_show (fileselection);
}


void
on_close_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gchar *warn;
  gchar title[256];

  if (cfile->changed==TRUE) {
    get_menu_text(cfile->menuentry,title);
    warn=g_strdup_printf(_ ("\n%s\nhas not been saved or backed up.\n\nReally close it ?"),title);
    if (!do_warning_dialog(warn)) {
      g_free(warn);
      return;
    }
    g_free(warn);
  }
  close_current_file();
}



void
on_restore_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;
  while (g_main_context_iteration(NULL,FALSE));

  fileselection = create_fileselection (_ ("Restore .lv1 file"),0,NULL);
  if (strlen(mainw->proj_load_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->proj_load_dir,-1,NULL,NULL,NULL));
  }
  gtk_file_selection_complete(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 ("*.lv1",-1,NULL,NULL,NULL));
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_restore_ok_clicked),NULL);
  gtk_widget_show (fileselection);
}



void
on_backup_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  while (g_main_context_iteration(NULL,FALSE));

  fileselection = create_fileselection (_ ("Backup as .lv1 file"),0,NULL);
  if (strlen(mainw->proj_save_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->proj_save_dir,-1,NULL,NULL,NULL));
  }
  gtk_file_selection_complete(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 ("*.lv1",-1,NULL,NULL,NULL));
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_backup_ok_clicked),NULL);
  gtk_widget_show (fileselection);

}


void
on_backup_ok_clicked                  (GtkButton       *button,
				       gpointer         user_data)
{
  g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  mainw->save_all=TRUE;
  backup_file(file_name);

  g_snprintf(mainw->proj_save_dir,256,"%s",file_name);
  get_dirname (mainw->proj_save_dir);
}

void
on_restore_ok_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  restore_file(file_name);

  g_snprintf(mainw->proj_load_dir,256,"%s",file_name);
  get_dirname (mainw->proj_load_dir);

}



void
on_quit_activate                      (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  gboolean changed=FALSE;
  int i;
  gchar *warn;
  
  // stop if playing
  if (mainw->playing_file>-1) {
    mainw->cancelled=CANCEL_APP_QUIT;
    return;
  }

  // check for unsaved files
  for (i=1;i<MAX_FILES;i++) {
    if (!(mainw->files[i]==NULL)) {
      if (mainw->files[i]->changed==TRUE) {
	changed=TRUE;
	break;
      }
    }
  }
  if (mainw->was_set==TRUE&&mainw->clips_available>0) {
    warn=g_strdup(_ ("\nIf you quit now, the saved set will be lost.\nTo keep the clips on disk, use Save Set again.\n\nAre you sure you want to quit ?"));
    if (!do_warning_dialog(warn)) {
      // reset mainw->cancelled in case we are playing
      mainw->cancelled=0;
      g_free(warn);
      return;
    }
    g_free(warn);
  }
  if (changed==TRUE) {
    warn=g_strdup(_ ("\nSome files have not been saved or backed up.\n\nQuit anyway ?"));
    if (!do_warning_dialog(warn)) {
      // reset mainw->cancelled in case we are playing
      mainw->cancelled=0;
      g_free(warn);
      return;
    }
    g_free(warn);
  }
  else {
    if (!(user_data==NULL)) {
      do_blocking_error_dialog (_ ("\n\nLiVES is exiting now\n"));
      while (g_main_context_iteration (NULL,FALSE));
    }
  }
  lives_exit();
}



// TODO - split into undo.c
void
on_undo_activate                      (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  gchar *com;
  gint ostart=cfile->start;
  gint oend=cfile->end;
  gchar msg[256];
  gint current_file=mainw->current_file;
  gint switch_file=current_file;

  gtk_widget_set_sensitive (mainw->undo, FALSE);
  gtk_widget_set_sensitive (mainw->redo, TRUE);
  cfile->undoable=FALSE;
  cfile->redoable=TRUE;
  gtk_widget_hide(mainw->undo);
  gtk_widget_show(mainw->redo);

  mainw->osc_block=TRUE;

  if (!(menuitem==NULL)) {
    get_menu_text (mainw->undo,msg);
    d_print(msg);
    d_print ("...");
  }

  // TODO - maybe use a switch()

  if (cfile->undo_action==UNDO_INSERT_SILENCE) {
    on_del_audio_activate(NULL,NULL);
    cfile->undo_action=UNDO_INSERT_SILENCE;
    set_redoable(_("Insert Silence"),TRUE);
  }

  if (cfile->undo_action==UNDO_CUT||cfile->undo_action==UNDO_DELETE||cfile->undo_action==UNDO_DELETE_AUDIO) {
    unlink(cfile->info_file);

    cfile->arate=cfile->undo_arate;
    cfile->signed_endian=cfile->undo_signed_endian;
    cfile->achans=cfile->undo_achans;
    cfile->asampsize=cfile->undo_asampsize;
    cfile->arps=cfile->undo_arps;

    if (cfile->frames==0) {
      cfile->hsize=cfile->ohsize;
      cfile->vsize=cfile->ovsize;
    }

    if (cfile->undo_action==UNDO_DELETE_AUDIO) {
      com=g_strdup_printf("smogrify insert %s %.8f 0. %.8f %s 2 0 0 0 0 %d %d %d -1",cfile->handle,cfile->undo1_dbl,cfile->undo2_dbl-cfile->undo1_dbl, cfile->handle, cfile->arps, cfile->achans, cfile->asampsize);
    }
    else {
      cfile->undo1_boolean&=mainw->ccpd_with_sound;
      com=g_strdup_printf("smogrify insert %s %d %d %d %s %d %d 0 0 %.3f %d %d %d -1",cfile->handle,cfile->undo_start-1,cfile->undo_start,cfile->undo_end,cfile->handle, cfile->undo1_boolean, cfile->frames, cfile->fps, cfile->arps, cfile->achans, cfile->asampsize);
    }

    system(com);
    // show a progress dialog, not cancellable
    do_auto_dialog(_ ("Undoing"));
    g_free(com);

    if (cfile->undo_action!=UNDO_DELETE_AUDIO) {
      cfile->insert_start=cfile->undo_start;
      cfile->insert_end=cfile->undo_end;
      
      if (cfile->start>=cfile->undo_start) {
	cfile->start+=cfile->undo_end-cfile->undo_start+1;
      }
      if (cfile->end>=cfile->undo_start) {
	cfile->end+=cfile->undo_end-cfile->undo_start+1;
      }
      
      cfile->frames+=cfile->undo_end-cfile->undo_start+1;
      if (cfile->frames>0) {
	if (cfile->start==0) {
	  cfile->start=1;
	}
	if (cfile->end==0) {
	  cfile->end=cfile->frames;
	}
      }
    }
    reget_afilesize (cfile);
    get_play_times();
  }


  if (cfile->undo_action==UNDO_RESIZABLE||cfile->undo_action==UNDO_RENDER||cfile->undo_action==UNDO_EFFECT||cfile->undo_action==UNDO_MERGE||(cfile->undo_action==UNDO_ATOMIC_RESAMPLE_RESIZE&&(cfile->frames!=cfile->old_frames||cfile->hsize!=cfile->ohsize||cfile->vsize!=cfile->ovsize||cfile->fps!=cfile->undo1_dbl))) {
    com=g_strdup_printf("smogrify undo %s %d %d",cfile->handle,cfile->undo_start,cfile->undo_end);
    unlink(cfile->info_file);
    system(com);
    // show a progress dialog, not cancellable
    cfile->progress_start=cfile->undo_start;
    cfile->progress_end=cfile->undo_end;
    do_progress_dialog(TRUE,FALSE,_ ("Undoing"));
    g_free(com);
  }

  if (cfile->undo_action==UNDO_ATOMIC_RESAMPLE_RESIZE&&(cfile->frames!=cfile->old_frames||cfile->hsize!=cfile->ohsize||cfile->vsize!=cfile->ovsize)) {
    cfile->frames=cfile->old_frames;
    cfile->hsize=cfile->ohsize;
    cfile->vsize=cfile->ovsize;
    cfile->fps=cfile->undo1_dbl;
    cfile->redoable=FALSE;
    // force a resize in switch_to_file
    switch_file=0;
  }

  if (cfile->undo_action==UNDO_RENDER) cfile->frames=cfile->old_frames;

  if (cfile->undo_action==UNDO_INSERT||cfile->undo_action==UNDO_MERGE) {
    gboolean ccpd_with_sound=mainw->ccpd_with_sound;
    if (!(cfile->undo_action==UNDO_MERGE&&cfile->insert_start==cfile->undo_start&&cfile->insert_end==cfile->undo_end)) {
      if (cfile->undo_action==UNDO_MERGE) {
	if (cfile->insert_start==cfile->undo_start) {
	  cfile->insert_start=cfile->undo_end+1;
	}
	if (cfile->insert_end==cfile->undo_end) {
	  cfile->insert_end=cfile->undo_start-1;
	}
      }
      cfile->start=cfile->insert_start;
      cfile->end=cfile->insert_end;
      on_delete_activate (NULL,NULL);
      cfile->start=ostart;
      if (ostart>=cfile->insert_start) {
	cfile->start-=cfile->insert_end-cfile->insert_start+1;
	if (cfile->start<cfile->insert_start-1) {
	  cfile->start=cfile->insert_start-1;
	}
      }
      cfile->end=oend;
      if (oend>=cfile->insert_start) {
	cfile->end-=cfile->insert_end-cfile->insert_start+1;
	if (cfile->end<cfile->insert_start-1) {
	  cfile->end=cfile->insert_start-1;
	}
      }
      // TODO - use lives_clip_start macro
      if (cfile->start<1) cfile->start=cfile->frames>0?1:0;
      if (cfile->end<1) cfile->end=cfile->frames>0?1:0;
      
      mainw->ccpd_with_sound=ccpd_with_sound;
      cfile->insert_start=cfile->insert_end=0;
    }
  }
  
  if (cfile->undo_action==UNDO_AUDIO_RESAMPLE||cfile->undo_action==UNDO_TRIM_AUDIO||cfile->undo_action==UNDO_APPEND_AUDIO||cfile->undo_action==UNDO_NEW_AUDIO||(cfile->undo_action==UNDO_ATOMIC_RESAMPLE_RESIZE&&cfile->arate!=cfile->undo1_int)) {
    // TODO - check arps
    unlink(cfile->info_file);
    com=g_strdup_printf("smogrify undo_audio %s",cfile->handle);
    system (com);
    do_auto_dialog(_ ("Undoing"));
    g_free (com);
  }

  if ((cfile->undo_action==UNDO_AUDIO_RESAMPLE)||(cfile->undo_action==UNDO_ATOMIC_RESAMPLE_RESIZE&&cfile->arate!=cfile->undo1_int)) {
    cfile->arate+=cfile->undo1_int;
    cfile->undo1_int=cfile->arate-cfile->undo1_int;
    cfile->arate-=cfile->undo1_int;

    cfile->achans+=cfile->undo2_int;
    cfile->undo2_int=cfile->achans-cfile->undo2_int;
    cfile->achans-=cfile->undo2_int;

    cfile->asampsize+=cfile->undo3_int;
    cfile->undo3_int=cfile->asampsize-cfile->undo3_int;
    cfile->asampsize-=cfile->undo3_int;

    cfile->arps+=cfile->undo4_int;
    cfile->undo4_int=cfile->arps-cfile->undo4_int;
    cfile->arps-=cfile->undo4_int;

    cfile->signed_endian+=cfile->undo1_uint;
    cfile->undo1_uint=cfile->signed_endian-cfile->undo1_uint;
    cfile->signed_endian-=cfile->undo1_uint;
  }
    
  if (cfile->undo_action==UNDO_CHANGE_SPEED) {
    cfile->fps+=cfile->undo1_dbl;
    cfile->undo1_dbl=cfile->fps-cfile->undo1_dbl;
    cfile->fps-=cfile->undo1_dbl;

    cfile->arate+=cfile->undo1_int;
    cfile->undo1_int=cfile->arate-cfile->undo1_int;
    cfile->arate-=cfile->undo1_int;
  }

  if (cfile->undo_action==UNDO_INSERT||cfile->undo_action==UNDO_MERGE||cfile->undo_action==UNDO_APPEND_AUDIO||cfile->undo_action==UNDO_NEW_AUDIO) {
    cfile->redoable=FALSE;
  }

  if (cfile->undo_action==UNDO_REORDER) {
    g_signal_handler_block(mainw->record_perf,mainw->record_perf_func);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->record_perf),FALSE);
    g_signal_handler_unblock(mainw->record_perf,mainw->record_perf_func);

    // deorder the frames 
    cfile->frames=deorder_frames(cfile->old_frames);
    com=g_strdup_printf(_ ("Length of video is now %d frames (with performance data).\n"),cfile->frames);
    d_print(_ ("done.\n"));
    d_print(com);
    g_free(com);
  }
  else {
    if (menuitem!=NULL) {
      d_print(_ ("done.\n"));
    }
  }

  if (cfile->undo_action==UNDO_RESAMPLE) {
    cfile->start=(gint)((cfile->start-1)/cfile->fps*cfile->undo1_dbl+1.);
    if ((cfile->end=(gint) (cfile->end/cfile->fps*cfile->undo1_dbl+.49999))<1) cfile->end=1;
    cfile->fps+=cfile->undo1_dbl; 
    cfile->undo1_dbl=cfile->fps-cfile->undo1_dbl;
    cfile->fps-=cfile->undo1_dbl;

    // deorder the frames
    cfile->frames=deorder_frames(cfile->old_frames);
    cfile->variable_fps=cfile->undo1_boolean;
    if (mainw->current_file>0) {
      if (cfile->event_list==NULL) com=g_strdup_printf(_ ("Length of video is now %d frames at %.3f frames per second.\n"),cfile->frames,cfile->fps);
      else com=g_strdup_printf(_ ("Length of video is now %d frames (with performance data).\n"),cfile->frames);
    }
    else {
      com=g_strdup_printf(_ ("Clipboard was resampled to %d frames.\n"),cfile->frames);
    }
    d_print(com);
    g_free(com);
  }

  if (cfile->end>cfile->frames) {
    cfile->end=cfile->frames;
  }

  if (cfile->undo_action==UNDO_RESIZABLE) {
    cfile->vsize+=cfile->ovsize;
    cfile->ovsize=cfile->vsize-cfile->ovsize;
    cfile->vsize-=cfile->ovsize;
    cfile->hsize+=cfile->ohsize;
    cfile->ohsize=cfile->hsize-cfile->ohsize;
    cfile->hsize-=cfile->ohsize;
    // force a resize in switch_to_file
    switch_file=0;
  }
  
  if (current_file>0) {
    switch_to_file((mainw->current_file=switch_file),current_file);
  }

  if (cfile->undo_action==UNDO_RENDER) {
    if (mainw->event_list!=NULL) event_list_free (mainw->event_list);
    mainw->event_list=cfile->event_list_back;
    cfile->event_list_back=NULL;
    if (cfile->event_list!=NULL) {
      event_list_free (cfile->event_list);
      cfile->event_list=NULL;
    }
    deal_with_render_choice(FALSE);
  }
  mainw->osc_block=FALSE;

}

void
on_redo_activate                      (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  gchar *com;
  gint ostart=cfile->start;
  gint oend=cfile->end;
  gint current_file=mainw->current_file;
  gchar msg[256];

  mainw->osc_block=TRUE;

  cfile->undoable=TRUE;
  cfile->redoable=FALSE;
  gtk_widget_hide(mainw->redo);
  gtk_widget_show(mainw->undo);
  gtk_widget_set_sensitive(mainw->undo,TRUE);
  gtk_widget_set_sensitive(mainw->redo,FALSE);

  if (!(menuitem==NULL)) {
    get_menu_text (mainw->redo,msg);
    d_print(msg);
    d_print ("...");
  }

  if (cfile->undo_action==UNDO_INSERT_SILENCE) {
    on_ins_silence_activate (NULL,NULL);
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_TRIM_AUDIO) {
    on_trim_audio_activate (NULL,NULL);
    d_print(_ ("done\n"));
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_CHANGE_SPEED) {
    on_change_speed_ok_clicked (NULL,NULL);
    d_print(_ ("done\n"));
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_REORDER) {
    on_reorder_activate (NULL,NULL);
    d_print(_ ("done\n"));
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_RESAMPLE) {
    on_resample_vid_ok (NULL,NULL);
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_AUDIO_RESAMPLE) {
    on_resaudio_ok_clicked (NULL,NULL);
    d_print(_ ("done\n"));
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_CUT||cfile->undo_action==UNDO_DELETE) {
    cfile->start=cfile->undo_start;
    cfile->end=cfile->undo_end;
    mainw->osc_block=FALSE;
  }
  if (cfile->undo_action==UNDO_CUT) {
    on_cut_activate(NULL,NULL);
    mainw->osc_block=FALSE;
  }
  if (cfile->undo_action==UNDO_DELETE) {
    on_delete_activate(NULL,NULL);
    mainw->osc_block=FALSE;
  }
  if (cfile->undo_action==UNDO_DELETE_AUDIO) {
    on_del_audio_activate(NULL,NULL);
    mainw->osc_block=FALSE;
    return;
  }
  if (cfile->undo_action==UNDO_CUT||cfile->undo_action==UNDO_DELETE) {
    cfile->start=ostart;
    cfile->end=oend;
    if (mainw->current_file==current_file) {
      if (cfile->start>=cfile->undo_start) {
	cfile->start-=cfile->undo_end-cfile->undo_start+1;
	if (cfile->start<cfile->undo_start-1) {
	  cfile->start=cfile->undo_start-1;
	}
      }
      if (cfile->end>=cfile->undo_start) {
	cfile->end-=cfile->undo_end-cfile->undo_start+1;
	if (cfile->end<cfile->undo_start-1) {
	  cfile->end=cfile->undo_start-1;
	}
      }
      switch_to_file (mainw->current_file,mainw->current_file);
    }
    mainw->osc_block=FALSE;
    return;
  }

  com=g_strdup_printf("smogrify redo %s %d %d",cfile->handle,cfile->undo_start,cfile->undo_end);
  unlink(cfile->info_file);
  system(com);
  cfile->progress_start=cfile->undo_start;
  cfile->progress_end=cfile->undo_end;
  // show a progress dialog, not cancellable
  do_progress_dialog(TRUE,FALSE,_ ("Redoing"));
  g_free(com);

  if (cfile->start>=cfile->undo_start) load_start_image(cfile->start);
  if (cfile->end<=cfile->undo_end) load_end_image(cfile->end);

  if (cfile->undo_action==UNDO_RESIZABLE) {
    cfile->vsize+=cfile->ovsize;
    cfile->ovsize=cfile->vsize-cfile->ovsize;
    cfile->vsize-=cfile->ovsize;
    cfile->hsize+=cfile->ohsize;
    cfile->ohsize=cfile->hsize-cfile->ohsize;
    cfile->hsize-=cfile->ohsize;
    switch_to_file ((mainw->current_file=0),current_file);
  }
  d_print(_ ("done\n"));
  mainw->osc_block=FALSE;
}



//////////////////////////////////////////////////

void
on_copy_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gchar *com;
  gint current_file=mainw->current_file;
  gint start,end;

  gchar *text=g_strdup_printf(_ ("Copying frames %d to %d%s to the clipboard..."),cfile->start,cfile->end,mainw->ccpd_with_sound==TRUE&&cfile->achans>0?" (with sound)":"");
  d_print(text);
  g_free(text);

  if (clipboard==NULL) {
    // here is where we create the clipboard
    // use get_new_handle(clipnumber,name);
    if (get_new_handle(0,"clipboard")==FALSE) {
      mainw->error=TRUE;
      return;
    }
  }
  else if (clipboard->frames>0) { 
    // clear old clipboard
    // need to set current file to 0 before monitoring progress
    mainw->current_file=0;
    com=g_strdup_printf("smogrify delete_all %s",clipboard->handle);
    unlink(clipboard->info_file);
    system(com);
    cfile->progress_start=cfile->start;
    cfile->progress_end=cfile->end;
    // show a progress dialog, not cancellable
    do_progress_dialog(TRUE,FALSE,_ ("Clearing the clipboard"));
    g_free(com);
  }

  unlink(cfile->info_file);
  mainw->current_file=current_file;
  mainw->last_transition_loops=1;

  start=cfile->start;
  end=cfile->end;

  mainw->fx1_val=1;
  mainw->fx1_bool=FALSE;
  com=g_strdup_printf("smogrify insert %s 0 %d %d %s %d 0 0 0 %.3f %d %d %d",clipboard->handle,start,end,cfile->handle, mainw->ccpd_with_sound, cfile->fps, cfile->arate, cfile->achans, cfile->asampsize);

  // we need to set this to look at the right info_file
  mainw->current_file=0;
  cfile->progress_start=clipboard->start=1;
  cfile->progress_end=clipboard->end=end-start+1;

  system(com);

  // stop the 'preview' and 'pause' buttons from appearing
  cfile->nopreview=TRUE;
  if (do_progress_dialog(TRUE,TRUE,_ ("Copying to the clipboard"))==FALSE) {
    g_free(com);
    // close clipboard, it is invalid
    com=g_strdup_printf ("smogrify close %s",clipboard->handle);
    system (com);
    g_free (com);
    clipboard=NULL;
    mainw->current_file=current_file;
    sensitize();
    mainw->cancelled=CANCEL_USER;
    return;
  }
  g_free(com);
  cfile->nopreview=FALSE;
  mainw->current_file=current_file;

  //set all clipboard details
  clipboard->frames=clipboard->end;
  clipboard->hsize=cfile->hsize;
  clipboard->vsize=cfile->vsize;
  clipboard->bpp=cfile->bpp;
  clipboard->fps=cfile->fps;
  clipboard->is_loaded=TRUE;
  g_snprintf(clipboard->type,40,"Frames");

  clipboard->asampsize=clipboard->arate=clipboard->achans=clipboard->afilesize=0;

  if (mainw->ccpd_with_sound==TRUE) {
    clipboard->achans=cfile->achans;
    clipboard->asampsize=cfile->asampsize;

    clipboard->arate=cfile->arate;
    clipboard->arps=cfile->arps;
    clipboard->signed_endian=cfile->signed_endian;
    reget_afilesize (clipboard);
  }

  clipboard->start=1;
  clipboard->end=clipboard->frames;

  get_total_time (clipboard);

  sensitize();
  d_print(_ ("done\n"));

}


void
on_cut_activate                       (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  gint current_file=mainw->current_file;
  if (!(cfile->event_list==NULL)) {
    do_error_dialog(_ ("\n\nThis file has frame changes recorded. You must click on Tools|Reorder Frames before you can cut/delete frames.\n\nYou can choose to Undo the reorder afterwards.\n"));
    return;
  }
  on_copy_activate(menuitem, user_data);
  if (mainw->cancelled) {
    return;
  }
  on_delete_activate(menuitem, user_data);
  if (mainw->current_file==current_file) {
    set_undoable ("Cut",TRUE);
    cfile->undo_action=UNDO_CUT;
  }
}


void
on_paste_as_new_activate                       (GtkMenuItem     *menuitem,
						gpointer         user_data)
{
  gchar *com;
  gchar *msg;
  gint old_file=mainw->current_file,current_file;

  mainw->current_file=mainw->first_free_file;
  
  if (!get_new_handle(mainw->current_file,NULL)) {
    mainw->current_file=old_file;
    return;
  }

  //set file details
  cfile->frames=clipboard->frames;
  cfile->hsize=clipboard->hsize;
  cfile->vsize=clipboard->vsize;
  cfile->bpp=clipboard->bpp;
  cfile->pb_fps=cfile->fps=clipboard->fps;
  cfile->progress_start=cfile->start=cfile->frames>0?1:0;
  cfile->progress_end=cfile->end=cfile->frames;
  cfile->changed=TRUE;
  cfile->is_loaded=TRUE;

  mainw->fx1_val=1;
  mainw->fx1_bool=FALSE;

  com=g_strdup_printf("smogrify insert %s 0 1 %d %s %d 0 0 0 %.3f %d %d %d",cfile->handle, clipboard->frames, clipboard->handle, mainw->ccpd_with_sound, clipboard->fps, clipboard->arate, clipboard->achans, clipboard->asampsize);
  
  system(com);
  // show a progress dialog, not cancellable
  do_progress_dialog(TRUE,FALSE,_ ("Pasting"));
  g_free(com);
  
  if (mainw->ccpd_with_sound==TRUE) {
    cfile->arate=clipboard->arate;
    cfile->arps=clipboard->arps;
    cfile->achans=clipboard->achans;
    cfile->asampsize=clipboard->asampsize;
    cfile->afilesize=clipboard->afilesize;
  }

  // add entry to window menu
  add_to_winmenu();
  current_file=mainw->current_file;
  switch_to_file((mainw->current_file=old_file),current_file);
  msg=g_strdup_printf (_ ("Pasted %d frames to new clip.\n"),cfile->frames);
  d_print (msg);
  g_free (msg);
}


void
on_insert_pre_activate                    (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  if (!(cfile->event_list==NULL)) {
    do_error_dialog(_ ("\n\nThis file has frame changes recorded. You must click on Tools|Reorder Frames before you can insert frames.\n\nYou can choose to Undo the reorder afterwards.\n"));
    return;
  }

  insertw = create_insert_dialog ();

  gtk_widget_show (insertw->insert_dialog);
  mainw->fx1_bool=FALSE;
  mainw->fx1_val=1;
  mainw->fx2_bool=clipboard->achans>0;
}



void
on_insert_activate                    (GtkButton     *button,
				       gpointer         user_data)
{
  gint where=cfile->start-1;
  gint start=cfile->start;
  gint end=cfile->end;

  gint hsize=cfile->hsize;
  gint vsize=cfile->vsize;

  gint cfile_signed,cfile_endian,clipboard_signed,clipboard_endian;
  gint current_file=mainw->current_file;

  gdouble times_to_insert=mainw->fx1_val;
  gdouble audio_stretch;

  // have we resampled ?
  gboolean cb_audio_change=FALSE;
  gboolean cb_video_change=FALSE;

  gint remainder_frames;
  gint cb_start=1,cb_end=clipboard->frames;
  
  gchar *msg,*com;
  gboolean with_sound=mainw->fx2_bool;
  gint ocarps=clipboard->arps;
  gint leave_backup=1;

  if (!(button==NULL)) {
    gtk_widget_destroy(insertw->insert_dialog);
    while (g_main_context_iteration (NULL,FALSE));
    g_free(insertw);
    if (cfile->fps!=clipboard->fps||(cfile->arps!=clipboard->arps&&with_sound==TRUE)) {
      if (do_clipboard_fps_warning()==FALSE) {
	return;
      }
    }
    if (prefs->ins_resample==TRUE&&clipboard->fps!=cfile->fps) {
      cb_end=count_resampled_frames(clipboard->frames,clipboard->fps,cfile->fps);
    }
  }
  else {
    // called from on_merge_activate()
    cb_start=mainw->fx1_start;
    cb_end=mainw->fx2_start;

    // we will use leave_backup as this will leave our
    // merge backup in place
    leave_backup=-1;
   }

  if (mainw->insert_after==TRUE) {
    cfile->insert_start=cfile->end+1;
  }
  else {
    cfile->insert_start=cfile->start;
  }
  
  cfile->insert_end=cfile->insert_start-1;

  if (mainw->insert_after==TRUE) where=cfile->end;

  // don't ask smogrify to resize if frames are the same size
  if (cfile->hsize==clipboard->hsize && cfile->vsize==clipboard->vsize) hsize=vsize=0;
  else {
    if (capable->has_convert==FALSE) {
      do_error_dialog(_ ("This operation requires resizing of frames. Please install 'convert' from the Image-magick package, and then restart LiVES.\n"));
      mainw->error=TRUE;
      return;
    }
  }

  // fit video to audio if requested
  if (mainw->fx1_bool==TRUE&&(cfile->asampsize*cfile->arate*cfile->achans)) {
    get_total_time(cfile);
    times_to_insert=(cfile->laudio_time*cfile->fps-cfile->frames)/clipboard->frames;
  }

  if (times_to_insert<0.&&(mainw->fx1_bool==TRUE)) {
    do_error_dialog(_ ("\n\nVideo is longer than audio.\nTry selecting all frames, and then using \nthe 'Trim Audio' function from the Audio menu."));
    mainw->error=TRUE;
    return;
  }

  // at least we should try to convert the audio to match...
  // with_sound should never be TRUE if clipboard has no audio
  if (with_sound==TRUE) {

    cfile_signed=!(cfile->signed_endian&AFORM_UNSIGNED);
    cfile_endian=!(cfile->signed_endian&AFORM_BIG_ENDIAN);

    clipboard_signed=!(clipboard->signed_endian&AFORM_UNSIGNED);
    clipboard_endian=!(clipboard->signed_endian&AFORM_BIG_ENDIAN);
    
    if ((cfile->achans*cfile->arps*cfile->asampsize>0)&&(cfile->achans!=clipboard->achans||cfile->arps!=clipboard->arps||cfile->asampsize!=clipboard->asampsize||cfile_signed!=clipboard_signed||cfile_endian!=clipboard_endian||cfile->arate!=clipboard->arate)) {
      if (!(prefs->audio_player==AUD_PLAYER_SOX)) {
	if (cfile->arps!=clipboard->arps) {
	  do_error_dialog(_ ("LiVES was unable to insert because the audio rates do not match.\nPlease switch to 'sox' as your audio player in Preferences, and try again."));
	  mainw->error=TRUE;
	  return;
	}
      }
      if (prefs->audio_player==AUD_PLAYER_SOX) {
	cb_audio_change=TRUE;

	if (clipboard->arps!=clipboard->arps||cfile->arate!=clipboard->arate) {
	  // pb rate != real rate - stretch to pb rate and resample
	  if ((audio_stretch=(gdouble)clipboard->arps/(gdouble)clipboard->arate*(gdouble)cfile->arate/(gdouble)cfile->arps)!=1.) {
	    unlink (clipboard->info_file);
	    com=g_strdup_printf ("smogrify resample_audio %s %d %d %d %d %d %d %d %d %d %d %.4f",clipboard->handle,clipboard->arps,clipboard->achans,clipboard->asampsize,clipboard_signed,clipboard_endian,cfile->arps,clipboard->achans,clipboard->asampsize,clipboard_signed,clipboard_endian,audio_stretch);
	    system (com);
	    mainw->current_file=0;
	    do_progress_dialog (TRUE,FALSE,_ ("Resampling clipboard audio"));
	    mainw->current_file=current_file;
	    g_free (com);

	    // not really, but we pretend...
	    clipboard->arps=cfile->arps;
	  }
	}
	
	if (cfile->achans!=clipboard->achans||cfile->arps!=clipboard->arps||cfile->asampsize!=clipboard->asampsize||cfile_signed!=clipboard_signed||cfile_endian!=clipboard_endian) {
	  unlink (clipboard->info_file);
	  com=g_strdup_printf ("smogrify resample_audio %s %d %d %d %d %d %d %d %d %d %d",clipboard->handle,clipboard->arps,clipboard->achans,clipboard->asampsize,clipboard_signed,clipboard_endian,cfile->arps,cfile->achans,cfile->asampsize,cfile_signed,cfile_endian);
	  system (com);
	  mainw->current_file=0;
	  do_progress_dialog (TRUE,FALSE,_ ("Resampling clipboard audio"));
	  mainw->current_file=current_file;
	  g_free (com);
	}
    
	if (clipboard->afilesize==0) {
	  if (prefs->conserve_space==TRUE) {
	    // oops...
	    clipboard->achans=clipboard->arate=clipboard->asampsize=0;
	    with_sound=FALSE;
	    do_error_dialog (_ ("\n\nLiVES was unable to resample the clipboard audio. \nClipboard audio has been erased.\n"));
	  }
	  else {
	    unlink (clipboard->info_file);
	    mainw->current_file=0;
	    com=g_strdup_printf ("smogrify undo_audio %s",clipboard->handle);
	    system (com);
	    g_free (com);
	    mainw->current_file=current_file;

	    clipboard->arps=ocarps;
	    reget_afilesize(clipboard);
	    
	    if (do_warning_dialog (_ ("\n\nLiVES was unable to resample the clipboard audio.\nDo you wish to continue with the insert \nusing unchanged audio ?\n"))==FALSE) {
	      mainw->error=TRUE;
	      return;
	    }}}}}}

  
  // if pref is set, resample clipboard video
  if (prefs->ins_resample==TRUE&&cfile->fps!=clipboard->fps) {
    cb_video_change=TRUE;
    clipboard->undo1_dbl=cfile->fps;
    mainw->current_file=0;
    
    on_resample_vid_ok(NULL,NULL);

    // check if it worked
    if (clipboard->fps!=cfile->fps) {
      d_print (_ ("resampling error..."));
      mainw->error=1;
      return;
    }

    if (mainw->fx1_bool==TRUE&&(cfile->asampsize*cfile->arate*cfile->achans)) {
      times_to_insert=(times_to_insert*cfile->fps-cfile->frames)/clipboard->frames;
    }
    switch_to_file(0,current_file);
    if (cb_end>clipboard->frames) {
      cb_end=clipboard->frames;
    }
  }

  if (with_sound==TRUE&&cfile->achans==0) {
    cfile->achans=clipboard->achans;
    cfile->asampsize=clipboard->asampsize;
    cfile->arps=cfile->arate=clipboard->arate;
  }


  // first remainder frames
  remainder_frames=(gint)(times_to_insert-(gdouble)(gint)times_to_insert)*clipboard->frames;

  if (mainw->insert_after==FALSE&&remainder_frames>0) {
    msg=g_strdup_printf(_ ("Inserting %d%s frames from the clipboard..."),remainder_frames,times_to_insert>1.?" remainder":"");
    d_print(msg);
    g_free(msg);

    com=g_strdup_printf("smogrify insert %s %d %d %d %s %d %d %d %d",cfile->handle, where, clipboard->frames-remainder_frames+1, clipboard->frames, clipboard->handle, 0, cfile->frames, hsize, vsize);
    
    cfile->progress_start=1;
    cfile->progress_end=remainder_frames;

    unlink(cfile->info_file);
    system(com);
    do_progress_dialog(TRUE,FALSE,_ ("Inserting"));

    cfile->frames+=remainder_frames;
    where+=remainder_frames;

    g_free(com);
    cfile->insert_end+=remainder_frames;

    if (mainw->insert_after==FALSE) {
      cfile->start+=remainder_frames;
      cfile->end+=remainder_frames;
    }

    if (with_sound==TRUE) {
      reget_afilesize (cfile);
    }
    get_play_times();
    d_print(_ ("done\n"));
  } 
  

  // inserts of whole clipboard
  if ((int)times_to_insert>1) {
    msg=g_strdup_printf(_ ("Inserting %d times from the clipboard%s..."),(int)times_to_insert,with_sound==TRUE?" (with sound)":"");
    d_print(msg);
    g_free(msg);
  }
  else if ((int)times_to_insert>0) {
    msg=g_strdup_printf(_ ("Inserting %d frames from the clipboard%s..."),cb_end-cb_start+1,with_sound==TRUE?" (with sound)":"");
    d_print(msg);
    g_free(msg);
  }
  
  // for an insert after a merge we set our start posn. -ve
  // this should indicate to the back end to leave our
  // backup frames alone

  com=g_strdup_printf("smogrify insert %s %d %d %d %s %d %d %d %d %.3f %d %d %d %d",cfile->handle, where, cb_start*leave_backup, cb_end, clipboard->handle, with_sound, cfile->frames, hsize, vsize, cfile->fps, cfile->arate, cfile->achans, cfile->asampsize, (int)times_to_insert);

  cfile->progress_start=1;
  cfile->progress_end=(cb_end-cb_start+1)*(int)times_to_insert+cfile->frames-where;

  unlink(cfile->info_file);
  system(com);

  // show a progress dialog
  cfile->nopreview=TRUE;
  if (do_progress_dialog(TRUE,TRUE,_ ("Inserting"))==FALSE) {
    // cancelled
    g_free (com);

    cfile->nopreview=FALSE;
    // clean up moved/inserted frames
    com=g_strdup_printf ("smogrify undo_insert %s %d %d %d",cfile->handle,where+1,where+(cb_end-cb_start+1)*(int)times_to_insert,cfile->frames);
    system (com);
    g_free(com);

    cfile->start=start;
    cfile->end=end;

    if (with_sound==TRUE) {
      // desample clipboard audio
      if (cb_audio_change==TRUE&&prefs->conserve_space==FALSE) {
	unlink (clipboard->info_file);
	com=g_strdup_printf ("smogrify undo_audio %s",clipboard->handle);
	mainw->current_file=0;
	system (com);
	g_free (com);
	mainw->current_file=current_file;
	clipboard->arps=ocarps;
	reget_afilesize(clipboard);
      }
    }
  
    if (cb_video_change==TRUE) {
      // desample clipboard video
      mainw->current_file=0;
      on_undo_activate(NULL,NULL);
      mainw->current_file=current_file;
    }
    
    switch_to_file (0,current_file);
    set_undoable (NULL,FALSE);
    mainw->cancelled=CANCEL_USER;
    return;
  }
  mainw->cancelled=0;
  cfile->nopreview=FALSE;
  g_free(com);
  
  cfile->frames+=(cb_end-cb_start+1)*(int)times_to_insert;
  where+=(cb_end-cb_start+1)*(int)times_to_insert;
  cfile->insert_end+=(cb_end-cb_start+1)*(int)times_to_insert;

  if (mainw->insert_after==FALSE) {
    cfile->start+=(cb_end-cb_start+1)*(int)times_to_insert;
    cfile->end+=(cb_end-cb_start+1)*(int)times_to_insert;
  }

  if (with_sound==1) {
    reget_afilesize(cfile);
  }
  get_play_times();
  d_print (_ ("done\n"));

  // last remainder frames

  if (mainw->insert_after==TRUE&&remainder_frames>0) {
    msg=g_strdup_printf(_ ("Inserting %d%s frames from the clipboard..."),remainder_frames,times_to_insert>1.?" remainder":"");
    d_print(msg);
    g_free(msg);

    com=g_strdup_printf("smogrify insert %s %d %d %d %s %d %d %d %d",cfile->handle,where, 1, remainder_frames, clipboard->handle, 0, cfile->frames, hsize, vsize);
    
    cfile->progress_start=1;
    cfile->progress_end=remainder_frames;

    unlink(cfile->info_file);
    system(com);
    do_progress_dialog(TRUE,FALSE,_ ("Inserting"));

    cfile->frames+=remainder_frames;
    cfile->insert_end+=remainder_frames;
    g_free(com);

    if (mainw->insert_after==FALSE) {
      cfile->start+=remainder_frames;
      cfile->end+=remainder_frames;
    }
    get_play_times();

    d_print(_ ("done\n"));
  } 
  
  
  // insert done

  // start or end can be zero if we inserted into pure audio
  if (cfile->start==0&&cfile->frames>0) cfile->start=1;
  if (cfile->end==0) cfile->end=cfile->frames;

  if (cfile->frames>0&&cfile->old_frames==0) {
    g_snprintf (cfile->type,40,"Frames");
    cfile->orig_file_name=FALSE;
    cfile->hsize=clipboard->hsize;
    cfile->vsize=clipboard->vsize;
    cfile->bpp=clipboard->bpp;
  }

  g_signal_handler_block(mainw->spinbutton_end,mainw->spin_end_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->frames==0?0:1,cfile->frames);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->end);
  g_signal_handler_unblock(mainw->spinbutton_end,mainw->spin_end_func);


  g_signal_handler_block(mainw->spinbutton_start,mainw->spin_start_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->frames==0?0:1,cfile->frames);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->start);
  g_signal_handler_unblock(mainw->spinbutton_start,mainw->spin_start_func);

  set_undoable ("Insert",TRUE);
  cfile->undo1_boolean=with_sound;
  cfile->undo_action=UNDO_INSERT;
  gtk_widget_set_sensitive (mainw->select_new, TRUE);

  // mark new file size as 'Unknown'
  cfile->f_size=0;
  cfile->changed=TRUE;

  if (with_sound==TRUE) {
    if (cb_audio_change==TRUE&&prefs->conserve_space==FALSE) {
      unlink (clipboard->info_file);
      mainw->current_file=0;
      com=g_strdup_printf ("smogrify undo_audio %s",clipboard->handle);
      system (com);
      g_free (com);
      mainw->current_file=current_file;
      clipboard->arps=ocarps;
      reget_afilesize(clipboard);
    }
  }
  
  if (cb_video_change==TRUE) {
    mainw->current_file=0;
    on_undo_activate(NULL,NULL);
    mainw->current_file=current_file;
  }

  switch_to_file(0,current_file);
  mainw->error=FALSE;
}



void
on_delete_activate                    (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  gint frames_cut=cfile->end-cfile->start+1;
  gint start=cfile->start;
  gint end=cfile->end;
  gchar *com;

  // occasionally we get a keyboard misread, so this should prevent that
  if (mainw->playing_file>-1) return;

  if (!(cfile->event_list==NULL)) {
    do_error_dialog(_ ("\n\nThis file has frame changes recorded. You must click on Tools|Reorder Frames before you can cut/delete frames.\n\nYou can choose to Undo the reorder afterwards.\n"));
    return;
  }

  if (cfile->start<=1 && cfile->end==cfile->frames) {
    if (menuitem!=GTK_MENU_ITEM(mainw->cut) && (cfile->achans==0||((gdouble)frames_cut/cfile->fps>=cfile->laudio_time && mainw->ccpd_with_sound))) {
      if (do_warning_dialog(_ ("\nDeleting all frames will close this file.\nAre you sure ?"))==TRUE) close_current_file();
      return;
    }
    cfile->ohsize=cfile->hsize;
    cfile->ovsize=cfile->vsize;
  }
  
  cfile->undo_start=cfile->start;
  cfile->undo_end=cfile->end;
  cfile->undo1_boolean=mainw->ccpd_with_sound;

  if (!(menuitem==NULL)) {
    com=g_strdup_printf(_ ("Deleting frames %d to %d%s..."),cfile->start,cfile->end,mainw->ccpd_with_sound==TRUE&&cfile->achans>0?" (with sound)":"");
    d_print(com);
    g_free(com);
  }

  com=g_strdup_printf("smogrify cut %s %d %d %d %d %.3f %d %d %d",cfile->handle,cfile->start,cfile->end, mainw->ccpd_with_sound, cfile->frames, cfile->fps, cfile->arate, cfile->achans, cfile->asampsize);

  cfile->progress_start=cfile->start;
  cfile->progress_end=cfile->frames;

  unlink(cfile->info_file);
  system(com);
 // show a progress dialog, not cancellable
  do_progress_dialog(TRUE,FALSE,_ ("Deleting"));
  g_free(com);

  cfile->frames-=frames_cut;

  cfile->undo_arate=cfile->arate;
  cfile->undo_signed_endian=cfile->signed_endian;
  cfile->undo_achans=cfile->achans;
  cfile->undo_asampsize=cfile->asampsize;
  cfile->undo_arps=cfile->arps;

  if (mainw->ccpd_with_sound==TRUE) {
    reget_afilesize(cfile);
  }

  if (cfile->frames==0) {
    if (cfile->afilesize==0) {
      close_current_file();
      return;
    }
    g_snprintf (cfile->type,40,"Audio");
    cfile->hsize=cfile->vsize=0;
    cfile->orig_file_name=FALSE;
    desensitize();
    sensitize();
  }

  if (mainw->selwidth_locked==FALSE||cfile->start>cfile->frames) {
    if (--start==0&&cfile->frames>0) {
      start=1;
    }
  }

  cfile->start=start;

  if (mainw->selwidth_locked==FALSE) {
    cfile->end=start;
  }
  else {
    cfile->end=end;
    if (cfile->end>cfile->frames) {
      cfile->end=cfile->frames;
    }
  }

  g_signal_handler_block(mainw->spinbutton_end,mainw->spin_end_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->frames==0?0:1,cfile->frames);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->end);
  g_signal_handler_unblock(mainw->spinbutton_end,mainw->spin_end_func);

  g_signal_handler_block(mainw->spinbutton_start,mainw->spin_start_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->frames==0?0:1,cfile->frames);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->start);
  g_signal_handler_unblock(mainw->spinbutton_start,mainw->spin_start_func);

  // menuitem is NULL if we came here from undo_insert
  if (menuitem==NULL) return;

  load_start_image(cfile->start);
  load_end_image(cfile->end);
  
  get_play_times();

  // mark new file size as 'Unknown'
  cfile->f_size=0;
  cfile->changed=TRUE;

  set_undoable ("Delete",TRUE);
  cfile->undo_action=UNDO_DELETE;
  d_print(_ ("done\n"));
}


void
on_select_all_activate                (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  if (mainw->current_file==-1) return;

  if (mainw->selwidth_locked) {
    if (menuitem!=NULL) do_error_dialog(_ ("\n\nSelection is locked.\n"));
    return;
  }

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->frames);

  cfile->start=cfile->frames>0?1:0;
  cfile->end=cfile->frames;

  get_play_times();

  load_start_image(cfile->start);
  load_end_image(cfile->end);
}


void
on_select_start_only_activate                (GtkMenuItem     *menuitem,
					      gpointer         user_data)
{
  if (mainw->current_file==-1) return;
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->start);
}

void
on_select_end_only_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (mainw->current_file==-1) return;
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->end);
}



void
on_select_invert_activate                (GtkMenuItem     *menuitem,
					  gpointer         user_data)
{
  if (cfile->start==1) {
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->end+1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->frames);
  }
  else {
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->start-1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),1);
  }

  get_play_times();

  load_start_image(cfile->start);
  load_end_image(cfile->end);
}

void
on_select_last_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (cfile->undo_start > cfile->frames) cfile->undo_start=cfile->frames;
  if (cfile->undo_end > cfile->frames) cfile->undo_end=cfile->frames;

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->undo_start);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->undo_end);

  cfile->start=cfile->undo_start;
  cfile->end=cfile->undo_end;

  get_play_times();

  load_start_image(cfile->start);
  load_end_image(cfile->end);
}


void
on_select_new_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (cfile->insert_start > cfile->frames) cfile->insert_start=cfile->frames;
  if (cfile->insert_end > cfile->frames) cfile->insert_end=cfile->frames;

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->insert_start);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->insert_end);

  cfile->start=cfile->insert_start;
  cfile->end=cfile->insert_end;

  get_play_times();

  load_start_image(cfile->start);
  load_end_image(cfile->end);
}

void
on_select_to_end_activate                (GtkMenuItem     *menuitem,
					  gpointer         user_data)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->frames);
  cfile->end=cfile->frames;
  get_play_times();
  load_end_image(cfile->end);
}

void
on_select_from_start_activate                (GtkMenuItem     *menuitem,
					      gpointer         user_data)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),1);
  cfile->start=cfile->frames>0?1:0;
  get_play_times();
  load_start_image(cfile->start);
}

void
on_lock_selwidth_activate                (GtkMenuItem     *menuitem,
					  gpointer         user_data)
{
  mainw->selwidth_locked=!mainw->selwidth_locked;
  gtk_widget_set_sensitive(mainw->select_submenu,!mainw->selwidth_locked);
}


void
on_playall_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (mainw->playing_file==-1) {
    if (cfile->proc_ptr!=NULL&&menuitem!=NULL) {
      on_preview_clicked (GTK_BUTTON (cfile->proc_ptr->preview_button),NULL);
      return;
    }
    mainw->play_start=(gint)(1+cfile->pointer_time*cfile->fps);
    if (!(cfile->event_list==NULL)) {
      mainw->play_start=calc_variable_frame(mainw->current_file,cfile->pointer_time);
    }
    mainw->play_end=cfile->frames;
    mainw->playing_sel=FALSE;
    unlink(cfile->info_file);

    play_file();
    GTK_RULER (mainw->hruler)->position=cfile->pointer_time;
    get_play_times();
  }
  else {
    cfile->play_paused=!cfile->play_paused;
    if (cfile->play_paused==TRUE) {
      system("smogrify pause_play");
    }
    else {
      system("smogrify resume_play");
    }
  }
}


void
on_playsel_activate                      (GtkMenuItem     *menuitem,
					  gpointer         user_data)
{

  if (cfile->proc_ptr!=NULL&&menuitem!=NULL) {
    on_preview_clicked (GTK_BUTTON (cfile->proc_ptr->preview_button),NULL);
    return;
  }
  
  if (!mainw->preview) {
    gint orig_play_frame=calc_variable_frame(mainw->current_file,cfile->pointer_time);
    if (orig_play_frame>mainw->play_start&&orig_play_frame<mainw->play_end) {
      mainw->play_start=orig_play_frame;
    }
  }

  if (!mainw->is_rendering) {
    mainw->play_start=cfile->start;
    mainw->play_end=cfile->end;
  }

  mainw->playing_sel=TRUE;
  unlink(cfile->info_file);
  play_file();
  mainw->playing_sel=FALSE;
  GTK_RULER (mainw->hruler)->position=cfile->pointer_time;
  gtk_widget_queue_draw (mainw->hruler);
  get_play_times();
}


void
on_playclip_activate                      (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  // play the clipboard
  gint current_file=mainw->current_file;
  gboolean oloop=mainw->loop;
  gboolean oloop_cont=mainw->loop_cont;

  // switch to the clipboard
  switch_to_file (current_file,0);
  gtk_widget_set_sensitive (mainw->loop_video, FALSE);
  gtk_widget_set_sensitive (mainw->loop_continue, FALSE);
  mainw->loop=mainw->loop_cont=FALSE;

  mainw->play_start=1;
  mainw->play_end=clipboard->frames;
  mainw->playing_sel=FALSE;
  mainw->loop=FALSE;

  unlink(cfile->info_file);
  play_file();
  mainw->loop=oloop;
  mainw->loop_cont=oloop_cont;

  if (current_file>-1) {
    switch_to_file(0,current_file);
  }
  else {
    mainw->current_file=current_file;
    close_current_file();
  }
}




void
on_record_perf_activate                      (GtkMenuItem     *menuitem,
					      gpointer         user_data)
{
  // TODO - rewrite all of this...

  // not yet, my friend...
  if (mainw->playing_file>0) return;

  // record performance
  if (!mainw->record&&(mainw->current_file==-1||cfile->event_list==NULL)) {
    gtk_widget_set_sensitive(mainw->reorder,FALSE);

    // TODO - change message depending on rec_opts
    d_print(_ ("Ready to record. Use 'control' and cursor keys during playback to record your performance.\n(To cancel, press 'r' or click on Play|Record/Clear Performance again before you play.)\n"));
    mainw->record=TRUE;
    get_play_times();
  }
  else {
    if (mainw->current_file==-1) d_print (_("Record cancelled.\n"));
    else {
      d_print(_ ("Performance data was cleared.\n"));
      cfile->next_event=NULL;
      if (cfile->event_list!=NULL) {
	if (cfile->event_list_back!=NULL) event_list_free (cfile->event_list_back);
	cfile->event_list_back=cfile->event_list;
	cfile->event_list=NULL;
      }
      gtk_widget_set_sensitive (mainw->reorder, FALSE);
      gtk_widget_set_sensitive (mainw->resample_video, TRUE);
      get_play_times();
      if (cfile->undo_action==UNDO_RESAMPLE) {
	set_undoable ("Resample",TRUE);
      }
      if (cfile->undo_action==UNDO_REORDER) {
	set_undoable ("Reorder",TRUE);
      }
    }
    mainw->record=FALSE;
  }
}


void
on_rewind_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  cfile->pointer_time=GTK_RULER (mainw->hruler)->position=0.;
  gtk_widget_queue_draw (mainw->hruler);
  gtk_widget_set_sensitive (mainw->rewind, FALSE);
  gtk_widget_set_sensitive (mainw->m_rewindbutton, FALSE);
  gtk_widget_set_sensitive (mainw->trim_to_pstart, FALSE);
  get_play_times();
}

void
on_stop_activate (GtkMenuItem *menuitem, gpointer user_data) {
  mainw->cancelled=CANCEL_USER;
  mainw->jack_can_stop=mainw->jack_can_start=FALSE;
}


gboolean
on_stop_activate_by_del                  (GtkWidget       *widget,
					  GdkEvent        *event,
					  gpointer         user_data)
{
  // called if the user closes the separate play window
  if (mainw->playing_file>-1) {
    mainw->cancelled=CANCEL_USER;
  }
  if (prefs->sepwin_type==1) {
    on_sepwin_pressed(NULL,NULL);
  }
  return TRUE;
}


void 
on_vidp_entry_changed (GtkWidget *vidp_entry, gpointer ptr) {
  const gchar *vidp=gtk_entry_get_text (GTK_ENTRY (vidp_entry));
  if (strcmp (vidp,"internal")) {
    gtk_widget_set_sensitive (prefsw->checkbutton_show_stats,FALSE);
  }
  else {
    gtk_widget_set_sensitive (prefsw->checkbutton_show_stats,TRUE);
  }
}


void 
on_encoder_entry_changed (GtkEntry *encoder_entry, gpointer ptr) {
  GList *encoder_capabilities=NULL;
  GList *ofmt_all=NULL;
  GList *ofmt=NULL;
  gchar *new_encoder_name=g_strdup(gtk_entry_get_text(encoder_entry));
  gchar *msg;
  gchar **array;
  int i;

  if (!strlen (new_encoder_name)) return;

  // finalise old plugin
  plugin_request(PLUGIN_ENCODERS,prefs->encoder.name,"finalise");
    
  clear_mainw_msg();
  // initialise new plugin
  if (plugin_request(PLUGIN_ENCODERS,new_encoder_name,"init")==NULL) {
    if (strlen (mainw->msg)) {
      msg=g_strdup_printf (_ ("\n\nThe '%s' plugin reports:\n%s\n"),new_encoder_name,mainw->msg);
    }
    else {
      msg=g_strdup_printf (_ ("\n\nUnable to find the 'init' method in the %s plugin.\nThe plugin may be broken or not installed correctly."),new_encoder_name);
    }
    g_free(new_encoder_name);
    gtk_widget_hide (gtk_widget_get_toplevel (GTK_COMBO (gtk_widget_get_parent (GTK_WIDGET (encoder_entry)))->list));
    do_blocking_error_dialog (msg);
    g_free (msg);
    g_signal_handler_block (encoder_entry,prefsw->encoder_name_fn);
    gtk_entry_set_text(encoder_entry,future_prefs->encoder.name);
    g_signal_handler_unblock (encoder_entry,prefsw->encoder_name_fn);
    return;
  }

  g_snprintf(future_prefs->encoder.name,51,"%s",new_encoder_name);
  g_free(new_encoder_name);

  if ((encoder_capabilities=plugin_request(PLUGIN_ENCODERS,future_prefs->encoder.name,"get_capabilities"))==NULL) {
    do_plugin_error(PLUGIN_ENCODERS,future_prefs->encoder.name);
  }
  prefs->encoder.capabilities=atoi (g_list_nth_data (encoder_capabilities,0));
  g_list_free (encoder_capabilities);

  // fill list with new formats
  if (!((ofmt_all=plugin_request_by_line(PLUGIN_ENCODERS,future_prefs->encoder.name,"get_formats"))==NULL)) {
    for (i=0;i<g_list_length(ofmt_all);i++) {
      if (get_token_count (g_list_nth_data (ofmt_all,i),'|')>2) {
	array=g_strsplit (g_list_nth_data (ofmt_all,i),"|",-1);
	ofmt=g_list_append(ofmt,g_strdup (array[1]));
	g_strfreev (array);
      }
    }

    // we have to block here, otherwise on_ofmt_changed gets called for every added entry !
    g_signal_handler_block (GTK_COMBO (prefsw->ofmt_combo)->entry,prefsw->encoder_ofmt_fn);
    combo_set_popdown_strings (GTK_COMBO (prefsw->ofmt_combo), ofmt);
    g_signal_handler_unblock (GTK_COMBO (prefsw->ofmt_combo)->entry,prefsw->encoder_ofmt_fn);
    g_list_free(ofmt);

    // set default (first) output type
    array=g_strsplit (g_list_nth_data (ofmt_all,0),"|",-1);
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->ofmt_combo))->entry),array[1]);
    on_encoder_ofmt_changed (GTK_ENTRY((GTK_COMBO(prefsw->ofmt_combo))->entry),NULL);
    g_strfreev (array);
    g_list_free(ofmt_all);
  }
}





void
on_insert_after_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  mainw->insert_after=!mainw->insert_after;

}

void
on_insertwsound_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton))==TRUE) {
    gtk_widget_set_sensitive (insertw->fit_checkbutton,FALSE);
  }
  else {
    gtk_widget_set_sensitive (insertw->fit_checkbutton,cfile->achans>0);
  }
  mainw->fx2_bool=!mainw->fx2_bool;
}


void 
on_insfitaudio_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  mainw->fx1_bool=!mainw->fx1_bool;

  if (gtk_toggle_button_get_active(togglebutton)==TRUE) {
    gtk_widget_set_sensitive(insertw->with_sound,FALSE);
    gtk_widget_set_sensitive(insertw->without_sound,FALSE);
    gtk_widget_set_sensitive(insertw->spinbutton_times,FALSE);
  }
  else {
    gtk_widget_set_sensitive(insertw->with_sound,clipboard->achans>0);
    gtk_widget_set_sensitive(insertw->without_sound,clipboard->achans>0);
    gtk_widget_set_sensitive(insertw->spinbutton_times,TRUE);
  }
}




gboolean dirchange_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {

  if (mainw->playing_file==-1) return TRUE;

  // change play direction
  if (cfile->play_paused==TRUE) {
    cfile->freeze_fps=-cfile->freeze_fps;
    return TRUE;
  }

  g_signal_handler_block(mainw->spinbutton_pb_fps,mainw->pb_fps_func);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps),-cfile->pb_fps);
  g_signal_handler_unblock(mainw->spinbutton_pb_fps,mainw->pb_fps_func);

  // make sure this is called, sometimes we switch clips too soon...
  changed_fps_during_pb (GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps), NULL);
  return TRUE;
}


gboolean prevclip_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  GList *list_index;
  gint i=0;

  // prev clip
  // if the effect is a transition, this will change the background clip

  if (mainw->current_file<1||mainw->preview||(mainw->is_processing&&cfile->is_loaded)||mainw->cliplist==NULL||(cfile->event_list!=NULL&&!mainw->is_rendering)) return TRUE;


  if (mainw->num_tr_applied>0&&mainw->playing_file>0) {
    list_index=g_list_find (mainw->cliplist, GINT_TO_POINTER (mainw->blend_file));
  }
  else {
    list_index=g_list_find (mainw->cliplist, GINT_TO_POINTER (mainw->current_file));
  }
  do {
    if ((list_index=g_list_previous(list_index))==NULL) list_index=g_list_last (mainw->cliplist);
    i=GPOINTER_TO_INT (list_index->data);
  } while ((mainw->files[i]==NULL||mainw->files[i]->opening==TRUE||mainw->files[i]->restoring==TRUE||(!mainw->files[i]->frames&&mainw->playing_file>-1))&&i!=((mainw->playing_file>0&&mainw->num_tr_applied>0)?mainw->blend_file:mainw->current_file));

  if (mainw->num_tr_applied>0&&mainw->playing_file>0) {
    if (i!=mainw->blend_file) {
      if (mainw->blend_file!=-1&&mainw->files[mainw->blend_file]->clip_type==CLIP_TYPE_GENERATOR&&mainw->blend_file!=mainw->current_file) {
	mainw->osc_block=TRUE;
	if (rte_window!=NULL) rtew_set_keych(rte_bg_gen_key(),FALSE);
	weed_generator_end (mainw->files[mainw->blend_file]->ext_src);
	mainw->osc_block=FALSE;
      }
      mainw->blend_file=i;
      mainw->blend_file_step=mainw->files[i]->pb_fps<0?-1:1;
      mainw->whentostop=NEVER_STOP;
    }
    return TRUE;
  }
  
  if (i==mainw->current_file) return TRUE;
  if (!cfile->is_loaded) mainw->cancelled=CANCEL_NO_PROPOGATE;

  if (mainw->playing_file>-1) {
    do_quick_switch (i);
  }
  else {
    switch_to_file (mainw->current_file,i);
  }
  return TRUE;
}


gboolean nextclip_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  GList *list_index;
  gint i;

  // next clip
  // if the effect is a transition, this will change the background clip

  if (mainw->current_file<1||mainw->preview||(mainw->is_processing&&cfile->is_loaded)||mainw->cliplist==NULL||(cfile->event_list!=NULL&&!mainw->is_rendering)) return TRUE;

  if (mainw->num_tr_applied>0&&mainw->playing_file>0) {
    list_index=g_list_find (mainw->cliplist, GINT_TO_POINTER (mainw->blend_file));
  }
  else {
    list_index=g_list_find (mainw->cliplist, GINT_TO_POINTER (mainw->current_file));
  }
  do {
    if ((list_index=g_list_next(list_index))==NULL) list_index=g_list_first (mainw->cliplist);
    i=GPOINTER_TO_INT (list_index->data);
  } while ((mainw->files[i]==NULL||mainw->files[i]->opening==TRUE||mainw->files[i]->restoring==TRUE||(!mainw->files[i]->frames&&mainw->playing_file>-1))&&i!=((mainw->playing_file>0&&mainw->num_tr_applied>0)?mainw->blend_file:mainw->current_file));
  
  if (mainw->num_tr_applied>0&&mainw->playing_file>0) {
    if (i!=mainw->blend_file) {
      if (mainw->blend_file!=-1&&mainw->files[mainw->blend_file]->clip_type==CLIP_TYPE_GENERATOR&&mainw->blend_file!=mainw->current_file) {
	mainw->osc_block=TRUE;
	if (rte_window!=NULL) rtew_set_keych(rte_bg_gen_key(),FALSE);
	weed_generator_end (mainw->files[mainw->blend_file]->ext_src);
	mainw->osc_block=FALSE;
      }
      mainw->blend_file=i;
      mainw->blend_file_step=mainw->files[i]->pb_fps<0?-1:1;
      mainw->whentostop=NEVER_STOP;
    }
    return TRUE;
  }

  if (i==mainw->current_file) return TRUE;
  if (!cfile->is_loaded) mainw->cancelled=CANCEL_NO_PROPOGATE;

  if (mainw->playing_file>-1) {
    do_quick_switch (i);
  }
  else {
    switch_to_file (mainw->current_file,i);
  }
  return TRUE;
}


void
on_save_set_activate            (GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
  if (do_warning_dialog_with_check (_ ("Saving the set will cause all loaded clips to remain on the disk, and LiVES will exit.\n\nPlease press 'Cancel' if that is not what you want.\n"),WARN_MASK_SAVE_SET)==FALSE) return;

  // get set name (use a modified rename window)
  renamew=create_rename_dialog(2);
  gtk_widget_show(renamew->rename_dialog);
}


void 
on_save_set_ok (void) {
  int i;
  int marker_fd;
  gchar *markerfile;
  gchar *oldmarkerfile;
  int pb_fps;
  gchar *old_set=g_strdup(mainw->set_name);

  g_snprintf(mainw->set_name,256,"%s",gtk_entry_get_text(GTK_ENTRY(renamew->entry)));
  gtk_widget_destroy(renamew->rename_dialog);
  g_free(renamew);

  gtk_widget_queue_draw (mainw->LiVES);
  while (g_main_context_iteration (NULL,FALSE));

  if (!(strlen(mainw->set_name))) g_snprintf (mainw->set_name,256,"%d.%d",getuid(),getgid());

  for (i=1;i<MAX_FILES;i++) {
    if (mainw->files[i]!=NULL&&mainw->files[i]->clip_type==CLIP_TYPE_DISK) {
      markerfile=g_strdup_printf("%s/%s/set.%s",prefs->tmpdir,mainw->files[i]->handle,mainw->set_name);
      if (strlen (old_set)) {
	oldmarkerfile=g_strdup_printf("%s/%s/set.%s",prefs->tmpdir,mainw->files[i]->handle,old_set);
	unlink (oldmarkerfile);
	g_free (oldmarkerfile);
      }

      pb_fps=1000*mainw->files[i]->pb_fps;
      marker_fd=creat(markerfile,S_IRUSR|S_IWUSR);
      write(marker_fd,&pb_fps,sizint);
      write(marker_fd,&mainw->files[i]->frameno,sizint);
      write(marker_fd,&mainw->files[i]->name,strlen (mainw->files[i]->name));
      close(marker_fd);
      g_free(markerfile);
      write_headers(mainw->files[i]);
    }
  }
  g_free (old_set);
  mainw->leave_files=TRUE;
  lives_exit();
}


void
on_load_set_activate            (GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
  // get set name (use a modified rename window)
  // TODO - create a set browser

  renamew=create_rename_dialog(3);
  gtk_widget_show(renamew->rename_dialog);
}



void 
on_load_set_ok (void) {
  gchar *com;
  gint last_file=-1,new_file;
  gint current_file=mainw->current_file;
  static gint clipnum=0;
  gchar *msg;

  g_snprintf(mainw->set_name,256,"%s",gtk_entry_get_text(GTK_ENTRY(renamew->entry)));
  gtk_widget_destroy(renamew->rename_dialog);
  g_free(renamew);


  if (!(strlen(mainw->set_name))) g_snprintf (mainw->set_name,256,"%d.%d",getuid(),getgid());
  g_snprintf (mainw->msg,256,"none");


  while (1) {
    com=g_strdup_printf ("smogrify get_next_in_set %s %s %d",mainw->msg,mainw->set_name,getpid());
    system (com);
    g_free (com);

    if ((new_file=mainw->first_free_file)==-1) {
      too_many_files();
      return;
    }
    mainw->current_file=new_file;

    get_handle_from_info_file();

    if (strlen (mainw->msg)==0||(!strncmp (mainw->msg,"none",4))) {
      g_free (cfile);
      cfile=NULL;
      if (last_file>0) {
	mainw->current_file=current_file;
	switch_to_file (current_file,last_file);
      }
      else {
	mainw->current_file=-1;
      }
      if (clipnum==0) {
	msg=g_strdup_printf (_ ("No clips were recovered for set (%s).\n"),mainw->set_name);
	memset (mainw->set_name,0,1);
      }
      else {
	mainw->was_set=TRUE;
	reset_clip_menu();
	gtk_widget_set_sensitive (mainw->vj_load_set, FALSE);
	msg=g_strdup_printf (_ ("%d clips were recovered from set (%s).\n"),clipnum,mainw->set_name);
      }
      d_print (msg);
      g_free (msg);
      return;
    }
    last_file=new_file;

    //create a new cfile and fill in the details
    create_cfile();

    // get file details
    read_headers(".");

    // read the plaback fps, play frame, and name
    open_set_file (mainw->set_name,++clipnum);
    get_total_time (cfile);

    // add to clip menu
    add_to_winmenu();
    get_next_free_file();
    cfile->start=cfile->frames>0?1:0;
    cfile->end=cfile->frames;
    cfile->is_loaded=TRUE;
    unlink (cfile->info_file);
    load_start_image (cfile->start);
    load_end_image (cfile->end);
  }
}


void
on_cleardisk_activate (GtkMenuItem *menuitem, gpointer user_data) {
  gint current_file=mainw->current_file;
  gchar *markerfile;
  gchar **array;
  int marker_fd;
  gint bytes;
  gchar *com;
  int i;

  if (!do_warning_dialog (_ ("LiVES will attempt to recover some disk space.\nYou should ONLY run this if you have no other copies of LiVES running on this machine.\nClick OK to proceed.\n"))) return;

  d_print(_ ("Cleaning up disk space..."));

  if (!get_temp_handle(mainw->first_free_file)) {
    d_print(_("failed.\n"));
    return;
  }

  for (i=0;i<MAX_FILES;i++) {
    if (mainw->files[i]!=NULL&&mainw->files[i]->clip_type==CLIP_TYPE_DISK) {
      markerfile=g_strdup_printf("%s/%s/set.",prefs->tmpdir,mainw->files[i]->handle);
      marker_fd=creat(markerfile,S_IRUSR|S_IWUSR);
      close(marker_fd);
      g_free(markerfile);
      if (mainw->files[i]->undo_action!=UNDO_NONE) {
	markerfile=g_strdup_printf("%s/%s/noprune",prefs->tmpdir,mainw->files[i]->handle);
	marker_fd=creat(markerfile,S_IRUSR|S_IWUSR);
	close(marker_fd);
	g_free(markerfile);
      }
    }
  }

  com=g_strdup_printf("smogrify bg_weed %s",cfile->handle);
  system(com);
  g_free(com);

  // show a progress dialog, not cancellable
  do_progress_dialog(TRUE,FALSE,_("Clearing disk space"));

  array=g_strsplit(mainw->msg,"|",2);
  bytes=atoi(array[1]);
  g_strfreev(array);

  com=g_strdup_printf("smogrify close %s",cfile->handle);
  system(com);
  g_free(com);
  g_free(cfile);
  cfile=NULL;

  for (i=0;i<MAX_FILES;i++) {
    if (mainw->files[i]!=NULL&&mainw->files[i]->clip_type==CLIP_TYPE_DISK) {
      markerfile=g_strdup_printf("%s/%s/set.",prefs->tmpdir,mainw->files[i]->handle);
      unlink (markerfile);
      g_free(markerfile);
      if (mainw->files[i]->undo_action!=UNDO_NONE) {
	markerfile=g_strdup_printf("%s/%s/noprune",prefs->tmpdir,mainw->files[i]->handle);
	unlink (markerfile);
	g_free(markerfile);
      }
    }
  }

  mainw->current_file=current_file;
  sensitize();
  d_print(_ ("done.\n"));

  do_error_dialog(g_strdup_printf(_("%d MB of disk space was recovered.\n"),bytes));

}



void
on_show_keys_activate            (GtkMenuItem     *menuitem,
				  gpointer         user_data)
{
  do_keys_window();
}

void
on_show_messages_activate            (GtkMenuItem     *menuitem,
				      gpointer         user_data)
{
  do_messages_window();
}


void
on_show_file_info_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  char buff[512];
  fileinfo *filew;
  
  if (mainw->current_file==-1) return;

  filew = create_info_window (cfile->achans);

  g_snprintf(buff,512,"LiVES - %s",cfile->name);
  gtk_window_set_title (GTK_WINDOW (filew->info_window), buff);
  
  if (cfile->frames>0) {
    // type
    g_snprintf(buff,512,"\n  %s",cfile->type);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview24)),buff, -1);
    // fps
    if (!cfile->variable_fps) {
      g_snprintf(buff,512,"\n  %.3f%s",cfile->fps,cfile->ratio_fps?"...":"");
    }
    else {
      g_snprintf(buff,512,_ ("\n  %.3f%s (variable)"),cfile->fps,cfile->ratio_fps?"...":"");
    }
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview25)),buff, -1);
    // image size
    g_snprintf(buff,512,"\n  %dx%d",cfile->hsize,cfile->vsize);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview26)),buff, -1);
    // frames
    if ((cfile->opening==TRUE&&cfile->opening_audio==FALSE&&cfile->frames==0)||cfile->frames==123456789) {
      g_snprintf(buff,512,_ ("\n  Opening..."));
    }
    else {
      g_snprintf(buff,512,"\n  %d",cfile->frames);
    }
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview27)),buff, -1);
    // video time
    if ((cfile->opening==TRUE&&cfile->opening_audio==FALSE&&cfile->frames==0)||cfile->frames==123456789) {
      g_snprintf(buff,512,_ ("\n  Opening..."));
    }
    else {
      g_snprintf(buff,512,"\n  %.2f sec",cfile->video_time);
    }
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview28)),buff, -1);
    // file size
    if (cfile->f_size) g_snprintf(buff,512,"\n  %.2f MB %s",cfile->f_size>0?cfile->f_size/1000000.:-cfile->f_size/1000000.,cfile->f_size<0?"?":"");
    else g_snprintf(buff,512,_ ("\n  Unknown"));
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview29)),buff, -1);
  }

  if (cfile->achans>0) {
    if (cfile->opening==TRUE) {
      g_snprintf(buff,512,_ ("\n  Opening..."));
    }
    else {
      g_snprintf(buff,512,"\n  %.2f sec",cfile->laudio_time);
    }
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview_ltime)),buff, -1);
    g_snprintf(buff,512,"\n  %d Hz %d bit",cfile->arate,cfile->asampsize);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview_lrate)),buff, -1);
  }
  
  if (cfile->achans>1) {
    g_snprintf(buff,512,"\n  %d Hz %d bit",cfile->arate,cfile->asampsize);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview_rrate)),buff, -1);
    if (cfile->opening==TRUE) {
      g_snprintf(buff,512,_ ("\n  Opening..."));
    }
    else {
      g_snprintf(buff,512,"\n  %.2f sec",cfile->raudio_time);
    }
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (filew->textview_rtime)),buff, -1);
  }
  gtk_widget_show (filew->info_window);
}

void
on_show_file_comments_activate            (GtkMenuItem     *menuitem,
					   gpointer         user_data)
{
  do_comments_dialog();
}



void
on_show_clipboard_info_activate            (GtkMenuItem     *menuitem,
					    gpointer         user_data)
{
  gint current_file=mainw->current_file;
  mainw->current_file=0;
  on_show_file_info_activate(menuitem,user_data);
  mainw->current_file=current_file;
}

void
switch_clip_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int i;
  if (mainw->current_file<1||mainw->preview||(mainw->is_processing&&cfile->is_loaded)||mainw->cliplist==NULL||(cfile->event_list!=NULL&&!mainw->is_rendering)) return;

  for (i=1;i<MAX_FILES;i++) {
    if (!(mainw->files[i]==NULL)) {
      if (GTK_MENU_ITEM(menuitem)==GTK_MENU_ITEM(mainw->files[i]->menuentry)) {
	if (!(i==mainw->current_file)) {
	  if (!cfile->is_loaded) mainw->cancelled=CANCEL_NO_PROPOGATE;
	  if (mainw->playing_file>-1) {
	    do_quick_switch (i);
	  }
	  else {
	    switch_to_file(mainw->current_file,i);
	  }
	}
	return;
      }
    }
  }
}


void
on_about_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gchar *mesg=g_strdup_printf(_ ("LiVES Version %s\nby G. Finch (salsaman) 2002-2005\n\nReleased under the GPL 2.0\n\nContact the author at:\nsalsaman@xs4all.nl\nHomepage: http://lives.sourceforge.net"),LiVES_VERSION);
  do_error_dialog(mesg);
  g_free(mesg);
}






void
on_fs_preview_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  // file selector preview
  gchar *com;
  unsigned int xwin=0;
  gint preview_frames=200;
  gint preview_type=GPOINTER_TO_INT (user_data);
  gdouble start_time=0.;
  FILE *ifile=NULL;
  gint height=0,width=0;
  gint pid=getpid();
  gchar *info_file=g_strdup_printf ("%s/thm%d/.status",prefs->tmpdir,pid);

  if (mainw->in_fs_preview==TRUE) {
    mainw->in_fs_preview=FALSE;
    com=g_strdup ("smogrify stop");
    system (com);
    g_free (com);
  }

  if (preview_type==3) {
    // open selection
    start_time=mainw->fx1_val;
    preview_frames=(gint)mainw->fx2_val;
  }
  else {
    // open file
    g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));
  }

  info_file=g_strdup_printf ("%s/thm%d/.status",prefs->tmpdir,getpid());
  unlink (info_file);

  if (preview_type==1) {
    gchar **array;
 
    clear_mainw_msg();
    
    // make thumb from any image file, c.f. multitrack.c make_thumb - makes thumb from a loaded clip
    com=g_strdup_printf("smogrify make_thumb thm%d \"%s\"",pid,file_name);
    system(com);
    g_free(com);
 
    while (!(ifile=fopen (info_file,"r"))) {
      g_usleep (prefs->sleep_time);
    }
    fgets(mainw->msg,512,ifile);
    fclose (ifile);

    array=g_strsplit(mainw->msg,"|",3);
    width=atoi(array[1]);
    height=atoi(array[2]);
    g_strfreev(array);
    
    if (height*width) {
      // draw image
      GError *error=NULL;
      gchar *thumb=g_strdup_printf("%s/thm%d/%08d.%s",prefs->tmpdir,pid,1,prefs->image_ext);
      GdkPixbuf *pixbuf=gdk_pixbuf_new_from_file(thumb,&error);

      if (error==NULL) {
	gtk_widget_hide (mainw->fs_playarea);
	gtk_widget_show (mainw->fs_playarea);
	gdk_draw_pixbuf (GDK_DRAWABLE (mainw->fs_playarea->window),mainw->gc,GDK_PIXBUF (pixbuf),0,0,0,0,-1,-1,GDK_RGB_DITHER_NONE,0,0);
      }	
      else {
	g_error_free(error);
      }
      if (pixbuf!=NULL) {
	gdk_pixbuf_unref(pixbuf);
      }
      g_free(thumb);
    }
    g_free(info_file);
    info_file=g_strdup_printf ("%s/thm%d/",prefs->tmpdir,getpid());
    com=g_strdup_printf("rm -rf %s",info_file);
    system(com);
    g_free(com);
    g_free(info_file);
  } 


  if (!(height*width)) {
    // media preview
    if (!capable->has_mplayer) {
      do_blocking_error_dialog(_ ("\n\nYou need to install mplayer to be able to preview this file.\n"));
      return;
    }


    info_file=g_strdup_printf ("%s/fsp%d/.status",prefs->tmpdir,getpid());

    if (preview_type==1||preview_type==3) {
      xwin=(unsigned int)GDK_WINDOW_XID (mainw->fs_playarea->window);
      preview_frames=100;
    }

    if (mainw->file_open_params!=NULL) {
      com=g_strdup_printf("smogrify fs_preview fsp%d %u %d %d %.2f %d \"%s\" \"%s\"",getpid(),xwin,DEFAULT_FRAME_HSIZE, DEFAULT_FRAME_VSIZE,start_time,preview_frames,file_name,mainw->file_open_params);
    }
    else {
      com=g_strdup_printf("smogrify fs_preview fsp%d %u %d %d %.2f %d \"%s\"",getpid(),xwin,DEFAULT_FRAME_HSIZE, DEFAULT_FRAME_VSIZE,start_time,preview_frames,file_name);
    }

    if (prefs->pause_xmms==TRUE) system("xmms -u");
    
    gtk_widget_set_app_paintable(mainw->fs_playarea,TRUE);
    
    mainw->in_fs_preview=TRUE;
    system(com);
    g_free(com);
    
    while ((!(ifile=fopen (info_file,"r")))&&mainw->in_fs_preview==TRUE) {
      while (g_main_context_iteration (NULL,FALSE));
      g_usleep (prefs->sleep_time);
    }

    if (ifile!=NULL) {
      fclose(ifile);
    }
    end_fs_preview();
    g_free(info_file);
  }
}



void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  // open a file

  g_snprintf(file_name,256,"%s",g_filename_from_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));

  g_snprintf(mainw->vid_load_dir,256,"%s",file_name);
  get_dirname(mainw->vid_load_dir);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  
  mainw->fs_playarea=NULL;
  end_fs_preview();

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  if (prefs->save_directories==TRUE) {
    set_pref ("vid_load_dir",mainw->vid_load_dir);
  }
  open_file(file_name);
}


void
ok_save_frame                  (GtkButton       *button,
				gpointer         frame)
{
  // save start frame
  g_snprintf(file_name,256,"%s",g_filename_from_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));

  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  if (save_frame(GPOINTER_TO_INT (frame),file_name)==FALSE) return;

  g_snprintf(mainw->image_dir,256,"%s",file_name);
  get_dirname(mainw->image_dir);
  if (prefs->save_directories==TRUE) {
    set_pref ("image_dir",mainw->image_dir);
  }
}



void
on_open_sel_ok_button_clicked                  (GtkButton       *button,
						gpointer         user_data)
{
  g_snprintf(file_name,256,"%s",g_filename_from_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));

  g_snprintf(mainw->vid_load_dir,256,"%s",file_name);
  get_dirname(mainw->vid_load_dir);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));
  if (prefs->save_directories==TRUE) {
    set_pref ("vid_load_dir",mainw->vid_load_dir);
  }
  open_sel_range_activate();
}


void
on_opensel_range_ok_clicked                  (GtkButton       *button,
						gpointer         user_data)
{
  // open file selection
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  mainw->fs_playarea=NULL;
  end_fs_preview();
  open_file_sel(file_name,mainw->fx1_val,(gint)mainw->fx2_val);

}


void
open_sel_range_activate()
{
  // open selection range dialog

  GtkWidget *opensel_dialog = create_opensel_dialog ();
  gtk_widget_show(opensel_dialog);
  mainw->fx1_val=0.;
  mainw->fx2_val=1000;

}


void
on_ok_button4_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  // open audio file
  gchar *a_type;
  gchar *com,*mesg;
  gchar **array;
  gint orate;
  gint ochans;
  gint osamps;
  guint osigned_endian;
  gint oundo_start;
  gint oundo_end;
  gint israw=1;

  mainw->noswitch=TRUE;

  orate=cfile->arate;
  ochans=cfile->achans;
  osamps=cfile->asampsize;
  osigned_endian=cfile->signed_endian;
  oundo_start=cfile->undo_start;
  oundo_end=cfile->undo_end;

  g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));
  g_snprintf(mainw->audio_dir,256,"%s",file_name);
  get_dirname(mainw->audio_dir);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  mainw->fs_playarea=NULL;
  end_fs_preview();

  a_type=file_name+strlen(file_name)-3;

  if (!strcmp(a_type,"mp3")||!strcmp(a_type,"ogg")||!strcmp(a_type,"wav")||!strcmp(a_type,"mod")||!strcmp(a_type,"xm")) com=g_strdup_printf("smogrify audioopen %s \"%s\"",cfile->handle,file_name);
  else {
    do_error_dialog(_ ("Sorry, unknown audio type.\n\n (Filenames must end in .mp3, .ogg, .wav, .mod or .xm)"));
    d_print(_ ("failed (unknown type)\n"));
    mainw->noswitch=FALSE;
    return;
  }

  if (!strcmp(a_type,"wav")) israw=0;

  if (capable->has_mplayer) {
    read_file_details (file_name);
    array=g_strsplit(mainw->msg,"|",15);
    cfile->arate=atoi(array[9]);
    cfile->achans=atoi(array[10]);
    cfile->asampsize=atoi(array[11]);
    cfile->signed_endian=get_signed_endian(atoi (array[12]), atoi (array[13]));
    g_strfreev(array);
  }
  else {
    // TODO !!! - need some way to identify audio without invoking mplayer
    cfile->arate=DEFAULT_AUDIO_RATE;
    cfile->achans=DEFAULT_AUDIO_CHANS;
    cfile->asampsize=DEFAULT_AUDIO_SAMPS;
    cfile->signed_endian=mainw->endian;
  }

  mesg=g_strdup_printf(_ ("Opening audio %s, type %s..."),file_name,a_type);
  d_print(mesg);
  g_free(mesg);

  unlink(cfile->info_file);
  system(com);
  g_free(com);

  cfile->opening=cfile->opening_audio=cfile->opening_only_audio=TRUE;

  cfile->undo_start=1;
  cfile->undo_end=cfile->frames;
  
  // show audio [opening...] in main window
  get_play_times();

  if (!(do_progress_dialog(TRUE,TRUE,_ ("Opening audio")))) {
    gtk_widget_queue_draw(mainw->LiVES);
    while (g_main_context_iteration(NULL,FALSE));
    com=g_strdup_printf("smogrify cancel_audio %s",cfile->handle);
    system(com);
    do_auto_dialog(_ ("Cancelling"));
    g_free(com);
    cfile->opening_audio=cfile->opening=cfile->opening_only_audio=FALSE;
    cfile->arate=orate;
    cfile->achans=ochans;
    cfile->asampsize=osamps;
    cfile->signed_endian=osigned_endian;
    cfile->undo_start=oundo_start;
    cfile->undo_end=oundo_end;
    sensitize();
    get_play_times();
    mainw->noswitch=FALSE;
    return;
  }

  cfile->opening_audio=cfile->opening=cfile->opening_only_audio=FALSE;

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  array=g_strsplit(mainw->msg,"|",7);
  cfile->afilesize=atoi(array[6]);
  g_strfreev(array);

  if (cfile->afilesize==0) {
    d_print(_ ("\nError loading audio file...\n"));
      
    com=g_strdup_printf("smogrify cancel_audio %s",cfile->handle);
    system(com);
    do_auto_dialog(_ ("Cancelling"));
    g_free(com);
    cfile->arate=orate;
    cfile->achans=ochans;
    cfile->asampsize=osamps;
    cfile->signed_endian=osigned_endian;
    cfile->undo_start=oundo_start;
    cfile->undo_end=oundo_end;
    sensitize();
    mainw->noswitch=FALSE;
    return;
  }

  cfile->arps=cfile->arate;
  cfile->changed=TRUE;
  d_print(_ ("done\n"));

  mesg=g_strdup_printf(_ ("New audio: %d Hz %d channel(s) %d bps\n"),cfile->arate,cfile->achans,cfile->asampsize);
  d_print(mesg);
  g_free(mesg);

  com=g_strdup_printf("smogrify commit_audio %s %d",cfile->handle,israw);
  system(com);
  do_auto_dialog(_ ("Committing audio"));
  g_free(com);
  if (prefs->save_directories==TRUE) {
    set_pref ("audio_dir",mainw->audio_dir);
  }
  if (!prefs->conserve_space) {
    cfile->undo_action=UNDO_NEW_AUDIO;
    set_undoable ("New Audio",TRUE);
  }
  switch_to_file (mainw->current_file,mainw->current_file);
  mainw->noswitch=FALSE;
}


void 
end_fs_preview(void) {
  // clean up if we were playing a preview - should be called from all callbacks 
  // where there is a possibility of fs preview still playing
  gchar *com;
  if (mainw->in_fs_preview==TRUE) {
    if (prefs->pause_xmms==TRUE) system("xmms -u");
    mainw->in_fs_preview=FALSE;
    com=g_strdup_printf ("smogrify stop");
    system (com);
    g_free (com);

    com=g_strdup_printf ("smogrify close fsp%d",getpid());
    system (com);
    g_free (com);
    if (!(mainw->fs_playarea==NULL)) {
      gtk_widget_set_app_paintable(mainw->fs_playarea,FALSE);
      gtk_widget_modify_bg (mainw->fs_playarea, GTK_STATE_NORMAL, &palette->normal_back);
      gtk_widget_hide (mainw->fs_playarea);
      gtk_widget_show (mainw->fs_playarea);
      while (g_main_context_iteration (NULL, FALSE));
    }
  }
}


void
on_cancel_button1_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  // generic cancel callback
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  while (g_main_context_iteration (NULL,FALSE));

  if (!(user_data==NULL)) {
    g_free(user_data);
  }

  mainw->fs_playarea=NULL;
  end_fs_preview();

  if (mainw->current_file>0) {
    if (cfile->opening==FALSE) {
      get_play_times();
    }
  }
}



void
on_cancel_opensel_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  mainw->fs_playarea=NULL;
  end_fs_preview();
}



void
on_button3_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  // Cancel/Keep from progress dialog
  gchar *com;
  guint keep_frames=0;
  FILE *infofile;

  clear_mainw_msg();

  if (!mainw->effects_paused||cfile->nokeep) {
    // Cancel
    if (mainw->cancel_type==CANCEL_KILL) {
      com=g_strdup_printf("smogrify cancel %s",cfile->handle);
    }
    else {
      // allow RFX effects to clean up
      com=g_strdup_printf("smogrify cancel_hup %s",cfile->handle);
    }
    if (!cfile->opening&&!mainw->internal_messaging) {
      // if we are opening, this is 'stop' in the preview, so don't cancel
      system(com);
    }

    mainw->cancelled=CANCEL_USER;
	
    if (mainw->is_rendering) cfile->frames=0;
    else {
      if ((infofile=fopen(cfile->info_file,"r"))>0) {
	fgets(mainw->msg,511,infofile);
	fclose(infofile);
      }
      
      if (strncmp (mainw->msg,"completed",9)) {
	d_print(_("cancelled\n"));
      }
    }
  }
  else {
    // Keep
    if (!mainw->is_rendering) keep_frames=cfile->proc_ptr->frames_done-cfile->progress_start+cfile->start-1+mainw->internal_messaging*2;
    else keep_frames=cfile->frames+1;
    if (keep_frames>mainw->internal_messaging) {
      gchar *msg=g_strdup_printf(_ ("%d frames are enough !\n"),keep_frames-cfile->start);
      d_print(msg);
      g_free(msg);
      if (!mainw->internal_messaging) {
	com=g_strdup_printf("smogrify keep %s %d %d",cfile->handle,cfile->start,keep_frames-1);
      }
      else {
	mainw->internal_messaging=FALSE;
	com=g_strdup_printf ("smogrify mv_mgk %s %d %d",cfile->handle,cfile->start,keep_frames);
      }
      if (!mainw->is_rendering) system(com);
      else mainw->cancelled=CANCEL_KEEP;
    }
    else {
      // no frames there
      d_print(_("cancelled\n"));
      com=g_strdup_printf("smogrify cancel_hup %s",cfile->handle);
      if (!mainw->internal_messaging&&!mainw->is_rendering) {
	system(com);
      }
      mainw->cancelled=CANCEL_USER;
    }
  }
  g_free(com);
}



void
on_ok_button3_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{   // save as
  gchar n_file_name[256];

  g_snprintf(n_file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));
  g_snprintf(mainw->vid_save_dir,256,"%s",n_file_name);
  get_dirname(mainw->vid_save_dir);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  if (prefs->save_directories==TRUE) {
    set_pref ("vid_save_dir",mainw->vid_save_dir);
  }
  save_file(FALSE,n_file_name);
}



void
on_info_ok_button_clicked            (GtkButton       *button,
				      gpointer         user_data)
{
  gint warn_mask_number=GPOINTER_TO_INT (user_data);
  if (mainw->warn_mask) {
    prefs->warning_mask|=warn_mask_number;
    set_int_pref("lives_warning_mask",prefs->warning_mask);
  }
}

void
on_info_ok_button_clicked2            (GtkButton       *button,
				      gpointer         user_data)
{
  on_info_ok_button_clicked(button,user_data);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void on_full_screen_pressed (GtkButton *button,
			     gpointer user_data) {
  // toolbar button (full screen)
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->full_screen),!mainw->fs);
}


void
on_full_screen_activate               (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  if (mainw->current_file>-1&&!cfile->frames&&mainw->multitrack==NULL) return;

  if (user_data==NULL) {
    mainw->fs=!mainw->fs;
  }
  if (mainw->current_file==-1) return;

  if (mainw->playing_file>-1){

    if (mainw->fs) {
      // switch TO full screen during pb
      if (mainw->multitrack==NULL) {
	if (!mainw->faded) {
	  fade_background();
	}
	
	fullscreen_internal();
	gtk_widget_hide(mainw->framebar);
      }

      if (mainw->sep_win) {
	if (prefs->sepwin_type==1) {
	  resize_play_window();
	}
	else {
	  kill_play_window();
	  make_play_window();
	  if (mainw->play_window!=NULL) {
	    hide_cursor(mainw->play_window->window);
	    gtk_widget_set_app_paintable(mainw->play_window,TRUE);
	  }
	}
	if (cfile->frames==1||cfile->play_paused) {
	  while (g_main_context_iteration (NULL,FALSE));
	}
      }
      if (mainw->ext_playback&&vid_playback_plugin->fheight>-1&&vid_playback_plugin->fwidth>-1) {	  
	// fixed o/p size for stream
	if (!(vid_playback_plugin->fwidth*vid_playback_plugin->fheight)) {
	  vid_playback_plugin->fwidth=cfile->hsize;
	  vid_playback_plugin->fheight=cfile->vsize;
	}
	mainw->pwidth=vid_playback_plugin->fwidth;
	mainw->pheight=vid_playback_plugin->fheight;
      }
      if ((cfile->frames==1||cfile->play_paused)&&!mainw->noswitch) {
	load_frame_image (cfile->frameno,cfile->frameno);
      }
    } else {
      if (mainw->multitrack==NULL) {
	// switch FROM fullscreen during pb
	gtk_widget_show(mainw->frame1);
	gtk_widget_show(mainw->frame2);
	gtk_widget_show(mainw->eventbox3);
	gtk_widget_show(mainw->eventbox4);
	
	if (prefs->show_framecount==TRUE) {
	  gtk_widget_show(mainw->framebar);
	}
	
	gtk_container_set_border_width (GTK_CONTAINER (mainw->playframe), 10);
	
	gtk_widget_set_sensitive(mainw->fade,TRUE);
	gtk_widget_set_sensitive(mainw->dsize,TRUE);
	
	gtk_widget_show(mainw->t_bckground);
	gtk_widget_show(mainw->t_double);

	resize(1);
	if (mainw->multitrack==NULL) {
	  if (cfile->is_loaded) {
	    load_start_image (cfile->start);
	    load_end_image (cfile->end);
	  }
	  else {
	    load_start_image (0);
	    load_end_image (0);
	  }
	}
      }

      if (mainw->sep_win) {
	// separate window
	if (prefs->sepwin_type==1) {
	  // sticky
	  if (mainw->ext_playback==TRUE) {
	    vid_playback_plugin_exit();
	  }
	}
	gtk_window_unfullscreen(GTK_WINDOW(mainw->play_window));

	if (!mainw->faded) {
	  unfade_background();
	}

	resize_play_window();

	if (mainw->opwx>-1) {
	  //opwx and opwy were stored when we first switched to full screen
	  gtk_window_move (GTK_WINDOW (mainw->play_window), mainw->opwx, mainw->opwy);
	  gtk_widget_queue_draw (mainw->play_window);
	  mainw->opwx=-1;
	  mainw->opwy=-1;
	}
	else {
	  // non-sticky
	  if (mainw->ext_playback==TRUE) {
	    vid_playback_plugin_exit();
	  }
	  kill_play_window();
	  make_play_window();
	  hide_cursor(mainw->play_window->window);
	  gtk_widget_set_app_paintable(mainw->play_window,TRUE);
	}
	if (mainw->multitrack==NULL&&(cfile->frames==1||cfile->play_paused)) {
	  while (g_main_context_iteration (NULL,FALSE));
	  if (mainw->play_window!=NULL&&!mainw->noswitch) {
	    load_frame_image (cfile->frameno,cfile->frameno);
	  }
	}
      }
      else {
	if (mainw->multitrack==NULL) {
	// in-frame window
	  while (g_main_context_iteration(NULL,FALSE));
	  
	  mainw->pwidth=mainw->playframe->allocation.width-H_RESIZE_ADJUST;
	  mainw->pheight=mainw->playframe->allocation.height-V_RESIZE_ADJUST;
	  
	  // double size
	  if (mainw->double_size) {
	    resize(2);
	    mainw->pheight*=2;
	    mainw->pheight++;
	    mainw->pwidth*=2;
	    mainw->pwidth+=2;
	  }
	}
	if (!mainw->faded) {
	  unfade_background();
	}
	else {
	  gtk_frame_set_label(GTK_FRAME(mainw->playframe), "");
	}
      }
      if ((cfile->frames==1||cfile->play_paused)&&!mainw->noswitch&&mainw->multitrack==NULL) {
	load_frame_image (cfile->frameno,cfile->frameno);
      }
    }
  }
  else {
    if (mainw->multitrack==NULL) {
      if (mainw->playing_file==-1) {
	if (mainw->fs) {
	  gtk_widget_set_sensitive(mainw->fade,FALSE);
	  gtk_widget_set_sensitive(mainw->dsize,FALSE);
	} else {
	  gtk_widget_set_sensitive(mainw->fade,TRUE);
	  gtk_widget_set_sensitive(mainw->dsize,TRUE);
	}
      }
    }
  }
}


void on_double_size_pressed (GtkButton *button,
			     gpointer user_data) {
  // toolbar button (separate window)
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->dsize),!mainw->double_size);
}



void
on_double_size_activate               (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  if (mainw->multitrack!=NULL||(mainw->current_file>-1&&cfile->frames==0)) return;

  if (user_data==NULL) {
    mainw->double_size=!mainw->double_size;
  }

  if (mainw->current_file==-1) return;

  if (mainw->playing_file>-1&&!mainw->fs) {
    // needed
    block_expose();
    do {
      while (g_main_context_iteration (NULL, FALSE));
      mainw->pwidth=mainw->playframe->allocation.width-H_RESIZE_ADJUST;
      mainw->pheight=mainw->playframe->allocation.height-V_RESIZE_ADJUST;
    } while (!(mainw->pwidth*mainw->pheight));
    unblock_expose();

    if (mainw->sep_win) {
      if (prefs->sepwin_type==1) {
	resize_play_window();
      }
      else {
	if (mainw->play_window!=NULL) {
	  kill_play_window();
	}
	make_play_window();
	if (mainw->play_window!=NULL) {
	  hide_cursor(mainw->play_window->window);
	  gtk_widget_set_app_paintable(mainw->play_window,TRUE);
	}
      }
      if (cfile->frames==1||cfile->play_paused) {
	while (g_main_context_iteration (NULL,FALSE));
	if (!(mainw->play_window==NULL)&&!mainw->noswitch) {
	  load_frame_image (cfile->frameno,cfile->frameno);
	}
      }
    }
    else {
      // in-frame
      if (mainw->double_size) {
	if (!mainw->sep_win) {
	  if (!mainw->faded) {
	    if (palette->style&STYLE_1) {
	      gtk_widget_hide(mainw->sep_image);
	    }
	    gtk_widget_hide(mainw->scrolledwindow);
	  }
	  resize(2);
	}
	mainw->pheight*=2;
	mainw->pheight++;
	mainw->pwidth*=2;
	mainw->pwidth+=2;
	gtk_widget_hide(mainw->t_double);
	gtk_widget_show(mainw->t_single);
      }
      else {
	resize(1);
	mainw->pheight--;
	mainw->pheight/=2;
	mainw->pwidth-=2;
	mainw->pwidth/=2;
	gtk_widget_show(mainw->t_double);
	gtk_widget_hide(mainw->t_single);
	if (!mainw->faded) {
	  if (palette->style&STYLE_1) {
	    gtk_widget_show(mainw->sep_image);
	  }
	  gtk_widget_show(mainw->scrolledwindow);
	}}}
  }
}



void on_sepwin_pressed (GtkButton *button,
			gpointer user_data) {

  // toolbar button (separate window)
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->sepwin),!mainw->sep_win);

}



void
on_sepwin_activate               (GtkMenuItem     *menuitem,
				  gpointer         user_data)
{
  mainw->sep_win=!mainw->sep_win;

  if (prefs->sepwin_type==1&&mainw->playing_file==-1) {
    if (mainw->sep_win) {
      make_play_window();
    }
    else {
      kill_play_window();
    }
  }
  else {
    if (mainw->playing_file>-1) {
      if (mainw->sep_win) {
	// switch to separate window during pb
	if (mainw->multitrack==NULL) {
	  if (mainw->fs&&!mainw->faded) {
	    if (mainw->ext_playback&&vid_playback_plugin->fheight>-1&&vid_playback_plugin->fwidth>-1) {	  
	      // fixed o/p size for stream
	      if (!(vid_playback_plugin->fwidth*vid_playback_plugin->fheight)) {
		vid_playback_plugin->fwidth=cfile->hsize;
		vid_playback_plugin->fheight=cfile->vsize;
	      }
	      mainw->pwidth=vid_playback_plugin->fwidth;
	      mainw->pheight=vid_playback_plugin->fheight;
	    }
	    
	    resize(1);
	    unfade_background();
	  }
	  else {
	    if (mainw->faded) {
	      gtk_widget_hide(mainw->playframe);
	    }
	    if (mainw->double_size&&mainw->multitrack==NULL) {
	      resize(1);
	      if (!mainw->faded) {
		if (palette->style&STYLE_1) {
		  gtk_widget_show(mainw->sep_image);
		}
		gtk_widget_show(mainw->scrolledwindow);
	      }
	    }
	  }
	}
	make_play_window();

	if (mainw->ext_playback&&vid_playback_plugin->fheight>-1&&vid_playback_plugin->fwidth>-1) {	  
	  // fixed o/p size for stream
	  if (!(vid_playback_plugin->fwidth*vid_playback_plugin->fheight)) {
	    vid_playback_plugin->fwidth=cfile->hsize;
	    vid_playback_plugin->fheight=cfile->vsize;
	  }
	  mainw->pwidth=vid_playback_plugin->fwidth;
	  mainw->pheight=vid_playback_plugin->fheight;
	}
	
	
	if (mainw->play_window!=NULL) {
	  hide_cursor(mainw->play_window->window);
	  gtk_widget_set_app_paintable(mainw->play_window,TRUE);
	}
	if (cfile->frames==1||cfile->play_paused) {
	  while (g_main_context_iteration (NULL,FALSE));
	  if (mainw->play_window!=NULL&&!mainw->noswitch&&mainw->multitrack==NULL) {
	    load_frame_image (cfile->frameno,cfile->frameno);
	  }
	}
      }
      else {
	// switch from separate window during playback
	if (cfile->frames>0&&mainw->multitrack==NULL) {
	  gtk_widget_show(mainw->playframe);
	}
	if (!mainw->fs&&mainw->multitrack==NULL) {
	  if (!mainw->double_size) {
	    resize(1);
	  }
	  else {
	    if (palette->style&STYLE_1) {
	      gtk_widget_hide(mainw->sep_image);
	    }
	    gtk_widget_hide(mainw->scrolledwindow);
	    resize(2);
	  }
	  
	  gtk_widget_queue_draw (mainw->playframe);
	  while (g_main_context_iteration (NULL, FALSE));
	  mainw->pwidth=mainw->playframe->allocation.width-H_RESIZE_ADJUST;
	  mainw->pheight=mainw->playframe->allocation.height-V_RESIZE_ADJUST;
	}
	else {
	  if (mainw->faded&&mainw->multitrack==NULL) {
	    fade_background();
	  }
	  if (mainw->ext_playback) {
	    vid_playback_plugin_exit();
	  }
	  
	  if (mainw->multitrack==NULL) {
	    gtk_widget_show(mainw->playframe);
	    fullscreen_internal();
	    while (g_main_context_iteration (NULL, FALSE));
	  }
	}
	if (mainw->multitrack!=NULL) {
	  mainw->must_resize=TRUE;
	  mainw->pwidth=MT_PLAY_WIDTH;
	  mainw->pheight=MT_PLAY_HEIGHT;
	}
	kill_play_window();
	add_to_playframe();
	hide_cursor(mainw->playarea->window);
	mainw->xwin = gtk_socket_get_id(GTK_SOCKET(mainw->playarea));
	if (mainw->multitrack==NULL&&((cfile->frames==1||cfile->play_paused)&&!mainw->noswitch)) load_frame_image (cfile->frameno,cfile->frameno);
      }
    }
  }
}


void
on_showfct_activate               (GtkMenuItem     *menuitem,
				   gpointer         user_data)
{
  prefs->show_framecount=!prefs->show_framecount;
  if (mainw->playing_file>-1) {
    if (prefs->show_framecount==TRUE&&mainw->fs==FALSE) {
      gtk_widget_show (mainw->framebar);
    }
    else {
      gtk_widget_hide (mainw->framebar);
    }
  }
}



void
on_sticky_activate               (GtkMenuItem     *menuitem,
				  gpointer         user_data)
{
  if (prefs->sepwin_type==0) {
    prefs->sepwin_type=1;
      if (mainw->sep_win==TRUE) {
	if (mainw->playing_file==-1) {
	  make_play_window();
	}
      }
      else {
	if (!(mainw->play_window==NULL)) {
	  gtk_window_set_title (GTK_WINDOW (mainw->play_window),_ ("LiVES: - Play Window"));
	}
      }
  }
  else {
    if (prefs->sepwin_type==1) {
      prefs->sepwin_type=0;
      if (mainw->sep_win==TRUE) {
	if (mainw->playing_file==-1) {
	  kill_play_window();
	}
	else {
	  gtk_window_set_title(GTK_WINDOW(mainw->play_window),gtk_window_get_title(GTK_WINDOW(mainw->LiVES)));
	}
      }
    }
  }
}



void on_fade_pressed (GtkButton *button,
		      gpointer user_data) 
{
  // toolbar button (unblank background)
  if (mainw->fs==TRUE) return;
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->fade),!mainw->faded);
}



void
on_fade_activate               (GtkMenuItem     *menuitem,
				gpointer         user_data)
{
  mainw->faded=!mainw->faded;
  if (mainw->playing_file>-1&&!mainw->fs) {
    if (mainw->faded==TRUE) fade_background();
    else unfade_background();
  }
}


void
on_save_with_sound_activate                (GtkMenuItem     *menuitem,
					    gpointer         user_data)
{
  mainw->save_with_sound=!mainw->save_with_sound;
}


void
on_ccpd_sound_activate                (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  mainw->ccpd_with_sound=!mainw->ccpd_with_sound;
}


void
on_loop_video_activate                (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  mainw->loop=!mainw->loop;
  gtk_widget_set_sensitive (mainw->playclip, !(clipboard==NULL));
  find_when_to_stop();
}

void
on_loop_button_activate                (GtkMenuItem     *menuitem,
					gpointer         user_data)
{
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->loop_continue), !mainw->loop_cont);
}


void
on_loop_cont_activate                (GtkMenuItem     *menuitem,
				      gpointer         user_data)
{
  mainw->loop_cont=!mainw->loop_cont;
  gtk_widget_set_sensitive (mainw->playclip, !(clipboard==NULL));
  find_when_to_stop();
}


void
on_ping_pong_activate                (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  mainw->ping_pong=!mainw->ping_pong;
}


void
on_mute_button_activate                (GtkMenuItem     *menuitem,
					gpointer         user_data)
{
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->mute_audio), !mainw->mute);
}

void
on_mute_activate                (GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
  gchar buff[256];

  mainw->mute=!mainw->mute;

  // change the mute icon
  if (mainw->mute==FALSE) {
    g_snprintf (buff,256,"%s%s/volume.png",prefs->prefix_dir,ICON_DIR);
  }
  else {
    g_snprintf (buff,256,"%s%s/volume_mute.png",prefs->prefix_dir,ICON_DIR);
  }
  gtk_image_set_from_file (GTK_IMAGE (mainw->mute_img),buff);
  gtk_widget_set_size_request (mainw->m_mutebutton, 28, 18);

  if (mainw->preview_box!=NULL) {
    gtk_image_set_from_file (GTK_IMAGE (mainw->p_mute_img),buff);
    gtk_widget_set_size_request (mainw->p_mutebutton, 28, 18);
  }
}

void
on_spin_value_changed           (GtkSpinButton   *spinbutton,
				 gpointer         user_data)
{
  // TODO - use array
  switch (GPOINTER_TO_INT (user_data)) {
  case 1 :
    mainw->fx1_val=gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 2 :
    mainw->fx2_val=gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 3 :
    mainw->fx3_val=gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 4 :
    mainw->fx4_val=gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton));
    break;
  }
}


void
on_spin_start_value_changed           (GtkSpinButton   *spinbutton,
				     gpointer         user_data)
{
  // generic
  // TODO - use array
  switch (GPOINTER_TO_INT (user_data)) {
  case 1 :
    mainw->fx1_start=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 2 :
    mainw->fx2_start=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 3 :
    mainw->fx3_start=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 4 :
    mainw->fx4_start=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  }
}
void
on_spin_step_value_changed           (GtkSpinButton   *spinbutton,
				     gpointer         user_data)
{
  // generic
  // TODO - use array
  switch (GPOINTER_TO_INT (user_data)) {
  case 1 :
    mainw->fx1_step=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 2 :
    mainw->fx2_step=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 3 :
    mainw->fx3_step=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 4 :
    mainw->fx4_step=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  }
}

void
on_spin_end_value_changed           (GtkSpinButton   *spinbutton,
				     gpointer         user_data)
{
  // generic
  // TODO - use array
  switch (GPOINTER_TO_INT (user_data)) {
  case 1 :
    mainw->fx1_end=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 2 :
    mainw->fx2_end=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 3 :
    mainw->fx3_end=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  case 4 :
    mainw->fx4_end=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
    break;
  }
}



void
on_boolean_toggled        (GtkToggleButton *togglebutton,
			    gpointer         user_data)
{
  // TODO - use array, like mainw->fx_bool[n]

  switch (GPOINTER_TO_INT (user_data)) {
  case 1 :
    mainw->fx1_bool=!mainw->fx1_bool;
    break;
  case 2 :
    mainw->fx2_bool=!mainw->fx2_bool;
    break;
  case 3 :
    mainw->fx3_bool=!mainw->fx3_bool;
    break;
  case 4 :
    mainw->fx4_bool=!mainw->fx4_bool;
    break;
  case 5 :
    mainw->fx5_bool=!mainw->fx5_bool;
    break;
  case 6 :
    mainw->fx6_bool=!mainw->fx6_bool;
    break;
  }
}




void
on_rev_clipboard_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // reverse the clipboard
  gchar *com;
  
  gint current_file=mainw->current_file;
  mainw->current_file=0;

  d_print(_ ("Reversing clipboard..."));
  com=g_strdup_printf("smogrify reverse %s %d %d",clipboard->handle,1,clipboard->frames);

  unlink(cfile->info_file);
  system(com);
  cfile->progress_start=1;
  cfile->progress_end=cfile->frames;

  // show a progress dialog, not cancellable
  do_progress_dialog(TRUE,FALSE,_ ("Reversing clipboard"));
  mainw->current_file=current_file;
  g_free(com);
  sensitize();
  d_print(_ ("done.\n"));

}



void
on_load_audio_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  // try to repaint the screen, as it may take a few seconds to get a direcotry listing
  while (g_main_context_iteration(NULL,FALSE));

  fileselection = create_fileselection (_ ("Select Audio File"),2,NULL);
  if (strlen(mainw->audio_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->audio_dir,-1,NULL,NULL,NULL));
  }
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_ok_button4_clicked),NULL);
  gtk_widget_show (fileselection);
}


void
on_load_cdtrack_activate                (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  GtkWidget *cdtrack_dialog;
  
  if (!strlen(prefs->cdplay_device)) {
    do_error_dialog(_ ("Please set your CD play device in Tools | Preferences | Misc\n"));
    return;
  }

  cdtrack_dialog = create_cdtrack_dialog (0);
  gtk_widget_show (cdtrack_dialog);
  mainw->fx1_val=1;

}


void
on_eject_cd_activate                (GtkMenuItem     *menuitem,
				     gpointer         user_data)
{
  gchar *com=g_strdup_printf("eject %s",prefs->cdplay_device);
  while(g_main_context_iteration(NULL,FALSE));
  system(com);
  g_free(com);
}


void 
on_load_cdtrack_ok_clicked                (GtkButton     *button,
					   gpointer         user_data)
{
  gchar *com,*mesg;
  gchar **array;
  gboolean was_new=FALSE;
  gint new_file=mainw->first_free_file;

  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  while(g_main_context_iteration(NULL,FALSE));

  mesg=g_strdup_printf(_ ("Opening CD track %d from %s..."),(int)mainw->fx1_val,prefs->cdplay_device);
  d_print(mesg);
  g_free(mesg);

  if (mainw->current_file==-1) {
    if (!get_new_handle(new_file,g_strdup_printf (_ ("CD track %d"),(int)mainw->fx1_val))) {
      return;
    }

    mainw->current_file=new_file;
    g_snprintf(cfile->type,40,"CD track %d on %s",(int)mainw->fx1_val,prefs->cdplay_device);
    get_play_times();
    add_to_winmenu();
    was_new=TRUE;
    cfile->opening=cfile->opening_audio=cfile->opening_only_audio=TRUE;
    cfile->hsize=DEFAULT_FRAME_HSIZE;
    cfile->vsize=DEFAULT_FRAME_VSIZE;
  }

  com=g_strdup_printf("smogrify cdopen %s %d",cfile->handle,(int)mainw->fx1_val);

  unlink(cfile->info_file);
  system(com);
  g_free(com);

  if (!(do_progress_dialog(TRUE,TRUE,_ ("Opening CD track...")))) {
    gtk_widget_queue_draw(mainw->LiVES);
    while (g_main_context_iteration(NULL,FALSE));
    com=g_strdup_printf("smogrify cancel_audio %s",cfile->handle);
    system(com);
    do_auto_dialog(_ ("Cancelling"));
    g_free(com);
    if (was_new) close_current_file();
    return;
  }

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  if (mainw->error==TRUE) {
    if (was_new) close_current_file();
    do_error_dialog(mainw->msg);
    return;
  }

  array=g_strsplit(mainw->msg,"|",5);
  cfile->arate=atoi(array[1]);
  cfile->achans=atoi(array[2]);
  cfile->asampsize=atoi(array[3]);
  cfile->afilesize=atoi(array[4]);
  g_strfreev(array);

  if (cfile->afilesize==0) {
    d_print(_ ("Error loading CD track\n"));
    com=g_strdup_printf("smogrify cancel_audio %s",cfile->handle);
    system(com);
    do_auto_dialog(_ ("Cancelling"));
    g_free(com);
    if (was_new) close_current_file();
    return;
  }

  cfile->arps=cfile->arate;
  cfile->opening=cfile->opening_audio=cfile->opening_only_audio=FALSE;

  com=g_strdup_printf("smogrify commit_audio %s",cfile->handle);
  system(com);
  do_auto_dialog(_ ("Committing audio"));
  g_free(com);

  reget_afilesize(cfile);
  get_play_times();
  cfile->changed=TRUE;
  d_print(_ ("done\n"));
  mesg=g_strdup_printf(_ ("New audio: %d Hz %d channel(s) %d bps\n"),cfile->arate,cfile->achans,cfile->asampsize);
  d_print(mesg);
  g_free(mesg);

  gtk_widget_set_sensitive(mainw->loop_video,TRUE);
}

void 
on_load_vcd_ok_clicked                (GtkButton     *button,
				       gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  if (GPOINTER_TO_INT (user_data)==1) {
    g_snprintf (file_name,256,"dvd://%d",(int)mainw->fx1_val);
    mainw->file_open_params=g_strdup_printf ("-chapter %d",(int)mainw->fx2_val);
  }
  else {
    g_snprintf (file_name,256,"vcd://%d",(int)mainw->fx1_val);
  }
  open_sel_range_activate();
}


void
on_xmms_play_audio_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  while (g_main_context_iteration(NULL,FALSE));

  fileselection = create_fileselection (_ ("Select Audio File"),2,NULL);
  g_signal_connect(GTK_FILE_SELECTION(fileselection)->ok_button, "clicked", G_CALLBACK (on_xmms_ok_clicked),NULL);
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->audio_dir,-1,NULL,NULL,NULL));
  gtk_widget_show (fileselection);
}

void
on_xmms_random_audio_activate                (GtkMenuItem     *menuitem,
					    gpointer         user_data)
{
  xranw = create_rp_dialog();
  gtk_widget_show (xranw->rp_dialog);
  g_snprintf(file_name,256,"%s",mainw->xmms_dir);
}


void on_xmms_ok_clicked                (GtkButton     *button,
					gpointer         user_data)
{
  gchar *com=g_strdup_printf("smogrify xmmsplay \"%s\"",gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))));
  system(com);
  g_free(com);
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}


void on_xmms_ran_ok_clicked                (GtkButton     *button,
					gpointer         user_data)
{
  gchar *com;
  gint curr_file=mainw->current_file;
  // we need to do some extra work, because we could be called whilst an effect is processing

  gchar *dir=g_strdup(gtk_entry_get_text(GTK_ENTRY(xranw->dir)));
  get_dirname(dir);

  // an example of using 'get_temp_handle()' ////////////////////
  if (get_temp_handle(mainw->first_free_file)==FALSE) {
    g_free(dir);
    return;
  }

  com=g_strdup_printf("smogrify xmmsrandom %s %d %d %d %d \"%s\"",cfile->handle,gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(xranw->numtracks)),gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(xranw->subdir_check)),gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(xranw->minsize)),gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(xranw->maxsize)),g_filename_from_utf8 (dir,-1,NULL,NULL,NULL));
  g_free(dir);

  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  while (g_main_context_iteration(NULL,FALSE));
  g_free(xranw);

  system(com);

  do_progress_dialog(TRUE,FALSE,_ ("Selecting tracks"));
  g_free(com);

  com=g_strdup_printf("smogrify close %s",cfile->handle);
  system(com);
  g_free(com);
  g_free(cfile);
  cfile=NULL;

  mainw->current_file=curr_file;
  ////////////////////////////////////////

  if (mainw->current_file>-1&&!(cfile->proc_ptr==NULL)) {
    // if we are in another processing dialogue
    desensitize();
    procw_desensitize();
  }
  else {
    // or not
    if (mainw->current_file>-1) {
      sensitize();
    }
    else {
      close_current_file();
    }
  }

  if (mainw->error==TRUE) {
    do_error_dialog(mainw->msg);
  }

  clear_mainw_msg();

  g_snprintf(mainw->xmms_dir,256,"%s",file_name);

}


void on_xmms_stop_audio_activate                (GtkMenuItem     *menuitem,
						 gpointer         user_data)
{
  gchar *com=g_strdup("smogrify xmmsstop");
  system(com);
  g_free(com);
}


void
on_rename_activate                    (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  renamew=create_rename_dialog(1);
  gtk_widget_show(renamew->rename_dialog);

}


void
on_rename_set_name                   (GtkButton       *button,
				      gpointer         user_data)
{
  gchar title[256];
  g_snprintf(title,256,"%s",gtk_entry_get_text(GTK_ENTRY(renamew->entry)));
  gtk_widget_destroy(renamew->rename_dialog);
  g_free(renamew);

  if (!(strlen(title))) return;

  set_menu_text(cfile->menuentry,title,FALSE);
  g_snprintf (cfile->name,256,"%s",title);
  set_main_title(title,0);
  cfile->was_renamed=TRUE;
}

void
on_toy_activate                (GtkMenuItem     *new_toy,
				gpointer         user_data)
{
  switch (mainw->toy_type) {
  case TOY_NONE : 
    g_signal_handler_block (mainw->toy_none, mainw->toy_func_none);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->toy_none),FALSE);
    gtk_widget_queue_draw (mainw->toy_none);
    g_signal_handler_unblock (mainw->toy_none, mainw->toy_func_none);
    break;
  case TOY_RANDOM_FRAMES : 
    g_signal_handler_block (mainw->toy_random_frames, mainw->toy_func_random_frames);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->toy_random_frames),FALSE);
    g_signal_handler_unblock (mainw->toy_random_frames, mainw->toy_func_random_frames);
    if (mainw->playing_file>-1) {
      if (mainw->faded==TRUE) {
	gtk_widget_hide(mainw->image272);
	gtk_widget_hide(mainw->image273);
      }
      load_start_image (cfile->start);
      load_end_image (cfile->end);
    }
    break;
  default :
    break;
  }
    
  if (GTK_WIDGET (new_toy)==GTK_WIDGET (mainw->toy_random_frames)) {
    mainw->toy_type=TOY_RANDOM_FRAMES;
    g_signal_handler_block (mainw->toy_random_frames, mainw->toy_func_random_frames);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->toy_random_frames),TRUE);
    g_signal_handler_unblock (mainw->toy_random_frames, mainw->toy_func_random_frames);
  }
  if (GTK_WIDGET (new_toy)==GTK_WIDGET (mainw->toy_tv)) {
#ifdef LIVES_TV_CHANNEL
    mainw->toy_type=TOY_TV;
    deduce_file (LIVES_TV_CHANNEL1,0.,0);
    if (prefs->discard_tv==TRUE) {
      close_current_file();
    }
#endif
  }
  else if (GTK_WIDGET (new_toy)==GTK_WIDGET (mainw->toy_random_frames)) {
    mainw->toy_type=TOY_RANDOM_FRAMES;
    g_signal_handler_block (mainw->toy_random_frames, mainw->toy_func_random_frames);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->toy_random_frames),TRUE);
    g_signal_handler_unblock (mainw->toy_random_frames, mainw->toy_func_random_frames);
  }
  else {
    mainw->toy_type=TOY_NONE;
    g_signal_handler_block (mainw->toy_none, mainw->toy_func_none);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->toy_none),TRUE);
    g_signal_handler_unblock (mainw->toy_none, mainw->toy_func_none);
    if (mainw->faded==TRUE&&mainw->foreign==FALSE) {
      gtk_widget_show(mainw->image272);
      gtk_widget_show(mainw->image273);
    }
  }
}



void
on_preview_spinbutton_changed          (GtkSpinButton   *spinbutton,
					gpointer         user_data)
{
  // update the play window preview
  gint preview_frame=gtk_spin_button_get_value_as_int(spinbutton);
  if ((preview_frame)==mainw->preview_frame) return;
  mainw->preview_frame=preview_frame;
  load_preview_image(TRUE);
}

void
on_prv_link_toggled                (GtkToggleButton *togglebutton,
				    gpointer         user_data)
{
  if (!gtk_toggle_button_get_active(togglebutton)) return;
  mainw->prv_link=GPOINTER_TO_INT(user_data);
  if (mainw->is_processing&&(mainw->prv_link==PRV_START||mainw->prv_link==PRV_END)) {
    // block spinbutton in play window
    gtk_widget_set_sensitive(mainw->preview_spinbutton,FALSE);
  }
  else {
    gtk_widget_set_sensitive(mainw->preview_spinbutton,TRUE);
  }
  load_preview_image(FALSE);
  gtk_widget_grab_focus (mainw->preview_spinbutton);
}

void
on_spinbutton_start_value_changed          (GtkSpinButton   *spinbutton,
					    gpointer         user_data)
{
  gint start,ostart=cfile->start;

  if (mainw->playing_file==-1&&mainw->current_file==0) return;

  if ((start=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton)))==cfile->start) return;
  cfile->start=start;

  if (mainw->selwidth_locked==TRUE) {
    cfile->end+=cfile->start-ostart;
    if (cfile->end>cfile->frames) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->start-cfile->end+cfile->frames);
    }
    else {
      mainw->selwidth_locked=FALSE;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->end);
      load_end_image (cfile->end);
      mainw->selwidth_locked=TRUE;
    }
  }

  else {
    if ((cfile->start==1||cfile->end==cfile->frames)&&!(cfile->start==1&&cfile->end==cfile->frames)) {
      gtk_widget_set_sensitive(mainw->select_invert,TRUE);
    }
    else {
      gtk_widget_set_sensitive(mainw->select_invert,FALSE);
    }
  }

  if (cfile->end!=mainw->preview_frame||mainw->prv_link!=PRV_START) {
    load_start_image(cfile->start);
  }
  if (cfile->start>cfile->end) {
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->start);
  }
  set_sel_label();
  get_play_times();
}



void
on_spinbutton_end_value_changed          (GtkSpinButton   *spinbutton,
					  gpointer         user_data)
{
  gint end,oend=cfile->end;

  if (mainw->playing_file==-1&&mainw->current_file==0) return;

  if ((end=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton)))==cfile->end) return;
  cfile->end=end;

  if (mainw->selwidth_locked==TRUE) {
    cfile->start+=cfile->end-oend;
    if (cfile->start<1) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->end-cfile->start+1);
    }
    else {
      mainw->selwidth_locked=FALSE;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->start);
      load_start_image (cfile->start);
      mainw->selwidth_locked=TRUE;
    }
  }

  else {
    if ((cfile->start==1||cfile->end==cfile->frames)&&!(cfile->start==1&&cfile->end==cfile->frames)) {
      gtk_widget_set_sensitive(mainw->select_invert,TRUE);
    }
    else {
      gtk_widget_set_sensitive(mainw->select_invert,FALSE);
    }
  }


  if (cfile->end!=mainw->preview_frame||mainw->prv_link!=PRV_END) {
    load_end_image(cfile->end);
  }
  if (cfile->end<cfile->start) {
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),cfile->end);
  }

  set_sel_label();
  get_play_times();

}


// for the timer bars

gint
expose_vid_event (GtkWidget *widget, GdkEventExpose *event)
{
  // only act on last event
  if (event->count>0||mainw->multitrack!=NULL) {
    return TRUE;
  }
  if (mainw->is_ready==FALSE||mainw->foreign) return FALSE;
  block_expose();

  if (mainw->current_file==-1) {
    if (mainw->video_drawable!=NULL)
      gdk_pixmap_unref(mainw->video_drawable);
     
    mainw->video_drawable = gdk_pixmap_new(mainw->video_draw->window,
					   mainw->video_draw->allocation.width,
					   mainw->video_draw->allocation.height,
					   -1);

    gdk_draw_rectangle (mainw->video_drawable,
			mainw->video_draw->style->bg_gc[GTK_WIDGET_STATE (widget)],
			TRUE,
			0, 0,
			mainw->video_draw->allocation.width,
			mainw->video_draw->allocation.height
			);
    
    gtk_widget_queue_draw_area(mainw->vidbar,event->area.x,event->area.y,event->area.width,event->area.height);
  }

  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		  mainw->video_drawable,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
  
  unblock_expose();
  return FALSE;
  
}

gint
expose_laud_event (GtkWidget *widget, GdkEventExpose *event)
{
  // only act on last event
  if (event->count>0||mainw->multitrack!=NULL) {
    return TRUE;
  }
  if (mainw->is_ready==FALSE||mainw->foreign) return FALSE;
  block_expose();

  if (mainw->current_file==-1) {
    if (mainw->laudio_drawable!=NULL)
      gdk_pixmap_unref(mainw->laudio_drawable);
    
    mainw->laudio_drawable = gdk_pixmap_new(mainw->laudio_draw->window,
					   mainw->laudio_draw->allocation.width,
					   mainw->laudio_draw->allocation.height,
					   -1);

    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->bg_gc[GTK_WIDGET_STATE (widget)],
			TRUE,
			0, 0,
			mainw->laudio_draw->allocation.width,
			mainw->laudio_draw->allocation.height
			);
    gtk_widget_queue_draw_area(mainw->laudbar,event->area.x,event->area.y,event->area.width,event->area.height);
  }
  
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		  mainw->laudio_drawable,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  unblock_expose();
  return FALSE;
}


gint
expose_raud_event (GtkWidget *widget, GdkEventExpose *event)
{
  // only act on last event
  if (event->count>0||mainw->multitrack!=NULL) {
    return TRUE;
  }
  if (mainw->is_ready==FALSE||mainw->foreign) return FALSE;
  block_expose();

  if (mainw->current_file==-1) {
    if (mainw->raudio_drawable!=NULL)
      gdk_pixmap_unref(mainw->raudio_drawable);
    
    mainw->raudio_drawable = gdk_pixmap_new(mainw->raudio_draw->window,
					    mainw->raudio_draw->allocation.width,
					    mainw->raudio_draw->allocation.height,
					    -1);

    gdk_draw_rectangle (mainw->raudio_drawable,
			mainw->raudio_draw->style->bg_gc[GTK_WIDGET_STATE (widget)],
			TRUE,
			0, 0,
			mainw->raudio_draw->allocation.width,
			mainw->raudio_draw->allocation.height);
    gtk_widget_queue_draw_area(mainw->raudbar,event->area.x,event->area.y,event->area.width,event->area.height);
  }
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		  mainw->raudio_drawable,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  unblock_expose();
  return FALSE;
}


gint
config_event (GtkWidget *widget, GdkEventConfigure *event)
{
  if (GTK_WIDGET_VISIBLE (mainw->LiVES)) {
    if (mainw->is_ready==TRUE) {
      if (mainw->gc==NULL) {
	mainw->gc=gdk_gc_new (GDK_DRAWABLE (mainw->LiVES->window));

	// this is nice, but it sets button text and entries too
	//gdk_gc_set_rgb_fg_color (GDK_GC (mainw->LiVES->style->fg_gc[2]),&palette->info_text);
      }
      if (mainw->current_file>-1) {
	if (mainw->video_draw->allocation.width!=mainw->LiVES->allocation.width) {
	  gtk_drawing_area_size(GTK_DRAWING_AREA(mainw->video_draw),mainw->LiVES->allocation.width,10);
	  while (g_main_context_iteration(NULL,FALSE));
	}
	get_play_times();
      }
    }
  }
  return FALSE;
}


gint
expose_play_window (GtkWidget *widget, GdkEventExpose *event) {
  // only act on last event
  if (event->count>0) {
    return TRUE;
  }
  if (((mainw->current_file==-1||!cfile->is_loaded)||(mainw->multitrack!=NULL&&mainw->playing_file<0))&&mainw->imframe!=NULL) {
    block_expose();
    gdk_draw_pixbuf (GDK_DRAWABLE (mainw->play_window->window),mainw->gc,GDK_PIXBUF (mainw->imframe),0,0,0,0,-1,-1,GDK_RGB_DITHER_NONE,0,0);
    unblock_expose();
  }
  return FALSE;
}





// these two really belong with the processing widget



void
on_effects_paused                     (GtkButton       *button,
				       gpointer         user_data)
{
  gchar *com;
 
  if (mainw->effects_paused==FALSE) {
    com=g_strdup_printf("smogrify pause %s",cfile->handle);
    if (mainw->preview==FALSE) {
      gtk_button_set_label(GTK_BUTTON(button),_ ("Resume"));
      if (!cfile->nokeep) {
	gtk_button_set_label(GTK_BUTTON(cfile->proc_ptr->cancel_button),_ ("Keep"));
	gtk_label_set_text(GTK_LABEL(cfile->proc_ptr->label2),_ ("\nPaused\n(click Keep to keep what you have and stop)\n(click Resume to continue processing)"));
      }
      d_print(_ ("paused..."));
    }
    

  } else {
    com=g_strdup_printf("smogrify resume %s",cfile->handle);
    if (mainw->preview==FALSE) {
      gtk_button_set_label(GTK_BUTTON(button),_ ("Pause"));
      gtk_button_set_label(GTK_BUTTON(cfile->proc_ptr->cancel_button), _("Cancel"));
      gtk_label_set_text(GTK_LABEL(cfile->proc_ptr->label2),_ ("\nPlease Wait"));
      d_print(_ ("resumed..."));
    }
  }
  if (mainw->internal_messaging==FALSE) {
    system(com);
  }
  g_free(com);
  mainw->effects_paused=!mainw->effects_paused;
}



void
on_preview_clicked                     (GtkButton       *button,
					gpointer         user_data)
{
  // play an effect/tool preview
  // IMPORTANT: cfile->undo_start and cfile->undo_end determine which frames
  // should be played

  gboolean resume_after;
  gint ostart=cfile->start;
  gint oend=cfile->end;
  gshort oaudp=prefs->audio_player;
  gint toy_type=mainw->toy_type;
  gboolean ointernal_messaging=mainw->internal_messaging;
  int i;
  guint64 old_rte; //TODO - block better
  gint current_file=mainw->current_file;

  mainw->preview=TRUE;
  old_rte=mainw->rte;

  if (mainw->internal_messaging) {
    mainw->internal_messaging=FALSE;
    // for realtime fx previews, we will switch all effects off and restore old
    // value after
    mainw->rte=EFFECT_NONE;
  }

  if (mainw->playing_file==-1) {
    if (cfile->opening) {
      // set vid player to int, and audio player to sox
      
      if (prefs->audio_player==AUD_PLAYER_MPLAYER) {
	switch_aud_to_sox();
      }
      if (cfile->opening_only_audio==FALSE) {
	mainw->toy_type=TOY_NONE;
	gtk_widget_set_sensitive(mainw->toys,FALSE);
      }
      gtk_widget_show (mainw->LiVES);

      if (!cfile->is_loaded) {
	if (mainw->play_window!=NULL) {
	  g_signal_handler_block(mainw->play_window,mainw->pw_exp_func);
	  cfile->is_loaded=TRUE;
	  resize_play_window();
	  cfile->is_loaded=FALSE;
	}
      }
    }

    resume_after=FALSE;

    if (mainw->multitrack!=NULL) {
      if (mainw->play_window==NULL) {
	mainw->must_resize=TRUE;
	mainw->pwidth=MT_PLAY_WIDTH;
	mainw->pheight=MT_PLAY_HEIGHT;
      }
      else {
	mainw->pwidth=cfile->hsize;
	mainw->pheight=cfile->vsize;
      }
    }


    if (user_data!=NULL) {
      if (mainw->play_window!=NULL) {
	g_signal_handler_block(mainw->play_window,mainw->pw_exp_func);
	resize_play_window();
      }
      if (mainw->multitrack!=NULL&&mainw->multitrack->is_rendering) {
	mainw->play_start=1;
	mainw->play_end=cfile->frames;
      }
      else {
	mainw->play_start=1;
	mainw->play_end=INT_MAX;
      }
    }
    else {
      if (!mainw->is_processing&&!mainw->is_rendering) {
	mainw->play_start=cfile->start=cfile->undo_start;
	mainw->play_end=cfile->end=cfile->undo_end;
      }
      else {
	mainw->play_start=calc_variable_frame (mainw->current_file,event_list_get_start_secs (cfile->event_list));
	mainw->play_end=INT_MAX;
      }
    }

    // stop effects processing (if preferred)
    if (prefs->pause_effect_during_preview) {
      if (!(mainw->effects_paused)) {
	on_effects_paused(GTK_BUTTON(cfile->proc_ptr->pause_button),NULL);
	resume_after=TRUE;
      }
    }

    if (button!=NULL) gtk_button_set_label(GTK_BUTTON(button),"Stop");
    if (cfile->proc_ptr!=NULL) {
      gtk_widget_set_sensitive(cfile->proc_ptr->pause_button,FALSE);
      gtk_widget_set_sensitive(cfile->proc_ptr->cancel_button,FALSE);
    }
    if (cfile->opening==FALSE) {
      gtk_widget_set_sensitive (mainw->showfct,FALSE);
    }

    unlink(cfile->info_file);
    desensitize();
   
    if (cfile->opening||cfile->opening_only_audio) {
      gtk_widget_hide (cfile->proc_ptr->processing);
      if (!cfile->opening_audio) {
	load_start_image (0);
	load_end_image (0);
      }
      resize (1);
    }

    if (ointernal_messaging) {
      system ("sync;sync;sync");
    }
    current_file=mainw->current_file;
    resize(1);

    // play the clip
    on_playsel_activate(NULL,NULL);

    if (current_file!=mainw->current_file) {
      if (cfile->proc_ptr!=NULL) {
	mainw->files[current_file]->proc_ptr=cfile->proc_ptr;
	cfile->proc_ptr=NULL;
      }
      if (mainw->is_rendering) {
	mainw->files[current_file]->event_list=cfile->event_list;
	mainw->files[current_file]->next_event=cfile->next_event;
	cfile->event_list=NULL;
	cfile->next_event=NULL;
	mainw->current_file=current_file;
      }
      else switch_to_file ((mainw->current_file=0),current_file);
    }

    // restart effects processing (if necessary)
    if (resume_after) on_effects_paused(GTK_BUTTON(cfile->proc_ptr->pause_button),NULL);

    // user_data is non-NULL if called from multitrack. We want to preserve the value of cancelled.
    if (user_data==NULL) mainw->cancelled=0;

    if (oaudp==AUD_PLAYER_MPLAYER&&prefs->audio_player!=oaudp) {
      switch_aud_to_mplayer();
    }

    cfile->start=ostart;
    cfile->end=oend;

    mainw->toy_type=toy_type;
    gtk_widget_set_sensitive(mainw->toys,TRUE);
    
    if (cfile->proc_ptr!=NULL) {
      // proc_ptr can be NULL if we finished loading with a bg generator running
      gtk_widget_show (cfile->proc_ptr->processing);
      gtk_button_set_label(GTK_BUTTON(button),_ ("Preview"));
      gtk_widget_set_sensitive(cfile->proc_ptr->pause_button,TRUE);
      gtk_widget_set_sensitive(cfile->proc_ptr->cancel_button,TRUE);
    }
    mainw->preview=FALSE;
    desensitize();
    procw_desensitize();

    if (cfile->opening==FALSE) {
      gtk_widget_set_sensitive (mainw->showfct,TRUE);
    }

    if (cfile->opening) {
      for (i=1;i<MAX_FILES;i++) {
	if (!(mainw->files[i]==NULL)) {
	  if (!(mainw->files[i]->menuentry==NULL)) {
	    gtk_widget_set_sensitive (mainw->files[i]->menuentry, TRUE);
	  }}}
      if (mainw->play_window!=NULL) {
	if (cfile->opening_audio==FALSE) {
	  g_signal_handler_unblock(mainw->play_window,mainw->pw_exp_func);
	}
	resize_play_window();
      }}}
  else {
    // this is a special value of cancel - don't propogate it to "open"
    mainw->cancelled=CANCEL_NO_PROPOGATE;
  }

  mainw->preview=FALSE;
    
  if (mainw->preview_box!=NULL) gtk_tooltips_set_tip (mainw->tooltips, mainw->p_playbutton,"Preview", NULL);
  gtk_tooltips_set_tip (mainw->tooltips, mainw->m_playbutton,"Preview", NULL);

  // redraw our bars for the clip 
  if (!mainw->merge) {
    get_play_times();
  }
  if (ointernal_messaging) {
    mainw->internal_messaging=TRUE;

    // switch realtime fx back on
    mainw->rte=old_rte;
  }
  
  if (mainw->play_window!=NULL&&mainw->fs) {
    // this prevents a hang when the separate window is visible
    // it may be the first time we have shown it
    block_expose();
    while (g_main_context_iteration(NULL,FALSE));
    unblock_expose();
    gtk_widget_queue_draw (mainw->LiVES);
  }
}


void
changed_fps_during_pb           (GtkSpinButton   *spinbutton,
				 gpointer         user_data)
{
  if (cfile->pb_fps==(gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton))*1000)/1000.) return;

  cfile->pb_fps=(gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton))*1000)/1000.;

  if (cfile->pb_fps==0.) {
    mainw->period=INT64_MAX;
  }
  else {
    mainw->period=U_SEC/cfile->pb_fps;
  }
}






// next few functions are for the timer bars
gboolean
on_mouse_sel_update           (GtkWidget       *widget,
			       GdkEventMotion  *event,
			       gpointer         user_data)
{
  if (mainw->current_file>-1&&mainw->sel_start>0) {
    gint x,sel_current;

    gdk_window_get_pointer(GDK_WINDOW (mainw->LiVES->window), &x, NULL, NULL);
    if ((sel_current=calc_variable_frame(mainw->current_file,(gdouble)x/(gdouble)mainw->vidbar->allocation.width*cfile->total_time))==0) sel_current=cfile->frames+1;

    if (mainw->sel_move==SEL_MOVE_SINGLE) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),sel_current);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),sel_current);
    }

    if (mainw->sel_move==SEL_MOVE_START||(mainw->sel_move==SEL_MOVE_AUTO&&sel_current<=mainw->sel_start)) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),sel_current);
    }
    if (mainw->sel_move==SEL_MOVE_END||(mainw->sel_move==SEL_MOVE_AUTO&&sel_current>=mainw->sel_start)) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),sel_current-1);
    }
  }
  return FALSE;
}


gboolean
on_mouse_sel_reset           (GtkWidget       *widget,
			      GdkEventButton  *event,
			      gpointer         user_data)
{
  if (mainw->current_file<=0) return FALSE;
  mainw->sel_start=0;
  if (!mainw->mouse_blocked) {
    g_signal_handler_block (mainw->eventbox2,mainw->mouse_fn1);
    mainw->mouse_blocked=TRUE;
  }
  return FALSE;
}


gboolean
on_mouse_sel_start           (GtkWidget       *widget,
			      GdkEventButton  *event,
			      gpointer         user_data)
{
  gint x;
  if (mainw->current_file<=0) return FALSE;

  gdk_window_get_pointer(GDK_WINDOW (mainw->LiVES->window), &x, NULL, NULL);
  if (!(mainw->sel_start=calc_variable_frame(mainw->current_file,(gdouble)x/(gdouble)mainw->vidbar->allocation.width*cfile->total_time))) mainw->sel_start=cfile->frames+1;
  
  if (event->button==3&&!mainw->selwidth_locked) {
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),mainw->sel_start);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),mainw->sel_start);
    mainw->sel_move=SEL_MOVE_AUTO;
  }
  
  else {
    if (event->button==2&&!mainw->selwidth_locked) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),mainw->sel_start);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),(gint)mainw->sel_start);
      mainw->sel_move=SEL_MOVE_SINGLE;
    }
    
    else {
      if (mainw->selwidth_locked==FALSE) {
	if (mainw->sel_start<cfile->end&&((mainw->sel_start-cfile->start)*(mainw->sel_start-cfile->start)<=(cfile->end-mainw->sel_start)*(cfile->end-mainw->sel_start)||(mainw->sel_start<cfile->start))) {
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),mainw->sel_start);
	  mainw->sel_move=SEL_MOVE_START;
	}
	else {
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),mainw->sel_start-1);
	  mainw->sel_move=SEL_MOVE_END;
	}
      }
      else {
	// locked selection
	if (mainw->sel_start>cfile->end) {
	  // past end
	  if (cfile->end+cfile->end-cfile->start+1<=cfile->frames) {
	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->end+cfile->end-cfile->start+1);
	    mainw->sel_move=SEL_MOVE_START;
	  }
	}
	else {
	  if (mainw->sel_start>=cfile->start) {
	    if (mainw->sel_start>cfile->start+(cfile->end-cfile->start+1)/2) {
	      // nearer to end
	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),mainw->sel_start);
	      mainw->sel_move=SEL_MOVE_END;
	    }
	    else {
	      // nearer to start
	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),mainw->sel_start);
	      mainw->sel_move=SEL_MOVE_START;
	    }
	  }
	  else {
	    // before start
	    if (cfile->start-cfile->end+cfile->start-1>=1) {
	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),cfile->start-1);
	      mainw->sel_move=SEL_MOVE_END;
	    }
	  }}}}}
  if (mainw->mouse_blocked) {// stops a warning if the user clicks around a lot...
    g_signal_handler_unblock (mainw->eventbox2,mainw->mouse_fn1);
    mainw->mouse_blocked=FALSE;
  }
  return FALSE;
}

gboolean
on_hrule_update           (GtkWidget       *widget,
			   GdkEventMotion  *event,
			   gpointer         user_data) {
  gint x;
  if (mainw->current_file<=0) return FALSE;

  gdk_window_get_pointer(GDK_WINDOW (mainw->LiVES->window), &x, NULL, NULL);
  if (x<0) x=0;

  // figure out where ptr should be even when > cfile->frames
  if ((GTK_RULER (mainw->hruler)->position=cfile->pointer_time=calc_variable_time(calc_variable_frame(mainw->current_file,(gdouble)x/mainw->vidbar->allocation.width*cfile->total_time)))<=0.) GTK_RULER (mainw->hruler)->position=cfile->pointer_time=(gdouble)x/mainw->vidbar->allocation.width*cfile->total_time;
  gtk_widget_queue_draw (mainw->hruler);
  get_play_times();
  return FALSE;
}


gboolean
on_hrule_reset           (GtkWidget       *widget,
			  GdkEventButton  *event,
			  gpointer         user_data)
{
  //button release
  gint x;
  if (mainw->current_file<=0) return FALSE;

  gdk_window_get_pointer(GDK_WINDOW (mainw->LiVES->window), &x, NULL, NULL);
  if (x<0) x=0;
  if ((GTK_RULER (mainw->hruler)->position=cfile->pointer_time=calc_variable_time(calc_variable_frame(mainw->current_file,(gdouble)x/mainw->vidbar->allocation.width*cfile->total_time)))<=0.) GTK_RULER (mainw->hruler)->position=cfile->pointer_time=(gdouble)x/mainw->vidbar->allocation.width*cfile->total_time;

  if (!mainw->hrule_blocked) {
    g_signal_handler_block (mainw->eventbox5,mainw->hrule_func);
    mainw->hrule_blocked=TRUE;
  }
  if (cfile->pointer_time>0.) {
    gtk_widget_set_sensitive (mainw->rewind, TRUE);
    gtk_widget_set_sensitive (mainw->trim_to_pstart, (cfile->achans*cfile->frames>0));
    gtk_widget_set_sensitive (mainw->m_rewindbutton, TRUE);
    if (mainw->preview_box!=NULL) {
      gtk_widget_set_sensitive (mainw->p_rewindbutton, TRUE);
    }
  }
  else {
    gtk_widget_set_sensitive (mainw->rewind, FALSE);
    gtk_widget_set_sensitive (mainw->trim_to_pstart, FALSE);
    gtk_widget_set_sensitive (mainw->m_rewindbutton, FALSE);
    if (mainw->preview_box!=NULL) {
      gtk_widget_set_sensitive (mainw->p_rewindbutton, FALSE);
    }
  }
  gtk_widget_queue_draw (mainw->hruler);
  get_play_times();
  return FALSE;
}


gboolean
on_hrule_set           (GtkWidget       *widget,
			GdkEventButton  *event,
			gpointer         user_data)
{
  // button press
  gint x;
  if (mainw->current_file<=0) return FALSE;
  gdk_window_get_pointer(GDK_WINDOW (mainw->LiVES->window), &x, NULL, NULL);
  if (x<0) x=0;
  if ((GTK_RULER (mainw->hruler)->position=cfile->pointer_time=calc_variable_time(calc_variable_frame(mainw->current_file,(gdouble)x/mainw->vidbar->allocation.width*cfile->total_time)))<=0.) GTK_RULER (mainw->hruler)->position=cfile->pointer_time=(gdouble)x/mainw->vidbar->allocation.width*cfile->total_time;
  gtk_widget_queue_draw (mainw->hruler);
  get_play_times();
  if (mainw->hrule_blocked) {
    g_signal_handler_unblock (mainw->eventbox5,mainw->hrule_func);
    mainw->hrule_blocked=FALSE;
  }
  return FALSE;
}


gboolean
frame_context (GtkWidget *widget, GdkEventButton  *event, gpointer which) {
  GtkWidget *fileselection;
  gint frame;
  gchar *compl=g_strdup_printf ("*.%s",prefs->image_ext);
  gchar *compl_str=g_strdup_printf (_ ("Save Frame as %s..."),compl);

  // check if a file is loaded
  if (mainw->current_file<=0) return FALSE;

  // only accept right mouse clicks
  if (event->button!=3) return FALSE;

  switch (GPOINTER_TO_INT (which)) {
  case 1:
    // start frame
    frame=cfile->start;
    break;
  case 2:
    // end frame
    frame=cfile->end;
    break;
  default:
    // preview frame
    frame=mainw->preview_frame;
    break;
  }

  fileselection = create_fileselection (compl_str,0,NULL);

  if (strlen(mainw->image_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->image_dir,-1,NULL,NULL,NULL));
  }
  gtk_file_selection_complete(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (compl,-1,NULL,NULL,NULL));
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (ok_save_frame),GINT_TO_POINTER (frame));
  gtk_widget_show (fileselection);

  g_free (compl);
  g_free (compl_str);

  return FALSE;
}




void
on_slower_pressed (GtkButton *button,
		   gpointer user_data)
{
  if (mainw->playing_file==-1||mainw->internal_messaging) return;

  if (mainw->rte_keys!=-1) {
    mainw->blend_factor-=(mainw->blend_factor>0.);
    mainw->blend_factor-=(mainw->blend_factor>0.);
    weed_set_blend_factor(mainw->rte_keys);
    return;
  }

  if (mainw->record&&!(prefs->rec_opts&REC_FPS)) return;
  if (cfile->next_event!=NULL) return;

#define PB_CHANGE_RATE .2
  if (cfile->pb_fps==0.) return;
  if (cfile->pb_fps>0.) {
    if (cfile->pb_fps<PB_CHANGE_RATE) cfile->pb_fps=PB_CHANGE_RATE;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps),cfile->pb_fps-PB_CHANGE_RATE);
    mainw->period=(cfile->pb_fps-PB_CHANGE_RATE)?U_SEC/(cfile->pb_fps-PB_CHANGE_RATE):INT_MAX;
  }
  else {
    if (cfile->pb_fps>-PB_CHANGE_RATE) cfile->pb_fps=-PB_CHANGE_RATE;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps),cfile->pb_fps+PB_CHANGE_RATE);
    mainw->period=(cfile->pb_fps+PB_CHANGE_RATE)?U_SEC/(cfile->pb_fps+PB_CHANGE_RATE):INT_MAX;
  }
}

void
on_faster_pressed (GtkButton *button,
		   gpointer user_data)
{
  if (mainw->playing_file==-1||mainw->internal_messaging) return;

  if (mainw->rte_keys!=-1) {
    mainw->blend_factor+=(mainw->blend_factor<255.);
    mainw->blend_factor+=(mainw->blend_factor<255.);
    weed_set_blend_factor(mainw->rte_keys);
    return;
  }

  if (cfile->play_paused==TRUE&&cfile->freeze_fps<0.) {
    cfile->pb_fps=-.00000001;
  }
  cfile->play_paused=FALSE;

  if (mainw->record&&!(prefs->rec_opts&REC_FPS)) return;
  if (cfile->next_event!=NULL) return;

  if (cfile->pb_fps>=0.) {
    if (cfile->pb_fps==FPS_MAX) return;
    if (cfile->pb_fps>FPS_MAX-PB_CHANGE_RATE) cfile->pb_fps=FPS_MAX-PB_CHANGE_RATE;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps),cfile->pb_fps+PB_CHANGE_RATE);
    mainw->period=(cfile->pb_fps+PB_CHANGE_RATE)?U_SEC/(cfile->pb_fps+PB_CHANGE_RATE):INT_MAX;
  }
  else {
    if (cfile->pb_fps==-FPS_MAX) return;
    if (cfile->pb_fps<-FPS_MAX+PB_CHANGE_RATE) cfile->pb_fps=-FPS_MAX+PB_CHANGE_RATE;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps),cfile->pb_fps-PB_CHANGE_RATE);
    mainw->period=(cfile->pb_fps-PB_CHANGE_RATE)?U_SEC/(cfile->pb_fps-PB_CHANGE_RATE):INT_MAX;
  }
}



//TODO - make pref
#define CHANGE_SPEED (cfile->pb_fps/KEY_RPT_INTERVAL*4)
void
on_back_pressed (GtkButton *button,
		   gpointer user_data)
{
  if (mainw->playing_file==-1||mainw->internal_messaging) return;
  if (mainw->record&&!(prefs->rec_opts&REC_FRAMES)) return;
  if (cfile->next_event!=NULL) return;

  mainw->deltaticks-=(gint64)(CHANGE_SPEED*mainw->period);

}

void
on_forward_pressed (GtkButton *button,
		   gpointer user_data)
{
  if (mainw->playing_file==-1||mainw->internal_messaging) return;
  if (mainw->record&&!(prefs->rec_opts&REC_FRAMES)) return;
  if (cfile->next_event!=NULL) return;

  mainw->deltaticks+=(gint64)(CHANGE_SPEED/2.*mainw->period);
}


gboolean freeze_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
  if (mainw->playing_file==-1) return TRUE;
  if (mainw->record&&!(prefs->rec_opts&REC_FRAMES)) return TRUE;

  mainw->rte_keys=EFFECT_NONE;

  if (cfile->play_paused==TRUE) {
    cfile->pb_fps=cfile->freeze_fps;
    cfile->play_paused=FALSE;
  }
  else {
    cfile->freeze_fps=cfile->pb_fps;
    cfile->play_paused=TRUE;
    cfile->pb_fps=0.;
    mainw->deltaticks=0.;
    if (!mainw->noswitch) load_frame_image (cfile->frameno+(mainw->loop==TRUE&&mainw->playing_sel\
==TRUE?cfile->start-1:mainw->play_start),cfile->last_frameno);
  }
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_pb_fps),cfile->pb_fps);
  return TRUE;
}



gboolean nervous_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer clip_number)
{
  mainw->nervous=!mainw->nervous;
  return TRUE;
}



gboolean storeclip_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer clip_number)
{
  // ctrl-fn key will store a clip for later switching
  gint clip=GPOINTER_TO_INT (clip_number)-1;
  int i;

  if (clip>=FN_KEYS-1) {
    // last fn key will clear all
    for (i=0;i<FN_KEYS-1;i++) {
      mainw->clipstore[i]=0;
    }
    return TRUE;
  }

  if (mainw->clipstore[clip]<1||mainw->files[mainw->clipstore[clip]]==NULL) {
    mainw->clipstore[clip]=mainw->current_file;
  }
  else {
  if (mainw->current_file<1||mainw->preview||(mainw->is_processing&&cfile->is_loaded)||mainw->cliplist==NULL||(cfile->event_list!=NULL&&!mainw->is_rendering)) return TRUE;
    if (mainw->playing_file==-1) {
      if (!cfile->is_loaded) mainw->cancelled=CANCEL_NO_PROPOGATE;
      switch_to_file (mainw->current_file,mainw->clipstore[clip]);
    }
    else {
      if (mainw->playing_file>0) {
	if (mainw->num_tr_applied>0) {
	  mainw->blend_file=mainw->clipstore[clip];
	  mainw->blend_file_step=mainw->files[mainw->blend_file]->pb_fps<0?-1:1;
	}
	else do_quick_switch (mainw->clipstore[clip]);
      }
    }
  }
  return TRUE;
}



void
on_toolbar_hide (GtkButton *button,
		 gpointer user_data)
{
  mainw->fixed_height+=mainw->toolbar->allocation.height+4;
  future_prefs->show_tool=FALSE;
  gtk_widget_hide (mainw->toolbar);
}





void
on_capture_activate                (GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
  gint curr_file=mainw->current_file;
  gchar *msg;
  gchar *com;
  gchar **array;

  if (mainw->first_free_file==-1) {
    too_many_files();
    return;
  }

  gtk_widget_hide(mainw->LiVES);

  if (!(do_warning_dialog(_ ("Capture an External Window:\n\nClick on 'OK', then click on any window to capture it\nClick 'Cancel' to cancel\n\n")))) {
    gtk_widget_show(mainw->LiVES);
    gtk_widget_show(mainw->record_perf);
    mainw->foreign=FALSE;
    d_print (_ ("External window was released.\n"));
    while (g_main_context_iteration(NULL,FALSE));
    return;
  }
  
  // an example of using 'get_temp_handle()' ////////
  if (get_temp_handle(mainw->first_free_file)==FALSE) {
    gtk_widget_show(mainw->LiVES);
    while (g_main_context_iteration(NULL,FALSE));
    return;
  }

  com=g_strdup_printf("smogrify get_window_id %s",cfile->handle);
  system(com);
  do_progress_dialog(TRUE,FALSE,_ ("Click on a Window to Capture it\nPress 'q' to stop recording.\n"));
  g_free(com);

  array=g_strsplit(mainw->msg,"|",5);
  mainw->foreign_id=atoi(array[1]);
  mainw->foreign_width=atoi(array[2]);
  mainw->foreign_height=atoi(array[3]);
  mainw->foreign_bpp=atoi(array[4]);
  g_strfreev(array);

  com=g_strdup_printf("smogrify close %s",cfile->handle);
  system(com);
  g_free(com);
  g_free(cfile);
  cfile=NULL;

  mainw->current_file=curr_file;
  ////////////////////////////////////////

  msg=g_strdup_printf(_ ("\nExternal window captured. Width=%d, height=%d, bpp=%d. *Do not resize*\n\nStop or 'q' to finish.\n(Default of %.3f frames per second will be used.)\n"),mainw->foreign_width,mainw->foreign_height,mainw->foreign_bpp,prefs->default_fps);
  d_print(msg);
  g_free (msg);

  // start another copy of LiVES and wait for it to return values
  com=g_strdup_printf("%s -capture %d %d %u %d %d",myname,getpid(),mainw->foreign_id,mainw->foreign_width,mainw->foreign_height,mainw->foreign_bpp);

  // force the dialog to disappear
  while (g_main_context_iteration(NULL,FALSE));

  system(com);

  gtk_widget_show(mainw->LiVES);
  mainw->noswitch=TRUE;
  while (g_main_context_iteration(NULL,FALSE));
  mainw->noswitch=FALSE;

  if (!after_foreign_play()&&mainw->cancelled==0) {
    do_error_dialog(_ ("LiVES was unable to capture this window. Sorry.\n"));
    if (mainw->current_file>-1) {
      sensitize();
    }
  }
}


gboolean
on_capture2_activate(gpointer data) {
  // this is in the second copy of lives, we are now going to grab frames from the X window
  int capture_fd;
  gchar *capfile=g_strdup_printf("%s/.capture.%d",prefs->tmpdir,mainw->foreign_key);
  int i;
  gchar buf[32];
  prepare_to_play_foreign();
  mainw->record_foreign=TRUE;  // for now...

  play_file();

  // pass the handle and frames back to the caller
  capture_fd=creat(capfile,S_IRUSR|S_IWUSR);
  for (i=1;i<MAX_FILES;i++) {
    if (mainw->files[i]==NULL) break;
    write(capture_fd,mainw->files[i]->handle,strlen(mainw->files[i]->handle));
    write(capture_fd,"|",1);
    g_snprintf (buf,32,"%d",cfile->frameno+1);
    write(capture_fd,buf,strlen (buf));
    write(capture_fd,"|",1);
    g_free (buf);
  }
  close(capture_fd);
  g_free(capfile);
  g_free(mainw);

  exit(0);
  return TRUE;
}


void 
on_vid_playback_plugin_changed (GtkEntry *vpp_entry, gpointer user_data) {
  g_snprintf (future_prefs->vpp_name,64,"%s",gtk_entry_get_text(GTK_ENTRY(vpp_entry)));
}





// TODO - move all encoder related stuff from here and plugins.c into encoders.c
void 
on_encoder_ofmt_changed (GtkEntry *fmt_entry, gpointer user_data) {
  // change encoder format in the encoder plugin
  gchar **array;
  GList *ofmt_all=NULL;
  int i;
  gchar *new_fmt=g_strdup(gtk_entry_get_text(GTK_ENTRY(fmt_entry)));

  if (!strlen (new_fmt)) return;

  if (!((ofmt_all=plugin_request_by_line(PLUGIN_ENCODERS,future_prefs->encoder.name,"get_formats"))==NULL)) {
    // get details for the current format
      for (i=0;i<g_list_length(ofmt_all);i++) {
	if (get_token_count (g_list_nth_data (ofmt_all,i),'|')>2) {
	  array=g_strsplit (g_list_nth_data (ofmt_all,i),"|",-1);
	  if (!strcmp(array[1],new_fmt)) {
	    g_signal_handler_block(fmt_entry,prefsw->encoder_ofmt_fn); // TODO
	    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->ofmt_combo))->entry),g_strdup(array[0]));
	    gtk_widget_queue_draw (GTK_COMBO(prefsw->ofmt_combo)->entry);
	    g_signal_handler_unblock(fmt_entry,prefsw->encoder_ofmt_fn);
	    g_snprintf(future_prefs->encoder.of_name,51,"%s",array[0]);

	    future_prefs->encoder.of_allowed_acodecs=atoi(array[2]);
	    g_strfreev (array);
	    break;
	  }
	  g_strfreev (array);
	}
      }
      if (ofmt_all!=NULL) g_list_free (ofmt_all);
      g_free(new_fmt);

      set_acodec_list_from_allowed(prefsw);
  }
  else {
    do_plugin_error(PLUGIN_ENCODERS,future_prefs->encoder.name);
  }
}







//














// TODO - move all this to audio.c

void
on_export_audio_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  if (cfile->end>0&&!GPOINTER_TO_INT (user_data)) {
    fileselection = create_fileselection (_ ("Export Selected Audio as..."),0,NULL);
  }
  else {
    fileselection = create_fileselection (_ ("Export Audio as..."),0,NULL);
  }

  if (strlen(mainw->audio_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->audio_dir,-1,NULL,NULL,NULL));
  }
  gtk_file_selection_complete(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 ("*.wav",-1,NULL,NULL,NULL));
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_ok_export_audio_clicked),user_data);
  gtk_widget_show (fileselection);
}


void
on_ok_export_audio_clicked                      (GtkButton *button,
						 gpointer user_data)
{
  gchar *com;
  gint nrate=cfile->arps;
  gdouble start,end;
  gint asigned=!(cfile->signed_endian&AFORM_UNSIGNED);

  gchar *filename=g_strdup (g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));

  if (rindex(filename,'.')==NULL) {
    g_snprintf(file_name,256,"%s.wav",filename);
  }
  else {
    g_snprintf(file_name,256,"%s",filename);
  }
  g_free (filename);
  
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));

  if (check_file(file_name)==FALSE) return;

  // warn if arps!=arate
  if (prefs->audio_player==AUD_PLAYER_SOX&&cfile->arate!=cfile->arps) {
    if (do_warning_dialog(_ ("\n\nThe audio playback speed has been altered for this clip.\nClick 'OK' to export at the new speed, or 'Cancel' to export at the original rate.\n"))==TRUE) {
      nrate=cfile->arate;
    }
  }
  
  if (cfile->start*cfile->end>0&&!GPOINTER_TO_INT (user_data)) {
    g_snprintf (mainw->msg,256,_ ("Exporting audio frames %d to %d as %s..."),cfile->start,cfile->end,file_name);
    start=calc_variable_time (cfile->start);
    end=calc_variable_time (cfile->end);
  }
  else {
    g_snprintf (mainw->msg,256,_ ("Exporting audio as %s..."),file_name);
    start=0.;
    end=cfile->laudio_time;
  }

  d_print (mainw->msg);
  
  com=g_strdup_printf ("smogrify export_audio %s %.8f %.8f %d %d %d %d %d \"%s\"",cfile->handle,start,end,cfile->arps,cfile->achans,cfile->asampsize,asigned,nrate,g_filename_from_utf8 (file_name,-1,NULL,NULL,NULL));
 
  unlink (cfile->info_file);
  system (com);
  do_progress_dialog (TRUE, FALSE, _("Exporting audio"));
  g_free (com);

  if (mainw->error==TRUE) {
    d_print (_ ("failed\n"));
    do_error_dialog (mainw->msg);
  }
  else {
    d_print (_ ("done\n"));
    get_dirname (file_name);
    g_snprintf (mainw->audio_dir,256,"%s",file_name);
  }
}


void
on_append_audio_activate (GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *fileselection;

  fileselection = create_fileselection (_ ("Append Audio File..."),2,NULL);
  if (strlen(mainw->audio_dir)) {
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), g_filename_from_utf8 (mainw->audio_dir,-1,NULL,NULL,NULL));
  }
  g_signal_connect (GTK_FILE_SELECTION(fileselection)->ok_button, "clicked",G_CALLBACK (on_ok_append_audio_clicked),NULL);
  gtk_widget_show (fileselection);
}




void
on_ok_append_audio_clicked                      (GtkButton *button,
						 GtkEntry *entry)
{
  gchar *com;
  gchar *a_type;
  gint asigned=!(cfile->signed_endian&AFORM_UNSIGNED);
  gint aendian=!(cfile->signed_endian&AFORM_BIG_ENDIAN);

  g_snprintf(file_name,256,"%s",g_filename_to_utf8 (gtk_file_selection_get_filename(GTK_FILE_SELECTION(gtk_widget_get_toplevel(GTK_WIDGET(button)))),-1,NULL,NULL,NULL));

  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

  gtk_widget_queue_draw(mainw->LiVES);
  while (g_main_context_iteration(NULL,FALSE));
  
  a_type=file_name+strlen(file_name)-3;

  if (!strcmp(a_type,"mp3")||!strcmp(a_type,"ogg")||!strcmp(a_type,"wav")||!strcmp(a_type,"mod")||!strcmp(a_type,"xm")) com=g_strdup_printf ("smogrify append_audio %s %s %d %d %d %d %d \"%s\"",cfile->handle,a_type,cfile->arate,cfile->achans,cfile->asampsize,asigned,aendian,g_filename_from_utf8 (file_name,-1,NULL,NULL,NULL));
  else {
    do_error_dialog(_ ("Sorry, unknown audio type.\n\n (Filenames must end in .mp3, .ogg, .wav, .mod or .xm)"));
    d_print(_ ("failed (unknown type)\n"));
    return;
  }

  g_snprintf (mainw->msg,256,_ ("Appending audio file %s..."),file_name);
  d_print (mainw->msg);
 
  unlink (cfile->info_file);
  system (com);

  if (do_progress_dialog (TRUE, TRUE,_ ("Appending audio"))==FALSE) {
    g_free (com);
    gtk_widget_queue_draw(mainw->LiVES);
    while (g_main_context_iteration(NULL,FALSE));
    com=g_strdup_printf ("smogrify cancel_audio %s",cfile->handle);
    system (com);
    do_auto_dialog(_ ("Cancelling"));
    g_free (com);
    return;
  }
  g_free (com);

  if (mainw->error==TRUE) {
    d_print (_ ("failed\n"));
    do_error_dialog (mainw->msg);
  }
  else {
    gtk_widget_queue_draw(mainw->LiVES);
    while (g_main_context_iteration(NULL,FALSE));
    com=g_strdup_printf ("smogrify commit_audio %s",cfile->handle);
    system (com);
    do_auto_dialog(_ ("Committing audio"));
    g_free (com);
    if (mainw->error==TRUE) {
      d_print (_ ("failed\n"));
      do_error_dialog (mainw->msg);
    }
    else {
      get_dirname (file_name);
      g_snprintf (mainw->audio_dir,256,"%s",file_name);
      reget_afilesize(cfile);
      cfile->changed=TRUE;
      get_play_times();
      d_print (_ ("done\n"));
    }
  }
  cfile->undo_action=UNDO_APPEND_AUDIO;
  set_undoable ("Append Audio",!prefs->conserve_space);
}



void
on_trim_audio_activate (GtkMenuItem     *menuitem,
			gpointer         user_data)
{
  gchar *com,*msg;
  gint type=GPOINTER_TO_INT (user_data);

  gdouble start,end;

  if (type==0) {
    start=calc_variable_time (cfile->start);
    end=calc_variable_time (cfile->end+1);
  }
  else {
    start=0.;
    end=cfile->pointer_time;
  }

  msg=g_strdup_printf(_ ("Trimming audio from %.2f to %.2f seconds..."),start,end);
  d_print(msg);
  g_free(msg);

  com=g_strdup_printf("smogrify trim_audio %s %.8f %.8f %d %d %d", cfile->handle, start, end, cfile->arate, cfile->achans, cfile->asampsize);
  unlink (cfile->info_file);
  system (com);
  do_progress_dialog(TRUE, FALSE, _("Trimming/Padding audio"));
  g_free(com);

  set_undoable ("Trim/Pad Audio",!prefs->conserve_space);

  cfile->undo_action=UNDO_TRIM_AUDIO;

  reget_afilesize(cfile);
  get_play_times();
  cfile->changed=TRUE;
  d_print(_ ("done\n"));
}


void
on_del_audio_activate (GtkMenuItem     *menuitem,
		       gpointer         user_data)
{
  gdouble start,end;
  gchar *com,*msg=NULL;

  if (menuitem==NULL) {
    // undo/redo
    start=cfile->undo1_dbl;
    end=cfile->undo2_dbl;
  }
  else {
    if (GPOINTER_TO_INT(user_data)) {
      if (!cfile->frames) {
	if (do_warning_dialog(_ ("\nDeleting all audio will close this file.\nAre you sure ?"))==TRUE) close_current_file();
	return;
      }
      start=0.;
      end=1000000000.;
      msg=g_strdup(_ ("Deleting all audio..."));
    }
    else {
      start=calc_variable_time (cfile->start);
      end=calc_variable_time (cfile->end+1);
      msg=g_strdup_printf(_ ("Deleting audio from %.2f to %.2f seconds..."),start,end);
      start*=(gdouble)cfile->arate/(gdouble)cfile->arps;
      end*=(gdouble)cfile->arate/(gdouble)cfile->arps;
    }
    cfile->undo1_dbl=start;
    cfile->undo2_dbl=end;
  }
  
  cfile->undo_arate=cfile->arate;
  cfile->undo_signed_endian=cfile->signed_endian;
  cfile->undo_achans=cfile->achans;
  cfile->undo_asampsize=cfile->asampsize;
  cfile->undo_arps=cfile->arps;

  if (msg!=NULL) {
    d_print(msg);
    g_free(msg);
  }

  com=g_strdup_printf("smogrify delete_audio %s %.8f %.8f %d %d %d", cfile->handle, start, end, cfile->arps, cfile->achans, cfile->asampsize);
  unlink (cfile->info_file);
  system (com);
  do_progress_dialog(TRUE, FALSE, _ ("Deleting Audio"));
  g_free(com);

  set_undoable ("Delete Audio",TRUE);
  cfile->undo_action=UNDO_DELETE_AUDIO;

  reget_afilesize(cfile);
  get_play_times();
  cfile->changed=TRUE;
  sensitize();

  if (menuitem!=NULL) {
    d_print(_ ("done\n"));
  }
}


void
on_ins_silence_activate (GtkMenuItem     *menuitem,
			 gpointer         user_data)
{
  gdouble start,end;
  gchar *com,*msg;

  if (menuitem==NULL) {
    // redo
    start=cfile->undo1_dbl;
    end=cfile->undo2_dbl;
    cfile->arate=cfile->undo_arate;
    cfile->signed_endian=cfile->undo_signed_endian;
    cfile->achans=cfile->undo_achans;
    cfile->asampsize=cfile->undo_asampsize;
    cfile->arps=cfile->undo_arps;
  }

  if (!cfile->achans) {
    mainw->fx1_val=DEFAULT_AUDIO_RATE;
    mainw->fx2_val=DEFAULT_AUDIO_CHANS;
    mainw->fx3_val=DEFAULT_AUDIO_SAMPS;
    mainw->fx4_val=mainw->endian;
    resaudw=create_resaudw(2);
    gtk_widget_show (resaudw->dialog);
    return;
  }

  start=calc_variable_time (cfile->start);
  end=calc_variable_time (cfile->end+1);

  if (menuitem!=NULL) {
    msg=g_strdup_printf(_ ("Inserting silence from %.2f to %.2f seconds..."),start,end);
    d_print(msg);
    g_free(msg);
  }

  cfile->undo1_dbl=start*=(gdouble)cfile->arate/(gdouble)cfile->arps;
  cfile->undo2_dbl=end*=(gdouble)cfile->arate/(gdouble)cfile->arps;

  com=g_strdup_printf("smogrify insert %s %.8f 0. %.8f %s 2 0 0 0 0 %d %d %d -1",cfile->handle, start, end-start, cfile->handle, -cfile->arps, cfile->achans, cfile->asampsize);

  unlink (cfile->info_file);
  system (com);
  do_progress_dialog(TRUE, FALSE, _("Inserting Silence"));
  g_free(com);

  set_undoable ("Insert Silence",TRUE);
  cfile->undo_action=UNDO_INSERT_SILENCE;

  reget_afilesize(cfile);
  get_play_times();
  cfile->changed=TRUE;
  sensitize();
  d_print(_ ("done\n"));
}



void
on_ins_silence_details_clicked                      (GtkButton *button,
						     GtkEntry *entry)
{
  int asigned=1,aendian=1;

  cfile->arps=cfile->arate=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_arate)));
  cfile->achans=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_achans)));
  cfile->asampsize=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_asamps)));
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->rb_unsigned))==TRUE) {
    asigned=0;
  }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->rb_bigend))==TRUE) {
    aendian=0;
  }
  cfile->signed_endian=get_signed_endian(asigned,aendian);
  gtk_widget_destroy (resaudw->dialog);
  while (g_main_context_iteration(NULL,FALSE));
  g_free (resaudw);
  if (cfile->arate<=0) {
    do_error_dialog (_ ("\n\nRate must be greater than 0\n"));
    cfile->achans=cfile->arate=cfile->asampsize=0;
    return;
  }
  on_ins_silence_activate(GTK_MENU_ITEM(mainw->ins_silence),NULL);
}



